/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

/*
 * Copyright (c) 2002-2010 Neterion, Inc.
 * All right Reserved.
 *
 * FileName :   vxge-os-template.h
 *
 * Description:  Template for creating platform-dependent "glue" code.
 *
 * Created:       27 December 2006
 */

#ifndef	VXGE_OS_TEMPLATE_H
#define	VXGE_OS_TEMPLATE_H

#ifndef	TEMPLATE
#error "should not be compiled for platforms other than TEMPLATE..."
#endif

/* ------------------------- includes and defines ------------------------- */

/*
 * Note:
 *
 *	 - on some operating systems like Linux & FreeBSD, there is a macro
 *	   by using which it is possible to determine endiennes automatically
 */
#define	VXGE_OS_HOST_BIG_ENDIAN				TEMPLATE

#define	VXGE_OS_HOST_PAGE_SIZE				TEMPLATE

#define	VXGE_OS_PCI_CONFIG_SIZE				TEMPLATE

/* ---------------------- fixed size primitive types ----------------------- */

/*
 * Note:
 *
 *	- u## - means ## bits unsigned int/long
 *	  - all names must be preserved since HAL using them.
 *	  - ulong_t is platform specific, i.e. for 64bit - 64bit size, for
 *	32bit - 32bit size
 */
#define	TEMPLATE					u8
#define	TEMPLATE					u16
#define	TEMPLATE					u32
#define	TEMPLATE					u64
#define	TEMPLATE					ptr_t
#define	TEMPLATE					ulong_t
#define	TEMPLATE					ptrdiff_t
#define	TEMPLATE					dma_addr_t
#define	TEMPLATE					spinlock_t
#define	TEMPLATE					event_t
typedef TEMPLATE					pci_dev_h;
typedef TEMPLATE					pci_reg_h;
typedef TEMPLATE					pci_dma_h;
typedef TEMPLATE					pci_irq_h;
typedef TEMPLATE					pci_cfg_h;
typedef TEMPLATE					pci_dma_acc_h;

/* -------------------------- "libc" functionality ------------------------- */

/*
 * Note:
 *
 *	- "libc" functionality maps one-to-one to be posix-like
 */
/* Note: use typedef: vxge_os_memzero(void* mem, int size); */
#define	vxge_os_memzero					TEMPLATE

/*
 * Note: the 1st argument MUST be destination, like in:
 * void *memcpy(void *dest, const void *src, ptr_t n);
 */
#define	vxge_os_memcpy					TEMPLATE

#define	vxge_os_strcpy(dest, src)			TEMPLATE

static inline u64 vxge_os_htonll(u64 val64)
{ TEMPLATE; }

static inline u64 vxge_os_ntohll(u64 val64)
{ TEMPLATE; }


/*
 * Note: should accept format (the 1st argument) and a variable
 * number of arguments thereafter..
 */
#define	vxge_os_printf(fmt...)				TEMPLATE

#define	vxge_os_vaprintf(hldev, vpid, fmt...)		TEMPLATE

#define	vxge_os_vasprintf(hldev, vpid, fmt...)		TEMPLATE

#define	vxge_os_sprintf(buf, fmt, ...)			TEMPLATE

#define	vxge_os_timestamp(buf)				TEMPLATE

#define	vxge_os_println					TEMPLATE

/* -------------------- synchronization primitives ------------------------- */

/*
 * Note:
 *
 *	- use spin_lock in interrupts or in threads when there is no races
 *	 with interrupt
 *	- use spin_lock_irqsave in threads if there is a race with interrupt
 *	- use spin_lock_irqsave for nested locks
 */

/*
 * Initialize the spin lock.
 */
#define	vxge_os_spin_lock_init(lockp, ctxh)		TEMPLATE
/*
 * Initialize the spin lock (IRQ version).
 */
#define	vxge_os_spin_lock_init_irq(lockp, ctxh)		TEMPLATE
/*
 * Destroy the lock.
 */
#define	vxge_os_spin_lock_destroy(lockp, ctxh)		TEMPLATE

/*
 * Destroy the lock (IRQ version).
 */
#define	vxge_os_spin_lock_destroy_irq(lockp, ctxh)	TEMPLATE
/*
 * Acquire the lock.
 */
#define	vxge_os_spin_lock(lockp)				TEMPLATE
/*
 * Release the lock.
 */
#define	vxge_os_spin_unlock(lockp)			TEMPLATE
/*
 * Acquire the lock(IRQ version).
 */
#define	vxge_os_spin_lock_irq(lockp, flags)		TEMPLATE
/*
 * Release the lock(IRQ version).
 */
#define	vxge_os_spin_unlock_irq(lockp, flags)		TEMPLATE
/*
 * Initialize the event.
 */
#define	vxge_os_event_init(eventp, ctx)			TEMPLATE
/*
 * Set the event).
 */
#define	vxge_os_event_set(eventp)			TEMPLATE
/*
 * Reset the event.
 */
#define	vxge_os_event_reset(eventp)			TEMPLATE
/*
 * Wait for event, ms=0 means indefinitely.
 */
#define	vxge_os_event_wait(eventp, ms)			TEMPLATE
/*
 * Destroy the event.
 */
#define	vxge_os_event_destroy(eventp, ctx)		TEMPLATE
/* ------------------------- misc primitives ------------------------------- */

#define	vxge_os_prefetch					TEMPLATE
#define	vxge_os_prefetchw				TEMPLATE
#define	vxge_os_bug(fmt...)				TEMPLATE

/* -------------------------- compiler stuffs ------------------------------ */

#define	__vxge_os_attr_cacheline_aligned			TEMPLATE

/* -------------------------- System stuffs ------------------------------ */
/*
 * Write memory barrier.
 */
#define	vxge_os_wmb()					TEMPLATE
/*
 * Delay (in micro seconds).
 */
#define	vxge_os_udelay(us)				TEMPLATE
/*
 * Delay (in milli seconds).
 */
#define	vxge_os_mdelay(ms)				TEMPLATE
/*
 * Compare and exchange.
 */
#define	vxge_os_cmpxchg(targetp, cmp, newval)		TEMPLATE

/*
 * exchange.
 */
#define	vxge_os_xchg(targetp, newval)			TEMPLATE

/*
 * Atomic increment
 */
#define	vxge_os_atomic_increment(counter)		TEMPLATE

/*
 * Atomic decrement
 */
#define	vxge_os_atomic_decrement(counter)		TEMPLATE

/*
 * Current time in milli seconds (tick count converted to absolute milliseconds
 * since start of the os specific origin date)
 */
static inline u64 vxge_os_curr_time()
{ TEMPLATE; }

/* ---------------------- memory primitives -------------------------------- */

/*
 * vxge_os_malloc - Allocate non DMA-able memory.
 * @pdev: Device context. Some OSs require device context to perform
 *	operations on memory.
 * @size: Size to allocate.
 *
 * Allocate @size bytes of memory. This allocation can sleep, and
 * therefore, and therefore it requires process context. In other words,
 * vxge_os_malloc() cannot be called from the interrupt context.
 * Use vxge_os_free() to free the allocated block.
 *
 * Returns: Pointer to allocated memory, NULL - on failure.
 *
 * See also: vxge_os_free().
 */
static inline void *vxge_os_malloc(IN  pci_dev_h pdev,
				IN  unsigned long size)
{ TEMPLATE; }

/*
 * vxge_os_free - Free non DMA-able memory.
 * @pdev: Device context. Some OSs require device context to perform
 *	operations on memory.
 * @vaddr: Address of the allocated memory block.
 * @size: Some OS's require to provide size on free
 *
 * Free the memory area obtained via vxge_os_malloc().
 * This call may also sleep, and therefore it cannot be used inside
 * interrupt.
 *
 * See also: vxge_os_malloc().
 */
static inline void vxge_os_free(IN  pci_dev_h pdev,
			IN  const void *vaddr,
			IN unsigned long size)
{ TEMPLATE; }

/*
 * vxge_os_vaddr - Get Virtual address for the given physical address.
 * @pdev: Device context. Some OSs require device context to perform
 *	operations on memory.
 * @vaddr: Physical Address of the memory block.
 * @size: Some OS's require to provide size
 *
 * Get the virtual address for physical address.
 * This call may also sleep, and therefore it cannot be used inside
 * interrupt.
 *
 * See also: vxge_os_malloc().
 */
static inline void vxge_os_vaddr(IN  pci_dev_h pdev,
			IN  const void *vaddr,
			IN unsigned long size)
{ TEMPLATE; }

/*
 * vxge_os_dma_malloc  -  Allocate DMA-able memory.
 * @pdev: Device context. Used to allocate/pin/map/unmap DMA-able memory.
 * @size: Size (in bytes) to allocate.
 * @dma_flags: VXGE_OS_DMA_CACHELINE_ALIGNED,
 *		VXGE_OS_DMA_STREAMING,
 *		VXGE_OS_DMA_CONSISTENT
 *	Note that the last two flags are mutually exclusive.
 * @p_dmah: Handle used to map the memory onto the corresponding device memory
 *	 space. See vxge_os_dma_map(). The handle is an out-parameter
 *	 returned by the function.
 * @p_dma_acch: One more DMA handle used subsequently to free the
 *		 DMA object (via vxge_os_dma_free()).
 *		 Note that this and the previous handle have
 *		 physical meaning for Solaris; on Windows and Linux the
 *		 corresponding value will be simply a pointer to PCI device.
 *		 The value is returned by this function.
 *
 * Allocate DMA-able contiguous memory block of the specified @size.
 * This memory can be subsequently freed using vxge_os_dma_free().
 * Note: can be used inside interrupt context.
 *
 * Returns: Pointer to allocated memory(DMA-able), NULL on failure.
 *
 */
static inline void *vxge_os_dma_malloc(IN  pci_dev_h pdev,
				IN  unsigned long size,
				IN  int dma_flags,
				OUT pci_dma_h *p_dmah,
				OUT pci_dma_acc_h *p_dma_acch)
{ TEMPLATE; }

/*
 * vxge_os_dma_malloc_async  -  Allocate DMA-able memory asynchronously.
 * @pdev: Device context. Used to allocate/pin/map/unmap DMA-able memory.
 * @devh: Handle to hal device to be passed to vxge_hal_blockpool_block_add()
 * @size: Size (in bytes) to allocate.
 * @dma_flags: VXGE_OS_DMA_CACHELINE_ALIGNED,
 *		VXGE_OS_DMA_STREAMING,
 *		VXGE_OS_DMA_CONSISTENT
 *	Note that the last two flags are mutually exclusive.
 *
 * Allocate DMA-able contiguous memory block of the specified @size.
 * This memory can be subsequently freed using vxge_os_dma_free().
 * Note: can be used inside interrupt context.
 *
 * Returns: calls the vxge_hal_blockpool_block_add() routine to add the block
 *
 */
static inline void vxge_os_dma_malloc_async(IN  pci_dev_h pdev,
				IN  pci_dev_h devh,
				IN  unsigned long size,
				IN  int dma_flags)
{ TEMPLATE; }

/*
 * vxge_os_dma_free - Free previously allocated DMA-able memory.
 * @pdev: Device context. Used to allocate/pin/map/unmap DMA-able memory.
 * @vaddr: Virtual address of the DMA-able memory.
 * @size: Size of the buffer
 * @dma_flags: flags used whiile allocating
 * @p_dma_acch: DMA handle used to free the resource.
 * @p_dmah: DMA handle used for mapping. See vxge_os_dma_malloc().
 *
 * Free DMA-able memory originally allocated by vxge_os_dma_malloc().
 * Note: can be used inside interrupt.
 * See also: vxge_os_dma_malloc().
 */
static inline void vxge_os_dma_free(IN  pci_dev_h pdev,
				IN  const void *vaddr,
				IN  ptr_t size,
				IN  int dma_flags,
				IN  pci_dma_h *p_dmah,
				IN  pci_dma_acc_h *p_dma_acch)
{ TEMPLATE; }

/* ----------------------- io/pci/dma primitives --------------------------- */

#define	VXGE_OS_DMA_DIR_TODEVICE			TEMPLATE
#define	VXGE_OS_DMA_DIR_FROMDEVICE			TEMPLATE
#define	VXGE_OS_DMA_DIR_BIDIRECTIONAL			TEMPLATE

/*
 * vxge_os_pci_write8 - Write one byte into device PCI configuration.
 * @pdev: Device context. Some OSs require device context to perform
 *	PIO and/or config space IO.
 * @cfgh: PCI configuration space handle.
 * @where: Offset in the PCI configuration space.
 * @val: Value to write.
 *
 * Write byte value into the specified PCI configuration space
 * Returns: 0 - success, non-zero - failure.
 */
static inline int vxge_os_pci_write8(IN  pci_dev_h pdev,
				IN  pci_cfg_h cfgh,
				IN  int where,
				IN  u8 val)
{ TEMPLATE; }

/*
 * vxge_os_pci_write16 - Write 16bit word into device PCI configuration.
 * @pdev: Device context. Some OSs require device context to perform
 *	PIO and/or config space IO.
 * @cfgh: PCI configuration space handle.
 * @where: Offset in the PCI configuration space.
 * @val: Value to write.
 *
 * Write 16bit value into the specified @offset in PCI
 * configuration space.
 * Returns: 0 - success, non-zero - failure.
 */
static inline int vxge_os_pci_write16(IN  pci_dev_h pdev,
				IN  pci_cfg_h cfgh,
				IN  int where,
				IN  u16 val)
{ TEMPLATE; }

/*
 * vxge_os_pci_write32 - Write 32bit word into device PCI configuration.
 * @pdev: Device context. Some OSs require device context to perform
 *	PIO and/or config space IO.
 * @cfgh: PCI configuration space handle.
 * @where: Offset in the PCI configuration space.
 * @val: Value to write.
 *
 * Write 32bit value into the specified @offset in PCI
 * configuration space.
 * Returns: 0 - success, non-zero - failure.
 */
static inline int vxge_os_pci_write32(IN  pci_dev_h pdev,
				IN  pci_cfg_h cfgh,
				IN  int where,
				IN  u32 val)
{ TEMPLATE; }

/*
 * vxge_os_pio_mem_read8 - Read 1 byte from device memory mapped space.
 * @pdev: Device context. Some OSs require device context to perform
 *	PIO and/or config space IO..
 * @regh: PCI configuration space handle.
 * @addr: Address in device memory space.
 *
 * Returns: 1 byte value read from the specified (mapped) memory space address.
 */
static inline u8 vxge_os_pio_mem_read8(IN  pci_dev_h pdev,
				IN  pci_reg_h regh,
				IN void *addr)
{ TEMPLATE; }

/*
 * vxge_os_pio_mem_write8 - Write 1 byte into device memory mapped
 * space.
 * @pdev: Device context. Some OSs require device context to perform
 *	PIO and/or config space IO..
 * @regh: PCI configuration space handle.
 * @val: Value to write.
 * @addr: Address in device memory space.
 *
 * Write byte value into the specified (mapped) device memory space.
 */
static inline void vxge_os_pio_mem_write8(IN  pci_dev_h pdev,
					IN  pci_reg_h regh,
					IN  u8 val,
					IN  void *addr)
{ TEMPLATE; }

/*
 * vxge_os_pio_mem_read16 - Read 16bit from device memory mapped space.
 * @pdev: Device context. Some OSs require device context to perform
 *	PIO.
 * @regh: PCI configuration space handle.
 * @addr: Address in device memory space.
 *
 * Returns: 16bit value read from the specified (mapped) memory space address.
 */
static inline u16 vxge_os_pio_mem_read16(IN  pci_dev_h pdev,
					IN  pci_reg_h regh,
					IN  void *addr)
{ TEMPLATE; }

/*
 * vxge_os_pio_mem_write16 - Write 16bit into device memory mapped space.
 * @pdev: Device context. Some OSs require device context to perform
 *	PIO.
 * @regh: PCI configuration space handle.
 * @val: Value to write.
 * @addr: Address in device memory space.
 *
 * Write 16bit value into the specified (mapped) device memory space.
 */
static inline void vxge_os_pio_mem_write16(IN  pci_dev_h pdev,
					IN  pci_reg_h regh,
					IN  u16 val,
					IN  void *addr)
{ TEMPLATE; }

/*
 * vxge_os_pio_mem_read32 - Read 32bit from device memory mapped space.
 * @pdev: Device context. Some OSs require device context to perform
 *	PIO.
 * @regh: PCI configuration space handle.
 * @addr: Address in device memory space.
 *
 * Returns: 32bit value read from the specified (mapped) memory space address.
 */
static inline u32 vxge_os_pio_mem_read32(IN  pci_dev_h pdev,
					IN  pci_reg_h regh,
					IN  void *addr)
{ TEMPLATE; }

/*
 * vxge_os_pio_mem_write32 - Write 32bit into device memory space.
 * @pdev: Device context. Some OSs require device context to perform
 *	PIO.
 * @regh: PCI configuration space handle.
 * @val: Value to write.
 * @addr: Address in device memory space.
 *
 * Write 32bit value into the specified (mapped) device memory space.
 */
static inline void vxge_os_pio_mem_write32(IN  pci_dev_h pdev,
					IN  pci_reg_h regh,
					IN  u32 val,
					IN  void *addr)
{ TEMPLATE; }

/*
 * vxge_os_pio_mem_read64 - Read 64bit from device memory mapped space.
 * @pdev: Device context. Some OSs require device context to perform
 *	PIO.
 * @regh: PCI configuration space handle.
 * @addr: Address in device memory space.
 *
 * Returns: 64bit value read from the specified (mapped) memory space address.
 */
static inline u64 vxge_os_pio_mem_read64(IN  pci_dev_h pdev,
					IN  pci_reg_h regh,
					IN  void *addr)
{ TEMPLATE; }

/*
 * vxge_os_pio_mem_write64 - Write 64bit into device memory space.
 * @pdev: Device context. Some OSs require device context to perform
 *	PIO.
 * @regh: PCI configuration space handle.
 * @val: Value to write.
 * @addr: Address in device memory space.
 *
 * Write 64bit value into the specified (mapped) device memory space.
 */
static inline void vxge_os_pio_mem_write64(IN  pci_dev_h pdev,
					IN  pci_reg_h regh,
					IN  u64 val,
					IN  void *addr)
{ TEMPLATE; }

/*
 * vxge_os_flush_bridge - Flush the bridge.
 * @pdev: Device context. Some OSs require device context to perform
 *	PIO.
 * @regh: PCI configuration space handle.
 * @addr: Address in device memory space.
 *
 * Flush the bridge.
 */
static inline void vxge_os_flush_bridge(IN  pci_dev_h pdev,
				IN  pci_reg_h regh,
				IN  void *addr)
{ TEMPLATE; }

/*
 * vxge_os_dma_map - Map DMA-able memory block to, or from, or
 * to-and-from device.
 * @pdev: Device context. Used to allocate/pin/map/unmap DMA-able memory.
 * @dmah: DMA handle used to map the memory block. Obtained via
 * vxge_os_dma_malloc().
 * @vaddr: Virtual address of the DMA-able memory.
 * @size: Size (in bytes) to be mapped.
 * @dir: Direction of this operation (VXGE_OS_DMA_DIR_TODEVICE, etc.)
 * @dma_flags: VXGE_OS_DMA_CACHELINE_ALIGNED,
 *		VXGE_OS_DMA_STREAMING,
 *		VXGE_OS_DMA_CONSISTENT
 *	Note that the last two flags are mutually exclusive.
 *
 * Map a single memory block.
 *
 * Returns: DMA address of the memory block,
 * VXGE_OS_INVALID_DMA_ADDR on failure.
 *
 * See also: vxge_os_dma_malloc(), vxge_os_dma_unmap(),
 * vxge_os_dma_sync().
 */
static inline dma_addr_t vxge_os_dma_map(IN  pci_dev_h pdev,
					IN  pci_dma_h dmah,
					IN  void *vaddr,
					IN  ptr_t size,
					IN  int dir,
					IN  int dma_flags)
{ TEMPLATE; }

/*
 * vxge_os_dma_unmap - Unmap DMA-able memory.
 * @pdev: Device context. Used to allocate/pin/map/unmap DMA-able memory.
 * @dmah: DMA handle used to map the memory block. Obtained via
 * vxge_os_dma_malloc().
 * @dma_addr: DMA address of the block. Obtained via vxge_os_dma_map().
 * @size: Size (in bytes) to be unmapped.
 * @dir: Direction of this operation (VXGE_OS_DMA_DIR_TODEVICE, etc.)
 *
 * Unmap a single DMA-able memory block that was previously mapped
 * using vxge_os_dma_map().
 * See also: vxge_os_dma_malloc(), vxge_os_dma_map().
 */
static inline void vxge_os_dma_unmap(IN  pci_dev_h pdev,
				IN  pci_dma_h dmah,
				IN  dma_addr_t dma_addr,
				IN  ptr_t size,
				IN  int dir)
{ TEMPLATE; }

/*
 * vxge_os_dma_sync - Synchronize mapped memory.
 * @pdev: Device context. Used to allocate/pin/map/unmap DMA-able memory.
 * @dmah: DMA handle used to map the memory block. Obtained via
 * vxge_os_dma_malloc().
 * @dma_addr: DMA address of the block. Obtained via vxge_os_dma_map().
 * @dma_offset: Offset from start of the blocke. Used by Solaris only.
 * @length: Size of the block.
 * @dir: Direction of this operation (VXGE_OS_DMA_DIR_TODEVICE, etc.)
 *
 * Make physical and CPU memory consistent for a single
 * streaming mode DMA translation.
 * This API compiles to NOP on cache-coherent platforms.
 * On non cache-coherent platforms, depending on the direction
 * of the "sync" operation, this API will effectively
 * either invalidate CPU cache (that might contain old data),
 * or  flush CPU cache to update physical memory.
 * See also: vxge_os_dma_malloc(), vxge_os_dma_map(),
 * vxge_os_dma_unmap().
 */
static inline void vxge_os_dma_sync(IN  pci_dev_h pdev,
				IN  pci_dma_h dmah,
				IN  dma_addr_t dma_addr,
				IN  u64 dma_offset,
				IN  ptr_t length,
				IN  int dir)
{ TEMPLATE; }

#endif /* VXGE_OS_TEMPLATE_H */
