/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

/*
 * Copyright (c) 2002-2010 Neterion Inc.
 * All right Reserved.
 *
 * FileName :   vxgell-rnic.h
 *
 * Description:
 *
 * Author:         Prasad Kanneganti
 *
 * Created:       October 5, 2006
 */

#ifndef	VXGE_RNIC_H
#define	VXGE_RNIC_H

#include "vxgehal-rnic.h"
#include "vxge-rnic-osdep.h"

__EXTERN_BEGIN_DECLS

#define	VXGE_RNIC_MAX_PHYADDR_LENGTH				32

#define	VXGE_RNIC_IF_ADMIN_STATUS_UP				1
#define	VXGE_RNIC_IF_ADMIN_STATUS_DOWN				2
#define	VXGE_RNIC_IF_ADMIN_STATUS_TESTING			3

#define	VXGE_RNIC_IF_OPER_STATUS_NON_OPERATIONAL		1
#define	VXGE_RNIC_IF_OPER_STATUS_UNREACHABLE WAN		2
#define	VXGE_RNIC_IF_OPER_STATUS_DISCONNECTED			3
#define	VXGE_RNIC_IF_OPER_STATUS_CONNECTING			4
#define	VXGE_RNIC_IF_OPER_STATUS_CONNECTED			5
#define	VXGE_RNIC_IF_OPER_STATUS_OPERATIONAL			6

typedef void *vxge_ll_adapter_h;

/*
 * struct vxge_ll_version_t - Link Layer interface version info
 * @version_major: Major version
 * @version_minor: Minor version
 * @version_fix: version fix
 * @version_build: Version Build
 *
 * Structure to store version info
 */
typedef struct vxge_ll_version_t {
	u32 version_major;
	u32 version_minor;
	u32 version_fix;
	u32 version_build;
} vxge_ll_version_t;

/*
 * struct vxge_ll_rnic_packet_t - Packet used to pass the unaligned data
 *			from the rnic to linklayer
 * @TotalLength: Total amount of data in the packet.
 * @buffer_list: List of buffers
 * @buffer_list_length: Length of buffer list
 * @vlan_tag: Vlan tag
 * @cksum: Checksum value
 * @flags: Set of flags for the packet
 *
 * Structure to pass unaligned data from rnic to link layer for reinsertion
 */
typedef struct vxge_ll_rnic_packet_t {
	u32 TotalLength;
	vxge_hal_sg_list_t *buffer_list;
	u32 buffer_list_length;
	u32 vlan_tag;
	u32 cksum;
	u32 flags;
#define	VXGE_LL_RNIC_PACKET_HIP_SEGMENT			0x00000001
#define	VXGE_LL_RNIC_PACKET_HIP_FPDU			0x00000002
#define	VXGE_LL_RNIC_PACKET_CKSUM_V4			0x00000004
#define	VXGE_LL_RNIC_PACKET_CKSUM_V6			0x00000008
#define	VXGE_LL_RNIC_PACKET_CKSUM_TCP			0x00000010
#define	VXGE_LL_RNIC_PACKET_CKSUM_UDP			0x00000020
#define	VXGE_LL_RNIC_PACKET_CKSUM_IP			0x00000040
} vxge_ll_rnic_packet_t;

/*
 * struct vxge_version_t - Link Layer version info
 * @version_major: Major version
 * @version_minor: Minor version
 * @version_fix: version fix
 * @version_build: Version Build
 *
 * Structure to store version info
 */
typedef struct vxge_version_t {
	u32 version_major;
	u32 version_minor;
	u32 version_fix;
	u32 version_build;
} vxge_version_t;

/*
 * struct vxge_ll_rnic_vp_info_t - Virtual Path Info
 * @vp_id: virtual Path Id
 *
 * This structure is used to pass the virtual path info from miniport
 * to rnic driver
 *
 */
typedef struct vxge_ll_rnic_vp_info_t {
	u32 vp_id;
} vxge_ll_rnic_vp_info_t;


/*
 * vxge_ll_rnic_device_notification_callback_f - Call back function for device
 *			 noptification
 * @hldev: Handle to the HAL device object
 * @pdev: Handle to adapter device object
 * @ifname: Interface name
 * @description: Interface description
 * @cur_phyaddr_len: Current MAC Address Length
 * @cur_phyaddr: Current MAC Address
 * @perm_phyaddr_len: Permanent MAC Address Length
 * @perm_phyaddr: Permanent MAC address
 * @number_of_vps: Number of virtual paths
 * @vp_info: Information of Virtual paths
 * @removed: TRUE if the device is removed. FALSE otherwise
 * @admin_status: Administrative status of interface
 * @operational_status: Operational status of interface
 * @userdata: User data specified at the registration of callback
 *
 * This function is the prototype of the callback to be registered by the
 * RNICIF to receive the configuration of initialized devices
 */
typedef void (*vxge_ll_rnic_device_notification_callback_f)(
			vxge_hal_device_h hldev,
			vxge_ll_adapter_h pdev,
			u8 *ifname,
			u8 *description,
			u32 cur_phyaddr_len,
			u8  cur_phyaddr[VXGE_RNIC_MAX_PHYADDR_LENGTH],
			u32 perm_phyaddr_len,
			u8  perm_phyaddr[VXGE_RNIC_MAX_PHYADDR_LENGTH],
			u32 number_of_vps,
			vxge_ll_rnic_vp_info_t *vp_info,
			u32 removed,
			u32 admin_status,
			u32 operational_status,
			void *userdata);

/*
 * vxge_ll_rnic_syn_callback_f - Call back function for SYN packets
 * @hldev: Handle to the device object
 * @msg: Packet
 * @msg_len: Packet length
 * @userdata: User data to be returned with the callback
 *
 * This function is the prototype of the callback to be registered by the
 * RNICIF to receive the SYN packets from legacy path
 */
typedef u32 (*vxge_ll_rnic_syn_callback_f)(
			vxge_hal_device_h hldev,
			u8 *msg,
			u32 msg_len,
			void *userdata);

/*
 * vxge_ll_rnic_ip_fragment_callback_f - Call back function for IP fragment
 * @hldev: Handle to the device object
 * @packet: Packet
 * @packet_len: Packet length
 * @userdata: User data to be returned with the callback
 *
 * This function is the prototype of the callback to be registered by the
 * RNICIF to receive the IP fragment packets from legacy path
 */
typedef u32 (*vxge_ll_rnic_ip_fragment_callback_f)(
			vxge_hal_device_h hldev,
			u8 *packet,
			u32 packet_len,
			void *userdata);

/*
 * vxge_ll_rnic_unaligned_send_callback_f - Call back function for Unaligned
 *                         send
 * @hldev: Handle to the device object
 * @packet: Packet
 * @userdata: User data to be returned with the callback
 *
 * This function is the prototype of the callback to be registered by the
 * RNICIF to receive the completion of unaligned send packet
 */
typedef u32 (*vxge_ll_rnic_unaligned_send_callback_f)(
			vxge_hal_device_h hldev,
			vxge_ll_rnic_packet_t *packet,
			void *userdata);

/*
 * vxge_ll_rnic_unload_callback_f - Call back function called while unloading
 *				ndis driver
 * @iov_hdl: iov handle returned when the communication is opened
 * @userdata: User data to be returned with the callback
 *
 * This function is the prototype of the callback to be registered by the
 * RNICIF to receive unload notification from the xge driver
 */
typedef u32 (*vxge_ll_rnic_unload_callback_f)(
			void *iov_hdl,
			void *userdata);

/*
 * struct vxge_ll_rnic_callbacks_t - set of callbacks registered by rnic driver
 * @version: Version of the interface
 * @device_notify_cb: Callback to be called whenever device status is changed
 * @syn_notify_cb: Callback to be called if a syn packet is received
 * @ip_fragment_cb: Callback to be called if a ip fragment packet is received
 * @unaligned_send_cb: Call back function for Unaligned send
 * @unload_cb: Call back function called while unloading ndis driver
 *
 * This structure is used to register set of callbacks with the IOV driver
 */
typedef struct vxge_ll_rnic_callbacks_t {
	vxge_ll_version_t version;
	vxge_ll_rnic_device_notification_callback_f device_notify_cb;
	vxge_ll_rnic_syn_callback_f syn_notify_cb;
	vxge_ll_rnic_ip_fragment_callback_f ip_fragment_cb;
	vxge_ll_rnic_unaligned_send_callback_f unaligned_send_cb;
	vxge_ll_rnic_unload_callback_f unload_cb;
} vxge_ll_rnic_callbacks_t;

/*
 * vxge_ll_rnic_if_open_f - Function to open communication with IOV driver
 * @rnic_cb: Structure containing callbacks
 * @userdata: User data to be returned in each of the callbacks
 * @iov_hdl: Buffer to return iov handle, which is to be passed when
 *		closing the communication
 *
 * This function is used to open communication with IOV driver
 */
typedef u32 (*vxge_ll_rnic_if_open_f)(
			vxge_ll_rnic_callbacks_t *rnic_cb,
			void *userdata,
			void **iov_hdl);

/*
 * vxge_ll_rnic_if_close_f - Function to close communication with IOV driver
 * @iov_hdl: iov handle returned when the communication is opened
 *
 * This function is used to close communication with IOV driver
 */
typedef u32 (*vxge_ll_rnic_if_close_f)(
			void *iov_hdl);

/*
 * vxge_ll_rnic_unaligned_send_f - Reinserts unaligned data through FIFO
 * @hldev: Handle to the device object
 * @packet: Packet array
 *
 * This function is the prototype of the function called by rnic clients to
 * reinsert unaligned data through FIFO
 */
typedef u32 (*vxge_ll_rnic_unaligned_send_f)(
			vxge_hal_device_h hldev,
			vxge_ll_rnic_packet_t *packet);

/*
 * vxge_ll_rnic_if_device_alive_check_f - Function to check if the device
 *				    is in hang state
 * @hldev: Handle to the device object
 * @vp_id: Virtual path identifier
 *
 * This function is used to check if the device is in hang state
 */
typedef u32 (*vxge_ll_rnic_if_device_alive_check_f)(
			vxge_hal_device_h hldev,
			u32 vp_id);

/*
 * vxge_ll_rnic_if_device_reset_f - Function to reset the device
 * @hldev: Handle to the device object
 * @vp_id: Virtual path identifier
 *
 * This function is used to reset the device
 */
typedef u32 (*vxge_ll_rnic_if_device_reset_f)(
			vxge_hal_device_h hldev,
			u32 vp_id);

/*
 * struct vxge_ll_rnic_if_t - Interface exported by link layor driver for rnic
 *			 interface driver
 * @vxge_ll_rnic_version: Version of the interface
 * @vxge_ll_rnic_if_open: Function to open communication with IOV driver
 * @vxge_ll_rnic_unaligned_send: Reinserts unaligned data through FIFO
 * @vxge_ll_rnic_if_device_alive_check: Function to check if the device
 *		is in hang state
 * @vxge_ll_rnic_if_device_reset: Function to reset the device
 * @vxge_ll_rnic_if_close: Function to close communication with IOV driver
 *
 * This structure is used to export link layer specific api for the RNIC
 */
typedef struct vxge_ll_rnic_if_t {
	vxge_ll_version_t		    vxge_ll_rnic_version;
	vxge_ll_rnic_if_open_f		    vxge_ll_rnic_if_open;
	vxge_ll_rnic_unaligned_send_f	    vxge_ll_rnic_unaligned_send;
	vxge_ll_rnic_if_device_alive_check_f vxge_ll_rnic_if_device_alive_check;
	vxge_ll_rnic_if_device_reset_f	    vxge_ll_rnic_if_device_reset;
	vxge_ll_rnic_if_close_f		    vxge_ll_rnic_if_close;
} vxge_ll_rnic_if_t;

/*
 * struct vxge_rnic_if_t - Interface exported for the use of rnic if driver
 * @version: Version of the interface
 * @vxge_ll_if: Interface exported by Link Layer driver
 * @vxge_hal_if: Interface exported by HAL
 *
 * This structure is used to export an interface of the miniport driver
 * that rnic driver uses.
 */
typedef struct vxge_rnic_if_t {
	vxge_version_t				version;
	vxge_ll_rnic_if_t			vxge_ll_if;
	vxge_hal_rnic_if_t			vxge_hal_if;
} vxge_rnic_if_t;

__EXTERN_END_DECLS

#endif /* VXGE_RNIC_H */
