#!/bin/bash
#  Copyright (c) 2002-2010 Neterion, Inc.
#  Script to install/re-install driver

usage() {
        echo "Usage: $0 <normal|jumbo>"
        echo 
        echo "    <normal|jumbo>     Enable specific MTU optimizations"
        echo
        exit 1
}
fatal() {
        echo " E $*"
        exit 2
}
if uname -a | grep '5\.11' >/dev/null; then
        os='opensol'
else
        os='sol'
fi
if isainfo | grep -i 'sparc' >/dev/null; then
        arch='sparc'
        archdir='/kernel/drv/sparcv9'
elif isainfo | grep -i 'amd64' >/dev/null; then
        archdir='/kernel/drv/amd64'
        arch='amd64'
elif isainfo | grep -i 'i386' >/dev/null; then
        arch='amd32'
        archdir='/kernel/drv'
else
    fatal "Unknown platform. Exiting"
fi

fwdir='/kernel/drv'
mtu=normal
if [ $# -eq 1 ]; then
    if [ x"$1" = xjumbo ]; then
        mtu=jumbo
    elif [ x"$1" != xjumbo ] && [ x"$1" != xnormal ]; then
        usage
    fi
elif [ $# -gt 1 ]; then
    usage
fi

echo " * Installation Wizard started"
test ! -f vxge.conf && fatal "Could not find ./vxge.conf. Exiting"
test ! -f vxge-${arch}/vxge && fatal "Could not find vxge-${arch}/vxge. Exiting"

dladmarg_opensol="show-ether"
dladmarg_sol="show-dev"
eval dladmarg=\$dladmarg_$os
if dladm show-aggr | awk '{print $1}' |grep 'vxge' >/dev/null; then
    fatal "Found link aggr, remove it before running this install script. Exiting"
fi

echo " * Removing previous driver"
interfaces=`dladm $dladmarg | awk '/vxge/ {print $1}'`
for i in $interfaces; do
        ifconfig $i unplumb 2>/dev/null
        ifconfig $i inet6 unplumb 2>/dev/null
done
rem_drv vxge 2>/dev/null

echo -n " * TCP windows and buffers optimized for "
if [ $mtu = normal ]; then
        default_mtu=1500
        ndd -set /dev/tcp tcp_recv_hiwat 65536
        ndd -set /dev/tcp tcp_xmit_hiwat 49152
        ndd -set /dev/tcp tcp_max_buf 1048576
        echo "NORMAL frames"
else
        default_mtu=9000
        ndd -set /dev/tcp tcp_recv_hiwat 393216
        ndd -set /dev/tcp tcp_xmit_hiwat 393216
        ndd -set /dev/tcp tcp_max_buf 1048576
        echo "JUMBO frames"
fi

echo " * Install driver for $arch"
cp -f vxge-${arch}/vxge $archdir

cat vxge.conf | sed -e "s/^.*default_mtu.*/default_mtu = $default_mtu;/" \
        > /kernel/drv/vxge.conf

echo " * Adding driver to the system "
if add_drv -i '"pciex17d5,5833"' vxge 2>/dev/null; then
        echo " * Driver loaded SUCCESSFULLY"
	echo " * TCP parameters have been changed"
else
        echo "error encountered"
        echo " E Driver copied, configuration changed but NOT LOADED"
        echo "   Reboot might be required."
        exit 1
fi
interfaces=`dladm $dladmarg | awk '/vxge/ {print $1}'`

echo " * Plumbing all devices "
iflist=""
for i in $interfaces; do
    if ifconfig $i plumb >/dev/null 2>&1; then
        iflist="$iflist $i"
    fi
done

if [ x"$iflist" != x ]; then
	echo " * Available X3100 devices are: $iflist"
else
	echo " * Firmware / Function mode changed !!"
	echo " * For more information check dmesg !!"
	echo " * POWERCYCLE SYSTEM REQUIRED !!"
fi

#set system parameters for driver if needed
setsys() {
    if ! cat /etc/system | grep "^[ ]*set[ ][ ]*$1" >/dev/null; then
        echo "set $1=$2" >>/etc/system
        updmsg=" Updated /etc/system, changes will take effect after reboot"
    fi
}

setsys ddi_msix_alloc_limit 20

test x"$updmsg" != x && echo $updmsg

exit 0
