/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

/*
 * Copyright (c) 2002-2010 Neterion, Inc.
 * All right Reserved.
 *
 * FileName :   vxgehal-bitmap.c
 *
 * Description:  HAL bitmap support functionality
 *
 * Created:       20 Nov 2006
 */
#include "vxgehal.h"

/*
 * __hal_bitmap_sq_set - Set SQ into bitmap
 * @sqh: Send Queue handle
 *
 * This routine sets the sq in the bitmap table
 */
vxge_hal_status_e
__hal_bitmap_sq_set(vxge_hal_sq_h  sqh)
{
	u32	sid;
	__hal_device_t *hldev;
	__hal_sq_t *sq = (__hal_sq_t *)sqh;

	vxge_assert(sq != NULL);

	hldev = (__hal_device_t *)sq->channel.devh;

	vxge_hal_trace_log_bitmap(hldev,
	    ((__hal_vpath_handle_t *)sq->channel.vph)->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_bitmap(hldev,
	    ((__hal_vpath_handle_t *)sq->channel.vph)->vpath->vp_id,
	    "sq = 0x"VXGE_OS_STXFMT, (ptr_t)sq);

	sid = VXGE_HAL_RNIC_OID_DB_SID_GET(sq->session_id);

	if (hldev->bitmap_table[sid].sq_ref_count++ == 0)
		hldev->bitmap_table[sid].sqh = (vxge_hal_sq_h)sq;

	vxge_hal_trace_log_bitmap(hldev,
	    ((__hal_vpath_handle_t *)sq->channel.vph)->vpath->vp_id,
	    "<==  %s:%s:%d Result = 0", __FILE__, __func__, __LINE__);

	return (VXGE_HAL_OK);
}

/*
 * __hal_bitmap_sq_reset - Reset SQ channel into bitmap
 * @sqh: Send Queue handle
 *
 * This routine resets the sq channel in the bitmap table
 */
vxge_hal_status_e
__hal_bitmap_sq_reset(vxge_hal_sq_h  sqh)
{
	u32	sid;
	__hal_device_t *hldev;
	__hal_sq_t *sq = (__hal_sq_t *)sqh;

	vxge_assert(sq != NULL);

	hldev = (__hal_device_t *)sq->channel.devh;

	vxge_hal_trace_log_bitmap(hldev,
	    ((__hal_vpath_handle_t *)sq->channel.vph)->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_bitmap(hldev,
	    ((__hal_vpath_handle_t *)sq->channel.vph)->vpath->vp_id,
	    "sq = 0x"VXGE_OS_STXFMT, (ptr_t)sq);

	sid = VXGE_HAL_RNIC_OID_DB_SID_GET(sq->session_id);

	hldev->bitmap_table[sid].sq_ref_count--;

	if (hldev->bitmap_table[sid].sq_ref_count == 0)
		hldev->bitmap_table[sid].sqh = NULL;

	vxge_hal_trace_log_bitmap(hldev,
	    ((__hal_vpath_handle_t *)sq->channel.vph)->vpath->vp_id,
	    "<==  %s:%s:%d Result = 0", __FILE__, __func__, __LINE__);

	return (VXGE_HAL_OK);
}

/*
 * __hal_bitmap_srq_set - Set SRQ channel into bitmap
 * @srqh: SRQ handle
 *
 * This routine sets the srq channel in the bitmap table
 */
vxge_hal_status_e
__hal_bitmap_srq_set(vxge_hal_srq_h  srqh)
{
	u32	sid;
	__hal_device_t *hldev;
	__hal_srq_t *srq = (__hal_srq_t *)srqh;

	vxge_assert(srq != NULL);

	hldev = (__hal_device_t *)srq->channel.devh;

	vxge_hal_trace_log_bitmap(hldev,
	    ((__hal_vpath_handle_t *)srq->channel.vph)->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_bitmap(hldev,
	    ((__hal_vpath_handle_t *)srq->channel.vph)->vpath->vp_id,
	    "srq = 0x"VXGE_OS_STXFMT, (ptr_t)srq);

	sid = VXGE_HAL_RNIC_OID_DB_SID_GET(srq->srq_id);

	if (hldev->bitmap_table[sid].srq_ref_count++ == 0)
		hldev->bitmap_table[sid].srqh = (vxge_hal_srq_h)srq;

	vxge_hal_trace_log_bitmap(hldev,
	    ((__hal_vpath_handle_t *)srq->channel.vph)->vpath->vp_id,
	    "<==  %s:%s:%d Result = 0", __FILE__, __func__, __LINE__);

	return (VXGE_HAL_OK);
}

/*
 * __hal_bitmap_srq_reset - Reset SRQ channel into bitmap
 * @srqh: SRQ handle
 *
 * This routine resets the srq channel in the bitmap table
 */
vxge_hal_status_e
__hal_bitmap_srq_reset(vxge_hal_srq_h  srqh)
{
	u32	sid;
	__hal_device_t *hldev;
	__hal_srq_t *srq = (__hal_srq_t *)srqh;

	vxge_assert(srq != NULL);

	hldev = (__hal_device_t *)srq->channel.devh;

	vxge_hal_trace_log_bitmap(hldev,
	    ((__hal_vpath_handle_t *)srq->channel.vph)->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_bitmap(hldev,
	    ((__hal_vpath_handle_t *)srq->channel.vph)->vpath->vp_id,
	    "srq = 0x"VXGE_OS_STXFMT, (ptr_t)srq);

	sid = VXGE_HAL_RNIC_OID_DB_SID_GET(srq->srq_id);

	hldev->bitmap_table[sid].srq_ref_count--;

	if (hldev->bitmap_table[sid].srq_ref_count == 0)
		hldev->bitmap_table[sid].srqh = NULL;

	vxge_hal_trace_log_bitmap(hldev,
	    ((__hal_vpath_handle_t *)srq->channel.vph)->vpath->vp_id,
	    "<==  %s:%s:%d Result = 0", __FILE__, __func__, __LINE__);

	return (VXGE_HAL_OK);
}

/*
 * __hal_bitmap_cqrq_set - Set CQRQ channel into bitmap
 * @cqrqh: CQRQ handle
 *
 * This routine sets the cqrq channel in the bitmap table
 */
vxge_hal_status_e
__hal_bitmap_cqrq_set(vxge_hal_cqrq_h  cqrqh)
{
	u32	sid;
	__hal_device_t *hldev;
	__hal_cqrq_t *cqrq = (__hal_cqrq_t *)cqrqh;

	vxge_assert(cqrq != NULL);

	hldev = (__hal_device_t *)cqrq->channel.devh;

	vxge_hal_trace_log_bitmap(hldev,
	    ((__hal_vpath_handle_t *)cqrq->channel.vph)->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_bitmap(hldev,
	    ((__hal_vpath_handle_t *)cqrq->channel.vph)->vpath->vp_id,
	    "cqrq = 0x"VXGE_OS_STXFMT, (ptr_t)cqrq);

	sid = VXGE_HAL_RNIC_OID_DB_SID_GET(cqrq->cqrq_id);

	if (hldev->bitmap_table[sid].cqrq_ref_count++ == 0)
		hldev->bitmap_table[sid].cqrqh = (vxge_hal_cqrq_h)cqrq;

	vxge_hal_trace_log_bitmap(hldev,
	    ((__hal_vpath_handle_t *)cqrq->channel.vph)->vpath->vp_id,
	    "<==  %s:%s:%d Result = 0", __FILE__, __func__, __LINE__);

	return (VXGE_HAL_OK);
}

/*
 * __hal_bitmap_cqrq_reset - Reset CQRQ channel into bitmap
 * @cqrqh: CQRQ handle
 *
 * This routine resets the cqrq channel in the bitmap table
 */
vxge_hal_status_e
__hal_bitmap_cqrq_reset(vxge_hal_cqrq_h  cqrqh)
{
	u32	sid;
	__hal_device_t *hldev;
	__hal_cqrq_t *cqrq = (__hal_cqrq_t *)cqrqh;

	vxge_assert(cqrq != NULL);

	hldev = (__hal_device_t *)cqrq->channel.devh;

	vxge_hal_trace_log_bitmap(hldev,
	    ((__hal_vpath_handle_t *)cqrq->channel.vph)->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_bitmap(hldev,
	    ((__hal_vpath_handle_t *)cqrq->channel.vph)->vpath->vp_id,
	    "cqrq = 0x"VXGE_OS_STXFMT, (ptr_t)cqrq);

	sid = VXGE_HAL_RNIC_OID_DB_SID_GET(cqrq->cqrq_id);

	hldev->bitmap_table[sid].cqrq_ref_count--;

	if (hldev->bitmap_table[sid].cqrq_ref_count == 0)
		hldev->bitmap_table[sid].cqrqh = NULL;

	vxge_hal_trace_log_bitmap(hldev,
	    ((__hal_vpath_handle_t *)cqrq->channel.vph)->vpath->vp_id,
	    "<==  %s:%s:%d Result = 0", __FILE__, __func__, __LINE__);

	return (VXGE_HAL_OK);
}

static struct {
	u32	num_of_bits;
	u32	set_bits[8];
} __hal_set_bit_numbers[256] = {
	    {0, {0, 0, 0, 0, 0, 0, 0, 0}},
	    {1, {7, 0, 0, 0, 0, 0, 0, 0}},
	    {1, {6, 0, 0, 0, 0, 0, 0, 0}},
	    {2, {6, 7, 0, 0, 0, 0, 0, 0}},
	    {1, {5, 0, 0, 0, 0, 0, 0, 0}},
	    {2, {5, 7, 0, 0, 0, 0, 0, 0}},
	    {2, {5, 6, 0, 0, 0, 0, 0, 0}},
	    {3, {5, 6, 7, 0, 0, 0, 0, 0}},
	    {1, {4, 0, 0, 0, 0, 0, 0, 0}},
	    {2, {4, 7, 0, 0, 0, 0, 0, 0}},
	    {2, {4, 6, 0, 0, 0, 0, 0, 0}},
	    {3, {4, 6, 7, 0, 0, 0, 0, 0}},
	    {2, {4, 5, 0, 0, 0, 0, 0, 0}},
	    {3, {4, 5, 7, 0, 0, 0, 0, 0}},
	    {3, {4, 5, 6, 0, 0, 0, 0, 0}},
	    {4, {4, 5, 6, 7, 0, 0, 0, 0}},
	    {1, {3, 0, 0, 0, 0, 0, 0, 0}},
	    {2, {3, 7, 0, 0, 0, 0, 0, 0}},
	    {2, {3, 6, 0, 0, 0, 0, 0, 0}},
	    {3, {3, 6, 7, 0, 0, 0, 0, 0}},
	    {2, {3, 5, 0, 0, 0, 0, 0, 0}},
	    {3, {3, 5, 7, 0, 0, 0, 0, 0}},
	    {3, {3, 5, 6, 0, 0, 0, 0, 0}},
	    {4, {3, 5, 6, 7, 0, 0, 0, 0}},
	    {2, {3, 4, 0, 0, 0, 0, 0, 0}},
	    {3, {3, 4, 7, 0, 0, 0, 0, 0}},
	    {3, {3, 4, 6, 0, 0, 0, 0, 0}},
	    {4, {3, 4, 6, 7, 0, 0, 0, 0}},
	    {3, {3, 4, 5, 0, 0, 0, 0, 0}},
	    {4, {3, 4, 5, 7, 0, 0, 0, 0}},
	    {4, {3, 4, 5, 6, 0, 0, 0, 0}},
	    {5, {3, 4, 5, 6, 7, 0, 0, 0}},
	    {1, {2, 0, 0, 0, 0, 0, 0, 0}},
	    {2, {2, 7, 0, 0, 0, 0, 0, 0}},
	    {2, {2, 6, 0, 0, 0, 0, 0, 0}},
	    {3, {2, 6, 7, 0, 0, 0, 0, 0}},
	    {2, {2, 5, 0, 0, 0, 0, 0, 0}},
	    {3, {2, 5, 7, 0, 0, 0, 0, 0}},
	    {3, {2, 5, 6, 0, 0, 0, 0, 0}},
	    {4, {2, 5, 6, 7, 0, 0, 0, 0}},
	    {2, {2, 4, 0, 0, 0, 0, 0, 0}},
	    {3, {2, 4, 7, 0, 0, 0, 0, 0}},
	    {3, {2, 4, 6, 0, 0, 0, 0, 0}},
	    {4, {2, 4, 6, 7, 0, 0, 0, 0}},
	    {3, {2, 4, 5, 0, 0, 0, 0, 0}},
	    {4, {2, 4, 5, 7, 0, 0, 0, 0}},
	    {4, {2, 4, 5, 6, 0, 0, 0, 0}},
	    {5, {2, 4, 5, 6, 7, 0, 0, 0}},
	    {2, {2, 3, 0, 0, 0, 0, 0, 0}},
	    {3, {2, 3, 7, 0, 0, 0, 0, 0}},
	    {3, {2, 3, 6, 0, 0, 0, 0, 0}},
	    {4, {2, 3, 6, 7, 0, 0, 0, 0}},
	    {3, {2, 3, 5, 0, 0, 0, 0, 0}},
	    {4, {2, 3, 5, 7, 0, 0, 0, 0}},
	    {4, {2, 3, 5, 6, 0, 0, 0, 0}},
	    {5, {2, 3, 5, 6, 7, 0, 0, 0}},
	    {3, {2, 3, 4, 0, 0, 0, 0, 0}},
	    {4, {2, 3, 4, 7, 0, 0, 0, 0}},
	    {4, {2, 3, 4, 6, 0, 0, 0, 0}},
	    {5, {2, 3, 4, 6, 7, 0, 0, 0}},
	    {4, {2, 3, 4, 5, 0, 0, 0, 0}},
	    {5, {2, 3, 4, 5, 7, 0, 0, 0}},
	    {5, {2, 3, 4, 5, 6, 0, 0, 0}},
	    {6, {2, 3, 4, 5, 6, 7, 0, 0}},
	    {1, {1, 0, 0, 0, 0, 0, 0, 0}},
	    {2, {1, 7, 0, 0, 0, 0, 0, 0}},
	    {2, {1, 6, 0, 0, 0, 0, 0, 0}},
	    {3, {1, 6, 7, 0, 0, 0, 0, 0}},
	    {2, {1, 5, 0, 0, 0, 0, 0, 0}},
	    {3, {1, 5, 7, 0, 0, 0, 0, 0}},
	    {3, {1, 5, 6, 0, 0, 0, 0, 0}},
	    {4, {1, 5, 6, 7, 0, 0, 0, 0}},
	    {2, {1, 4, 0, 0, 0, 0, 0, 0}},
	    {3, {1, 4, 7, 0, 0, 0, 0, 0}},
	    {3, {1, 4, 6, 0, 0, 0, 0, 0}},
	    {4, {1, 4, 6, 7, 0, 0, 0, 0}},
	    {3, {1, 4, 5, 0, 0, 0, 0, 0}},
	    {4, {1, 4, 5, 7, 0, 0, 0, 0}},
	    {4, {1, 4, 5, 6, 0, 0, 0, 0}},
	    {5, {1, 4, 5, 6, 7, 0, 0, 0}},
	    {2, {1, 3, 0, 0, 0, 0, 0, 0}},
	    {3, {1, 3, 7, 0, 0, 0, 0, 0}},
	    {3, {1, 3, 6, 0, 0, 0, 0, 0}},
	    {4, {1, 3, 6, 7, 0, 0, 0, 0}},
	    {3, {1, 3, 5, 0, 0, 0, 0, 0}},
	    {4, {1, 3, 5, 7, 0, 0, 0, 0}},
	    {4, {1, 3, 5, 6, 0, 0, 0, 0}},
	    {5, {1, 3, 5, 6, 7, 0, 0, 0}},
	    {3, {1, 3, 4, 0, 0, 0, 0, 0}},
	    {4, {1, 3, 4, 7, 0, 0, 0, 0}},
	    {4, {1, 3, 4, 6, 0, 0, 0, 0}},
	    {5, {1, 3, 4, 6, 7, 0, 0, 0}},
	    {4, {1, 3, 4, 5, 0, 0, 0, 0}},
	    {5, {1, 3, 4, 5, 7, 0, 0, 0}},
	    {5, {1, 3, 4, 5, 6, 0, 0, 0}},
	    {6, {1, 3, 4, 5, 6, 7, 0, 0}},
	    {2, {1, 2, 0, 0, 0, 0, 0, 0}},
	    {3, {1, 2, 7, 0, 0, 0, 0, 0}},
	    {3, {1, 2, 6, 0, 0, 0, 0, 0}},
	    {4, {1, 2, 6, 7, 0, 0, 0, 0}},
	    {3, {1, 2, 5, 0, 0, 0, 0, 0}},
	    {4, {1, 2, 5, 7, 0, 0, 0, 0}},
	    {4, {1, 2, 5, 6, 0, 0, 0, 0}},
	    {5, {1, 2, 5, 6, 7, 0, 0, 0}},
	    {3, {1, 2, 4, 0, 0, 0, 0, 0}},
	    {4, {1, 2, 4, 7, 0, 0, 0, 0}},
	    {4, {1, 2, 4, 6, 0, 0, 0, 0}},
	    {5, {1, 2, 4, 6, 7, 0, 0, 0}},
	    {4, {1, 2, 4, 5, 0, 0, 0, 0}},
	    {5, {1, 2, 4, 5, 7, 0, 0, 0}},
	    {5, {1, 2, 4, 5, 6, 0, 0, 0}},
	    {6, {1, 2, 4, 5, 6, 7, 0, 0}},
	    {3, {1, 2, 3, 0, 0, 0, 0, 0}},
	    {4, {1, 2, 3, 7, 0, 0, 0, 0}},
	    {4, {1, 2, 3, 6, 0, 0, 0, 0}},
	    {5, {1, 2, 3, 6, 7, 0, 0, 0}},
	    {4, {1, 2, 3, 5, 0, 0, 0, 0}},
	    {5, {1, 2, 3, 5, 7, 0, 0, 0}},
	    {5, {1, 2, 3, 5, 6, 0, 0, 0}},
	    {6, {1, 2, 3, 5, 6, 7, 0, 0}},
	    {4, {1, 2, 3, 4, 0, 0, 0, 0}},
	    {5, {1, 2, 3, 4, 7, 0, 0, 0}},
	    {5, {1, 2, 3, 4, 6, 0, 0, 0}},
	    {6, {1, 2, 3, 4, 6, 7, 0, 0}},
	    {5, {1, 2, 3, 4, 5, 0, 0, 0}},
	    {6, {1, 2, 3, 4, 5, 7, 0, 0}},
	    {6, {1, 2, 3, 4, 5, 6, 0, 0}},
	    {7, {1, 2, 3, 4, 5, 6, 7, 0}},
	    {1, {0, 0, 0, 0, 0, 0, 0, 0}},
	    {2, {0, 7, 0, 0, 0, 0, 0, 0}},
	    {2, {0, 6, 0, 0, 0, 0, 0, 0}},
	    {3, {0, 6, 7, 0, 0, 0, 0, 0}},
	    {2, {0, 5, 0, 0, 0, 0, 0, 0}},
	    {3, {0, 5, 7, 0, 0, 0, 0, 0}},
	    {3, {0, 5, 6, 0, 0, 0, 0, 0}},
	    {4, {0, 5, 6, 7, 0, 0, 0, 0}},
	    {2, {0, 4, 0, 0, 0, 0, 0, 0}},
	    {3, {0, 4, 7, 0, 0, 0, 0, 0}},
	    {3, {0, 4, 6, 0, 0, 0, 0, 0}},
	    {4, {0, 4, 6, 7, 0, 0, 0, 0}},
	    {3, {0, 4, 5, 0, 0, 0, 0, 0}},
	    {4, {0, 4, 5, 7, 0, 0, 0, 0}},
	    {4, {0, 4, 5, 6, 0, 0, 0, 0}},
	    {5, {0, 4, 5, 6, 7, 0, 0, 0}},
	    {2, {0, 3, 0, 0, 0, 0, 0, 0}},
	    {3, {0, 3, 7, 0, 0, 0, 0, 0}},
	    {3, {0, 3, 6, 0, 0, 0, 0, 0}},
	    {4, {0, 3, 6, 7, 0, 0, 0, 0}},
	    {3, {0, 3, 5, 0, 0, 0, 0, 0}},
	    {4, {0, 3, 5, 7, 0, 0, 0, 0}},
	    {4, {0, 3, 5, 6, 0, 0, 0, 0}},
	    {5, {0, 3, 5, 6, 7, 0, 0, 0}},
	    {3, {0, 3, 4, 0, 0, 0, 0, 0}},
	    {4, {0, 3, 4, 7, 0, 0, 0, 0}},
	    {4, {0, 3, 4, 6, 0, 0, 0, 0}},
	    {5, {0, 3, 4, 6, 7, 0, 0, 0}},
	    {4, {0, 3, 4, 5, 0, 0, 0, 0}},
	    {5, {0, 3, 4, 5, 7, 0, 0, 0}},
	    {5, {0, 3, 4, 5, 6, 0, 0, 0}},
	    {6, {0, 3, 4, 5, 6, 7, 0, 0}},
	    {2, {0, 2, 0, 0, 0, 0, 0, 0}},
	    {3, {0, 2, 7, 0, 0, 0, 0, 0}},
	    {3, {0, 2, 6, 0, 0, 0, 0, 0}},
	    {4, {0, 2, 6, 7, 0, 0, 0, 0}},
	    {3, {0, 2, 5, 0, 0, 0, 0, 0}},
	    {4, {0, 2, 5, 7, 0, 0, 0, 0}},
	    {4, {0, 2, 5, 6, 0, 0, 0, 0}},
	    {5, {0, 2, 5, 6, 7, 0, 0, 0}},
	    {3, {0, 2, 4, 0, 0, 0, 0, 0}},
	    {4, {0, 2, 4, 7, 0, 0, 0, 0}},
	    {4, {0, 2, 4, 6, 0, 0, 0, 0}},
	    {5, {0, 2, 4, 6, 7, 0, 0, 0}},
	    {4, {0, 2, 4, 5, 0, 0, 0, 0}},
	    {5, {0, 2, 4, 5, 7, 0, 0, 0}},
	    {5, {0, 2, 4, 5, 6, 0, 0, 0}},
	    {6, {0, 2, 4, 5, 6, 7, 0, 0}},
	    {3, {0, 2, 3, 0, 0, 0, 0, 0}},
	    {4, {0, 2, 3, 7, 0, 0, 0, 0}},
	    {4, {0, 2, 3, 6, 0, 0, 0, 0}},
	    {5, {0, 2, 3, 6, 7, 0, 0, 0}},
	    {4, {0, 2, 3, 5, 0, 0, 0, 0}},
	    {5, {0, 2, 3, 5, 7, 0, 0, 0}},
	    {5, {0, 2, 3, 5, 6, 0, 0, 0}},
	    {6, {0, 2, 3, 5, 6, 7, 0, 0}},
	    {4, {0, 2, 3, 4, 0, 0, 0, 0}},
	    {5, {0, 2, 3, 4, 7, 0, 0, 0}},
	    {6, {0, 2, 3, 4, 6, 0, 0, 0}},
	    {6, {0, 2, 3, 4, 6, 7, 0, 0}},
	    {5, {0, 2, 3, 4, 5, 0, 0, 0}},
	    {6, {0, 2, 3, 4, 5, 7, 0, 0}},
	    {6, {0, 2, 3, 4, 5, 6, 0, 0}},
	    {7, {0, 2, 3, 4, 5, 6, 7, 0}},
	    {2, {0, 1, 0, 0, 0, 0, 0, 0}},
	    {3, {0, 1, 7, 0, 0, 0, 0, 0}},
	    {3, {0, 1, 6, 0, 0, 0, 0, 0}},
	    {4, {0, 1, 6, 7, 0, 0, 0, 0}},
	    {3, {0, 1, 5, 0, 0, 0, 0, 0}},
	    {4, {0, 1, 5, 7, 0, 0, 0, 0}},
	    {4, {0, 1, 5, 6, 0, 0, 0, 0}},
	    {5, {0, 1, 5, 6, 7, 0, 0, 0}},
	    {3, {0, 1, 4, 0, 0, 0, 0, 0}},
	    {4, {0, 1, 4, 7, 0, 0, 0, 0}},
	    {4, {0, 1, 4, 6, 0, 0, 0, 0}},
	    {5, {0, 1, 4, 6, 7, 0, 0, 0}},
	    {4, {0, 1, 4, 5, 0, 0, 0, 0}},
	    {5, {0, 1, 4, 5, 7, 0, 0, 0}},
	    {5, {0, 1, 4, 5, 6, 0, 0, 0}},
	    {6, {0, 1, 4, 5, 6, 7, 0, 0}},
	    {3, {0, 1, 3, 0, 0, 0, 0, 0}},
	    {4, {0, 1, 3, 7, 0, 0, 0, 0}},
	    {4, {0, 1, 3, 6, 0, 0, 0, 0}},
	    {5, {0, 1, 3, 6, 7, 0, 0, 0}},
	    {4, {0, 1, 3, 5, 0, 0, 0, 0}},
	    {5, {0, 1, 3, 5, 7, 0, 0, 0}},
	    {5, {0, 1, 3, 5, 6, 0, 0, 0}},
	    {6, {0, 1, 3, 5, 6, 7, 0, 0}},
	    {4, {0, 1, 3, 4, 0, 0, 0, 0}},
	    {5, {0, 1, 3, 4, 7, 0, 0, 0}},
	    {5, {0, 1, 3, 4, 6, 0, 0, 0}},
	    {6, {0, 1, 3, 4, 6, 7, 0, 0}},
	    {5, {0, 1, 3, 4, 5, 0, 0, 0}},
	    {6, {0, 1, 3, 4, 5, 7, 0, 0}},
	    {6, {0, 1, 3, 4, 5, 6, 0, 0}},
	    {7, {0, 1, 3, 4, 5, 6, 7, 0}},
	    {3, {0, 1, 2, 0, 0, 0, 0, 0}},
	    {4, {0, 1, 2, 7, 0, 0, 0, 0}},
	    {4, {0, 1, 2, 6, 0, 0, 0, 0}},
	    {5, {0, 1, 2, 6, 7, 0, 0, 0}},
	    {4, {0, 1, 2, 5, 0, 0, 0, 0}},
	    {5, {0, 1, 2, 5, 7, 0, 0, 0}},
	    {5, {0, 1, 2, 5, 6, 0, 0, 0}},
	    {6, {0, 1, 2, 5, 6, 7, 0, 0}},
	    {4, {0, 1, 2, 4, 0, 0, 0, 0}},
	    {5, {0, 1, 2, 4, 7, 0, 0, 0}},
	    {5, {0, 1, 2, 4, 6, 0, 0, 0}},
	    {6, {0, 1, 2, 4, 6, 7, 0, 0}},
	    {5, {0, 1, 2, 4, 5, 0, 0, 0}},
	    {6, {0, 1, 2, 4, 5, 7, 0, 0}},
	    {6, {0, 1, 2, 4, 5, 6, 0, 0}},
	    {7, {0, 1, 2, 4, 5, 6, 7, 0}},
	    {4, {0, 1, 2, 3, 0, 0, 0, 0}},
	    {5, {0, 1, 2, 3, 7, 0, 0, 0}},
	    {5, {0, 1, 2, 3, 6, 0, 0, 0}},
	    {6, {0, 1, 2, 3, 6, 7, 0, 0}},
	    {5, {0, 1, 2, 3, 5, 0, 0, 0}},
	    {6, {0, 1, 2, 3, 5, 7, 0, 0}},
	    {6, {0, 1, 2, 3, 5, 6, 0, 0}},
	    {7, {0, 1, 2, 3, 5, 6, 7, 0}},
	    {5, {0, 1, 2, 3, 4, 0, 0, 0}},
	    {6, {0, 1, 2, 3, 4, 7, 0, 0}},
	    {6, {0, 1, 2, 3, 4, 6, 0, 0}},
	    {7, {0, 1, 2, 3, 4, 6, 7, 0}},
	    {6, {0, 1, 2, 3, 4, 5, 0, 0}},
	    {7, {0, 1, 2, 3, 4, 5, 7, 0}},
	    {7, {0, 1, 2, 3, 4, 5, 6, 0}},
	    {8, {0, 1, 2, 3, 4, 5, 6, 7}}};


/*
 * __hal_number_of_bits_set_get - Get the number of bits set in the 64 bit value
 * @val: 64 bit value
 *
 * This routine returns the number of bits set in the 64 bit value
 */
u32
__hal_number_of_bits_set_get(u64  val)
{
	u64 nbits = 0;

	nbits = val - ((val >> 1) & 0x7777777777777777LL)
	    - ((val >> 2) & 0x3333333333333333LL)
	    - ((val >> 3) & 0x1111111111111111LL);

	nbits = ((nbits + (nbits >> 4)) & 0x0F0F0F0F0F0F0F0FLL);

	nbits = nbits - ((nbits/255)*255);

	return ((u32)nbits);
}

/*
 * __hal_set_bits_64_get - Returns the locations of the set bits
 *		in a 64 bit value
 * @val: 64 bit value
 * @loc: Array to return locations of the set bits
 *
 * This routine returns the locations of the set bits in a 64 bit value
 */
void
__hal_set_bits_64_get(u64  val, u32 loc[64])
{
	u32 i, j;
	u32 index;
	u32 nbits;
	u32 *set_bits;
	u32 *bit_loc = loc;

	for (i = 0; i < 64; i += 8) {
		index = (u32)bVAL8(val, i);
		nbits = __hal_set_bit_numbers[index].num_of_bits;
		set_bits = __hal_set_bit_numbers[index].set_bits;

		for (j = 0; j < nbits; j++) {
			*bit_loc++ = i + set_bits[i];
		}
	}
}

/*
 * __hal_set_bits_16_get - Returns the locations of the set bits
 *		in a 16 bit value
 * @val: 16 bit value
 * @loc: Array to return locations of the set bits
 *
 * This routine returns the locations of the set bits in a 16 bit value
 */
void
__hal_set_bits_16_get(u16  val, u32 loc[16])
{
	u32 i;
	u32 index;
	u32 nbits;
	u32 *set_bits;
	u32 *bit_loc = loc;

	index = (u32)bVAL8(val, 0);
	nbits = __hal_set_bit_numbers[index].num_of_bits;
	set_bits = __hal_set_bit_numbers[index].set_bits;

	for (i = 0; i < nbits; i++) {
		*bit_loc++ = set_bits[i];
	}

	index = (u32)bVAL8(val, 8);
	nbits = __hal_set_bit_numbers[index].num_of_bits;
	set_bits = __hal_set_bit_numbers[index].set_bits;

	for (i = 0; i < nbits; i++) {
		*bit_loc++ = 8 + set_bits[i];
	}
}

/*
 * __hal_bitmap_sessions_process - Processes bitmap to get the channels that
 *		need service and invokes the call backs of the channels
 * @vpath_handle: Virtual Path Handle
 * @bitmap: The bitmap data
 *
 * This routine processes the bit map and invokes the callbacks on the channels.
 */
vxge_hal_status_e
vxge_hal_bitmap_sessions_process(vxge_hal_vpath_h vpath_handle, u64 *bitmap)
{
	__hal_sq_t *sq;
	__hal_cqrq_t *cqrq;
	__hal_device_t *hldev;
	__hal_vpath_handle_t *vp = (__hal_vpath_handle_t *)vpath_handle;
	u32 rbits_set[64];
	u32 rbits_count;
	u32 nbits_set[16];
	u32 nbits_count;
	u32 lbits_set[64];
	u32 lbits_count;
	u32 bit_id;
	u64 root;
	u64 *nodes;
	u16 node_val;
	u32 nodes_count;
	u64 *leafs;
	u32 r, n, l;
	u32 current_node_idx;

	vxge_assert((vpath_handle != NULL) && (bitmap != NULL));

	hldev = vp->vpath->hldev;

	vxge_hal_trace_log_bitmap(hldev, vp->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_bitmap(hldev, vp->vpath->vp_id,
	    "vpath_handle = 0x"VXGE_OS_STXFMT",  bitmap = 0x"VXGE_OS_STXFMT,
	    (ptr_t)vpath_handle, (ptr_t)bitmap);

	root = *bitmap & 0xFFFFFFFF00000000LL;

	rbits_count = __hal_number_of_bits_set_get(root);

	if (rbits_count == 0) {
		vxge_hal_trace_log_bitmap(hldev, vp->vpath->vp_id,
		    "<==  %s:%s:%d Result = 0", __FILE__, __func__, __LINE__);
		return (VXGE_HAL_OK);
	}

	__hal_set_bits_64_get(root, rbits_set);

	nodes = bitmap + 1;

	nodes_count = ((rbits_set[rbits_count-1]/4) - (rbits_set[0]/4)) + 1;

	current_node_idx = rbits_set[0]/4;

	leafs = bitmap + nodes_count + 1;

	for (r = 0; r < rbits_count; r++) {

		if (current_node_idx != rbits_set[r]/4) {
			current_node_idx = rbits_set[r]/4;
			nodes++;
		}

		node_val = (u16)bVAL16((*nodes), ((rbits_set[r]%4)*16));

		nbits_count = __hal_number_of_bits_set_get((u64)node_val);

		if (nbits_count == 0) {
			vxge_assert(nbits_count != 0);
			continue;
		}

		__hal_set_bits_16_get(node_val, nbits_set);

		for (n = 0; n < nbits_count; n++, leafs++) {

			lbits_count = __hal_number_of_bits_set_get(*leafs);

			if (lbits_count == 0) {
				vxge_assert(lbits_count != 0);
				continue;
			}

			__hal_set_bits_64_get(*leafs, lbits_set);

			for (l = 0; l < lbits_count; l++) {

				Bit_id = rbits_set[r] * 1024 +
				    nbits_set[n] * 64 +
				    lbits_set[l];

				sq = (__hal_sq_t *)
				    vp->vpath->hldev->bitmap_table[bit_id].sqh;

				if (vxge_hal_sq_completed_towi_check(
				    vpath_handle,
				    (vxge_hal_obj_id_t)(ptr_t)sq) ==
				    VXGE_HAL_OK) {
					u64 opaque_handle;
					if (vxge_hal_sq_next_completed_towi_get(
					    vpath_handle,
					    (vxge_hal_obj_id_t)(ptr_t)sq,
					    &opaque_handle) == VXGE_HAL_OK) {
						sq->callback(vpath_handle,
						    (vxge_hal_obj_id_t)(ptr_t)
						    sq,
						    opaque_handle,
						    sq->channel.userdata);
					}
				}

				cqrq = (__hal_cqrq_t *)
				    vp->vpath->hldev->bitmap_table[bit_id].
				    cqrqh;

				if (cqrq->cqrq_type == VXGE_HAL_CQRQ_TYPE_LRO) {
				vxge_hal_cqrq_lro_cqe_Info_t cqrq_info;
				if ((vxge_hal_cqrq_ready_lro_cqe_check(
				    vpath_handle,
				    (vxge_hal_obj_id_t)(ptr_t)cqrq) ==
				    VXGE_HAL_OK) &&
				    (vxge_hal_cqrq_next_ready_lro_cqe_get(
				    vpath_handle,
				    (vxge_hal_obj_id_t)(ptr_t)cqrq,
				    &cqrq_info) == VXGE_HAL_OK)) {
					((vxge_hal_cqrq_lro_cqe_callback_f)
					    cqrq->callback)(vpath_handle,
					    (vxge_hal_obj_id_t)(ptr_t)cqrq,
					    &cqrq_info,
					    cqrq->channel.userdata);

				}
				} else {
				vxge_hal_cqrq_hw_cqe_Info_t cqrq_info;
				if ((vxge_hal_cqrq_ready_hw_cqe_check(
				    vpath_handle,
				    (vxge_hal_obj_id_t)(ptr_t)cqrq) ==
				    VXGE_HAL_OK) &&
				    (vxge_hal_cqrq_next_ready_hw_cqe_get(
				    vpath_handle,
				    (vxge_hal_obj_id_t)(ptr_t)cqrq,
				    &cqrq_info) == VXGE_HAL_OK)) {
					((vxge_hal_cqrq_hw_cqe_callback_f)
					    cqrq->callback)(vpath_handle,
					    (vxge_hal_obj_id_t)(ptr_t)cqrq,
					    &cqrq_info,
					    cqrq->channel.userdata);

				}
				}
			}

		}
	}

	vxge_hal_trace_log_bitmap(hldev, vp->vpath->vp_id,
	    "<==  %s:%s:%d Result = 0", __FILE__, __func__, __LINE__);

	return (VXGE_HAL_OK);

}

/*
 * __hal_bitmap_llbits_process - Processes lowletency bits of bitmap
 * @vpath_handle: Virtual Path Handle
 * @root: Root bits
 *
 * This routine processes the bit map  lowletency bits.
 */
vxge_hal_status_e
vxge_hal_bitmap_llbits_process(vxge_hal_vpath_h vpath_handle, u64 root)
{
	u32  ll_bits;
	__hal_device_t *hldev;
	__hal_vpath_handle_t *vp = (__hal_vpath_handle_t *)vpath_handle;

	vxge_assert(vpath_handle != NULL);

	hldev = vp->vpath->hldev;

	vxge_hal_trace_log_bitmap(hldev, vp->vpath->vp_id,
	    "<==  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_bitmap(hldev, vp->vpath->vp_id,
	    "vpath_handle = 0x"VXGE_OS_STXFMT", "
	    "root = 0x"VXGE_OS_LLXFMT, (ptr_t)vpath_handle, root);

	ll_bits = (u32) (root & 0xFFFFFFFF);

	// TOdo: Process bits

	vxge_hal_trace_log_bitmap(hldev, vp->vpath->vp_id,
	    "<==  %s:%s:%d Result = 0", __FILE__, __func__, __LINE__);

	return (VXGE_HAL_OK);
}

/*
 * __hal_bitmap_process - Processes bits of bitmap
 * @vpath_handle: Virtual Path Handle
 *
 * This routine processes the bit map  lowletency bits.
 */
vxge_hal_status_e
vxge_hal_bitmap_process(vxge_hal_vpath_h vpath_handle)
{
	vxge_hal_status_e status;
	u64 *bitmap = NULL;
	__hal_device_t *hldev;
	__hal_vpath_handle_t *vp = (__hal_vpath_handle_t *)vpath_handle;

	vxge_assert(vpath_handle != NULL);

	hldev = vp->vpath->hldev;

	vxge_hal_trace_log_bitmap(hldev, vp->vpath->vp_id,
	    "<==  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_bitmap(hldev, vp->vpath->vp_id,
	    "vpath_handle = 0x"VXGE_OS_STXFMT, (ptr_t)vpath_handle);

	// TODO: bitmap = vp->vpath->bitmap_mem;

	status = vxge_hal_bitmap_llbits_process(vpath_handle, *bitmap);

	status = vxge_hal_bitmap_sessions_process(vpath_handle, bitmap);

	vxge_hal_trace_log_bitmap(hldev, vp->vpath->vp_id,
	    "<==  %s:%s:%d Result = 0", __FILE__, __func__, __LINE__);

	return (VXGE_HAL_OK);
}
