/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

/*
 * Copyright (c) 2002-2010 Neterion, Inc.
 * All right Reserved.
 *
 * FileName :   vxgehal-bitmap.h
 *
 * Description:  HAL bitmap support structures
 *
 * Created:       20 Nov 2006
 */

#ifndef	VXGE_HAL_BITMAP_H
#define	VXGE_HAL_BITMAP_H

__EXTERN_BEGIN_DECLS

#define	VXGE_HAL_MAX_BITMAP_BITS		32*1024

/*
 * struct __hal_bitmap_entry_t - Entry to map the SQ, SRQ, CQRQ to a bitmap bit
 * @sqh: SQ channel using this bit
 * @sq_ref_count: SQ ref count
 * @srqh: SRQ channel using this bit
 * @srq_ref_count: SRQ ref count
 * @cqrqh: CQRQ channel using this bit
 * @cqrq_ref_count: CQRQ ref count
 *
 * Structure to maintain the mapping between the queues and bitmap bit
 */
typedef struct __hal_bitmap_entry_t {
	vxge_hal_sq_h			sqh;
	u32				sq_ref_count;
	vxge_hal_srq_h			srqh;
	u32				srq_ref_count;
	vxge_hal_cqrq_h			cqrqh;
	u32				cqrq_ref_count;
} __hal_bitmap_entry_t;

vxge_hal_status_e
__hal_bitmap_sq_set(vxge_hal_sq_h sqh);

vxge_hal_status_e
__hal_bitmap_sq_reset(vxge_hal_sq_h sqh);

vxge_hal_status_e
__hal_bitmap_srq_set(vxge_hal_srq_h srqh);

vxge_hal_status_e
__hal_bitmap_srq_reset(vxge_hal_srq_h srqh);

vxge_hal_status_e
__hal_bitmap_cqrq_set(vxge_hal_cqrq_h cqrqh);

vxge_hal_status_e
__hal_bitmap_cqrq_reset(vxge_hal_cqrq_h cqrqh);

u32
__hal_number_of_bits_set_get(u64  val);

void
__hal_set_bits_64_get(u64  val, u32 loc[64]);

void
__hal_set_bits_16_get(u16  val, u32 loc[16]);

vxge_hal_status_e
vxge_hal_bitmap_sessions_process(vxge_hal_vpath_h vpath_handle, u64 *bitmap);

vxge_hal_status_e
vxge_hal_bitmap_llbits_process(vxge_hal_vpath_h vpath_handle, u64 root);

vxge_hal_status_e
vxge_hal_bitmap_process(vxge_hal_vpath_h vpath_handle);

__EXTERN_END_DECLS

#endif /* VXGE_HAL_BITMAP_H */
