/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

/*
 * Copyright (c) 2002-2010 Neterion, Inc.
 * All right Reserved.
 *
 * FileName :   vxgehal-cqrq.h
 *
 * Description:  HAL offload CQRQ list structures
 *
 * Created:       15 Dec 2005
 */

#ifndef	VXGE_HAL_CQRQ_H
#define	VXGE_HAL_CQRQ_H

__EXTERN_BEGIN_DECLS

/*
 * struct __hal_cqrq_t - CQRQ channel
 * @channel: Channel "base" of this Queue,  the common part of all HAL
 *		channels.
 * @callback: Call back function to be called when a CQE is ready in CQRQ
 * @high_cqe_number: The high cqe number posted by adapter in the RR register.
 *		This is used to specify the high cqe number
 *		(always modulo 2^32).
 * @cqrq_opaque_handle: Opaque handle for this CQRQ.(Allocated by the ULD)
 * @opaque_handle: Opaque handle passed for creation of this CQRQ
 *		(Allocated by ULD)
 * @mempool: Memory pool,  the pool from which cqe lsit blocks get allocated
 *		(See vxge_hal_mm.h).
 * @cq_rq_ir: The adapter will overwrite and update this location as CQEs
 *		are returned to the host.
 * @cq_rq_ir_reg_entry: Reg entry of cq_rq_ir_t
 * @first_block: The address of first block in the CQRQ.
 * @last_block: The address of last block in the CQRQ.
 * @first_block_address: The DMA Address of the first block.
 * @first_block_length: The Length of the first block.
 * @cqrq_id: The object Id of the CQRQ channel
 * @cqrq_type: CQRQ type.
 * @cqrq_length: The length of the CQRQ.
 * @is_shared: If this channel is shared.
 * @immed_data_length: Length of Immediate data bytes to follow CQE.
 *		This has to be multiple of cache_alignment size.
 * @cqe_size: size of CQE.
 * @cqe_priv_size: Per CQE size reserved (by HAL) for ULD to keep per-wqe
 *		data (e.g.,  DMA handle for Solaris).
 * @cqes_per_block: Number of cqes per hardware-defined RxD
 *		block.
 * @cqeblock_priv_size: Reserved at the end of each list block. HAL internal
 *		usage. Not to confuse with @cqe_priv_size.
 * @low_threshold_notify: CQE Low Threshold to generate interrupt/message.
 * @stats: Statistics of this cqrq
 *
 * CQE List or CQRQ channel. This channel is used for the completion of
 * WQEs that were queued through channel __hal_srq_t {}.
 *
 */
typedef struct __hal_cqrq_t  {
	__hal_channel_t				channel;
	vxge_hal_callback_h			callback;
	u64 __vxge_os_attr_cacheline_aligned	high_cqe_number;
	vxge_hal_opaque_handle_t			*cqrq_opaque_handle;
	vxge_hal_opaque_handle_t			*opaque_handle;
	vxge_hal_mempool_t			*mempool;
	__hal_cq_rq_ir_t			*cq_rq_ir;
	__hal_reg_entry_t			*cq_rq_ir_reg_entry;
	void					*first_block;
	void					*last_block;
	u64					first_block_address;
	u32					first_block_length;
	u32					cqrq_id;
	u32					cqrq_type;
	u32					cqrq_length;
	u32					is_shared;
	u32					immed_data_length;
	u32					cqe_size;
	u32					cqe_priv_size;
	u32					cqes_per_block;
	u32					cqeblock_priv_size;
	u32					low_threshold_notify;
	vxge_hal_vpath_stats_sw_cqrq_info_t	stats;
} __hal_cqrq_t;

vxge_hal_status_e
__hal_cqrq_create(
	vxge_hal_vpath_h vpath_handle,
	__hal_cqrq_t *cqrq);

vxge_hal_status_e
__hal_cqrq_delete(
	vxge_hal_vpath_h vpath_handle,
	__hal_cqrq_t *cqrq);

vxge_hal_status_e
__hal_cqrq_replenish(
	__hal_cqrq_t *cqrq);

vxge_hal_status_e
__hal_cqrq_ready_cqe_check(
	vxge_hal_obj_id_t cqrq_id);

vxge_hal_status_e
__hal_cqrq_fcqebw(
	vxge_hal_vpath_h vpath_handle,
	vxge_hal_obj_id_t cqrq_id,
	vxge_hal_opaque_handle_t *opaque_handle);

vxge_hal_status_e
vxge_hal_cqrq_ready_hw_cqe_check(
	vxge_hal_vpath_h vpath_handle,
	vxge_hal_obj_id_t cqrq_id);

vxge_hal_status_e
vxge_hal_cqrq_next_ready_hw_cqe_get(
	vxge_hal_vpath_h vpath_handle,
	vxge_hal_obj_id_t cqrq_id,
	vxge_hal_cqrq_hw_cqe_Info_t *cqe_info);

__EXTERN_END_DECLS

#endif /* VXGE_HAL_CQRQ_H */
