/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

/*
 * Copyright (c) 2002-2010 Neterion, Inc.
 * All right Reserved.
 *
 * FileName :   vxgehal-dmq.c
 *
 * Description:  HAL Down Message Queue functionality
 *
 * Created:       14 December 2005
 */
#include "vxgehal.h"

#ifdef	VXGE_HAL_INTERNAL_COMPILER_ERROR

#pragma optimize("", off)

#endif
/*
 * __hal_dmq_first_block_address_get - Get the dma address of the first block
 * @umqh: Handle of UMQ
 *
 * This function returns the dma address of the first block
 */
u64
__hal_dmq_first_block_address_get(
	vxge_hal_dmq_h dmqh)
{
	__hal_dmq_t *dmq = (__hal_dmq_t *)dmqh;

	return (dmq->first_block_address);
}

/*
 * __hal_dmq_first_block_length_get - Get the length of the first block
 * @dmqh: Handle of DMQ
 *
 * This function returns the address of the first block
 */
u32
__hal_dmq_first_block_length_get(
	vxge_hal_dmq_h dmqh)
{
	__hal_dmq_t *dmq = (__hal_dmq_t *)dmqh;

	return (dmq->first_block_length);
}

/*
 * __hal_dmsg_mempool_item_alloc - Allocate List blocks for down messages
 *				callback
 * @mempoolh: Handle to memory pool
 * @memblock: Address of this memory block
 * @memblock_index: Index of this memory block
 * @dma_object: dma object for this block
 * @item: Pointer to this item
 * @index: Index of this item in memory block
 * @is_last: If this is last item in the block
 * @userdata: Specific data of user
 *
 * This function is callback passed to __hal_mempool_create to create memory
 * pool for messages
 */
static vxge_hal_status_e
__hal_dmsg_mempool_item_alloc(
	vxge_hal_mempool_h mempoolh,
	void *memblock,
	u32 memblock_index,
	vxge_hal_mempool_dma_t *dma_object,
	void *item,
	u32 index,
	u32 is_last,
	void *userdata)
{
	u32 memblock_item_idx;
	__hal_dmq_t *dmqdata = (__hal_dmq_t *)userdata;
	__hal_list_block_t *prev_block, *next_block;
	__hal_list_block_wrapper_t *prev_wrapper;
	__hal_dmq_blk_priv_t *dmq_blk_priv;
	__hal_device_t *hldev = (__hal_device_t *)dmqdata->channel.devh;

	vxge_assert(dmqdata != NULL);

	vxge_hal_trace_log_pool(hldev,
	    ((__hal_vpath_handle_t *)dmqdata->channel.vph)->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_pool(hldev,
	    ((__hal_vpath_handle_t *)dmqdata->channel.vph)->vpath->vp_id,
	    "mempoolh = 0x"VXGE_OS_STXFMT", "
	    "memblock = 0x"VXGE_OS_STXFMT", memblock_index = %d, "
	    "dma_object = 0x"VXGE_OS_STXFMT", item = 0x"VXGE_OS_STXFMT", "
	    "index = %d, is_last = %d, userdata = 0x"VXGE_OS_STXFMT,
	    (ptr_t)mempoolh, (ptr_t)memblock, memblock_index,
	    (ptr_t)dma_object, (ptr_t)item, index, is_last,
	    (ptr_t)userdata);

	dmq_blk_priv = (__hal_dmq_blk_priv_t *)__hal_mempool_item_priv(
	    mempoolh, memblock_index, item, &memblock_item_idx);

	vxge_assert(dmq_blk_priv != NULL);

	dmqdata->channel.dtr_arr[index].dtr = item;

	dmq_blk_priv->dma_offset = 0;
	dmq_blk_priv->dma_addr = dma_object->addr;
	dmq_blk_priv->dma_handle = dma_object->handle;
	dmq_blk_priv->memblock = memblock;
	dmq_blk_priv->wrapper = (__hal_list_block_wrapper_t *)
	    (((char *)memblock) + VXGE_HAL_LIST_BLOCK_ITEMS_SIZE);
	dmq_blk_priv->dmqh = (vxge_hal_dmq_h)userdata;
	dmq_blk_priv->compl_offset = VXGE_HAL_LIST_BLOCK_ITEMS_SIZE;
	dmq_blk_priv->post_offset = 0;

	__hal_list_block_memblock_priv_data_set(memblock,
	    (u64)(ptr_t)dmq_blk_priv);
	__hal_list_block_memblock_idx_set(memblock, memblock_index);

	if (index > 0) {
		prev_block = __hal_mempool_item(mempoolh, index-1);
		next_block = __hal_mempool_item(mempoolh, 0);
		prev_wrapper = (__hal_list_block_wrapper_t *)
		    (((char *)prev_block) + VXGE_HAL_LIST_BLOCK_ITEMS_SIZE);
		__hal_wrapper_link(prev_wrapper, dmq_blk_priv->wrapper,
		    dma_object->addr, VXGE_HAL_LIST_BLOCK_ITEMS_SIZE);
		__hal_wrapper_htonll(prev_wrapper);
	} else {
		prev_block = memblock;
		next_block = memblock;
		__hal_wrapper_link(dmq_blk_priv->wrapper, dmq_blk_priv->wrapper,
		    dma_object->addr, VXGE_HAL_LIST_BLOCK_ITEMS_SIZE);
		__hal_wrapper_htonll(dmq_blk_priv->wrapper);
	}

	__hal_list_block_link(memblock, prev_block, next_block);

	vxge_hal_trace_log_pool(hldev,
	    ((__hal_vpath_handle_t *)dmqdata->channel.vph)->vpath->vp_id,
	    "<==  %s:%s:%d  Result: 0", __FILE__, __func__, __LINE__);

	return (VXGE_HAL_OK);
}

/*
 * __hal_dmsg_mempool_item_free - Free List blocks for message list callback
 * @mempoolh: Handle to memory pool
 * @memblock: Address of this memory block
 * @memblock_index: Index of this memory block
 * @dma_object: dma object for this block
 * @item: Pointer to this item
 * @index: Index of this item in memory block
 * @is_last: If this is last item in the block
 * @userdata: Specific data of user
 *
 * This function is callback passed to __hal_mempool_free to destroy memory
 * pool for message list
 */
static vxge_hal_status_e
__hal_dmsg_mempool_item_free(
	vxge_hal_mempool_h mempoolh,
	void *memblock,
	u32 memblock_index,
	vxge_hal_mempool_dma_t *dma_object,
	void *item,
	u32 index,
	u32 is_last,
	void *userdata)
{
	u32 memblock_item_idx;
	__hal_list_block_t *prev_block;
	__hal_list_block_wrapper_t *prev_wrapper;
	__hal_dmq_blk_priv_t *dmq_blk_priv;
	__hal_dmq_t *dmqdata = (__hal_dmq_t *)userdata;
	__hal_device_t *hldev = (__hal_device_t *)dmqdata->channel.devh;

	vxge_assert(dmqdata != NULL);

	vxge_hal_trace_log_pool(hldev,
	    ((__hal_vpath_handle_t *)dmqdata->channel.vph)->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_pool(hldev,
	    ((__hal_vpath_handle_t *)dmqdata->channel.vph)->vpath->vp_id,
	    "mempoolh = 0x"VXGE_OS_STXFMT", "
	    "memblock = 0x"VXGE_OS_STXFMT", memblock_index = %d, "
	    "dma_object = 0x"VXGE_OS_STXFMT", item = 0x"VXGE_OS_STXFMT", "
	    "index = %d, is_last = %d, userdata = 0x"VXGE_OS_STXFMT,
	    (ptr_t)mempoolh, (ptr_t)memblock, memblock_index,
	    (ptr_t)dma_object, (ptr_t)item, index, is_last,
	    (ptr_t)userdata);

	dmq_blk_priv = (__hal_dmq_blk_priv_t *)__hal_mempool_item_priv(
	    mempoolh, memblock_index, item, &memblock_item_idx);

	vxge_assert(dmq_blk_priv != NULL);

	prev_block = __hal_list_block_prev_pointer(memblock);

	vxge_assert(prev_block != NULL);

	prev_wrapper = (__hal_list_block_wrapper_t *)
	    (((char *)prev_block) + VXGE_HAL_LIST_BLOCK_ITEMS_SIZE);

	__hal_wrapper_unlink(prev_wrapper, dmq_blk_priv->wrapper);

	__hal_list_block_unlink(memblock);

	vxge_hal_trace_log_pool(hldev,
	    ((__hal_vpath_handle_t *)dmqdata->channel.vph)->vpath->vp_id,
	    "<==  %s:%s:%d  Result: 0", __FILE__, __func__, __LINE__);

	return (VXGE_HAL_OK);
}

/*
 * __hal_dmq_create - Create down message Queue
 * @vpath_handle: Virtual path handle to which this queue belongs
 *
 * This function creates down message queue with the configuration set
 */
vxge_hal_status_e
__hal_dmq_create(
	vxge_hal_vpath_h vpath_handle)
{
	vxge_hal_status_e status;
	__hal_list_block_t *compl_block;
	__hal_dmq_blk_priv_t *compl_priv;
	__hal_dmq_t *dmq;
	__hal_device_t *hldev;
	__hal_vpath_handle_t *vp = (__hal_vpath_handle_t *)vpath_handle;
	vxge_hal_mempool_dma_t *dma_object;
	u32 dmq_length;

	vxge_assert(vpath_handle != NULL);

	hldev = vp->vpath->hldev;

	vxge_hal_trace_log_dmq(hldev, vp->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_dmq(hldev, vp->vpath->vp_id,
	    "vpath_handle = 0x"VXGE_OS_STXFMT, (ptr_t)vpath_handle);

	if (vpath_handle == NULL) {
		vxge_hal_err_log_dmq(hldev, vp->vpath->vp_id,
		    "<==  %s:%s:%d  Result: %d", __FILE__, __func__,
		    __LINE__, VXGE_HAL_ERR_INVALID_HANDLE);
		return (VXGE_HAL_ERR_INVALID_HANDLE);
	}

	dmq_length = vp->vpath->hldev->
	    header.config.vp_config[vp->vpath->vp_id].dmq.dmq_length;

	dmq = (__hal_dmq_t *)__hal_channel_allocate(
	    (vxge_hal_device_h)vp->vpath->hldev,
	    vpath_handle,
	    VXGE_HAL_CHANNEL_TYPE_DOWN_MESSAGE_QUEUE,
	    dmq_length,
	    TRUE,
	    FALSE,
	    FALSE,
	    0,
	    NULL);

	if (dmq == NULL) {
		vxge_hal_err_log_dmq(hldev, vp->vpath->vp_id,
		    "Memory allocation failed %s : %d",
		    __func__, __LINE__);
		vxge_hal_trace_log_dmq(hldev, vp->vpath->vp_id,
		    "<==  %s:%s:%d  Result: %d",
		    __FILE__, __func__, __LINE__,
		    VXGE_HAL_ERR_OUT_OF_MEMORY);
		return (VXGE_HAL_ERR_OUT_OF_MEMORY);
	}

	vp->vpath->dmqh = (vxge_hal_dmq_h)dmq;

	dmq->stats = &vp->vpath->sw_stats->dmq_stats;

	dmq->dmq_length = dmq_length;
	dmq->gen_compl = vp->vpath->hldev->
	    header.config.vp_config[vp->vpath->vp_id].dmq.gen_compl;
	dmq->intr_ctrl = vp->vpath->hldev->
	    header.config.vp_config[vp->vpath->vp_id].dmq.intr_ctrl;

#if defined(VXGE_HAL_DMQ_POST)
	vxge_os_spin_lock_init(&dmq->channel.post_lock,
	    vp->vpath->hldev->pdev);
#elif defined(VXGE_HAL_DMQ_POST_IRQ)
	vxge_os_spin_lock_init_irq(&dmq->channel.post_lock,
	    vp->vpath->hldev->irqh);
#endif

	dmq->msgblock_priv_size = sizeof (__hal_dmq_blk_priv_t);

	dmq->mempool = __hal_mempool_create(
	    (vxge_hal_device_h)vp->vpath->hldev,
	    VXGE_OS_HOST_PAGE_SIZE,
	    VXGE_OS_HOST_PAGE_SIZE,
	    dmq->msgblock_priv_size,
	    dmq->dmq_length,
	    dmq->dmq_length,
	    __hal_dmsg_mempool_item_alloc,
	    __hal_dmsg_mempool_item_free,
	    dmq);

	if (dmq->mempool == NULL) {
		vxge_hal_err_log_dmq(hldev, vp->vpath->vp_id,
		    "Memory allocation for dmq failed %s : %d",
		    __func__, __LINE__);
		__hal_dmq_delete(vpath_handle, dmq);
		vxge_hal_trace_log_dmq(hldev, vp->vpath->vp_id,
		    "<==  %s:%s:%d  Result: %d",
		    __FILE__, __func__, __LINE__,
		    VXGE_HAL_ERR_OUT_OF_MEMORY);
		return (VXGE_HAL_ERR_OUT_OF_MEMORY);
	}

	status = __hal_channel_initialize(&dmq->channel);

	if (status != VXGE_HAL_OK) {
		vxge_hal_err_log_dmq(hldev, vp->vpath->vp_id,
		    "dmq channel initialization failed == > %s : %d",
		    __func__, __LINE__);
		__hal_dmq_delete(vpath_handle, dmq);
		vxge_hal_trace_log_dmq(hldev, vp->vpath->vp_id,
		    "<==  %s:%s:%d  Result: %d",
		    __FILE__, __func__, __LINE__, status);
		return (status);
	}

	dmq->channel.compl_index = 0;

	compl_block = dmq->channel.dtr_arr[dmq->channel.compl_index].dtr;
	compl_priv = (__hal_dmq_blk_priv_t *)(ptr_t)
	    __hal_list_block_priv_data(compl_block);

	vxge_assert(compl_priv != NULL);

	compl_priv->compl_offset = 0;

	dmq->high_msg_address = compl_priv->dma_addr + compl_priv->compl_offset;

	dmq->bytes_used = 0;
	dmq->bytes_free = dmq->dmq_length * VXGE_HAL_LIST_BLOCK_ITEMS_SIZE;

	dma_object = __hal_mempool_memblock_dma(dmq->mempool, 0);

	vxge_assert(dma_object != NULL);

	dmq->first_block_address = dma_object->addr;

	dmq->first_block_length = VXGE_HAL_LIST_BLOCK_ITEMS_SIZE;

	vxge_hal_trace_log_dmq(hldev, vp->vpath->vp_id,
	    "<==  %s:%s:%d  Result: 0", __FILE__, __func__, __LINE__);

	return (VXGE_HAL_OK);
}

/*
 * __hal_dmq_reset - Resets the dmq
 * @dmqh: dmq to be reset
 *
 * This function resets the dmq during vpath reset operation
 */
vxge_hal_status_e
__hal_dmq_reset(
	vxge_hal_dmq_h dmqh)
{
	u32 index;
	vxge_hal_status_e status;
	__hal_device_t *hldev;
	__hal_dmq_t *dmq = (__hal_dmq_t *)dmqh;
	__hal_list_block_t *block;
	__hal_dmq_blk_priv_t *block_priv;

	vxge_assert(dmq != NULL);

	hldev = (__hal_device_t *)dmq->channel.devh;

	vxge_hal_trace_log_dmq(hldev,
	    ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_dmq(hldev,
	    ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath->vp_id,
	    "dmq = 0x"VXGE_OS_STXFMT, (ptr_t)dmq);

	status = __hal_channel_reset(&dmq->channel);

	if (status != VXGE_HAL_OK) {

		vxge_hal_err_log_dmq(hldev,
		    ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath->vp_id,
		    "<==  %s:%s:%d  Result: %d",
		    __FILE__, __func__, __LINE__, status);
		return (status);
	}

	for (index = 0;
	    index < __hal_mempool_item_count(dmq->mempool);
	    index++) {

		block = __hal_mempool_item(dmq->mempool, index);

		vxge_assert(block != NULL);

		dmq->channel.dtr_arr[index].dtr = block;

		block_priv = (__hal_dmq_blk_priv_t *)(ptr_t)
		    __hal_list_block_priv_data(block);

		vxge_assert(block_priv != NULL);

		block_priv->compl_offset = VXGE_HAL_LIST_BLOCK_ITEMS_SIZE;

		block_priv->post_offset = 0;
	}

	dmq->channel.compl_index = 0;

	block = dmq->channel.dtr_arr[dmq->channel.compl_index].dtr;

	block_priv = (__hal_dmq_blk_priv_t *)(ptr_t)
	    __hal_list_block_priv_data(block);

	vxge_assert(block_priv != NULL);

	block_priv->compl_offset = 0;

	dmq->high_msg_address = block_priv->dma_addr + block_priv->compl_offset;

	dmq->bytes_used = 0;

	dmq->bytes_free = dmq->dmq_length * VXGE_HAL_LIST_BLOCK_ITEMS_SIZE;

	vxge_hal_trace_log_dmq(hldev,
	    ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath->vp_id,
	    "<==  %s:%s:%d  Result: 0", __FILE__, __func__, __LINE__);

	return (VXGE_HAL_OK);
}

/*
 * __hal_dmq_doorbell_reset - Resets the doorbell fifo
 * @dmqh: DMQ to be reset
 *
 * This function resets the doorbell fifo during if fifo error occurs
 */
vxge_hal_status_e
__hal_dmq_doorbell_reset(
	vxge_hal_dmq_h dmqh)
{
	u32 index;
	u32 num_bytes;
	u64 high_msg_addr;
	__hal_device_t *hldev;
	__hal_virtualpath_t *vpath;
	__hal_list_block_t *compl_block, *post_block;
	__hal_dmq_blk_priv_t *compl_priv, *post_priv;
	__hal_dmq_t *dmq = (__hal_dmq_t *)dmqh;
	vxge_hal_status_e status = VXGE_HAL_OK;

	vxge_assert(dmqh != NULL);

	vpath = ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath;

	hldev = vpath->hldev;

	vxge_hal_trace_log_dmq(hldev, vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_dmq(hldev, vpath->vp_id,
	    "dmq = 0x"VXGE_OS_STXFMT, (ptr_t)dmqh);

	status =  __hal_message_db_reset(dmq->channel.vph);

	if (status != VXGE_HAL_OK) {
		vxge_hal_err_log_dmq(hldev, vpath->vp_id,
		    "<==  %s:%s:%d  Result: 0", __FILE__, __func__, __LINE__);
		return (status);
	}

	high_msg_addr = VXGE_HAL_UMQ_DMQ_IR_GET_DMQ_LAST_NOT_USED_BLOCK_ADDR(
	    vpath->umq_dmq_ir->control_1);

	num_bytes = 0;

	compl_block = dmq->channel.dtr_arr[dmq->channel.compl_index].dtr;
	compl_priv = (__hal_dmq_blk_priv_t *)(ptr_t)
	    __hal_list_block_priv_data(compl_block);

	vxge_assert(compl_priv != NULL);

	post_block = dmq->channel.dtr_arr[dmq->channel.post_index].dtr;

	post_priv = (__hal_dmq_blk_priv_t *)(ptr_t)
	    __hal_list_block_priv_data(post_block);

	vxge_assert(post_priv != NULL);

	for (index = dmq->channel.compl_index;
	    index <= dmq->channel.post_index;
	    index++) {

		if (!((high_msg_addr >= compl_priv->dma_addr) &&
		    (high_msg_addr <= (compl_priv->dma_addr +
		    VXGE_HAL_LIST_BLOCK_ITEMS_SIZE)))) {
				continue;
		}

		if (dmq->channel.compl_index == dmq->channel.post_index) {
			num_bytes += (u32)(post_priv->post_offset -
			    compl_priv->compl_offset);
		} else {
			if (index == dmq->channel.post_index)
				num_bytes += (u32)post_priv->post_offset;
			else
				num_bytes +=
				    (u32)(VXGE_HAL_LIST_BLOCK_ITEMS_SIZE -
				    compl_priv->compl_offset);
		}

	}

	if (num_bytes > 0) {
		__hal_message_db_post(dmq->channel.vph,
		    num_bytes,
		    NULL,
		    0);
	}

	vxge_hal_trace_log_dmq(hldev, vpath->vp_id,
	    "<==  %s:%s:%d  Result: 0", __FILE__, __func__, __LINE__);
	return (VXGE_HAL_OK);
}

/*
 * __hal_dmq_delete - Removes the down Message Queue
 * @vpath_handle: Handle to the virtual path
 * @dmq: dmq to be deleted
 *
 * This function freeup the memory pool and removes the
 * message queue.
 */

vxge_hal_status_e
__hal_dmq_delete(
	vxge_hal_vpath_h vpath_handle,
	__hal_dmq_t *dmq)
{
	__hal_device_t *hldev;
	__hal_vpath_handle_t *vp = (__hal_vpath_handle_t *)vpath_handle;

	vxge_assert((vpath_handle != NULL) && (dmq != NULL));

	hldev = vp->vpath->hldev;

	vxge_hal_trace_log_dmq(hldev, vp->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_dmq(hldev, vp->vpath->vp_id,
	    "vpath_handle = 0x"VXGE_OS_STXFMT", dmq = 0x"VXGE_OS_STXFMT,
	    (ptr_t)vpath_handle, (ptr_t)dmq);

	if (dmq == NULL) {
		vxge_hal_err_log_dmq(hldev, vp->vpath->vp_id,
		    "<==  %s:%s:%d  Result: %d",
		    __FILE__, __func__, __LINE__, VXGE_HAL_ERR_INVALID_HANDLE);
		return (VXGE_HAL_ERR_INVALID_HANDLE);
	}

	if (dmq->mempool) {
		__hal_mempool_destroy(dmq->mempool);
	}

	__hal_channel_terminate(&dmq->channel);

#if defined(VXGE_HAL_DMQ_POST)
	vxge_os_spin_lock_destroy(&dmq->channel.post_lock,
	    vp->vpath->hldev->header.pdev);
#elif defined(VXGE_HAL_DMQ_POST_IRQ)
	vxge_os_spin_lock_destroy_irq(&dmq->channel.post_lock,
	    vp->vpath->hldev->header.pdev);
#endif

	__hal_channel_free(&dmq->channel);

	vp->vpath->dmqh = (vxge_hal_dmq_h)NULL;

	vxge_hal_trace_log_dmq(hldev, vp->vpath->vp_id,
	    "<==  %s:%s:%d  Result: 0", __FILE__, __func__, __LINE__);

	return (VXGE_HAL_OK);
}

/*
 * __hal_dmq_complete - Process completed messages in the DMQ
 * @dmq: Down Message queue
 *
 * This function reads IR register and adjusts the compl_index and
 * compl_offset so that the space can be reused.
 */

vxge_hal_status_e
__hal_dmq_complete(
	__hal_dmq_t *dmq)
{
	__hal_list_block_t *compl_block;
	__hal_dmq_blk_priv_t *compl_priv;
	__hal_device_t *hldev;
	__hal_virtualpath_t *vpath =
	    ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath;
	u64 prev_offset;

	vxge_assert(dmq != NULL);

	hldev = vpath->hldev;

	vxge_hal_trace_log_dmq(hldev, vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_dmq(hldev, vpath->vp_id,
	    "dmq = 0x"VXGE_OS_STXFMT, (ptr_t)dmq);

	compl_block = dmq->channel.dtr_arr[dmq->channel.compl_index].dtr;
	compl_priv = (__hal_dmq_blk_priv_t *)(ptr_t)
	    __hal_list_block_priv_data(compl_block);

	vxge_assert(compl_priv != NULL);

	vxge_hal_info_log_dmq(hldev, vpath->vp_id,
	    "compl_index = %d, Compl_block = 0x"VXGE_OS_LLXFMT", "
	    "Compl_offset = 0x"VXGE_OS_LLXFMT,
	    dmq->channel.compl_index, (u64)compl_priv->dma_addr,
	    compl_priv->compl_offset);

	dmq->high_msg_address =
	    VXGE_HAL_UMQ_DMQ_IR_GET_DMQ_LAST_NOT_USED_BLOCK_ADDR(
	    vpath->umq_dmq_ir->control_1);

	vxge_hal_info_log_dmq(hldev, vpath->vp_id,
	    "High Message Address = 0x"VXGE_OS_LLXFMT,
	    dmq->high_msg_address);

	if (dmq->high_msg_address ==
	    compl_priv->dma_addr + compl_priv->compl_offset) {
		vxge_hal_info_log_dmq(hldev, vpath->vp_id,
		    "No new completed messages == > %s : %d",
		    __func__, __LINE__);
		vxge_hal_trace_log_dmq(hldev, vpath->vp_id,
		    "<==  %s:%s:%d  Result: %d",
		    __FILE__, __func__, __LINE__,
		    VXGE_HAL_INF_NO_MORE_COMPLETED_DESCRIPTORS);
		return (VXGE_HAL_OK);
	}

	for (;;) {

		if ((dmq->high_msg_address >= compl_priv->dma_addr) &&
		    (dmq->high_msg_address <= (compl_priv->dma_addr +
		    VXGE_HAL_LIST_BLOCK_ITEMS_SIZE))) {

			prev_offset = compl_priv->compl_offset;
			compl_priv->compl_offset = dmq->high_msg_address -
			    compl_priv->dma_addr;

			if (compl_priv->compl_offset ==
			    VXGE_HAL_LIST_BLOCK_ITEMS_SIZE) {

				compl_priv->post_offset = 0;

				dmq->channel.compl_index++;
				if (dmq->channel.compl_index ==
				    dmq->channel.length)
					dmq->channel.compl_index = 0;
			}

			dmq->bytes_used -=
			    (compl_priv->compl_offset - prev_offset);
			dmq->bytes_free +=
			    (compl_priv->compl_offset - prev_offset);

			break;

		} else {

			dmq->bytes_used -=
			    (VXGE_HAL_LIST_BLOCK_ITEMS_SIZE -
			    compl_priv->compl_offset);
			dmq->bytes_free +=
			    (VXGE_HAL_LIST_BLOCK_ITEMS_SIZE -
			    compl_priv->compl_offset);

			compl_priv->compl_offset =
			    VXGE_HAL_LIST_BLOCK_ITEMS_SIZE;
			compl_priv->post_offset = 0;

			dmq->channel.compl_index++;
			if (dmq->channel.compl_index == dmq->channel.length)
				dmq->channel.compl_index = 0;

			compl_block =
			    dmq->channel.dtr_arr[dmq->channel.compl_index].dtr;
			compl_priv = (__hal_dmq_blk_priv_t *)(ptr_t)
			    __hal_list_block_priv_data(compl_block);
			vxge_hal_info_log_dmq(hldev, vpath->vp_id,
			    "compl_index = %d, Compl_block = 0x"VXGE_OS_LLXFMT
			    ", Compl_offset = 0x"VXGE_OS_LLXFMT,
			    dmq->channel.compl_index, (u64)compl_priv->dma_addr,
			    compl_priv->compl_offset);

		}

	}

	vxge_hal_info_log_dmq(hldev, vpath->vp_id,
	    "compl_index = %d, Compl_block = 0x"VXGE_OS_LLXFMT", "
	    "Compl_offset = 0x"VXGE_OS_LLXFMT, dmq->channel.compl_index,
	    (u64)compl_priv->dma_addr, compl_priv->compl_offset);

	vxge_hal_info_log_dmq(hldev, vpath->vp_id,
	    "completed messages processed. free 0x"VXGE_OS_LLXFMT
	    " used 0x"VXGE_OS_LLXFMT,
	    dmq->bytes_free, dmq->bytes_used);

	vxge_hal_trace_log_dmq(hldev, vpath->vp_id,
	    "<==  %s:%s:%d  Result: 0", __FILE__, __func__, __LINE__);

	return (VXGE_HAL_OK);
}

/*
 * __hal_dmq_msg_copy - Copy the message to the DMQ
 * @dmq: Down message queue
 * @dmsgh: Handle to the message that needs to be posted.
 *
 * This function copies the message from the buffer into DMAable
 * channel blocks.
 */

vxge_hal_status_e
__hal_dmq_msg_copy(
	__hal_dmq_t *dmq,
	vxge_hal_down_msg_h dmsgh)
{
	__hal_list_block_t *post_block;
	__hal_dmq_blk_priv_t *post_priv;
	__hal_device_t *hldev;
	__hal_down_msg_t *dmsg = (__hal_down_msg_t *)dmsgh;
	u32 msg_length;
	char *tmp_ptr;

	vxge_assert((dmq != NULL) && (dmsg != NULL));

	hldev = (__hal_device_t *)dmq->channel.devh;

	vxge_hal_trace_log_dmq(hldev,
	    ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_dmq(hldev,
	    ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath->vp_id,
	    "dmq = 0x"VXGE_OS_STXFMT", dmsgh = 0x"VXGE_OS_STXFMT,
	    (ptr_t)dmq, (ptr_t)dmsgh);

	msg_length = VXGE_HAL_DMSG_LENGTH_GET((__hal_down_msg_t *)dmsgh);

	vxge_hal_trace_log_dmq(hldev,
	    ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath->vp_id,
	    "Msg Length = %d", msg_length);

	if (msg_length > dmq->bytes_free) {

		vxge_hal_err_log_dmq(hldev,
		    ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath->vp_id,
		    "No more space in the queue == > %s : %d",
		    __func__, __LINE__);
		vxge_hal_trace_log_dmq(hldev,
		    ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath->vp_id,
		    "<==  %s:%s:%d  Result: %d", __FILE__, __func__, __LINE__,
		    VXGE_HAL_ERR_OUT_OF_SPACE);
		return (VXGE_HAL_ERR_OUT_OF_SPACE);

	}

	post_block = dmq->channel.dtr_arr[dmq->channel.post_index].dtr;

	post_priv = (__hal_dmq_blk_priv_t *)(ptr_t)
	    __hal_list_block_priv_data(post_block);

	vxge_hal_info_log_dmq(hldev,
	    ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath->vp_id,
	    "post_index = %d, post_block = 0x"VXGE_OS_LLXFMT", "
	    "post_offset = 0x"VXGE_OS_LLXFMT, dmq->channel.post_index,
	    (u64)post_priv->dma_addr, post_priv->post_offset);

	vxge_assert(post_priv != NULL);

	tmp_ptr = ((char *)post_block) + post_priv->post_offset;

	for (;;) {

		if (post_priv->post_offset + msg_length <=
		    VXGE_HAL_LIST_BLOCK_ITEMS_SIZE) {

			if (msg_length > 0)
				vxge_os_memcpy(tmp_ptr, dmsg, msg_length);

			post_priv->post_offset += msg_length;

			dmq->bytes_used += msg_length;
			dmq->bytes_free -= msg_length;

			if (post_priv->post_offset ==
			    VXGE_HAL_LIST_BLOCK_ITEMS_SIZE) {

				dmq->channel.post_index++;
				if (dmq->channel.post_index ==
				    dmq->channel.length)
					dmq->channel.post_index = 0;

				post_block = dmq->channel.dtr_arr[
				    dmq->channel.post_index].dtr;

				post_priv = (__hal_dmq_blk_priv_t *)(ptr_t)
				    __hal_list_block_priv_data(post_block);

				post_priv->compl_offset = 0;

			}

			break;
		} else {
			u32 length;

			length = (u32)(VXGE_HAL_LIST_BLOCK_ITEMS_SIZE -
			    post_priv->post_offset);

			if (length >= msg_length) {

				vxge_os_memcpy(tmp_ptr, dmsg, msg_length);

				msg_length = 0;

			} else {
				vxge_os_memcpy(tmp_ptr, dmsg, length);

				msg_length -= length;

				dmsg = (__hal_down_msg_t *)
				    (((char *)dmsg) + length);

			}

			post_priv->post_offset = VXGE_HAL_LIST_BLOCK_ITEMS_SIZE;

			dmq->bytes_used += length;
			dmq->bytes_free -= length;

			dmq->channel.post_index++;
			if (dmq->channel.post_index == dmq->channel.length)
				dmq->channel.post_index = 0;

			post_block = dmq->channel.dtrr[dmq->channel.post_index];

			post_priv = (__hal_dmq_blk_priv_t *)(ptr_t)
			    __hal_list_block_priv_data(post_block);

			post_priv->compl_offset = 0;

			if (msg_length == 0)
				break;

			vxge_hal_info_log_dmq(hldev, ((__hal_vpath_handle_t *)
			    dmq->channel.vph)->vpath->vp_id,
			    "post_index = %d, post_block = 0x"VXGE_OS_LLXFMT", "
			    "post_offset = 0x"VXGE_OS_LLXFMT,
			    dmq->channel.post_index, (u64)post_priv->dma_addr,
			    post_priv->post_offset);

			tmp_ptr = (char *)post_block;

		}

	}


	vxge_hal_info_log_dmq(hldev,
	    ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath->vp_id,
	    "post_index = %d, post_block = 0x"VXGE_OS_LLXFMT", "
	    "post_offset = 0x"VXGE_OS_LLXFMT, dmq->channel.post_index,
	    (u64)post_priv->dma_addr, post_priv->post_offset);

	vxge_hal_info_log_dmq(hldev,
	    ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath->vp_id,
	    "Posted message. free 0x"VXGE_OS_LLXFMT
	    "used 0x"VXGE_OS_LLXFMT, dmq->bytes_free, dmq->bytes_used);

	vxge_hal_trace_log_dmq(hldev,
	    ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath->vp_id,
	    "<==  %s:%s:%d  Result: 0", __FILE__, __func__, __LINE__);

	return (VXGE_HAL_OK);
}


/*
 * __hal_dmq_post - Post the message to the DMQ
 * @dmq: Down message queue
 * @dmsgh: Handle to the message that needs to be posted.
 *
 * This function calls __hal_dmq_msg_copy to copy message into
 * DMQ and posts doorbell.
 */

vxge_hal_status_e
__hal_dmq_post(
	__hal_dmq_t *dmq,
	vxge_hal_down_msg_h dmsgh)
{
	__hal_device_t *hldev;
	vxge_hal_status_e status = VXGE_HAL_OK;
	u32 msg_length, pad_length = 0;

	vxge_assert((dmq != NULL) && (dmsgh != NULL));

	hldev = (__hal_device_t *)dmq->channel.devh;

	vxge_hal_trace_log_dmq(hldev,
	    ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_dmq(hldev,
	    ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath->vp_id,
	    "dmq = 0x"VXGE_OS_STXFMT", dmsgh = 0x"VXGE_OS_STXFMT,
	    (ptr_t)dmq, (ptr_t)dmsgh);

	if (dmsgh == NULL) {
		vxge_hal_info_log_dmq(hldev,
		    ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath->vp_id,
		    "null pointer passed dmsgh = null %s : %d",
		    __func__, __LINE__);
		vxge_hal_trace_log_dmq(hldev,
		    ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath->vp_id,
		    "<==  %s:%s:%d  Result: %d", __FILE__, __func__, __LINE__,
		    VXGE_HAL_ERR_INVALID_HANDLE);
		return (VXGE_HAL_ERR_INVALID_HANDLE);
	}

#if defined(VXGE_HAL_DMQ_POST)
	vxge_os_spin_lock(&dmq->channel.post_lock);
#elif defined(VXGE_HAL_DMQ_POST_IRQ)
	vxge_os_spin_lock_irq(&dmq->channel.post_lock, flags);
#endif

	__hal_dmq_complete(dmq);

	msg_length = VXGE_HAL_DMSG_LENGTH_GET((__hal_down_msg_t *)dmsgh);

#ifndef	VXGE_HAL_TITAN_EMULATION

	if (msg_length % __vxge_os_cacheline_size) {
		pad_length = __vxge_os_cacheline_size -
		    (msg_length % __vxge_os_cacheline_size);
	} else {
		pad_length = 0;
	}

#endif

	VXGE_HAL_DMSG_LENGTH_SET((__hal_down_msg_t *)dmsgh,
	    msg_length+pad_length);

	status = __hal_dmq_msg_copy(dmq, dmsgh);

	if (status == VXGE_HAL_OK) {

		__hal_message_db_post(dmq->channel.vph,
		    msg_length+pad_length,
		    NULL,
		    0);
	}

#if defined(VXGE_HAL_DMQ_POST)
	vxge_os_spin_unlock(&dmq->channel.post_lock);
#elif defined(VXGE_HAL_DMQ_POST_IRQ)
	vxge_os_spin_unlock_irq(&dmq->channel.post_lock, flags);
#endif

	vxge_hal_trace_log_dmq(hldev,
	    ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath->vp_id,
	    "<==  %s:%s:%d  Result: %d",
	    __FILE__, __func__, __LINE__, status);

	return (status);
}

/*
 * __hal_dmq_is_empty - Is the dmq empty
 * @dmq: Down message queue
 *
 * This function returns VXGE_HAL_OK if the queue is empty.
 */

vxge_hal_status_e
__hal_dmq_is_empty(
	__hal_dmq_t *dmq)
{
	__hal_device_t *hldev;
	__hal_list_block_t *compl_block;
	__hal_dmq_blk_priv_t *compl_priv;
	vxge_hal_status_e status;

	vxge_assert(dmq != NULL);

	hldev = (__hal_device_t *)dmq->channel.devh;

	vxge_hal_trace_log_dmq(hldev,
	    ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_dmq(hldev,
	    ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath->vp_id,
	    "dmq = 0x"VXGE_OS_STXFMT, (ptr_t)dmq);

#if defined(VXGE_HAL_DMQ_POST)
	vxge_os_spin_lock(&dmq->channel.post_lock);
#elif defined(VXGE_HAL_DMQ_POST_IRQ)
	vxge_os_spin_lock_irq(&dmq->channel.post_lock, flags);
#endif

	compl_block = dmq->channel.dtr_arr[dmq->channel.compl_index].dtr;
	compl_priv = (__hal_dmq_blk_priv_t *)(ptr_t)
	    __hal_list_block_priv_data(compl_block);

	vxge_assert(compl_priv != NULL);

	if ((dmq->high_msg_address !=
	    compl_priv->dma_addr + compl_priv->compl_offset) ||
	    (dmq->channel.compl_index != dmq->channel.post_index) ||
	    (compl_priv->compl_offset != compl_priv->post_offset))
		status = VXGE_HAL_FAIL;
	else
		status = VXGE_HAL_OK;

#if defined(VXGE_HAL_DMQ_POST)
	vxge_os_spin_unlock(&dmq->channel.post_lock);
#elif defined(VXGE_HAL_DMQ_POST_IRQ)
	vxge_os_spin_unlock_irq(&dmq->channel.post_lock, flags);
#endif

	vxge_hal_trace_log_dmq(hldev,
	    ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath->vp_id,
	    "Posted message. free 0x"VXGE_OS_LLXFMT" used "
	    "0x"VXGE_OS_LLXFMT, dmq->bytes_free, dmq->bytes_used);

	vxge_hal_trace_log_dmq(hldev,
	    ((__hal_vpath_handle_t *)dmq->channel.vph)->vpath->vp_id,
	    "<==  %s:%s:%d  Result: %d",
	    __FILE__, __func__, __LINE__, status);
	return (status);

}
