/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

/*
 * Copyright (c) 2002-2010 Neterion, Inc.
 * All right Reserved.
 *
 * FileName :   vxgehal-dmq.h
 *
 * Description:  HAL Message Queue list structures
 *
 * Created:       15 Dec 2005
 */

#ifndef	VXGE_HAL_DMQ_H
#define	VXGE_HAL_DMQ_H

__EXTERN_BEGIN_DECLS

/*
 * struct __hal_dmq_blk_priv_t - Down Message Block HAL-private data.
 * @dma_addr: DMA (mapped) address of _this_ Message block.
 * @dma_handle: DMA handle used to map the Message block to the device.
 * @dma_offset: Message block's offset in the memory block(always 0).
 * @memblock: Virtual address of the message block.
 * @wrapper: Address of the List Block wrapper for this list.
 * @dmqh: Queue identifier for this structure.
 * @compl_offset: Number of bytes from the beginning of the block processed.
 * @post_offset: Number of bytes used from the beginning of the block for the
 *		 messages so far.
 * @pad: Padding to make it cache-aligned.
 *
 * Per-Message block HAL-private data. HAL uses the space to keep DMA
 * information associated with the Message block.
 *
 */
typedef struct __hal_dmq_blk_priv_t {
	dma_addr_t			dma_addr;
	pci_dma_h			dma_handle;
	ptrdiff_t			dma_offset;
	void				*memblock;
	__hal_list_block_wrapper_t	*wrapper;
	vxge_hal_dmq_h			dmqh;
	u64				compl_offset;
	u64				post_offset;
	u8				pad[(((63+__vxge_os_cacheline_size)/
						__vxge_os_cacheline_size)*
						__vxge_os_cacheline_size)-64];
} __hal_dmq_blk_priv_t;

/*
 * struct __hal_dmq_t - Downward Message Queue channel.
 * @channel: Channel "base" of this Queue, the common part of all HAL
 *	  channels.
 * @high_msg_address: The physical address of the last byte of the last
 *		   message processed.
 * @mempool: Memory pool, the pool from which message blocks get allocated.
 *	  (See vxge_hal_mm.h).
 * @bytes_used: The number of bytes used in message queue.
 * @bytes_free: The number of bytes free in message queue.
 * @first_block_address: The DMA Address of the first block.
 * @first_block_length: The Length of the first block.
 * @dmq_id: The object Id of the DMQ channel
 * @dmq_length: The number of blocks in the Message queue.
 * @intr_ctrl: Enable/Disable Interrupt generation.
 * @gen_compl: Enable/Disable completion message.
 * @msgblock_priv_size: Reserved at the end of each list block for HAL internal
 *			 usage.
 * @stats: Statistics of this dmq
 *
 * Contains the host generated messages for the adapter or the completions
 * for the upward messages. The signaling of the downward message is done
 * through down message doorbell
 *
 * See also: vxge_hal_messaging_db_wrapper_t {}.
 *
 */
typedef struct __hal_dmq_t {
	__hal_channel_t				channel;
	u64 __vxge_os_attr_cacheline_aligned	high_msg_address;
	vxge_hal_mempool_t			*mempool;
	u64					bytes_used;
	u64					bytes_free;
	u64					first_block_address;
	u32					first_block_length;
	u32					dmq_id;
	u32					dmq_length;
	u32					intr_ctrl;
	u32					gen_compl;
	u32					msgblock_priv_size;
	vxge_hal_vpath_stats_sw_dmq_info_t	*stats;
} __hal_dmq_t;

u64
__hal_dmq_first_block_address_get(
	vxge_hal_dmq_h dmqh);

u32
__hal_dmq_first_block_length_get(
	vxge_hal_dmq_h dmqh);

vxge_hal_status_e
__hal_dmq_create(
	vxge_hal_vpath_h vpath_handle);

vxge_hal_status_e
__hal_dmq_reset(
	vxge_hal_dmq_h dmqh);

vxge_hal_status_e
__hal_dmq_doorbell_reset(
	vxge_hal_dmq_h dmqh);

vxge_hal_status_e
__hal_dmq_delete(
	vxge_hal_vpath_h vpath_handle,
	__hal_dmq_t *dmq);

vxge_hal_status_e
__hal_dmq_complete(
	__hal_dmq_t *dmq);

vxge_hal_status_e
__hal_dmq_msg_copy(
	__hal_dmq_t *dmq,
	vxge_hal_down_msg_h dmsgh);

vxge_hal_status_e
__hal_dmq_post(
	__hal_dmq_t *dmq,
	vxge_hal_down_msg_h dmsgh);

vxge_hal_status_e
__hal_dmq_is_empty(__hal_dmq_t *dmq);

__EXTERN_END_DECLS

#endif /* VXGE_HAL_DMQ_H */
