/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */
/*
 * Copyright (c) 2002-2010 Neterion, Inc.
 * All right Reserved.
 *
 * FileName :    vxgehal-llloopback.c
 *
 * Description:  Loopback functionality
 *
 * Created:      05 June 2007
 */
#include "vxgehal.h"
#include "xgehal-device.h"
#include "xgehal-fifo.h"
#include "xgehal-llloopback.h"


void
vxge_hal_dmq_db_post(void)
{
}

void
vxge_hal_sq_db_post(void)
{
}

vxge_hal_status_e vxge_hal_rldram_test(vxge_hal_device_h devh,
    u64 *data)
{
	return (VXGE_HAL_OK);
}

vxge_hal_status_e vxge_hal_device_rts_section_enable(vxge_hal_device_h devh,
    u32 index)
{
	return (VXGE_HAL_OK);
}

vxge_hal_status_e vxge_hal_aux_stats_pci_read(IN vxge_hal_device_h devh,
    IN  int	bufsize,
    OUT char	*retbuf,
    OUT int	*retsize)
{
	return (VXGE_HAL_OK);
}

vxge_hal_status_e vxge_hal_aux_stats_hal_read(IN vxge_hal_device_h devh,
    IN  int	bufsize,
    OUT char	*retbuf,
    OUT int	*retsize)
{
	return (VXGE_HAL_OK);
}

vxge_hal_status_e vxge_hal_aux_stats_tmac_read(IN  vxge_hal_device_h devh,
    IN  int	bufsize,
    OUT char	*retbuf,
    OUT int	*retsize)
{
	return (VXGE_HAL_OK);
}

vxge_hal_status_e vxge_hal_aux_stats_rmac_read(IN  vxge_hal_device_h devh,
    IN  int	bufsize,
    OUT char	*retbuf,
    OUT int	*retsize)
{
	return (VXGE_HAL_OK);
}

vxge_hal_status_e vxge_hal_aux_stats_sw_dev_read(IN vxge_hal_device_h devh,
    IN  int	bufsize,
    OUT char	*retbuf,
    OUT int	*retsize)
{
	return (VXGE_HAL_OK);
}

vxge_hal_status_e vxge_hal_pma_loopback(vxge_hal_device_h devh,
    u32 enable)
{
	return (VXGE_HAL_OK);
}

#ifdef LL_LOOPBACK
/*
 * displayQ - display elements of queue
 */
void
displayQ(llloopback_t *lpbk)
{
	lpbkqueue_t **front, **rear;
	lpbkqueue_t *q, *p = NULL;
	vxge_hal_dtr_h dtrh;
	vxge_hal_fifo_txd_t *txdp;
	front = &lpbk->front;
	rear = &lpbk->rear;
	q = *front;

	if (*front == NULL) {
		return;
	}
	while (q != p) {
		dtrh = q->dtr_ptr;
		txdp = (vxge_hal_fifo_txd_t *)dtrh;
		vxge_debug_ll(VXGE_TRACE, "%s contrl_1=0x%x \n",
		    __FUNCTION__, (u32)txdp->control_1);
		q = q->link;
		p = *front;
	}
}

/*
 * enqueue_dtr - add a dtr element to queue.
 * this function is part of HAL
 */
void
enqueue_dtr(vxge_hal_dtr_h dtrh, llloopback_t *lpbk)
{
	lpbkqueue_t *q, **front, **rear;
	front = &lpbk->front;
	rear = &lpbk->rear;

	q = vxge_os_malloc(NULL, sizeof (lpbkqueue_t));
		if (q == NULL) {
			vxge_debug_ll(VXGE_TRACE, "%s:malloc failed\n",
			    __FUNCTION__);
			return;
		}
	q->link = NULL;
	q->dtr_ptr = (void*)dtrh;
	if (*front == NULL) {
		*front = *rear = q;
	} else {
		(*rear)->link = q;
		*rear = q;
	}
	(*rear)->link = *front;
}

/*
 * dequeue_dtr - delete an element from queue.
 * this function is part of HAL
 */
vxge_hal_dtr_h
*dequeue_dtr(llloopback_t *lpbk)
{
	lpbkqueue_t *q, **front = &lpbk->front, **rear = &lpbk->rear;
	vxge_hal_dtr_h dtrh;

	if (*front == NULL) {
		return (0);
	} else if (*front == *rear) {
		dtrh = (*front)->dtr_ptr;
		kfree(*front);
		*front = *rear = NULL;
	} else {
		q = *front;
		dtrh = (*front)->dtr_ptr;
		(*front) = (*front)->link;
		kfree(q);
	}
	return (dtrh);
}

/*
 * calucate IP header checksum
 */
u16
ip_csum_calc(u16 buff[])
{
	u32 sum = 0;
	int i;
	buff[5] = 0x0;

	vxge_debug_ll(VXGE_TRACE, "%s:\n", __FUNCTION__);
	for (i = 0; i < 20; i++)
		vxge_debug_ll(VXGE_TRACE, "%s:0x%02x, ", __FUNCTION__, buff[i]);

	for (i = 0; i < 10; i++) {
		sum = sum + buff[i];
		vxge_debug_ll(VXGE_TRACE, "%s:buff[%d] = 0x%x sum=0x%x\n",
		    __FUNCTION__, i, buff[i], sum);
	}
	while (sum >> 16)
		sum = (sum & 0xffff) + (sum >> 16);
	sum = ~sum;
	vxge_debug_ll(VXGE_TRACE, "iphcsum = 0x%x\n", sum);
	return ((u16)sum);
}

/*
 * print pkt headers
 */
void
print_pkt(char *pkt)
{
	int i;
	vxge_debug_ll(VXGE_TRACE, "\n%s:Ethernet Header:\n", __FUNCTION__);
	for (i = 0; i < 14; i++)
		vxge_debug_ll(VXGE_TRACE, "%s:0x%02x ",
		    __FUNCTION__, (u8)*(pkt+ i));

	vxge_debug_ll(VXGE_TRACE, "\n%s:IP Header:\n\t", __FUNCTION__);
	for (i = 14; i < 34; i++)
		vxge_debug_ll(VXGE_TRACE, "%s:0x%02x ",
		    __FUNCTION__, *(pkt+ i));

	vxge_debug_ll(VXGE_TRACE, "\n%s:ICMP Header:\n\t", __FUNCTION__);
	for (i = 34; i < 42; i++)
		vxge_debug_ll(VXGE_TRACE, "%s:0x%02x ",
		    __FUNCTION__, (u8)*(pkt+ i));
}

/*
 * returns pointer to current RXDTR element from work queue
 * __hal_lpbk_channel_dtr_try_complete function uses lpbk_index
 * to point to current DTR
 */
void
vxge_hal_ring_dtr_get(__hal_channel_t *channelh, vxge_hal_dtr_h *rx_dtrh)
{
	__hal_lpbk_channel_dtr_try_complete(channelh, rx_dtrh);
}

/*
 * Called from __hal_fifo_dtr_post_single function
 * marks the DTR as completed and queues it.
 */
// void lllpbk_fifo_dtr_post(__hal_fifo_t *fifo, vxge_hal_dtr_h dtrh)
void
lllpbk_fifo_dtr_post(vxge_hal_fifo_h fifoh, vxge_hal_dtr_h dtrh)
{
	vxge_hal_device_t *hldev;
	vxge_hal_fifo_txd_t *txdp = (vxge_hal_fifo_txd_t *)dtrh;
	llloopback_t *lpbk;
	static u8 t_code = 0;
	__hal_fifo_t *fifo = (__hal_fifo_t *)fifoh;

	hldev = fifo->channel.devh;
	lpbk = hldev->lpbk;

	/* mark dtr as completed */
	txdp->control_1 &= ~VXGE_HAL_FIFO_TXD_LIST_OWN_ADAPTER;
	txdp->control_1 |= VXGE_HAL_FIFO_TXD_T_CODE(++t_code);

	/* queue the TX DTR */
	enqueue_dtr(dtrh, lpbk);
}

/*
 * tx_timer_handler - tx timer funciton
 * channelh - fifo_channel handler
 * This function simulates TX_DONE interrupt
 * Process the queue serially, marks TX DTRs are completed and calls ULD
 * callback.
 * It also creates a RX DTR, copees relavent data from TX-DTR to RX-DTR
 * by calling fill_rx_dtr function.
 */
void
lllpbk_tx_timer_handler(unsigned long data)
{
	llloopback_t *lpbk;
	vxge_hal_dtr_h tx_dtrh, rx_dtrh;
	int t_code = 0;
	vxge_hal_fifo_txd_t *txdp;
	__hal_vpath_handle_t *vp = (__hal_vpath_handle_t *)data;
	__hal_device_t *hldev = (__hal_device_t *)vp->devh;
	__hal_fifo_t *fifo;
	__hal_ring_t *ring;

	lpbk = hldev->header.lpbk;
	ring = (__hal_ring_t *)vp->vpath->ringh;
	fifo = (__hal_fifo_t *)vp->vpath->fifoh;

	/* process all queued DTRs */
	while ((tx_dtrh = dequeue_dtr(hldev->header.lpbk))) {
		/*
		 * RX DTR is created here so that the header information can be
		 * copied from TX DTR. This can be moved to rx_timer if header
		 * copy is not required
		 */
		vxge_hal_ring_dtr_get(&ring->channel, &rx_dtrh);

		/*
		 * Copy header and data from TX DTR
		 */
		lllpbk_fill_rx_dtr(ring, tx_dtrh, rx_dtrh);

		/*
		 * Call ULD callback - can this function be called or is there
		 * any other way of calling ULD callback ?
		 */
		txdp = (vxge_hal_fifo_txd_t *)tx_dtrh;
		t_code = (u8)VXGE_HAL_FIFO_TXD_T_CODE_GET(txdp->control_1);

		fifo->callback(vp, tx_dtrh, NULL, t_code,
		    fifo->channel.userdata);
	}

	MOD_TIMER(lpbk->tx_timer, 20);
}

/*
 * rx_timer_handler
 * this timer simulates RX interrupt.
 * calls poll_rx_channels for further processing
 */
void
lllpbk_rx_timer_handler(unsigned long data)
{
	llloopback_t *lpbk;
	u8 t_code = 0;
	vxge_hal_dtr_h dtrh;
	__hal_vpath_handle_t *vp = (__hal_vpath_handle_t *)data;
	__hal_device_t *hldev = (__hal_device_t *)vp->devh;
	__hal_ring_t *ring = (__hal_ring_t *)vp->vpath->ringh;

	lpbk = hldev->header.lpbk;

	// vxge_hal_device_poll_rx_channels(hldev, &got_rx);
	if ((vxge_hal_ring_dtr_next_completed(vp,
	    &dtrh, NULL, &t_code) ==  VXGE_HAL_OK))
		ring->callback(vp, dtrh, NULL, t_code, ring->channel.userdata);
#if 0
	/* for each opened rx channel */
	vxge_list_for_each(item, &hldev->ring_channels) {
		channel = vxge_container_of(item, vxge_hal_channel_t, item);
		/* check for completed DTRs */
		vxge_hal_device_poll_rx_channel(channel, &got_rx);
		if ((vxge_hal_ring_dtr_next_completed(channel, &dtrh,
		    &t_code)) == VXGE_HAL_OK) {
			t_code = 0;
			vxge_debug_ll(VXGE_TRACE, "calling rx_compl\n");
			/* call RX ULD callback to process the DTR */
			channel->callback(channel, dtrh, t_code,
			    channel->userdata);
		}
	}
#endif
	MOD_TIMER(lpbk->rx_timer, 20);
}

/*
 * lllpbk_init
 * This fucntion is called from ULD driver.
 * Initalizes loopback structure and starts RX and TX timers.
 */
int
lllpbk_init(vxge_hal_vpath_h vpath_handle)
{
	__hal_device_t *hldev;
	llloopback_t *lpbk = NULL;
	__hal_vpath_handle_t *vp = (__hal_vpath_handle_t *)vpath_handle;
	hldev = (__hal_device_t *)vp->devh;
	vxge_debug_ll(VXGE_TRACE, "%s:Loopback initialization\n", __FUNCTION__);
	lpbk = vxge_os_malloc(NULL, sizeof (llloopback_t));
	if (lpbk == NULL) {
		vxge_debug_ll(VXGE_ERR, "%s:can not allocate memory for lpbk",
		    __FUNCTION__);
		return (0);
	}

	/* initialize members of loopback sturcture */
	lpbk->front = lpbk->rear = NULL;
	hldev->header.lpbk = lpbk;

	/* start RX and TX timers */
	TIMER_CONF(lpbk->tx_timer, lllpbk_tx_timer_handler, vp, 50);
	TIMER_CONF(lpbk->rx_timer, lllpbk_rx_timer_handler, vp, 50);

	return (1);
}
/*
 * modifies mac/ip/udp header
 * modifies ip and icmp pkts to simulate ping
 */
static void lllpbk_copy_headers(u8 *tx_pkt, u8 *rx_pkt, int pktlen)
{
	icmp_h *txicmph, *rxicmph;
	ip_h *rx_iph, *tx_iph;
	struct ethhdr *eh;

	/* first copy the entire tx_pkt to rx_pkt */
	vxge_os_memcpy(rx_pkt, tx_pkt, pktlen);

	eh = (struct ethhdr *)tx_pkt;
	/* if IPV6 packet don't modify the header */
	if (eh->h_proto != vxge_os_htons(0x08)) {
		vxge_debug_ll(VXGE_TRACE, "%s:Transmit PKT:", __FUNCTION__);
		print_pkt(tx_pkt);

	    /* copy the src & dst mac addresses to rx dtr */
		vxge_os_memcpy(rx_pkt + 6, tx_pkt, 6);
		vxge_os_memcpy(rx_pkt, tx_pkt + 6, 6);

	    /* get IP headers */
		tx_iph = (ip_h *) (tx_pkt + ETH_HLEN);
		rx_iph = (ip_h *) (rx_pkt + ETH_HLEN);

	    /* update iphdr part of rx dtr */
		rx_iph->saddr = tx_iph->daddr;
		rx_iph->daddr = tx_iph->saddr;
		rx_iph->frag_off = 0x0;
		rx_iph->id += 0x0001;

		rx_iph = (ip_h *) (rx_pkt + ETH_HLEN);
		rx_iph->check = ip_csum_calc((u16*)rx_iph);

		/* ICMP header */
		txicmph = (icmp_h *) (tx_pkt + 34);
		rxicmph = (icmp_h *) (rx_pkt + 34);

		rxicmph->type = 0x0; // reply code
		*(rx_pkt + 36) += 8; // check sum of icmp add 8

		vxge_debug_ll(VXGE_TRACE, "%s:Receive PKT:", __FUNCTION__);
		print_pkt(rx_pkt);
	}
}
/* farword declaration of uld function to get rid of compilation error */
int	xgeuld_lllpbk_fill_rx_dtr(void*, void*, u8*, u8*);

/*
 * fill the rx-DTR data by modifying headers and update RX-DTR bits.
 */
void
lllpbk_fill_rx_dtr(vxge_hal_ring_h channelh, vxge_hal_dtr_h tx_dtrh,
    vxge_hal_dtr_h rx_dtrh)
{
	void *tx_priv, *rx_priv;
	vxge_hal_ring_rxd_1_t *rxdp;
	int pktlen;
	u8 *rx_pkt, *tx_pkt;
	__hal_ring_t *ring = (__hal_ring_t *)channelh;

	rx_priv = vxge_hal_ring_dtr_1b_3b_private_get(ring,
	    rx_dtrh); // rx private data
	tx_priv = vxge_hal_fifo_dtr_private_get(tx_dtrh); // tx private data

	pktlen = xgeuld_lllpbk_fill_rx_dtr(tx_priv, rx_priv, &tx_pkt, &rx_pkt);

	lllpbk_copy_headers(tx_pkt, rx_pkt, pktlen);

	/* modify rx DTR bits accordingly */
	rxdp = (vxge_hal_ring_rxd_1_t *)rx_dtrh;

	/* set owner ship bit to host */
	rxdp->control_0 &= ~BIT(7);

	/* set pkt len in control2 */
	rxdp->control_1 = VXGE_HAL_RING_RXD_1_BUFFER0_SIZE(pktlen);

	/* reset t_code */
	rxdp->control_1 |= VXGE_HAL_RING_RXD_T_CODE
	    (VXGE_HAL_RING_RXD_T_CODE_OK);
}
#endif /* LL_LOOPBACK */
