/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */
/*
 * Copyright (c) 2002-2010 Neterion, Inc.
 * All right Reserved.
 *
 * FileName :    vxgehal-llloopback.h
 *
 * Description:  Loopback functionality
 *
 * Created:      8 June 2007
 */

#ifndef VXGE_HAL_LL_LOOPBACK
#define	VXGE_HAL_LL_LOOPBACK
#ifdef LL_LOOPBACK

#define	TIMER_CONF(timer, handle, arg, exp) \
		vxge_os_timer(timer, handle, arg, exp)
#define	MOD_TIMER(timer, exp) vxge_os_mod_timer(timer, exp)

/* queue structures */
typedef struct lpbkqueue_t {
	void *dtr_ptr;
	struct lpbkqueue_t *link;
}lpbkqueue_t;

/* structure to hold queue pointers and timer elements */
typedef struct llloopback_t {
	lpbkqueue_t *front;
	lpbkqueue_t *rear;
	vxge_hal_timer tx_timer;
	vxge_hal_timer rx_timer;
}llloopback_t;

/* timer handlers */
void lllpbk_tx_timer_handler(unsigned long data);
void lllpbk_rx_timer_handler(unsigned long data);

/* queue management functions */
void displayQ(llloopback_t *lpbk);
void enqueue_dtr(vxge_hal_dtr_h dtrh,
		    llloopback_t *lpbk);
vxge_hal_dtr_h *dequeue_dtr(llloopback_t *lpbk);

/* New HAL functions for lopback functionality */
void lllpbk_fifo_dtr_post(vxge_hal_fifo_h fifoh,
			vxge_hal_dtr_h dtrh);
// void vxge_hal_ring_dtr_get(__hal_channel_t* channelh,
//			vxge_hal_dtr_h *rx_dtrh);
void lllpbk_fill_rx_dtr(vxge_hal_ring_h channelh,
			vxge_hal_dtr_h tx_dtrh,
			vxge_hal_dtr_h rx_dtrh);
int lllpbk_init(vxge_hal_vpath_h vpath_handle);

/* helper functions */
u16 ip_csum_calc(u16 buff[]);
void print_pkt(char *pkt);

#endif /* LL_LOOPBACK */
#endif /* VXGE_HAL_LL_LOOPBACK */
