/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */
/*
 * Copyright (c) 2002-2010 Neterion, Inc.
 * All right Reserved.
 *
 * FileName :   vxgehal-lro.h
 *
 * Description:  LRO offload related include
 *
 * Created:       11 January 2006
 */

#ifndef	VXGE_HAL_LRO_H
#define	VXGE_HAL_LRO_H

__EXTERN_BEGIN_DECLS

#ifdef	VXGE_HAL_TITAN_2

#define	VXGE_HAL_LRO_MODE_AUTO					0
#define	VXGE_HAL_LRO_MODE_ADAPTIVE				1

/*
 * struct __hal_lro_session_t : LRO session object
 * @item: List item: used to keep this object in a list.
 * @session_opaque_handle: Opaque handle used by the host to reference data
 *		structures in host memory related to this session
 * @time_last_accessed: Time stamp for the last access
 * @ip_sa: Source ip address
 * @ip_da: Destination ip address
 * @src_port: Source port
 * @dest_port: Dest port
 * @session_id: Session Id
 * @session_state: The state of this session
 *	    - VXGE_HAL_LRO_SESSION_STATE_UNKNOWN-Unknown
 *	    - VXGE_HAL_LRO_SESSION_STATE_INITIALIZED-Initialized
 *	    - VXGE_HAL_LRO_SESSION_STATE_OPEN_PENDING-Open req sent
 *	    - VXGE_HAL_LRO_SESSION_STATE_OPENED-Open response recvd
 *	    - VXGE_HAL_LRO_SESSION_STATE_DECLASSIFY_PENDING-Declassify req sent
 *	    - VXGE_HAL_LRO_SESSION_STATE_DECLASSIFIED-Declassify resp received
 *	    - VXGE_HAL_LRO_SESSION_STATE_CLOSE_PENDING-Close req sent
 *	    - VXGE_HAL_LRO_SESSION_STATE_CLOSED-Closed
 *	    - VXGE_HAL_LRO_SESSION_STATE_FREED-Freed
 * @total_cqes_recvd: Total number of CQEs on this session
 * @total_bytes_recvd: Total number of bytes received on this LRO
 *
 * LRO session object.
 *
 * See also: __hal_lro_session_t {}
 */
typedef struct __hal_lro_session_t {
	vxge_list_t		item;
	vxge_hal_opaque_handle_t *session_opaque_handle;
	u64			time_last_accessed;
	struct {
		u64		high;
		u64		low;
	} ip_sa;
	struct {
		u64		high;
		u64		low;
	} ip_da;
	u32			src_port;
	u32			dest_port;
	u32			session_id;
	u32			session_state;
#define	VXGEHAL_LRO_SESS_STATE_UNKNOWN			0
#define	VXGEHAL_LRO_SESS_STATE_INITED			1
#define	VXGEHAL_LRO_SESS_STATE_OPEN_PENDING		2
#define	VXGEHAL_LRO_SESS_STATE_OPENED			3
#define	VXGEHAL_LRO_SESS_STATE_DECLASS_PENDING		4
#define	VXGEHAL_LRO_SESS_STATE_DECLASSED		5
#define	VXGEHAL_LRO_SESS_STATE_CLOSE_PENDING		6
#define	VXGEHAL_LRO_SESS_STATE_CLOSED			7
#define	VXGEHAL_LRO_SESS_STATE_FREED			8
	u32			total_cqes_recvd;
	u32			total_bytes_recvd;
} __hal_lro_session_t;


#define	VXGE_HAL_LRO_SESSION_STATE_SET(lro, st)	(lro)->session_state = st;

#define	VXGE_HAL_LRO_SESSION_STATE_GET(lro)	(lro)->session_state

vxge_hal_status_e
__hal_lro_session_pool_create(
	vxge_hal_device_h devh,
	__hal_virtualpath_t *vpath,
	u32 pool_size);

vxge_hal_status_e
__hal_lro_session_pool_reset(
	vxge_hal_device_h devh,
	__hal_virtualpath_t *vpath);

vxge_hal_status_e
__hal_lro_session_pool_destroy(
	vxge_hal_device_h devh,
	__hal_virtualpath_t *vpath);

__hal_lro_session_t *
__hal_lro_session_allocate(
	vxge_hal_device_h devh,
	__hal_virtualpath_t *vpath,
	vxge_hal_opaque_handle_t *session_opaque_handle,
	u32 src_port,
	u32 dest_port,
	u64 ip_sa_high,
	u64 ip_sa_low,
	u64 ip_da_high,
	u64 ip_da_low);

vxge_hal_status_e
__hal_lro_session_timestamp(
	vxge_hal_device_h devh,
	__hal_virtualpath_t *vpath,
	vxge_hal_obj_id_t session_id);

vxge_hal_status_e
__hal_lro_sessions_timeout(
	vxge_hal_device_h devh,
	__hal_virtualpath_t *vpath);

vxge_hal_status_e
__hal_lro_session_free(
	vxge_hal_device_h devh,
	__hal_virtualpath_t *vpath,
	vxge_hal_obj_id_t session_id);

#endif  /* VXGE_HAL_TITAN_2 */

#if defined(VXGE_HAL_USE_SW_LRO)

#define	VXGE_HAL_TS_SAVE	2
#define	VXGE_HAL_TS_VERIFY	1
#define	VXGE_HAL_TS_UPDATE	0


#define	VXGE_HAL_SW_LRO_SESSION_SIZE		(sizeof (vxge_hal_sw_lro_t)+\
						    sizeof (vxge_list_t))

#define	VXGE_HAL_SW_LRO_SESSION_ITEM(lro) ((lro == NULL) ? NULL : ((lro) + 1))

#define	VXGE_HAL_SW_LRO_SESSION(item) \
		    ((item == NULL) ? NULL : (((vxge_hal_sw_lro_t *)item)-1))
#define	VXGE_HAL_INET_ECN_MASK		3
#define	VXGE_HAL_INET_ECN_CE		3

vxge_hal_status_e
__hal_sw_lro_init(__hal_ring_t *ring);

vxge_hal_status_e
__hal_sw_lro_reset(__hal_ring_t *ring);

vxge_hal_status_e
__hal_sw_lro_terminate(__hal_ring_t *ring);

vxge_hal_status_e
__hal_lro_capable(__hal_virtualpath_t *vpath,
		    u8 *buffer,
		    vxge_hal_ip_hdr_t **ip,
		    vxge_hal_tcp_hdr_t **tcp,
		    vxge_hal_ring_rxd_info_t *ext_info);

#endif

__EXTERN_END_DECLS

#endif /* VXGE_HAL_LRO_H */
