/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */
/*
 * Copyright (c) 2002-2010 Neterion, Inc.
 * All right Reserved.
 *
 * FileName :   vxgehal-odlist.h
 *
 * Description:  HAL HW Full od list structures
 *
 * Created:       15 Dec 2005
 */

#ifndef	VXGE_HAL_ODLIST_H
#define	VXGE_HAL_ODLIST_H

__EXTERN_BEGIN_DECLS

/*
 * struct __hal_od_t - Offload Descriptor
 * @control_0: Bits 0 to 31 - The RDMA Stag. An all-0s value indicates that
 *			   ptr_to is a pointer into host memory.
 *	       Bits 32 to 63 - The number of bytes described by this OD.
 * @ptr_to: If this OD's Stag is 0, this is a pointer into host memory
 *	 If the OD's Stag is not 0, this is the starting tagged offset.
 * @pad: Padding to make it cache-aligned.
 *
 * The adapter DMA reads an Offload Descriptor when Fetching Transmit data
 * and Fetching RxWQEs. The adapter never writes to an Offload Descriptor.
 * On Tx side, the Towi points to OD List.
 */
typedef __vxge_os_attr_cacheline_aligned struct __hal_od_t {
	u64		control_0;
#define	VXGE_HAL_OD_GET_STAG(ctrl0)			bVAL32(ctrl0, 0)
#define	VXGE_HAL_OD_STAG(val)				vBIT(val, 0, 32)

#define	VXGE_HAL_OD_GET_LENGTH(ctrl0)			bVAL32(ctrl0, 32)
#define	VXGE_HAL_OD_LENGTH(val)				vBIT(val, 32, 32)

	u64		ptr_to;
} __hal_od_t;

/*
 * __hal_odlist_dma_addr_get - Get the dma address of first block in the list
 * @odlist: odlist.
 *
 * This function is returns the dma address of the first block.
 */
static inline u64
__hal_odlist_dma_addr_get(
	vxge_hal_odlist_t *odlist)
{
	__hal_blockpool_entry_t *entry;

	entry = (__hal_blockpool_entry_t *)odlist->od_list;

	if (entry != NULL)
		return (entry->dma_addr);

	return (0);
}

/*
 * __hal_odlist_length_get - Get the size of the OD List block
 *			  to be placed in the LB Wrapper
 * @odlist: odlist.
 *
 * This function is returns the dma address of the first block.
 */
static inline u32
__hal_odlist_length_get(
	vxge_hal_odlist_t *odlist)
{
	return (odlist->num_ods * odlist->od_size);
}

vxge_hal_status_e
vxge_hal_odlist_allocate(
	vxge_hal_vpath_h vpath_handle,
	vxge_hal_odlist_t *odlist,
	u32 length);

vxge_hal_status_e
vxge_hal_odlist_set(
	vxge_hal_odlist_t *odlist,
	u64 ptr_to,
	u32 stag,
	u32 length);

void
vxge_hal_odlist_free(
	vxge_hal_vpath_h vpath_handle,
	vxge_hal_odlist_t *odlist);

__EXTERN_END_DECLS

#endif /* VXGE_HAL_ODLIST_H */
