/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */
/*
 * Copyright (c) 2002-2010 Neterion, Inc.
 * All right Reserved.
 *
 * FileName :   vxgehal-offload.h
 *
 * Description:  HAL HW Full offload structures
 *
 * Created:       05 Dec 2005
 */

#ifndef	VXGE_HAL_OFFLOAD_H
#define	VXGE_HAL_OFFLOAD_H

__EXTERN_BEGIN_DECLS

/*
 * struct __hal_nsmr_ctrl_block_t - NSMR control block
 * @control_0: Bits 0 to 24 - The identity of the STag to be registered.
 *	       Bits 24 to 32 - The key for the newly-registered STag.
 *	       Bits 32 to 63 - The byte offset into the first physical
 *				buffer where the NSMR begins.
 * @control_1: The length in bytes of the NSMR.
 * @control_2: The TO of the first byte of the NSMR. This only applies
 *		if the addressing type is VA-Based.
 * @control_3: Bits 0 to 7 - Addressing Type.
 *	       Bits 8 to 15 - Allow local write access.
 *	       Bits 16 to 23 - Allow remote write access.
 *	       Bits 24 to 31 - Allow local read access.
 *	       Bits 32 to 39 - Allow remote read access.
 *	       Bits 40 to 47 - Allow MWs to be bound to this NSMR.
 *	       Bits 48 to 63 - Reserved.
 * @control_4: Bits 0 to 31 - The total number of PBLEs in the address list.
 *	       Bits 32 to 63 - The size in bytes of each physical buffer in
 *				the address list.
 * @pbl_block_wrapper0:The location in host memory where the PBL block starts
 * @pbl_block_wrapper1:Bits 0 to 31 - The length in bytes of the PBL block
 *			- not including that block's list block wrapper.
 *		       Bits 32 to 63 - Reserved.
 * @pad: Padding to make it cache-aligned.
 *
 * When the adapter receives a TOWI that contains a FastRegisterNSMR operation,
 * it first DMA reads the MR_Control block and then DMA reads the PBL blocks
 * that contain the PBLEs.
 */
typedef __vxge_os_attr_cacheline_aligned struct __hal_nsmr_ctrl_block_t {
	u64		control_0;
#define	VXGE_HAL_NSMR_GET_STAG_INDEX(ctrl0)		bVAL24(ctrl0, 0)
#define	VXGE_HAL_NSMR_STAG_INDEX(val)			vBIT(val, 0, 24)

#define	VXGE_HAL_NSMR_GET_STAG_KEY(ctrl0)		bVAL8(ctrl0, 24)
#define	VXGE_HAL_NSMR_STAG_KEY(val)			vBIT(val, 24, 8)

#define	VXGE_HAL_NSMR_GET_FBO(ctrl0)			bVAL32(ctrl0, 32)
#define	VXGE_HAL_NSMR_STAG_FBO(val)			vBIT(val, 32, 32)

	u64		control_1;
#define	VXGE_HAL_NSMR_GET_LENGTH(ctrl1)			(ctrl1)
#define	VXGE_HAL_NSMR_LENGTH(val)			vBIT(val, 0, 64)

	u64		control_2;
#define	VXGE_HAL_NSMR_GET_VA_BASE_TO(ctrl2)		(ctrl2)
#define	VXGE_HAL_NSMR_VA_BASE_TO(val)			vBIT(val, 0, 64)

	u64		control_3;
#define	VXGE_HAL_NSMR_GET_ADDR_TYPE(ctrl3)		bVAL8(ctrl3, 0)
#define	VXGE_HAL_NSMR_ADDR_TYPE(val)			vBIT(val, 0, 8)
#define	VXGE_HAL_NSMR_ADDR_TYPE_ZERO_BASED		0
#define	VXGE_HAL_NSMR_ADDR_TYPE_VA_BASED		1

#define	VXGE_HAL_NSMR_GET_LOCAL_WRITE(ctrl3)		bVAL8(ctrl3, 8)
#define	VXGE_HAL_NSMR_LOCAL_WRITE(val)			vBIT(val, 8, 8)
#define	VXGE_HAL_NSMR_LOCAL_WRITE_ENABLE		0x1

#define	VXGE_HAL_NSMR_GET_REMOTE_WRITE(ctrl3)		bVAL8(ctrl3, 16)
#define	VXGE_HAL_NSMR_REMOTE_WRITE(val)			vBIT(val, 16, 8)
#define	VXGE_HAL_NSMR_REMOTE_WRITE_ENABLE		0x1

#define	VXGE_HAL_NSMR_GET_LOCAL_READ(ctrl3)		bVAL8(ctrl3, 24)
#define	VXGE_HAL_NSMR_LOCAL_READ(val)			vBIT(val, 24, 8)
#define	VXGE_HAL_NSMR_LOCAL_READ_ENABLE			0x1

#define	VXGE_HAL_NSMR_GET_REMOTE_READ(ctrl3)		bVAL8(ctrl3, 32)
#define	VXGE_HAL_NSMR_REMOTE_READ(val)			vBIT(val, 32, 8)
#define	VXGE_HAL_NSMR_REMOTE_READ_ENABLE		0x1

#define	VXGE_HAL_NSMR_GET_MW_BINDING(ctrl3)		bVAL8(ctrl3, 40)
#define	VXGE_HAL_NSMR_MW_BINDING(val)			vBIT(val, 40, 8)
#define	VXGE_HAL_NSMR_MW_BINDING_ENABLE			0x1

	u64		control_4;
#define	VXGE_HAL_NSMR_GET_ADDR_LIST_LENGTH(ctrl4)	bVAL32(ctrl4, 0)
#define	VXGE_HAL_NSMR_ADDR_LIST_LENGTH(val)		vBIT(val, 0, 32)

#define	VXGE_HAL_NSMR_GET_PBLE_SIZE(ctrl4)		bVAL32(ctrl4, 32)
#define	VXGE_HAL_NSMR_PBLE_SIZE(val)			vBIT(val, 32, 32)

	u64		pbl_block_wrapper0;
#define	VXGE_HAL_NSMR_GET_PBL_BLOCK_ADDRESSH(pbw0)	(pbw0)
#define	VXGE_HAL_NSMR_PBL_BLOCK_ADDRESS(val)		vBIT(val, 0, 64)

	u64		pbl_block_wrapper1;
#define	VXGE_HAL_NSMR_GET_PBL_BLOCK_LENGTH(pbw1)	bVAL32(pbw1, 0)
#define	VXGE_HAL_NSMR_PBL_BLOCK_LENGTH(val)		vBIT(val, 0, 32)

	u8		pad[(((55+__vxge_os_cacheline_size)/
		__vxge_os_cacheline_size)*__vxge_os_cacheline_size)-56];
} __hal_nsmr_ctrl_block_t;

/*
 * struct __hal_towi_t - Transmit Offload Work Item
 * @control_0 : Bits 0 to 7 - Describes the payload locator type in this TOWI.
 *	       Bits 8 to 15 - TCP push flag.
 *	       Bits 16 to 23 - Describes the message type.
 *	       Bits 24 to 31 - Reserved.
 *	       Bits 32 to 39 - Specifies how to indicate TOWI completion.
 *	       Bits 40 to 47 - Specifies how to indicate RDMA-READ completion.
 *	       Bits 48 to 63 - Reserved.
 * @control_1 : Bits 0 to 31 - conveys the number of payload bytes.
 *	       Bits 21 to 63 - Reserved.
 * @ddp_info_0 : DDP information 1 comes in four forms - Tagged form, Untagged
 *		  form, Bind window form and Invalidate local STag form.
 *		  In Tagged and Untagged forms
 *	       Bits 0 to 7 - opcode value determines which form is used.
 *	       Bits 8 to 15 - DDP L-bit of final segment of this TOWI.
 *	       Bits 16 to 31 - Reserved.
 *	       Bits 32 to 63 - The wire STag used with this TOWI or The remote
 *				  STag to be invalidated.
 *		  In Bind window form
 *	       Bits 0 to 31 - Memory Window STag value.
 *	       Bits 32 to 63 - Memory Region STag value.
 *		  In Invalidate local STag form
 *	       Bits 0 to 31 - Reserved.
 *	       Bits 32 to 63 - Local STag to invalidate.
 * @ddp_info_1 : DDP information w comes in four forms - Tagged form, Untagged
 *		  form, Bind window form and Invalidate local STag form.
 *		  In Tagged form
 *	       Bits 0 to 63 - Tagged Offset of the first byte of this TOWI.
 *		  In Untagged form
 *	       Bits 0 to 31 - Message Offset of the first byte of this TOWI.
 *	       Bits 32 to 63 - Message sequence number of this TOWI.
 *		  In Bind window form
 *	       Bits 0 to 23 - The existing Memory Windows STag Index..
 *	       Bits 24 to 32 - The new STag Key to be used for the newly
 *				  bound MW.
 *	       Bits 32 to 64 - Memory Region STag value..
 *		  In Invalidate local STag form
 *	       Bits 0 to 63 - Reserved.
 * @payload_locator_0 :
 * @payload_locator_1 : Payload locators 1 and  2 come in two forms:
 *		Bind Memory Window Control structure, and Fast
 *		Register Non-Shared Memory Region pointer structure.
 *		The TOWI_Type determines which form is present.
 *		In Bind Memory Window form
 *		payload_locator_0 -  Memory Window Length in bytes.
 *		payload_locator_1 - Memory Window Tagged Offset in bytes.
 *		In Fast Register Non-Shared Memory Region pointer
 *		structure.
 *		payload_locator_0 -  Pointer to Fast Register NSMR control
 *		block in host memory.
 * @payload_locator_2 - Reserved
 * @host_control_0 : Reserved for host use. Ignored by the adapter
 * @host_control_1 : Reserved for host use. Ignored by the adapter
 * @pad: Padding to make it cache-aligned.
 *
 * Towi is the fundamental unit of offload work that is constructed by the host
 * and operated on by the adapter. Two basic types of TOWIs are memory
 * management TOWIs and data movement TOWIs. The message type field in Control_0
 * identifies TOWI type. A single TOWI list __hal_sq_t {} exists per session and
 * for only that session. The adapter DMA reads a TOWI to obtain a unit of
 * transmit work but never writes to a TOWI. The TOWI is queued in to TOWI list
 * and indicated to the hardware by a write to the doorbell.
 *
 */
typedef __vxge_os_attr_cacheline_aligned struct __hal_towi_t {
	u64		control_0;
#define	VXGE_HAL_TOWI_GET_TYPE(ctrl0)			bVAL8(ctrl0, 0)
#define	VXGE_HAL_TOWI_TYPE(val)				vBIT(val, 0, 8)
#define	VXGE_HAL_TOWI_TYPE_OD				0x0
#define	VXGE_HAL_TOWI_TYPE_OD_LB_WRAPPER		0x1

#define	VXGE_HAL_TOWI_GET_TCP_PUSH(ctrl0)		bVAL8(ctrl0, 8)
#define	VXGE_HAL_TOWI_TCP_PUSH(val)			vBIT(val, 8, 8)
#define	VXGE_HAL_TOWI_TCP_PUSH_NOT_INCLUDED		0x0
#define	VXGE_HAL_TOWI_PUSH_INCLUDED			0x1

#define	VXGE_HAL_TOWI_GET_MSG_TYPE(ctrl0)		bVAL8(ctrl0, 16)
#define	VXGE_HAL_TOWI_MSG_TYPE(val)			vBIT(val, 16, 8)
#define	VXGE_HAL_TOWI_MSG_TYPE_FIN			0x0
#define	VXGE_HAL_TOWI_MSG_TYPE_FAST_REG_NSMR		0x1
#define	VXGE_HAL_TOWI_MSG_TYPE_INVALID_LOCAL_STAG	0x2
#define	VXGE_HAL_TOWI_MSG_TYPE_BIND_MEM_WINDOW		0x3
#define	VXGE_HAL_TOWI_MSG_TYPE_BLOCK_FOR_LIRR		0x4
#define	VXGE_HAL_TOWI_MSG_TYPE_INVALID_LOCAL_STAG_FENCE	0x5
#define	VXGE_HAL_TOWI_MSG_TYPE_LIRR_INVALID		0x6
#define	VXGE_HAL_TOWI_MSG_TYPE_TERMINATE		0x7
#define	VXGE_HAL_TOWI_MSG_TYPE_DDP_TAGGED		0x80
#define	VXGE_HAL_TOWI_MSG_TYPE_DDP_UNTAGGED		0x81
#define	VXGE_HAL_TOWI_MSG_TYPE_BYTE_STREAM		0x82
#define	VXGE_HAL_TOWI_MSG_TYPE_LIRR			0x83

#define	VXGE_HAL_TOWI_GET_TOWI_COMPLETION(ctrl0)	bVAL8(ctrl0, 32)
#define	VXGE_HAL_TOWI_COMPLETION(val)			vBIT(val, 32, 8)
#define	VXGE_HAL_TOWI_COMPLETION_INTERRUPT		0x2
#define	VXGE_HAL_TOWI_COMPLETION_UPDATE_HW_CQ_SQ	0x1

#define	VXGE_HAL_TOWI_GET_READ_COMPLETION(ctrl0)	bVAL8(ctrl0, 40)
#define	VXGE_HAL_TOWI_READ_COMPLETION(val)		vBIT(val, 40, 8)
#define	VXGE_HAL_TOWI_READ_COMPLETION_INTERRUPT		0x2
#define	VXGE_HAL_TOWI_READ_COMPLETION_UPDATE_HW_CQ_SQ	0x1

	u64		control_1;
#define	VXGE_HAL_TOWI_GET_PAYLOAD_BYTES(ctrl1)		bVAL32(ctrl1, 0)
#define	VXGE_HAL_TOWI_PAYLOAD_BYTES(val)		vBIT(val, 0, 32)

#define	VXGE_HAL_TOWI_GET_LOCAL_STAG_TO_INVALID(ctrl1)	bVAL32(ctrl1, 32)
#define	VXGE_HAL_TOWI_LOCAL_STAG_TO_INVALID(val)	vBIT(val, 32, 32)

	u64		ddp_info_0;
#define	VXGE_HAL_TOWI_GET_DDP_RDMA_OPCODE(ddpinf0)	bVAL8(ddpinf0, 0)
#define	VXGE_HAL_TOWI_DDP_RDMA_OPCODE(val)		vBIT(val, 0, 8)
#define	VXGE_HAL_TOWI_DDP_RDMA_OPCODE_WRITE		0x0
#define	VXGE_HAL_TOWI_DDP_RDMA_OPCODE_READ		0x1
#define	VXGE_HAL_TOWI_DDP_RDMA_OPCODE_SEND		0x3
#define	VXGE_HAL_TOWI_DDP_RDMA_OPCODE_SEND_INVALIDATE	0x4
#define	VXGE_HAL_TOWI_DDP_RDMA_OPCODE_SEND_SE		0x5
#define	VXGE_HAL_TOWI_DDP_RDMA_OPCODE_SEND_SE_INVALIDATE 0x6
#define	VXGE_HAL_TOWI_DDP_RDMA_OPCODE_TERMINATE		0x7

#define	VXGE_HAL_TOWI_GET_DDP_LAST_SEGMENT_BIT(ddpinf0)	bVAL8(ddpinf0, 8)
#define	VXGE_HAL_TOWI_DDP_LAST_SEGMENT_BIT(val)		vBIT(val, 8, 8)
#define	VXGE_HAL_TOWI_DDP_LAST_SEGMENT_SET		0x1

#define	VXGE_HAL_TOWI_GET_DDP_MW_STAG_INDEX(ddpinf0)	bVAL24(ddpinf0, 0)
#define	VXGE_HAL_TOWI_DDP_MW_STAG_INDEX(val)		vBIT(val, 0, 24)

#define	VXGE_HAL_TOWI_GET_DDP_MW_STAG_KEY(ddpinf0)	bVAL8(ddpinf0, 24)
#define	VXGE_HAL_TOWI_DDP_MW_STAG_KEY(val)		vBIT(val, 24, 8)

#define	VXGE_HAL_TOWI_GET_DDP_REMOTE_STAG(ddpinf0)	bVAL32(ddpinf0, 32)
#define	VXGE_HAL_TOWI_DDP_REMOTE_STAG(val)		vBIT(val, 32, 32)

#define	VXGE_HAL_TOWI_GET_DDP_MR_STAG(ddpinf0)		bVAL32(ddpinf0, 32)
#define	VXGE_HAL_TOWI_DDP_MR_STAG(val)			vBIT(val, 32, 32)

	u64		ddp_info_1;
#define	VXGE_HAL_TOWI_GET_DDP_START_TAG_OFFSET(ddpinf1)	(ddpinf1)
#define	VXGE_HAL_TOWI_DDP_START_TAG_OFFSET(val)		val

#define	VXGE_HAL_TOWI_GET_DDP_MW_CONTROL(ddpinf1)	bVAL8(ddpinf1, 0)
#define	VXGE_HAL_TOWI_DDP_MW_CONTROL(val)		vBIT(val, 0, 8)
#define	VXGE_HAL_TOWI_DDP_MW_CONTROL_VADDR_TYPE	VXGE_HAL_MW_CTRL_VADDR_TYPE
#define	VXGE_HAL_TOWI_DDP_MW_CONTROL_RW_ENABLE	VXGE_HAL_MW_CTRL_RW_ENABLE
#define	VXGE_HAL_TOWI_DDP_MW_CONTROL_RR_ENABLE	VXGE_HAL_MW_CTRL_RR_ENABLE

#define	VXGE_HAL_TOWI_GET_DDP_START_MSG_OFFSET(ddpinf1)	bVAL32(ddpinf1, 0)
#define	VXGE_HAL_TOWI_DDP_START_MSG_OFFSET(val)		vBIT(val, 0, 32)

#define	VXGE_HAL_TOWI_GET_DDP_MSG_SEQ_NO(ddpinf1)	bVAL32(ddpinf1, 32)
#define	VXGE_HAL_TOWI_DDP_MSG_SEQ_NO(val)		vBIT(val, 32, 32)

	u64		payload_locator_0;
#define	VXGE_HAL_TOWI_GET_DDP_MW_LENGTH(pl0)		pl0
#define	VXGE_HAL_TOWI_DDP_MW_LENGTH(val)		vBIT(val, 0, 64)

#define	VXGE_HAL_TOWI_GET_DDP_FR_NMSR_CTRL_LENGTH(pl0)	bVAL32(pl0, 32)
#define	VXGE_HAL_TOWI_DDP_FR_NMSR_CTRL_LENGTH(val)	vBIT(val, 32, 32)

#define	VXGE_HAL_TOWI_GET_LOCAL_STAG(pl0)		bVAL32(pl0, 0)
#define	VXGE_HAL_TOWI_LOCAL_STAG(val)			vBIT(val, 0, 32)

#define	VXGE_HAL_TOWI_GET_LENGTH(pl0)			bVAL32(pl0, 32)
#define	VXGE_HAL_TOWI_LENGTH(val)			vBIT(val, 32, 32)

	u64		payload_locator_1;
#define	VXGE_HAL_TOWI_GET_DDP_MW_TAGGED_OFFSET(pl1)	pl1
#define	VXGE_HAL_TOWI_DDP_MW_TAGGED_OFFSET(val)		vBIT(val, 0, 64)

#define	VXGE_HAL_TOWI_GET_DDP_FR_NMSR_CTRL_PTR(pl1)	pl1
#define	VXGE_HAL_TOWI_DDP_FR_NMSR_CTRL_PTR(val)		vBIT(val, 0, 64)

	u64		host_control_0;
	u64		host_control_1;
} __hal_towi_t;

/*
 * vxge_hal_usd_t - User Session Doorbells
 * @control_0:Bits 0 to 7 - The incarnation number of this session
 *	   Bits 8 to 31 -The number of TOWIs that have been posted to the
 *			   per-session TOWI list since the creation of this
 *			   session 2 modulo 232
 *
 * The host has the option, on a per-session basis, to notify the adapter
 * of new TOWIs by performing a PIO write of the updated High TOWI Number
 * to a session specific adapter address. That write must consist of a
 * single 32-bit bus transaction consisting of nothing other than the updated
 * HTN. The adapter will service those updates in accordance with its internal
 * scheduling algorithms.
 */

typedef __vxge_os_attr_cacheline_aligned struct __hal_usd_t {
	u32		control_0;
#define	VXGE_HAL_USD_GET_SESSION_INST_NUMBER(ctrl0)	bVAL8(ctrl0, 0)
#define	VXGE_HAL_USD_SESSION_INST_NUMBER(val)		vBIT(val, 0, 8)

#define	VXGE_HAL_USD_GET_HIGH_TOWI_NUMBER(ctrl0)	bVAL24((ctrl0, 8)
#define	VXGE_HAL_USD_HIGH_TOWI_NUMBER(val)		vBIT32(val, 8, 24)
}__hal_usd_t;

/*
 * struct __hal_towi_priv_t - TOWI HAL-private data
 * @dma_addr: DMA (mapped) address of _this_ TOWI.
 * @dma_handle: DMA handle used to map the TOWI onto device.
 * @dma_offset: TOWI's offset in the memory block.
 * @memblock: Pointer to the TOWI LIST memory block or memory page.
 * @wrapper: Address of the List Block wrapper for this list.
 * @sqh: The queue to which the descriptor of this priv belongs to.
 * @nsmr_ctrl_block: Address of nsmr control block for fast register
 * @nsmr_ctrl_block_reg_entry: Reg entry of nsmr_ctrl_block
 * @pad: Padding to make it cache-aligned.
 *
 * Per-towi HAL-private data.
 * HAL uses the space to keep DMA information associated with the towi.
 *
 * See also: __hal_fifo_txdl_priv_t {}, __hal_ring_rxd_priv_t {}
 */
typedef struct __hal_towi_priv_t {
	dma_addr_t			dma_addr;
	pci_dma_h			dma_handle;
	ptrdiff_t			dma_offset;
	void				*memblock;
	__hal_list_block_wrapper_t	*wrapper;
	vxge_hal_sq_h			sqh;
	__hal_nsmr_ctrl_block_t		*nsmr_ctrl_block;
	__hal_reg_entry_t		*nsmr_ctrl_block_reg_entry;
	u8				pad[(((63+__vxge_os_cacheline_size)/
						__vxge_os_cacheline_size)*
						__vxge_os_cacheline_size)-64];
} __hal_towi_priv_t;

/*
 * struct __hal_hw_cq_sq_ir_t - HW Completion Queue for Send Queue Indication
 *			Register
 * @control_0: The number of TOWIs that have been completely ACKed since the
 *		creation of this session 2 modulo 232.
 * @pad: Padding to make it cache-aligned.
 *
 * The adapter DMA writes a HW-CQ-SQ-IR to inform the host of transmit progress.
 * The adapter never reads from a HW-CQ-SQ-IR.
 *
 */
typedef __vxge_os_attr_cacheline_aligned struct __hal_hw_cq_sq_ir_t {
	u64		control_0;
#define	VXGE_HAL_HW_CQ_SQ_IR_GET_TOWI_COMPLETED(ctrl0)	bVAL32(ctrl0, 0)
#define	VXGE_HAL_HW_CQ_SQ_IR_TOWI_COMPLETED(val)	vBIT(val, 0, 32)

	u8		pad[(((7+__vxge_os_cacheline_size)/
		    __vxge_os_cacheline_size)*__vxge_os_cacheline_size)-8];
}__hal_hw_cq_sq_ir_t;

/*
 * struct __hal_hw_cq_rr_ir_t - HW Completion Queue for Read Response
 * @control_0: The number of RDMA Reads that have been completed since the
 *		creation of this session 2 modulo 232.  Completion of an RDMA
 *		Read means that an in-order L-bit has been received for the RDMA
 *		Read Response.
 * @pad: Padding to make it cache-aligned.
 *
 * The adapter DMA writes a HW-CQ-RR-IR to inform the host of RDMA Read
 * progress. The adapter never reads from a HW-CQ-RR.
 */
typedef __vxge_os_attr_cacheline_aligned struct __hal_hw_cq_rr_ir_t {
	u64		control_0;
#define	VXGE_HAL_HW_CQ_RR_IR_GET_READ_COMPLETED(ctrl0)	bVAL32(ctrl0, 0)
#define	VXGE_HAL_HW_CQ_RR_IR_READ_COMPLETED(val)	vBIT(val, 0, 32)

	u8		pad[(((7+__vxge_os_cacheline_size)/
		    __vxge_os_cacheline_size)*__vxge_os_cacheline_size)-8];
}__hal_hw_cq_rr_ir_t;

/*
 * struct __hal_hw_wqe_t - HW Work Queue Element
 * @host_control: This field is exclusively for host use and is read-only
 *		from the adapter's perspective
 * @opaque_handle: This field is copied, in its entirety, into the Completion
 *		Queue Element that is generated when consumption of this WQE
 *		is complete
 * @control_0: Bits 0 to 6 - Reserved.
 *	       Bit 7 - If set, the adapter will generate an interrupt when this
 *			HW-WQE is completed.
 *	       Bits 8 to 14 - Reserved.
 *	       Bit 15 - If set, the adapter will place the received message
 *			 contents into the CQE as immediate data - if possible
 *			 - INSTEAD of placing it into this WQE.
 *	       Bits 16 to 31 - Reserved.
 *	       Bits 32 to 63 - The number of message payload bytes that this
 *				WQE can hold.
 * @control_1: Bits 0 to 7 - The number (N) of ODs that this WQE contains.
 *	       Bits 8 to 31 - Reserved.
 *	       Bits 32 to 63 - The WQE Sequence Number attached to this WQE.
 *
 * @pad: Padding to make it cache-aligned.
 *
 * HW-WQE specifies the application buffers into which the adapter should place
 * the received data. HW-WQEs are posted into Receive Queue vxge_hal_hw_rq for
 * the adapter. The receive queue can either be dedicated to one session or
 * shared by several sessions. Each session must contain at least one receive
 * queue. HW-WQE is Created by the hostand Read by the adapter (CMG) via DMA.
 * Not overwritten in any way.(Consumption is implicit by completions sent to
 * the host.)
 */
typedef __vxge_os_attr_cacheline_aligned struct __hal_hw_wqe_t {
	u64		host_control;
	u64		opaque_handle;
	u64		control_0;
#define	VXGE_HAL_HW_WQE_GET_INTERRUPT_CONTROL(ctrl0)	bVAL1(ctrl0, 7)
#define	VXGE_HAL_HW_WQE_INTERRUPT_CONTROL		mBIT(7)

#define	VXGE_HAL_HW_WQE_GET_LOW_LATENCY(ctrl0)		bVAL1(ctrl0, 15)
#define	VXGE_HAL_HW_WQE_LOW_LATENCY			mBIT(15)

#define	VXGE_HAL_HW_WQE_GET_CAPACITY(ctrl0)		bVAL32(ctrl0, 32)
#define	VXGE_HAL_HW_WQE_CAPACITY(val)			vBIT(val, 32, 32)

	u64		control_1;
#define	VXGE_HAL_HW_WQE_GET_NUM_OID(ctrl1)		bVAL8(ctrl1, 0)
#define	VXGE_HAL_HW_WQE_NUM_OID(val)			vBIT(val, 0, 8)

#define	VXGE_HAL_HW_WQE_GET_WSN(ctrl1)			bVAL32(ctrl1, 32)
#define	VXGE_HAL_HW_WQE_WSN(val)			vBIT(val, 32, 32)


	u8		pad[(((31+__vxge_os_cacheline_size)/
		    __vxge_os_cacheline_size)*__vxge_os_cacheline_size)-32];
}__hal_hw_wqe_t;

/*
 * struct __hal_lro_wqe_t - LRO Work Queue Element
 * @wqe_host_control: Reserved for host use. Ignored by the adapter.
 * @wqe_opaque_handle: The opaque handle of this WQE.
 * @control_0: Bits 0 to 7 - Flag to force consumption of this WQE to
 *			  generate an interrupt.
 *	       Bits 8 to 63 - Reserved.
 * @buffer0_ptr: Buffer 0 address
 * @buffer1_ptr: Buffer 1 address
 * @buffer2_ptr: Buffer 2 address
 * @buffer3_ptr: Buffer 3 address
 * @buffer4_ptr: Buffer 4 address
 *
 * The adapter DMA reads an LRO WQE to obtain buffer space for one LRO frame.
 * The adapter never writes to an LRO WQE. All LRO WQE fields are read by the
 * adapter as part of the LRO WQE read.
 */
typedef __vxge_os_attr_cacheline_aligned struct __hal_lro_wqe_t {
	u64		wqe_host_control;
	u64		wqe_opaque_handle;
	u64		control_0;
#define	VXGE_HAL_LRO_WQE_GET_INTERRUPT_OVERRIDE(ctrl0)	bVAL8(ctrl0, 0)
#define	VXGE_HAL_LRO_WQE_INTERRUPT_OVERRIDE(val)	vBIT(val, 0, 8)
#define	VXGE_HAL_LRO_WQE_INTERRUPT_OVERRIDE_FORCE	0x1
#define	VXGE_HAL_LRO_WQE_INTERRUPT_OVERRIDE_DONT_FORCE	0x0
	u64		buffer0_ptr;
	u64		buffer1_ptr;
	u64		buffer2_ptr;
	u64		buffer3_ptr;
	u64		buffer4_ptr;
} __hal_lro_wqe_t;

/*
 * struct __hal_lro_wqe_priv_t - LRO_WQE HAL-private data.
 * @dma_addr: DMA (mapped) address of _this_ LRO_WQE.
 * @dma_handle: DMA handle used to map the LRO_WQE onto device.
 * @dma_offset: LRO_WQE's offset in the memory block.
 * @memblock: Pointer to the LRO_WQE LIST memory block or memory page.
 * @wrapper: Address of the List Block wrapper for this list
 * @srqh: The queue to which the descriptor of this priv belongs to.
 * @first: Is this wqe first in the block
 * @owner: Flag to identify the completed LRO WQEs
 * @opaque_handle: Opaque handle passed by client while post wqe
 * @pad: Padding to make it cache-aligned.
 *
 * Per-lro_wqe HAL-private data. HAL uses the space to keep DMA
 * information associated with the lro-wqe.
 *
 * See also: __hal_fifo_txdl_priv_t {}, __hal_ring_rxd_priv_t {},
 *	  __hal_towi_priv_t {}, __hal_cqe_priv_t {}.
 */
typedef struct __hal_lro_wqe_priv_t {
	dma_addr_t			dma_addr;
	pci_dma_h			dma_handle;
	ptrdiff_t			dma_offset;
	void				*memblock;
	__hal_list_block_wrapper_t	*wrapper;
	vxge_hal_srq_h			srqh;
	u32				first;
	u32				owner;
#define	VXGE_HAL_LRO_WQE_PRIV_OWNER_UNKNOWN	0
#define	VXGE_HAL_LRO_WQE_PRIV_OWNER_ADAPTER	1
#define	VXGE_HAL_LRO_WQE_PRIV_OWNER_HOST		2
	u64				opaque_handle;
	u8				pad[(((63+__vxge_os_cacheline_size)/
						__vxge_os_cacheline_size)*
						__vxge_os_cacheline_size)-48];
} __hal_lro_wqe_priv_t;

/*
 * struct __hal_rq_ir_t - HW/LRO (Shared) Receive Queue Indication Register
 * @control_0: The DMA Address upto which adapter read in the WQEs.
 *
 * @pad: Padding to make it cache-aligned.
 *
 * This space is prepared and initialized by the host at startup. Its location
 * is signaled at the time of CQ-RQ creation, in the DMsg (TBD). The adapter
 * will overwrite and update as WQEs are read in by adapter.
 *
 */
typedef __vxge_os_attr_cacheline_aligned struct __hal_rq_ir_t {
	u64		control_0;
#define	VXGE_HAL_RQ_IR_GET_WQE_ADDRESS(ctrl0)		bVAL32(ctrl0, 0)
#define	VXGE_HAL_RQ_IR_WQE_ADDRESS(val)			vBIT(val, 0, 32)

	u8		pad[(((7+__vxge_os_cacheline_size)/
		    __vxge_os_cacheline_size)*__vxge_os_cacheline_size)-8];
}__hal_rq_ir_t;

/*
 * struct __hal_hw_cqe_t - HW Completion Queue Element
 * @opaque_handle_1: This field is copied from the HW-WQE corresponding
 *		  to this Completion
 * @opaque_handle_2: This field is copied from the Session Context
 * @control_0: Bits 0 to 2 - Reserved.
 *	       Bit 3 - receive frame contained a TCP segment with PUSH flag set.
 *	       Bits 4 to 5 - Reserved.
 *	       Bits 6 to 7 - Describes the RDMA Send type of this CQE.
 *	       Bits 8 to 15 - The number of Immediate Data Bytes included in
 *			   this CQE.
 *	       Bits 16 to 22 - Reserved.
 *	       Bit 23 - If set, TCP out of order data
 *	       Bits 24 to 30 - Reserved.
 *	       Bit 31 - Set to indicate that the Stag_To_Invalidate indicated
 *			 by the received message has been invalidated on the
 *			 adapter.
 *	       Bits 32 to 63 - The local STag to invalidate.
 * @control_1: Bits 0 to 31 - For iWarp sessions, this is the RDMA message
 *			   sequence number (MSN) of the DDP message described
 *			   by this CQE. For bytestream sessions, this is TCP
 *			   SN of the first data.
 *	       Bits 32 to 63 - The number of DDP Payload bytes placed in the
 *				corresponding HW-WQE.
 * @pad: Padding to make it cache-aligned.
 *
 * Created by the adapter and written into the host-prepared HW-(S-)CQ-RQ space
 * when data is received and either paced into HW_WQE or directly into HW-CQE.
 * Host allocates memory for HW-CQE in receive completion queue vxge_hal_hw_rcq.
 * The receive completion queue can either be dedicated to a session or shared
 * by several sessions. Each session must have a receive completion queue.
 *
 */
typedef __vxge_os_attr_cacheline_aligned struct __hal_hw_cqe_t {
	u64		opaque_handle_1;
	u64		opaque_handle_2;
	u64		control_0;
#define	VXGE_HAL_HW_CQE_GET_PUSH(ctrl0)			bVAL1(ctrl0, 3)
#define	VXGE_HAL_HW_CQE_PUSH				mBIT(3)

#define	VXGE_HAL_HW_CQE_GET_SEND_TYPE(ctrl0)		bVAL2(ctrl0, 6)
#define	VXGE_HAL_HW_CQE_SEND_TYPE(val)			vBIT(val, 6, 2)
#define	VXGE_HAL_HW_CQE_SEND				0x3
#define	VXGE_HAL_HW_CQE_SEND_INVALIDATE			0x4
#define	VXGE_HAL_HW_CQE_SEND_SOLICITED			0x5
#define	VXGE_HAL_HW_CQE_SEND_SOLICITED_INVALIDATE	0x6

#define	VXGE_HAL_HW_CQE_GET_IMMEDIATE_DATA_LEN(ctrl0)	bVAL8(ctrl0, 8)
#define	VXGE_HAL_HW_CQE_IMMEDIATE_DATA_LEN(val)		vBIT(val, 8, 8)

#define	VXGE_HAL_HW_CQE_GET_INO(ctrl0)			bVAL1(ctrl0, 23)
#define	VXGE_HAL_HW_CQE_INO				mBIT(23)

#define	VXGE_HAL_HW_CQE_GET_INVALIDATE(ctrl0)		bVAL1(ctrl0, 31)
#define	VXGE_HAL_HW_CQE_INVALIDATE			mBIT(31)

#define	VXGE_HAL_HW_CQE_GET_STAG(ctrl0)			bVAL32(ctrl0, 32)
#define	VXGE_HAL_HW_CQE_STAG(val)			vBIT(val, 32, 32)

	u64		control_1;
#define	VXGE_HAL_HW_CQE_GET_MSN(ctrl1)			bVAL32(ctrl1, 0)
#define	VXGE_HAL_HW_CQE_MSN(val)			vBIT(val, 0, 32)

#define	VXGE_HAL_HW_CQE_GET_BYTES_PLACED(ctrl1)		bVAL32(ctrl1, 32)
#define	VXGE_HAL_HW_CQE_BYTES_PLACED(val)		vBIT(val, 32, 32)

	u8		pad[(((31+__vxge_os_cacheline_size)/
		    __vxge_os_cacheline_size)*__vxge_os_cacheline_size)-32];
}__hal_hw_cqe_t;

/*
 * struct __hal_lro_cqe_t - LRO Completion Queue Element
 * @wqe_opaque_handle: The opaque handle of the WQE where the merged frame
 *			described by this CQE is located. Allows the host to
 *			locate the WQE
 * @session_opaque_handle: The opaque handle of the session to which this CQE
 *			belongs. Allows the host to locate the session
 * @control_0: Bits 0 to 15 - IPv4 header checksum value from the merged
 *			   frame's IPv4 header.
 *	       Bits 16 to 31 - TCP checksum from the merged frame's TCP header.
 *	       Bits 32 to 47 - The VLAN tag that arrived with the Rx frame(s).
 *	       Bits 48 to 63 - The number of bytes placed into Buffer0 of the
 *				corresponding WQE.
 * @control_1: Bits 0 to 15 - bytes placed into Buffer1.
 *	       Bits 16 to 31 - bytes placed into Buffer2.
 *	       Bits 32 to 47 - bytes placed into Buffer3.
 *	       Bits 48 to 63 - bytes placed into Buffer4.
 * @control_2: Bits 0 to 7 - 0x0 for IPv4, 0x1 for IPv6
 *	       Bits 8 to 15 - The number of Rx frames which went into the merged
 *			   frame described by this CQE.
 *	       Bits 16 to 23 - Reserved.
 *	       Bits 24 to 31 - The ethernet encapsulation used by the (merged)
 *				frame descibed by this CQE.
 *	       Bits 32 to 39 - The reason this CQE was generated.
 *	       Bits 40 to 47 - VLAN tag flag.
 *	       Bits 48 to 63 - Reserved.
 * @pad: Padding to make it cache-aligned.
 *
 * The adapter DMA writes an LRO-CQE to describe one (merged) frame to the
 * host. The adapter never reads from an LRO-CQE. All LRO CQE fields are
 * written by the adapter as part of the CQE write.
 */
typedef __vxge_os_attr_cacheline_aligned struct __hal_lro_cqe_t {
	u64		wqe_opaque_handle;
	u64		session_opaque_handle;
	u64		control_0;
#define	VXGE_HAL_LRO_CQE_GET_IPV4_HEADER_CHECKSUM(ctrl0) bVAL16(ctrl0, 0)
#define	VXGE_HAL_LRO_CQE_IPV4_HEADER_CHECKSUM(val)	vBIT(val, 0, 16)

#define	VXGE_HAL_LRO_CQE_GET_TCP_CHECKSUM(ctrl0)	bVAL16(ctrl0, 16)
#define	VXGE_HAL_LRO_CQE_TCP_CHECKSUM(val)		vBIT(val, 16, 16)

#define	VXGE_HAL_LRO_CQE_GET_VLAN_TAG(ctrl0)		bVAL16(ctrl0, 16)
#define	VXGE_HAL_LRO_CQE_VLAN_TAG(val)			vBIT(val, 32, 16)

#define	VXGE_HAL_LRO_CQE_GET_BUF0_BYTES_PLACED(ctrl0)	bVAL16(ctrl0, 32)
#define	VXGE_HAL_LRO_CQE_BUF0_BYTES_PLACED(val)		vBIT(val, 48, 16)

	u64		control_1;
#define	VXGE_HAL_LRO_CQE_GET_BUF1_BYTES_PLACED(ctrl1)	bVAL16(ctrl1, 0)
#define	VXGE_HAL_LRO_CQE_BUF1_BYTES_PLACED(val)		vBIT(val, 0, 16)

#define	VXGE_HAL_LRO_CQE_GET_BUF2_BYTES_PLACED(ctrl1)	bVAL16(ctrl1, 16)
#define	VXGE_HAL_LRO_CQE_BUF2_BYTES_PLACED(val)		vBIT(val, 16, 16)

#define	VXGE_HAL_LRO_CQE_GET_BUF3_BYTES_PLACED(ctrl1)	bVAL16(ctrl1, 32)
#define	VXGE_HAL_LRO_CQE_BUF3_BYTES_PLACED(val)		vBIT(val, 32, 16)

#define	VXGE_HAL_LRO_CQE_GET_BUF4_BYTES_PLACED(ctrl1)	bVAL16(ctrl1, 48)
#define	VXGE_HAL_LRO_CQE_BUF4_BYTES_PLACED(val)		vBIT(val, 48, 16)

	u64		control_2;
#define	VXGE_HAL_LRO_CQE_GET_IPV6(ctrl2)		bVAL8(ctrl2, 0)
#define	VXGE_HAL_LRO_CQE_IPV6(val)			vBIT(val, 0, 8)
#define	VXGE_HAL_LRO_CQE_IPV6_IPV4			0x0
#define	VXGE_HAL_LRO_CQE_IPV6_IPV6			0x1

#define	VXGE_HAL_LRO_CQE_GET_NUMBER_OF_FRAMES(ctrl2)	bVAL8(ctrl2, 8)
#define	VXGE_HAL_LRO_CQE_NUMBER_OF_FRAMES(val)		vBIT(val, 8, 8)

#define	VXGE_HAL_LRO_CQE_GET_ETH_ENCAPSULATION(ctrl2)	bVAL8(ctrl2, 24)
#define	VXGE_HAL_LRO_CQE_ETH_ENCAPSULATION(val)		vBIT(val, 24, 8)
#define	VXGE_HAL_LRO_CQE_ETH_ENCAPSULATION_DIX		0x0
#define	VXGE_HAL_LRO_CQE_ETH_ENCAPSULATION_LLC		0x1
#define	VXGE_HAL_LRO_CQE_ETH_ENCAPSULATION_SNAP		0x2

#define	VXGE_HAL_LRO_CQE_GET_REASON(ctrl2)		bVAL8(ctrl2, 32)
#define	VXGE_HAL_LRO_CQE_REASON(val)			vBIT(val, 32, 8)
#define	VXGE_HAL_LRO_CQE_REASON_TIMEOUT			0x0
#define	VXGE_HAL_LRO_CQE_REASON_MERGE_DISABLED		0x1
#define	VXGE_HAL_LRO_CQE_REASON_SYN_FIN_RST_URG		0x2
#define	VXGE_HAL_LRO_CQE_REASON_FR_CNT_TH_REACHED	0x3
#define	VXGE_HAL_LRO_CQE_REASON_PSH_FLAG_TH_REACHED	0x4
#define	VXGE_HAL_LRO_CQE_REASON_OUT_OF_SPACE		0x5
#define	VXGE_HAL_LRO_CQE_REASON_FIRST_FR_TCP_DISCON	0x6
#define	VXGE_HAL_LRO_CQE_REASON_LAST_FR_TCP_DISCONT	0x7

#define	VXGE_HAL_LRO_CQE_GET_VLAN_FLAG(ctrl2)		bVAL8(ctrl2, 40)
#define	VXGE_HAL_LRO_CQE_VLAN_FLAG(val)			vBIT(val, 40, 8)
#define	VXGE_HAL_LRO_CQE_VLAN_FLAG_PRESENT		0x1
#define	VXGE_HAL_LRO_CQE_VLAN_FLAG_NOT_PRESENT		0x0

	u8		pad[(((39+__vxge_os_cacheline_size)/
		    __vxge_os_cacheline_size)*__vxge_os_cacheline_size)-40];
} __hal_lro_cqe_t;

/*
 * struct __hal_cqe_priv_t - HW/LRO_CQE HAL-private data.
 * @dma_addr: DMA (mapped) address of _this_ HW/LRO_CQE.
 * @dma_handle: DMA handle used to map the HW/LRO_CQE onto device.
 * @dma_offset: HW/LRO_CQE's offset in the memory block.
 * @memblock: Pointer to the HW/LRO_CQE LIST memory block or memory page.
 * @wrapper: Address of the List Block wrapper for this list
 * @cqrql: The queue to which the descriptor of this priv belongs to.
 * @pad: Padding to make it cache-aligned.
 *
 * Per-hw/lro_cqe HAL-private data. HAL uses the space to keep DMA
 * information associated with the hw/lro-cqe.
 *
 * See also: __hal_fifo_txdl_priv_t {}, __hal_ring_rxd_priv_t {},
 *	  __hal_towi_priv_t {}.
 */
typedef struct __hal_cqe_priv_t {
	dma_addr_t			dma_addr;
	pci_dma_h			dma_handle;
	ptrdiff_t			dma_offset;
	void				*memblock;
	__hal_list_block_wrapper_t	*wrapper;
	vxge_hal_cqrq_h			cqrqh;
	u8				pad[(((47+__vxge_os_cacheline_size)/
						__vxge_os_cacheline_size)*
						__vxge_os_cacheline_size)-48];
} __hal_cqe_priv_t;

/*
 * struct __hal_cq_rq_ir_t - HW/LRO (Shared) Completion Queue for Receive Queue
 *			  Indication Register
 * @control_0: The number of CQEs that have been written to this CQRQ
 *		since t=0, mod 232
 *
 * @pad: Padding to make it cache-aligned.
 *
 * This space is prepared and initialized by the host at startup. Its location
 * is signaled at the time of CQ-RQ creation, in the DMsg (TBD). The adapter
 * will overwrite and update as CQEs are returned to the host.
 *
 */
typedef __vxge_os_attr_cacheline_aligned struct __hal_cq_rq_ir_t {
	u64		control_0;
#define	VXGE_HAL_CQ_RQ_IR_GET_NUM_CQE(ctrl0)		bVAL32(ctrl0, 0)
#define	VXGE_HAL_CQ_RQ_IR_NUM_CQE(val)			vBIT(val, 0, 32)

	u8		pad[(((7+__vxge_os_cacheline_size)/
		    __vxge_os_cacheline_size)*__vxge_os_cacheline_size)-8];
}__hal_cq_rq_ir_t;

__EXTERN_END_DECLS

#endif /* VXGE_HAL_OFFLOAD_H */
