/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */
/*
 * Copyright (c) 2002-2010 Neterion, Inc.
 * All right Reserved.
 *
 * FileName :   vxgehal-regpool.h
 *
 * Description:  HAL Register pool for IR Register and NSMR registers
 *
 * Created:       15 September 2006
 */

#ifndef	VXGE_HAL_REGPOOL_H
#define	VXGE_HAL_REGPOOL_H

__EXTERN_BEGIN_DECLS

#define	VXGE_HAL_REGPOOL_SIZE		64
#define	VXGE_HAL_REGPOOL_REG_SIZE	((__vxge_os_cacheline_size <= 64)? 64:\
					    __vxge_os_cacheline_size)

/*
 * struct __hal_reg_entry_t - Register private data structure
 * @item: List header used to link.
 * @reg_addr: Virtual Address of the register
 * @dma_offset: Register offset in memory block.
 * @dma_addr: DMA Address of the register block.
 * @dma_handle: DMA handle of the register block.
 * @memblock: Virtual address memory block
 * @memblock_index: Index of the memory block
 *
 * register entry is allocated with a header to put the registers into list.
 *
 */
typedef struct __hal_reg_entry_t {
	vxge_list_t			item;
	void				*reg_addr;
	ptrdiff_t			dma_offset;
	dma_addr_t			dma_addr;
	pci_dma_h			dma_handle;
	void				*memblock;
	u32				memblock_index;
} __hal_reg_entry_t;

/*
 * struct __hal_regpool_t - Register Pool
 * @mempool: Memory pool, the pool from which registers get allocated.
 * @pool_size: Number of registers in the pool
 * @reg_size: size of each register.
 * @regs_per_block: Registers per block
 * @reg_entry_size: Reserved at the end of each list block. HAL internal
 *			 usage.
 * @regblock_priv_size: Block private size
 * @free_reg_list: List of free registers
 * @pool_lock: Spin lock for the pool
 *
 * Reg pool contains the registers preallocated to be used for ir registers
 * and nsmr ctrl.
 *
 */
typedef struct __hal_regpool_t {
	vxge_hal_mempool_t		*mempool;
	u32				pool_size;
	u32				reg_size;
	u32				regs_per_block;
	u32				reg_entry_size;
	u32				regblock_priv_size;
	vxge_list_t			free_reg_list;
	spinlock_t			pool_lock;
} __hal_regpool_t;

vxge_hal_status_e
__hal_regpool_create(vxge_hal_device_h devh,
			__hal_regpool_t *regpool,
			u32 pool_size);

void
__hal_regpool_destroy(vxge_hal_device_h devh,
			    __hal_regpool_t *regpool);


vxge_hal_status_e
__hal_regpool_allocate(__hal_regpool_t *regpool,
			    __hal_reg_entry_t **reg_entry);

vxge_hal_status_e
__hal_regpool_free(__hal_regpool_t *regpool,
			__hal_reg_entry_t *reg_entry);

__EXTERN_END_DECLS

#endif /* VXGE_HAL_REGPOOL_H */
