/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */
/*
 * Copyright (c) 2002-2010 Neterion, Inc.
 * All right Reserved.
 *
 * FileName :   vxgehal-rnic.c
 *
 * Description:  Routines to support rnicif
 *
 * Created:       65 Sep 2006
 */
#include "vxgehal.h"

void
vxge_hal_rnicif_init(void *buf)
{
	vxge_hal_rnic_if_t *rnicif = (vxge_hal_rnic_if_t *)buf;

	vxge_assert(rnicif != NULL);

	vxge_hal_trace_log_driver(NULL_HLDEV, NULL_VPID,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_driver(NULL_HLDEV, NULL_VPID,
	    "rnicif = "VXGE_OS_STXFMT, (ptr_t)rnicif);

	rnicif->vxge_hal_version.version_major = VXGE_HAL_VERSION_MAJOR;
	rnicif->vxge_hal_version.version_minor = VXGE_HAL_VERSION_MINOR;
	rnicif->vxge_hal_version.version_fix   = VXGE_HAL_VERSION_FIX;
	rnicif->vxge_hal_version.version_build = VXGE_HAL_VERSION_BUILD;

	rnicif->vxge_hal_vpath_mgmt_if.vxge_hal_vpath_attach =
	    vxge_hal_vpath_attach;
	rnicif->vxge_hal_vpath_mgmt_if.vxge_hal_vpath_detach =
	    vxge_hal_vpath_detach;

	rnicif->vxge_hal_odlist_mgmt_if.vxge_hal_odlist_allocate =
	    vxge_hal_odlist_allocate;
	rnicif->vxge_hal_odlist_mgmt_if.vxge_hal_odlist_set =
	    vxge_hal_odlist_set;
	rnicif->vxge_hal_odlist_mgmt_if.vxge_hal_odlist_free =
	    vxge_hal_odlist_free;

	rnicif->vxge_hal_pblist_mgmt_if.vxge_hal_pblist_allocate =
	    vxge_hal_pblist_allocate;
	rnicif->vxge_hal_pblist_mgmt_if.vxge_hal_pblist_set =
	    vxge_hal_pblist_set;
	rnicif->vxge_hal_pblist_mgmt_if.vxge_hal_pblist_free =
	    vxge_hal_pblist_free;

	rnicif->vxge_hal_nce_mgmt_if.vxge_hal_nce_create =
	    vxge_hal_nce_create;
	rnicif->vxge_hal_nce_mgmt_if.vxge_hal_nce_update_mac_addr =
	    vxge_hal_nce_update_mac_addr;
	rnicif->vxge_hal_nce_mgmt_if.vxge_hal_nce_update_rch_time =
	    vxge_hal_nce_update_rch_time;
	rnicif->vxge_hal_nce_mgmt_if.vxge_hal_nce_query =
	    vxge_hal_nce_query;
	rnicif->vxge_hal_nce_mgmt_if.vxge_hal_nce_query_resp_get =
	    vxge_hal_nce_query_resp_umsg_get;
	rnicif->vxge_hal_nce_mgmt_if.vxge_hal_nce_delete =
	    vxge_hal_nce_delete;

	rnicif->vxge_hal_stag_mgmt_if.vxge_hal_stag_allocate_nsmr =
	    vxge_hal_stag_allocate_nsmr;
	rnicif->vxge_hal_stag_mgmt_if.vxge_hal_stag_allocate_nsmr_resp_get =
	    vxge_hal_stag_allocate_nsmr_resp_umsg_get;
	rnicif->vxge_hal_stag_mgmt_if.vxge_hal_stag_allocate_mw =
	    vxge_hal_stag_allocate_mw;
	rnicif->vxge_hal_stag_mgmt_if.vxge_hal_stag_allocate_mw_resp_get =
	    vxge_hal_stag_allocate_mw_resp_umsg_get;
	rnicif->vxge_hal_stag_mgmt_if.vxge_hal_stag_deallocate =
	    vxge_hal_stag_deallocate;
	rnicif->vxge_hal_stag_mgmt_if.vxge_hal_stag_deallocate_resp_get =
	    vxge_hal_stag_deallocate_resp_umsg_get;
	rnicif->vxge_hal_stag_mgmt_if.vxge_hal_stag_register_nsmr =
	    vxge_hal_stag_register_nsmr;
	rnicif->vxge_hal_stag_mgmt_if.vxge_hal_stag_register_nsmr_resp_get =
	    vxge_hal_stag_register_nsmr_resp_umsg_get;
	rnicif->vxge_hal_stag_mgmt_if.vxge_hal_stag_re_register_nsmr =
	    vxge_hal_stag_re_register_nsmr;
	rnicif->vxge_hal_stag_mgmt_if.vxge_hal_stag_re_register_nsmr_resp_get =
	    vxge_hal_stag_re_register_nsmr_resp_umsg_get;
	rnicif->vxge_hal_stag_mgmt_if.vxge_hal_stag_register_smr =
	    vxge_hal_stag_register_smr;
	rnicif->vxge_hal_stag_mgmt_if.vxge_hal_stag_register_smr_resp_get =
	    vxge_hal_stag_register_smr_resp_umsg_get;
	rnicif->vxge_hal_stag_mgmt_if.vxge_hal_stag_query =
	    vxge_hal_stag_query;
	rnicif->vxge_hal_stag_mgmt_if.vxge_hal_stag_query_resp_get =
	    vxge_hal_stag_query_resp_umsg_get;
	rnicif->vxge_hal_stag_mgmt_if.vxge_hal_stag_validate =
	    vxge_hal_stag_validate;
	rnicif->vxge_hal_stag_mgmt_if.vxge_hal_stag_validate_resp_get =
	    vxge_hal_stag_validate_resp_umsg_get;
	rnicif->vxge_hal_stag_mgmt_if.vxge_hal_stag_invalidate =
	    vxge_hal_stag_invalidate;
	rnicif->vxge_hal_stag_mgmt_if.vxge_hal_stag_invalidate_resp_get =
	    vxge_hal_stag_invalidate_resp_umsg_get;
	rnicif->vxge_hal_stag_mgmt_if.vxge_hal_stag_bind_mw =
	    vxge_hal_stag_bind_mw;
	rnicif->vxge_hal_stag_mgmt_if.vxge_hal_stag_bind_mw_resp_get =
	    vxge_hal_stag_bind_mw_resp_umsg_get;
	rnicif->vxge_hal_stag_mgmt_if.vxge_hal_stag_fast_register_nsmr =
	    vxge_hal_stag_fast_register_nsmr;
	rnicif->
	    vxge_hal_stag_mgmt_if.vxge_hal_stag_fast_register_nsmr_resp_get =
	    vxge_hal_stag_fast_register_nsmr_resp_umsg_get;

	rnicif->vxge_hal_cqrq_mgmt_if.vxge_hal_cqrq_create =
	    vxge_hal_cqrq_create;
	rnicif->vxge_hal_cqrq_mgmt_if.vxge_hal_cqrq_modify =
	    vxge_hal_cqrq_modify;
	rnicif->vxge_hal_cqrq_mgmt_if.vxge_hal_cqrq_query =
	    vxge_hal_cqrq_query;
	rnicif->vxge_hal_cqrq_mgmt_if.vxge_hal_cqrq_query_resp_get =
	    vxge_hal_cqrq_query_resp_umsg_get;
	rnicif->vxge_hal_cqrq_mgmt_if.vxge_hal_cqrq_arm =
	    vxge_hal_cqrq_arm;
	rnicif->vxge_hal_cqrq_mgmt_if.vxge_hal_cqrq_event_notification_get =
	    vxge_hal_cqrq_event_notification_umsg_get;
	rnicif->vxge_hal_cqrq_mgmt_if.vxge_hal_cqrq_ready_hw_cqe_check =
	    vxge_hal_cqrq_ready_hw_cqe_check;
	rnicif->vxge_hal_cqrq_mgmt_if.vxge_hal_cqrq_next_ready_hw_cqe_get =
	    vxge_hal_cqrq_next_ready_hw_cqe_get;
	rnicif->vxge_hal_cqrq_mgmt_if.vxge_hal_cqrq_delete =
	    vxge_hal_cqrq_delete;

	rnicif->vxge_hal_srq_mgmt_if.vxge_hal_srq_create =
	    vxge_hal_srq_create;
	rnicif->vxge_hal_srq_mgmt_if.vxge_hal_srq_modify =
	    vxge_hal_srq_modify;
	rnicif->vxge_hal_srq_mgmt_if.vxge_hal_srq_query =
	    vxge_hal_srq_query;
	rnicif->vxge_hal_srq_mgmt_if.vxge_hal_srq_query_resp_get =
	    vxge_hal_srq_query_resp_umsg_get;
	rnicif->vxge_hal_srq_mgmt_if.vxge_hal_srq_arm =
	    vxge_hal_srq_arm;
	rnicif->vxge_hal_srq_mgmt_if.vxge_hal_srq_event_notification_get =
	    vxge_hal_srq_event_notification_umsg_get;
	rnicif->vxge_hal_srq_mgmt_if.vxge_hal_srq_wqebadd =
	    vxge_hal_srq_wqebadd;
	rnicif->vxge_hal_srq_mgmt_if.vxge_hal_srq_hw_wqe_post =
	    vxge_hal_srq_hw_wqe_post;
	rnicif->vxge_hal_srq_mgmt_if.vxge_hal_srq_return_unused_wqes =
	    vxge_hal_srq_return_unused_wqes;
	rnicif->vxge_hal_srq_mgmt_if.vxge_hal_srq_return_unused_wqes_resp_get =
	    vxge_hal_srq_return_unused_wqes_resp_umsg_get;
	rnicif->vxge_hal_srq_mgmt_if.vxge_hal_srq_delete =
	    vxge_hal_srq_delete;

	rnicif->vxge_hal_sq_mgmt_if.vxge_hal_sq_create =
	    vxge_hal_sq_create;
	rnicif->vxge_hal_sq_mgmt_if.vxge_hal_send_towi_post =
	    vxge_hal_send_towi_post;
	rnicif->vxge_hal_sq_mgmt_if.vxge_hal_send_towi_odlist_post =
	    vxge_hal_send_towi_odlist_post;
	rnicif->vxge_hal_sq_mgmt_if.vxge_hal_write_towi_post =
	    vxge_hal_write_towi_post;
	rnicif->vxge_hal_sq_mgmt_if.vxge_hal_write_towi_odlist_post =
	    vxge_hal_write_towi_odlist_post;
	rnicif->vxge_hal_sq_mgmt_if.vxge_hal_read_towi_post =
	    vxge_hal_read_towi_post;
	rnicif->vxge_hal_sq_mgmt_if.vxge_hal_bind_mw_towi_post =
	    vxge_hal_bind_mw_towi_post;
	rnicif->vxge_hal_sq_mgmt_if.vxge_hal_fmr_towi_post =
	    vxge_hal_fmr_towi_post;
	rnicif->vxge_hal_sq_mgmt_if.vxge_hal_local_invalidate_towi_post =
	    vxge_hal_local_invalidate_towi_post;
	rnicif->vxge_hal_sq_mgmt_if.vxge_hal_terminate_towi_post =
	    vxge_hal_terminate_towi_post;
	rnicif->vxge_hal_sq_mgmt_if.vxge_hal_bytestream_towi_post =
	    vxge_hal_bytestream_towi_post;
	rnicif->vxge_hal_sq_mgmt_if.vxge_hal_bytestream_towi_odlist_post =
	    vxge_hal_bytestream_towi_odlist_post;
	rnicif->vxge_hal_sq_mgmt_if.vxge_hal_tcp_fin_towi_post =
	    vxge_hal_tcp_fin_towi_post;
	rnicif->vxge_hal_sq_mgmt_if.vxge_hal_sq_completed_towi_check =
	    vxge_hal_sq_completed_towi_check;
	rnicif->vxge_hal_sq_mgmt_if.vxge_hal_sq_next_completed_towi_get =
	    vxge_hal_sq_next_completed_towi_get;
	rnicif->vxge_hal_sq_mgmt_if.vxge_hal_sq_delete =
	    vxge_hal_sq_delete;

	rnicif->vxge_hal_tcp_mgmt_if.vxge_hal_tcp_open =
	    vxge_hal_tcp_open;
	rnicif->vxge_hal_tcp_mgmt_if.vxge_hal_tcp_promote_to_iwarp =
	    vxge_hal_tcp_promote_to_iwarp;
	rnicif->vxge_hal_tcp_mgmt_if.vxge_hal_tcp_promote_to_iwarp_resp_get =
	    vxge_hal_tcp_promote_to_iwarp_resp_umsg_get;
	rnicif->vxge_hal_tcp_mgmt_if.vxge_hal_tcp_modify =
	    vxge_hal_tcp_modify;
	rnicif->vxge_hal_tcp_mgmt_if.vxge_hal_tcp_abort =
	    vxge_hal_tcp_abort;
	rnicif->vxge_hal_tcp_mgmt_if.vxge_hal_tcp_update_rxwin =
	    vxge_hal_tcp_update_rxwin;
	rnicif->vxge_hal_tcp_mgmt_if.vxge_hal_tcp_update_mss =
	    vxge_hal_tcp_update_mss;
	rnicif->vxge_hal_tcp_mgmt_if.vxge_hal_tcp_update_ip_header =
	    vxge_hal_tcp_update_ip_header;
	rnicif->vxge_hal_tcp_mgmt_if.vxge_hal_tcp_update_keepalive =
	    vxge_hal_tcp_update_keepalive;
	rnicif->vxge_hal_tcp_mgmt_if.vxge_hal_tcp_relink_to_nce =
	    vxge_hal_tcp_relink_to_nce;
	rnicif->vxge_hal_tcp_mgmt_if.vxge_hal_tcp_delete =
	    vxge_hal_tcp_delete;

	rnicif->vxge_hal_session_mgmt_if.vxge_hal_session_query =
	    vxge_hal_session_query;
	rnicif->vxge_hal_session_mgmt_if.vxge_hal_session_query_resp_get =
	    vxge_hal_session_query_resp_umsg_get;
	rnicif->vxge_hal_session_mgmt_if.
	    vxge_hal_session_return_in_progress_wqes_get =
	    vxge_hal_session_return_in_progress_wqes_umsg_get;
	rnicif->vxge_hal_session_mgmt_if.vxge_hal_session_frame_write_get =
	    vxge_hal_session_frame_write_umsg_get;

	vxge_hal_trace_log_driver(NULL_HLDEV, NULL_VPID,
	    "<==  %s:%s:%d  Result: 0", __FILE__, __func__, __LINE__);
}
