/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */
/*
 * Copyright (c) 2002-2010 Neterion, Inc.
 * All right Reserved.
 *
 * FileName :   vxgehal-session.c
 *
 * Description:  session(TCP, LRO, SPDM) related routines
 *
 * Created:       11 January 2006
 */
#include "vxgehal.h"

/*
 * vxge_hal_session_query - Query the information of a session
 *
 * @vpath_handle: Handle returned by message queue register
 * @session_id: Session Id
 * @opaque_handle: HAL Opaque handle
 *
 * Queries the information of a session
 */
vxge_hal_status_e
vxge_hal_session_query(
	vxge_hal_vpath_h vpath_handle,
	vxge_hal_obj_id_t session_id,
	vxge_hal_opaque_handle_t *opaque_handle)
{
	__hal_vpath_handle_t *vp = (__hal_vpath_handle_t *)vpath_handle;
	__hal_device_t *hldev;
	__hal_dmq_t *dmq;
	__hal_down_msg_t dmsg;
	vxge_hal_status_e status;

	vxge_assert((vpath_handle != NULL) && (session_id != 0) &&
	    (opaque_handle != NULL));

	hldev = (__hal_device_t *)vp->vpath->hldev;

	vxge_hal_trace_log_session(hldev, vp->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_session(hldev, vp->vpath->vp_id,
	    "vpath_handle = 0x"VXGE_OS_STXFMT", session_id = 0x"VXGE_OS_STXFMT
	    ", opaque_handle = 0x"VXGE_OS_STXFMT, (ptr_t)vpath_handle,
	    (ptr_t)session_id, (ptr_t)opaque_handle);

	dmq = (__hal_dmq_t *)vp->vpath->dmqh;

	vxge_assert(dmq != NULL);

	VXGE_HAL_OPAQUE_HANDLE_VPATH_HANDLE(opaque_handle, vpath_handle);
	VXGE_HAL_OPAQUE_HANDLE_HAL_PRIV(opaque_handle, session_id);

	__hal_session_query_req_dmsg_set(
	    (vxge_hal_down_msg_h)&dmsg,
	    (u32)session_id,
	    dmq->intr_ctrl,
	    dmq->gen_compl,
	    opaque_handle);

	status =  __hal_dmq_post(dmq, (vxge_hal_down_msg_h)&dmsg);

	if (status == VXGE_HAL_OK)
		status = VXGE_HAL_PENDING;

	vxge_hal_trace_log_session(hldev, vp->vpath->vp_id,
	    "<==  %s:%s:%d  Result: %d", __FILE__, __func__, __LINE__, status);
	return (status);
}
