/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */
/*
 * Copyright (c) 2002-2010 Neterion, Inc.
 * All right Reserved.
 *
 * FileName :   vxgehal-umq.c
 *
 * Description:  HAL Up Message Queue functionality
 *
 * Created:       14 December 2005
 */
#include "vxgehal.h"

#ifdef	VXGE_HAL_INTERNAL_COMPILER_ERROR

#pragma optimize("", off)

#endif
/*
 * __hal_umq_first_block_address_get - Get the dma address of the first block
 * @umqh: Handle of UMQ
 *
 * This function returns the dma address of the first block
 */
u64
__hal_umq_first_block_address_get(
	vxge_hal_umq_h umqh)
{
	__hal_umq_t *umq = (__hal_umq_t *)umqh;

	return (umq->first_block_address);
}

/*
 * __hal_umq_first_block_length_get - Get the length of the first block
 * @umqh: Handle of UMQ
 *
 * This function returns the length of the first block
 */
u32
__hal_umq_first_block_length_get(
	vxge_hal_umq_h umqh)
{
	__hal_umq_t *umq = (__hal_umq_t *)umqh;

	return (umq->first_block_length);
}

/*
 * __hal_umsg_mempool_item_alloc - Allocate List blocks for messages callback
 * @mempoolh: Handle to memory pool
 * @memblock: Address of this memory block
 * @memblock_index: Index of this memory block
 * @dma_object: dma object for this block
 * @item: Pointer to this item
 * @index: Index of this item in memory block
 * @is_last: If this is last item in the block
 * @userdata: Specific data of user
 *
 * This function is callback passed to __hal_mempool_create to create memory
 * pool for messages
 */
static vxge_hal_status_e
__hal_umsg_mempool_item_alloc(
	vxge_hal_mempool_h mempoolh,
	void *memblock,
	u32 memblock_index,
	vxge_hal_mempool_dma_t *dma_object,
	void *item,
	u32 index,
	u32 is_last,
	void *userdata)
{
	u32 memblock_item_idx;
	__hal_umq_t *umqdata = (__hal_umq_t *)userdata;
	__hal_list_block_t *prev_block, *next_block;
	__hal_list_block_wrapper_t *prev_wrapper;
	__hal_umq_blk_priv_t *umq_blk_priv;
	__hal_device_t *hldev = (__hal_device_t *)umqdata->channel.devh;

	vxge_assert(umqdata != NULL);

	vxge_hal_trace_log_pool(hldev,
	    ((__hal_vpath_handle_t *)umqdata->channel.vph)->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_pool(hldev,
	    ((__hal_vpath_handle_t *)umqdata->channel.vph)->vpath->vp_id,
	    "mempoolh = 0x"VXGE_OS_STXFMT", memblock = 0x"VXGE_OS_STXFMT", "
	    "memblock_index = %d, dma_object = 0x"VXGE_OS_STXFMT", "
	    "item = 0x"VXGE_OS_STXFMT", index = %d, is_last = %d, "
	    "userdata = 0x"VXGE_OS_STXFMT, (ptr_t)mempoolh,
	    (ptr_t)memblock, memblock_index, (ptr_t)dma_object,
	    (ptr_t)item, index, is_last, (ptr_t)userdata);

	umq_blk_priv = (__hal_umq_blk_priv_t *)
	    __hal_mempool_item_priv(mempoolh, memblock_index, item,
	    &memblock_item_idx);

	vxge_assert(umq_blk_priv != NULL);

	umqdata->channel.dtr_arr[index].dtr = item;

	umq_blk_priv->dma_offset = 0;
	umq_blk_priv->dma_addr = dma_object->addr;
	umq_blk_priv->dma_handle = dma_object->handle;
	umq_blk_priv->memblock = memblock;
	umq_blk_priv->wrapper = (__hal_list_block_wrapper_t *)
	    (((char *)memblock) + VXGE_HAL_LIST_BLOCK_ITEMS_SIZE);
	umq_blk_priv->umqh = (vxge_hal_umq_h)userdata;
	umq_blk_priv->compl_offset = 0;
	umq_blk_priv->post_offset = VXGE_HAL_LIST_BLOCK_ITEMS_SIZE;

	__hal_list_block_memblock_priv_data_set(memblock,
	    (u64)(ptr_t)umq_blk_priv);
	__hal_list_block_memblock_idx_set(memblock, memblock_index);

	if (index > 0) {
		prev_block = __hal_mempool_item(mempoolh, index-1);
		next_block = __hal_mempool_item(mempoolh, 0);
		prev_wrapper = (__hal_list_block_wrapper_t *)
		    (((char *)prev_block) + VXGE_HAL_LIST_BLOCK_ITEMS_SIZE);
		__hal_wrapper_link(prev_wrapper, umq_blk_priv->wrapper,
		    dma_object->addr, VXGE_HAL_LIST_BLOCK_ITEMS_SIZE);
		__hal_wrapper_htonll(prev_wrapper);
	} else {
		prev_block = memblock;
		next_block = memblock;
		__hal_wrapper_link(umq_blk_priv->wrapper, umq_blk_priv->wrapper,
		    dma_object->addr, VXGE_HAL_LIST_BLOCK_ITEMS_SIZE);
		__hal_wrapper_htonll(umq_blk_priv->wrapper);
	}

	if (is_last) {
		umq_blk_priv->wrapper->control_0 = 0;
	}

	__hal_list_block_link(memblock, prev_block, next_block);

	vxge_hal_trace_log_pool(hldev,
	    ((__hal_vpath_handle_t *)umqdata->channel.vph)->vpath->vp_id,
	    "<==  %s:%s:%d  Result: 0", __FILE__, __func__, __LINE__);

	return (VXGE_HAL_OK);
}

/*
 * __hal_umsg_mempool_item_free - Free List blocks for message list callback
 * @mempoolh: Handle to memory pool
 * @memblock: Address of this memory block
 * @memblock_index: Index of this memory block
 * @dma_object: dma object for this block
 * @item: Pointer to this item
 * @index: Index of this item in memory block
 * @is_last: If this is last item in the block
 * @userdata: Specific data of user
 *
 * This function is callback passed to __hal_mempool_free to destroy memory
 * pool for message list
 */
static vxge_hal_status_e
__hal_umsg_mempool_item_free(
	vxge_hal_mempool_h mempoolh,
	void *memblock,
	u32 memblock_index,
	vxge_hal_mempool_dma_t *dma_object,
	void *item,
	u32 index,
	u32 is_last,
	void *userdata)
{
	u32 memblock_item_idx;
	__hal_list_block_t *prev_block;
	__hal_list_block_wrapper_t *prev_wrapper;
	__hal_umq_blk_priv_t *umq_blk_priv;
	__hal_umq_t *umqdata = (__hal_umq_t *)userdata;
	__hal_device_t *hldev = (__hal_device_t *)umqdata->channel.devh;

	vxge_assert(umqdata != NULL);

	vxge_hal_trace_log_pool(hldev,
	    ((__hal_vpath_handle_t *)umqdata->channel.vph)->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_pool(hldev,
	    ((__hal_vpath_handle_t *)umqdata->channel.vph)->vpath->vp_id,
	    "mempoolh = 0x"VXGE_OS_STXFMT", memblock = 0x"VXGE_OS_STXFMT", "
	    "memblock_index = %d, dma_object = 0x"VXGE_OS_STXFMT", "
	    "item = 0x"VXGE_OS_STXFMT", index = %d, is_last = %d, "
	    "userdata = 0x"VXGE_OS_STXFMT, (ptr_t)mempoolh,
	    (ptr_t)memblock, memblock_index, (ptr_t)dma_object,
	    (ptr_t)item, index, is_last, (ptr_t)userdata);

	umq_blk_priv = (__hal_umq_blk_priv_t *)__hal_mempool_item_priv(
	    mempoolh, memblock_index, item, &memblock_item_idx);

	vxge_assert(umq_blk_priv != NULL);

	prev_block = __hal_list_block_prev_pointer(memblock);

	vxge_assert(prev_block != NULL);

	prev_wrapper = (__hal_list_block_wrapper_t *)
	    (((char *)prev_block) + VXGE_HAL_LIST_BLOCK_ITEMS_SIZE);

	__hal_wrapper_unlink(prev_wrapper, umq_blk_priv->wrapper);

	__hal_list_block_unlink(memblock);

	vxge_hal_trace_log_pool(hldev,
	    ((__hal_vpath_handle_t *)umqdata->channel.vph)->vpath->vp_id,
	    "<==  %s:%s:%d  Result: 0", __FILE__, __func__, __LINE__);

	return (VXGE_HAL_OK);
}

/*
 * __hal_umq_create - Create up message Queue
 * @vpath_handle: Virtual path handle to which this queue belongs
 *
 * This function creates up message queue with the configuration set
 */
vxge_hal_status_e
__hal_umq_create(
	vxge_hal_vpath_h vpath_handle)
{
	vxge_hal_status_e status;
	__hal_umq_t *umq;
	__hal_device_t *hldev;
	__hal_vpath_handle_t *vp = (__hal_vpath_handle_t *)vpath_handle;
	vxge_hal_mempool_dma_t *dma_object;
	vxge_list_t *list_item;
	u32 i, umq_length;

	/*
	 * Note: at this point we have channel.devh and channel.pdev
	 *	 pre-set only!
	 */

	vxge_assert(vpath_handle != NULL);

	hldev = vp->vpath->hldev;

	vxge_hal_trace_log_umq(hldev, vp->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_umq(hldev, vp->vpath->vp_id,
	    "vpath_handle = 0x"VXGE_OS_STXFMT, (ptr_t)vpath_handle);

	if (vpath_handle == NULL) {
		vxge_hal_err_log_umq(hldev, vp->vpath->vp_id,
		    "<==  %s:%s:%d  Result: %d", __FILE__, __func__,
		    __LINE__, VXGE_HAL_ERR_INVALID_HANDLE);
		return (VXGE_HAL_ERR_INVALID_HANDLE);
	}

	umq_length = vp->vpath->hldev->
	    header.config.vp_config[vp->vpath->vp_id].umq.umq_length;

	umq = (__hal_umq_t *)__hal_channel_allocate(
	    (vxge_hal_device_h)vp->vpath->hldev,
	    vpath_handle,
	    VXGE_HAL_CHANNEL_TYPE_UP_MESSAGE_QUEUE,
	    umq_length,
	    TRUE,
	    FALSE,
	    FALSE,
	    0,
	    NULL);

	if (umq == NULL) {
		vxge_hal_err_log_umq(hldev, vp->vpath->vp_id,
		    "Memory allocation failed == > %s : %d ",
		    __func__, __LINE__);
		vxge_hal_trace_log_umq(hldev, vp->vpath->vp_id,
		    "<==  %s:%s:%d  Result: %d", __FILE__, __func__,
		    __LINE__, VXGE_HAL_ERR_OUT_OF_MEMORY);
		return (VXGE_HAL_ERR_OUT_OF_MEMORY);
	}

	vp->vpath->umqh = (vxge_hal_umq_h)umq;

	umq->stats = &vp->vpath->sw_stats->umq_stats;

	umq->umq_length = umq_length;
	umq->gen_compl = vp->vpath->hldev->
	    header.config.vp_config[vp->vpath->vp_id].umq.gen_compl;
	umq->intr_ctrl = vp->vpath->hldev->
	    header.config.vp_config[vp->vpath->vp_id].umq.intr_ctrl;

#if defined(VXGE_HAL_UMQ_POST)
	vxge_os_spin_lock_init(&umq->channel.post_lock, hldev->pdev);
#elif defined(VXGE_HAL_UMQ_POST_IRQ)
	vxge_os_spin_lock_init_irq(&umq->channel.post_lock, hldev->irqh);
#endif

	vxge_list_init(&umq->free_msg_list);
	vxge_list_init(&umq->alloc_msg_list);

	umq->msgblock_priv_size = sizeof (__hal_umq_blk_priv_t);

	umq->mempool = __hal_mempool_create(
	    (vxge_hal_device_h)vp->vpath->hldev,
	    VXGE_OS_HOST_PAGE_SIZE,
	    VXGE_OS_HOST_PAGE_SIZE,
	    umq->msgblock_priv_size,
	    umq->umq_length,
	    umq->umq_length,
	    __hal_umsg_mempool_item_alloc,
	    __hal_umsg_mempool_item_free,
	    umq);

	if (umq->mempool == NULL) {
		vxge_hal_err_log_umq(hldev, vp->vpath->vp_id,
		    "Memory allocation for umq failed == > %s : %d ",
		    __func__, __LINE__);
		__hal_umq_delete(vpath_handle, umq);
		vxge_hal_trace_log_umq(hldev, vp->vpath->vp_id,
		    "<==  %s:%s:%d  Result: %d", __FILE__, __func__,
		    __LINE__, VXGE_HAL_ERR_OUT_OF_MEMORY);
		return (VXGE_HAL_ERR_OUT_OF_MEMORY);
	}

	status = __hal_channel_initialize(&umq->channel);

	if (status != VXGE_HAL_OK) {
		vxge_hal_err_log_umq(hldev, vp->vpath->vp_id,
		    "umq channel initialization failed %s : %d ",
		    __func__, __LINE__);
		__hal_umq_delete(vpath_handle, umq);
		vxge_hal_trace_log_umq(hldev, vp->vpath->vp_id,
		    "<==  %s:%s:%d  Result: %d",
		    __FILE__, __func__, __LINE__, status);
		return (status);
	}

	umq->channel.post_index = umq->channel.length - 1;

	umq->bytes_processed = 0;

	for (i = 0; i < VXGE_HAL_UMQ_MAX_MESSAGES; i++) {

		list_item = (vxge_list_t *)vxge_os_malloc(
		    vp->vpath->hldev->header.pdev,
		    VXGE_HAL_UMQ_MSG_BUFFER_LENGTH+sizeof (vxge_list_t));

		if (list_item == NULL) {
			vxge_hal_err_log_umq(hldev, vp->vpath->vp_id,
			    "umq message pool memory alloc failed %s : %d ",
			    __func__, __LINE__);
			__hal_umq_delete(vpath_handle, umq);
			vxge_hal_trace_log_umq(hldev, vp->vpath->vp_id,
			    "<==  %s:%s:%d  Result: %d", __FILE__, __func__,
			    __LINE__, VXGE_HAL_ERR_OUT_OF_MEMORY);
			return (VXGE_HAL_ERR_OUT_OF_MEMORY);
		}

		vxge_os_memzero(list_item,
		    VXGE_HAL_UMQ_MSG_BUFFER_LENGTH+sizeof (vxge_list_t));

		vxge_list_insert(list_item, &umq->free_msg_list);
	}

	dma_object = __hal_mempool_memblock_dma(umq->mempool, 0);

	vxge_assert(dma_object != NULL);

	umq->first_block_address = dma_object->addr;

	umq->first_block_length = VXGE_HAL_LIST_BLOCK_ITEMS_SIZE;

	vxge_hal_trace_log_umq(hldev, vp->vpath->vp_id,
	    "<==  %s:%s:%d  Result: 0", __FILE__, __func__, __LINE__);

	return (VXGE_HAL_OK);
}

/*
 * __hal_umq_reset - Resets the dmq
 * @dmqh: dmq to be reset
 *
 * This function resets the umq during vpath reset operation
 */
vxge_hal_status_e
__hal_umq_reset(
	vxge_hal_umq_h umqh)
{
	u32 index;
	vxge_list_t *item, *next;
	vxge_hal_status_e status;
	__hal_device_t *hldev;
	__hal_umq_t *umq = (__hal_umq_t *)umqh;
	__hal_list_block_t *block;
	__hal_umq_blk_priv_t *block_priv = NULL;

	vxge_assert(umq != NULL);

	hldev = (__hal_device_t *)umq->channel.devh;

	vxge_hal_trace_log_umq(hldev,
	    ((__hal_vpath_handle_t *)umq->channel.vph)->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_umq(hldev,
	    ((__hal_vpath_handle_t *)umq->channel.vph)->vpath->vp_id,
	    "umq = 0x"VXGE_OS_STXFMT, (ptr_t)umq);

	vxge_list_for_each_safe(item, next, &umq->alloc_msg_list) {

		vxge_list_remove(item);

		vxge_os_memzero(item,
		    VXGE_HAL_UMQ_MSG_BUFFER_LENGTH+sizeof (vxge_list_t));

		vxge_list_insert(item, &umq->free_msg_list);

	}

	status = __hal_channel_reset(&umq->channel);

	if (status != VXGE_HAL_OK) {

		vxge_hal_err_log_umq(hldev,
		    ((__hal_vpath_handle_t *)umq->channel.vph)->vpath->vp_id,
		    "<==  %s:%s:%d  Result: %d",
		    __FILE__, __func__, __LINE__, status);
		return (status);
	}

	for (index = 0;
	    index < __hal_mempool_item_count(umq->mempool);
	    index++) {

		block = __hal_mempool_item(umq->mempool, index);

		vxge_assert(block != NULL);

		umq->channel.dtr_arr[index].dtr = block;

		block_priv = (__hal_umq_blk_priv_t *)(ptr_t)
		    __hal_list_block_priv_data(block);

		vxge_assert(block_priv != NULL);

		block_priv->compl_offset = 0;

		block_priv->post_offset = VXGE_HAL_LIST_BLOCK_ITEMS_SIZE;
	}

	block_priv->wrapper->control_0 = 0;

	umq->channel.post_index = umq->channel.length - 1;

	umq->bytes_processed = 0;

	vxge_hal_trace_log_umq(hldev,
	    ((__hal_vpath_handle_t *)umq->channel.vph)->vpath->vp_id,
	    "<==  %s:%s:%d  Result: 0", __FILE__, __func__, __LINE__);

	return (VXGE_HAL_OK);
}

/*
 * __hal_umq_delete - Removes the Up Message Queue
 * @vpath_handle: Handle to the virtual path
 * @umq: umq to be deleted
 *
 * This function freeup the memory pool and removes the
 * message queue.
 */

vxge_hal_status_e
__hal_umq_delete(
	vxge_hal_vpath_h vpath_handle,
	__hal_umq_t *umq)
{
	__hal_device_t *hldev;
	__hal_vpath_handle_t *vp = (__hal_vpath_handle_t *)vpath_handle;
	vxge_list_t *item, *next;

	vxge_assert((vpath_handle != NULL) && (umq != NULL));

	hldev = vp->vpath->hldev;

	vxge_hal_trace_log_umq(hldev, vp->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_umq(hldev, vp->vpath->vp_id,
	    "vpath_handle = 0x"VXGE_OS_STXFMT", umq = 0x"VXGE_OS_STXFMT,
	    (ptr_t)vpath_handle, (ptr_t)umq);

	if (umq == NULL) {
		vxge_hal_trace_log_umq(hldev, vp->vpath->vp_id,
		    "<==  %s:%s:%d  Result: %d",
		    __FILE__, __func__, __LINE__,
		    VXGE_HAL_ERR_INVALID_HANDLE);
		return (VXGE_HAL_ERR_INVALID_HANDLE);
	}

	vxge_assert(umq != NULL);

	vxge_list_for_each_safe(item, next, &umq->free_msg_list) {
		vxge_list_remove(item);
		vxge_os_free(vp->vpath->hldev->header.pdev,
		    item,
		    VXGE_HAL_UMQ_MSG_BUFFER_LENGTH+sizeof (vxge_list_t));
	}

	vxge_list_for_each_safe(item, next, &umq->alloc_msg_list) {
		vxge_list_remove(item);
		vxge_os_free(vp->vpath->hldev->header.pdev,
		    item,
		    VXGE_HAL_UMQ_MSG_BUFFER_LENGTH+sizeof (vxge_list_t));
	}

	if (umq->mempool) {
		__hal_mempool_destroy(umq->mempool);
	}

	__hal_channel_terminate(&umq->channel);

#if defined(VXGE_HAL_UMQ_POST)
	vxge_os_spin_lock_destroy(&umq->channel.post_lock,
	    vp->vpath->hldev->header.pdev);
#elif defined(VXGE_HAL_UMQ_POST_IRQ)
	vxge_os_spin_lock_destroy_irq(&umq->channel.post_lock,
	    vp->vpath->hldev->header.pdev);
#endif

	__hal_channel_free(&umq->channel);

	vp->vpath->umqh = (vxge_hal_umq_h)NULL;

	vxge_hal_trace_log_umq(hldev, vp->vpath->vp_id,
	    "<==  %s:%s:%d  Result: 0", __FILE__, __func__, __LINE__);

	return (VXGE_HAL_OK);
}


/*
 * __hal_umq_complete - Processing of the messages is complete
 * @umq: up message queue
 * @umsgh: Handle to the message passed in vxge_hal_umq_next_ready_msg_get
 *
 * This function free up the block used to pass the message and return it
 * to free_msg_list.
 */

vxge_hal_status_e
__hal_umq_complete(
	__hal_umq_t *umq,
	vxge_hal_up_msg_h umsgh)
{
	__hal_device_t *hldev;
	vxge_list_t *item;
	u32 msg_length;

	vxge_assert((umq != NULL) && (umsgh != NULL));

	hldev = (__hal_device_t *)umq->channel.devh;

	vxge_hal_trace_log_umq(hldev,
	    ((__hal_vpath_handle_t *)umq->channel.vph)->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_umq(hldev,
	    ((__hal_vpath_handle_t *)umq->channel.vph)->vpath->vp_id,
	    "umq = 0x"VXGE_OS_STXFMT", umsgh = "VXGE_OS_STXFMT,
	    (ptr_t)umq, (ptr_t)umsgh);

#if defined(VXGE_HAL_UMQ_POST)
	vxge_os_spin_lock(&umq->channel.post_lock);
#elif defined(VXGE_HAL_UMQ_POST_IRQ)
	vxge_os_spin_lock_irq(&umq->channel.post_lock, flags);
#endif

	msg_length = VXGE_HAL_UMSG_LENGTH_GET((__hal_up_msg_t *)umsgh);

	if (msg_length <= VXGE_HAL_UMQ_MSG_BUFFER_LENGTH) {

		item = (vxge_list_t *)(((char *)umsgh) - sizeof (vxge_list_t));

		vxge_list_remove(item);

		vxge_os_memzero(item,
		    VXGE_HAL_UMQ_MSG_BUFFER_LENGTH+sizeof (vxge_list_t));

		vxge_list_insert(item, &umq->free_msg_list);

	} else {
		vxge_os_free(umq->channel.pdev, umsgh, msg_length);
	}

#if defined(VXGE_HAL_UMQ_POST)
	vxge_os_spin_unlock(&umq->channel.post_lock);
#elif defined(VXGE_HAL_UMQ_POST_IRQ)
	vxge_os_spin_unlock_irq(&umq->channel.post_lock, flags);
#endif
	vxge_hal_trace_log_umq(hldev,
	    ((__hal_vpath_handle_t *)umq->channel.vph)->vpath->vp_id,
	    "<==  %s:%s:%d  Result: 0", __FILE__, __func__, __LINE__);

	return (VXGE_HAL_OK);

}

/*
 * __hal_umq_ready_msg_check - Check if any up messages are ready
 * @umq: up message queue
 *
 * This function reads IR register to see if any new messages are ready.
 */

vxge_hal_status_e
__hal_umq_ready_msg_check(
	__hal_umq_t *umq)
{
	__hal_device_t *hldev;
	__hal_list_block_t *compl_block;
	__hal_umq_blk_priv_t *compl_priv;
	__hal_virtualpath_t *vpath =
	    ((__hal_vpath_handle_t *)umq->channel.vph)->vpath;
	vxge_hal_status_e status;

	vxge_assert(umq != NULL);

	hldev = (__hal_device_t *)umq->channel.devh;

	vxge_hal_trace_log_umq(hldev,
	    ((__hal_vpath_handle_t *)umq->channel.vph)->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_umq(hldev,
	    ((__hal_vpath_handle_t *)umq->channel.vph)->vpath->vp_id,
	    "umq = 0x"VXGE_OS_STXFMT, (ptr_t)umq);

#if defined(VXGE_HAL_UMQ_POST)
	vxge_os_spin_lock(&umq->channel.post_lock);
#elif defined(VXGE_HAL_UMQ_POST_IRQ)
	vxge_os_spin_lock_irq(&umq->channel.post_lock, flags);
#endif

	compl_block = umq->channel.dtr_arr[umq->channel.compl_index].dtr;
	compl_priv = (__hal_umq_blk_priv_t *)(ptr_t)
	    __hal_list_block_priv_data(compl_block);

	vxge_assert(compl_priv != NULL);

	vxge_hal_info_log_umq(hldev,
	    ((__hal_vpath_handle_t *)umq->channel.vph)->vpath->vp_id,
	    "high_msg_address = %d",
	    (u32)VXGE_HAL_UMQ_DMQ_IR_GET_NO_OF_BYTES_WRITTEN(
	    vpath->umq_dmq_ir->control_0));

	if (umq->bytes_processed != VXGE_HAL_UMQ_DMQ_IR_GET_NO_OF_BYTES_WRITTEN(
	    vpath->umq_dmq_ir->control_0)) {
		status = VXGE_HAL_OK;
	} else {
		status = VXGE_HAL_INF_NO_MORE_COMPLETED_DESCRIPTORS;
	}

#if defined(VXGE_HAL_UMQ_POST)
	vxge_os_spin_unlock(&umq->channel.post_lock);
#elif defined(VXGE_HAL_UMQ_POST_IRQ)
	vxge_os_spin_unlock_irq(&umq->channel.post_lock, flags);
#endif

	vxge_hal_trace_log_umq(hldev,
	    ((__hal_vpath_handle_t *)umq->channel.vph)->vpath->vp_id,
	    "<==  %s:%s:%d  Result: %d",
	    __FILE__, __func__, __LINE__, status);

	return (status);

}

/*
 * __hal_umq_next_ready_msg_get - Get the next ready message
 * @umq: up message queue
 * @umsgh: Pointer in which handle of the next message is returned
 *
 * This function copies the message from DMA able memory to a buffer
 * and returns the buffer to the client.
 */

vxge_hal_status_e
__hal_umq_next_ready_msg_get(
	__hal_umq_t *umq,
	vxge_hal_up_msg_h *umsgh)
{
	__hal_list_block_t *compl_block, *post_block;
	__hal_umq_blk_priv_t *compl_priv, *post_priv;
	__hal_up_msg_t *umsg, *umsg_buf;
	__hal_device_t *hldev;
	__hal_virtualpath_t *vpath =
	    ((__hal_vpath_handle_t *)umq->channel.vph)->vpath;
	u32 msg_length, pad_length = 0, bytes_written;
	vxge_hal_status_e status;
	vxge_list_t *item;
	char *tmp_ptr;

	vxge_assert((umq != NULL) && (umsgh != NULL));

	hldev = (__hal_device_t *)umq->channel.devh;

	vxge_hal_trace_log_umq(hldev, vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_umq(hldev, vpath->vp_id,
	    "umq = 0x"VXGE_OS_STXFMT", umsgh = 0x"VXGE_OS_STXFMT,
	    (ptr_t)umq, (ptr_t)umsgh);

#if defined(VXGE_HAL_UMQ_POST)
	vxge_os_spin_lock(&umq->channel.post_lock);
#elif defined(VXGE_HAL_UMQ_POST_IRQ)
	vxge_os_spin_lock_irq(&umq->channel.post_lock, flags);
#endif

	compl_block = umq->channel.dtr_arr[umq->channel.compl_index].dtr;
	compl_priv = (__hal_umq_blk_priv_t *)(ptr_t)
	    __hal_list_block_priv_data(compl_block);

	vxge_assert(compl_priv != NULL);

	if (umq->bytes_processed == VXGE_HAL_UMQ_DMQ_IR_GET_NO_OF_BYTES_WRITTEN(
	    vpath->umq_dmq_ir->control_0)) {

		status = VXGE_HAL_INF_NO_MORE_COMPLETED_DESCRIPTORS;
	} else {
		vxge_hal_info_log_umq(hldev, vpath->vp_id,
		    "compl_index = %d, block_addr = 0x"VXGE_OS_STXFMT", "
		    "compl_offset = 0x"VXGE_OS_LLXFMT,
		    umq->channel.compl_index, compl_priv->dma_addr,
		    compl_priv->compl_offset);

		if (umq->bytes_processed <=
		    (u32)VXGE_HAL_UMQ_DMQ_IR_GET_NO_OF_BYTES_WRITTEN(
		    vpath->umq_dmq_ir->control_0)) {
			bytes_written =
			    (u32)VXGE_HAL_UMQ_DMQ_IR_GET_NO_OF_BYTES_WRITTEN(
			    vpath->umq_dmq_ir->control_0) -
			    umq->bytes_processed;
		} else {
			bytes_written =
			    ((u32)0xFFFFFFFF) - umq->bytes_processed;
			bytes_written +=
			    (u32)VXGE_HAL_UMQ_DMQ_IR_GET_NO_OF_BYTES_WRITTEN(
			    vpath->umq_dmq_ir->control_0);
		}

		umsg = (__hal_up_msg_t *)(((char *)compl_block) +
		    compl_priv->compl_offset);

		msg_length = VXGE_HAL_UMSG_LENGTH_GET((__hal_up_msg_t *)umsgh);

		vxge_assert(msg_length <= bytes_written);

		if (msg_length <= VXGE_HAL_UMQ_MSG_BUFFER_LENGTH) {

			item = vxge_list_first_get(&umq->free_msg_list);

			if (item == NULL) {

				item = (vxge_list_t *)
				    vxge_os_malloc(umq->channel.pdev,
				    VXGE_HAL_UMQ_MSG_BUFFER_LENGTH +
				    sizeof (vxge_list_t));

				vxge_os_memzero(item,
				    VXGE_HAL_UMQ_MSG_BUFFER_LENGTH +
				    sizeof (vxge_list_t));

			} else {
				vxge_list_remove(item);
			}

			vxge_list_insert(item, &umq->alloc_msg_list);

			umsg_buf = (__hal_up_msg_t *)(((char *)item) +
			    sizeof (vxge_list_t));
		} else {
			umsg_buf = (__hal_up_msg_t *)vxge_os_malloc(
			    umq->channel.pdev,
			    msg_length);

			if (umsg_buf == NULL) {
				status = VXGE_HAL_ERR_OUT_OF_MEMORY;
				goto _exit_now;
			}

			vxge_os_memzero(umsg_buf, msg_length);
		}

#ifndef	VXGE_HAL_TITAN_EMULATION

		if (msg_length % __vxge_os_cacheline_size) {
			pad_length = __vxge_os_cacheline_size -
			    (msg_length % __vxge_os_cacheline_size);
		} else {
			pad_length = 0;
		}

#endif

		tmp_ptr = (char *)umsg_buf;

		for (;;) {
			if (compl_priv->compl_offset + msg_length + pad_length <
			    compl_priv->post_offset) {

				if (msg_length > 0) {
					vxge_os_memcpy(tmp_ptr,
					    umsg, msg_length);
				}

				compl_priv->compl_offset +=
				    (msg_length + pad_length);

				break;
			} else {
				u32 length;

				length = (u32)(compl_priv->post_offset -
				    compl_priv->compl_offset);

				if (length >= msg_length) {

					vxge_os_memcpy(tmp_ptr,
					    umsg, msg_length);

					if (pad_length >
					    (length - msg_length)) {
						pad_length -=
						    (length - msg_length);
					} else {
						pad_length = 0;
					}

					msg_length = 0;

				} else {

					vxge_os_memcpy(tmp_ptr, umsg, length);
					msg_length -= length;
					tmp_ptr = tmp_ptr + length;

				}

				post_block = umq->channel.dtr_arr[
				    umq->channel.post_index].dtr;

				post_priv = (__hal_umq_blk_priv_t *)(ptr_t)
				    __hal_list_block_priv_data(post_block);

				__hal_wrapper_link(post_priv->wrapper,
				    compl_priv->wrapper,
				    compl_priv->dma_addr,
				    VXGE_HAL_LIST_BLOCK_ITEMS_SIZE);

				__hal_wrapper_htonll(post_priv->wrapper);

				compl_priv->compl_offset = 0;

				umq->channel.post_index++;
				if (umq->channel.post_index ==
				    umq->channel.length)
					umq->channel.post_index = 0;

				umq->channel.compl_index++;
				if (umq->channel.compl_index ==
				    umq->channel.length)
					umq->channel.compl_index = 0;

				if ((msg_length == 0) && (pad_length == 0))
					break;

				compl_block = umq->channel.dtr_arr[
				    umq->channel.compl_index].dtr;

				compl_priv = (__hal_umq_blk_priv_t *)(ptr_t)
				    __hal_list_block_priv_data(compl_block);

				umsg = (__hal_up_msg_t *)compl_block;

				vxge_hal_info_log_umq(hldev, vpath->vp_id,
				    "compl_index = %d, "
				    "block_addr = 0x"VXGE_OS_STXFMT", "
				    "compl_offset = "VXGE_OS_LLXFMT,
				    umq->channel.compl_index,
				    compl_priv->dma_addr,
				    compl_priv->compl_offset);

			}

		}

		status = VXGE_HAL_OK;

		*umsgh = (vxge_hal_up_msg_h)umsg_buf;

		if ((((u32)0xFFFFFFFF)-umq->bytes_processed) >=
		    (msg_length+pad_length)) {
			umq->bytes_processed += (msg_length+pad_length);
		} else {
			umq->bytes_processed = (msg_length+pad_length) -
			    (((u32)0xFFFFFFFF)-umq->bytes_processed);
		}

		vxge_hal_info_log_umq(hldev, vpath->vp_id,
		    "compl_index = %d, block_addr = 0x"VXGE_OS_STXFMT",  "
		    "compl_offset = 0x"VXGE_OS_LLXFMT,
		    umq->channel.compl_index, (ptr_t)compl_priv->dma_addr,
		    compl_priv->compl_offset);

	}

_exit_now:

#if defined(VXGE_HAL_UMQ_POST)
	vxge_os_spin_unlock(&umq->channel.post_lock);
#elif defined(VXGE_HAL_UMQ_POST_IRQ)
	vxge_os_spin_unlock_irq(&umq->channel.post_lock, flags);
#endif

	vxge_hal_trace_log_umq(hldev, vpath->vp_id,
	    "<==  %s:%s:%d  Result: %d",
	    __FILE__, __func__, __LINE__, status);

	return (status);

}

/*
 * __hal_umq_is_empty - Is the umq empty
 * @umq: up message queue
 *
 * This function returns VXGE_HAL_OK if the queue is empty.
 */

vxge_hal_status_e
__hal_umq_is_empty(
	__hal_umq_t *umq)
{
	__hal_list_block_t *compl_block;
	__hal_umq_blk_priv_t *compl_priv;
	__hal_device_t *hldev;
	__hal_virtualpath_t *vpath =
	    ((__hal_vpath_handle_t *)umq->channel.vph)->vpath;
	vxge_hal_status_e status;

	vxge_assert(umq != NULL);

	hldev = (__hal_device_t *)umq->channel.devh;

	vxge_hal_trace_log_umq(hldev, vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_umq(hldev, vpath->vp_id,
	    "umq = 0x"VXGE_OS_STXFMT, (ptr_t)umq);

#if defined(VXGE_HAL_UMQ_POST)
	vxge_os_spin_lock(&umq->channel.post_lock);
#elif defined(VXGE_HAL_UMQ_POST_IRQ)
	vxge_os_spin_lock_irq(&umq->channel.post_lock, flags);
#endif

	compl_block = umq->channel.dtr_arr[umq->channel.compl_index].dtr;
	compl_priv = (__hal_umq_blk_priv_t *)(ptr_t)
	    __hal_list_block_priv_data(compl_block);

	vxge_assert(compl_priv != NULL);

	if ((umq->bytes_processed !=
	    VXGE_HAL_UMQ_DMQ_IR_GET_NO_OF_BYTES_WRITTEN(
	    vpath->umq_dmq_ir->control_0)) ||
	    (umq->channel.compl_index != umq->channel.post_index) ||
	    (compl_priv->compl_offset != compl_priv->post_offset))
		status = VXGE_HAL_FAIL;
	else
		status = VXGE_HAL_OK;

#if defined(VXGE_HAL_UMQ_POST)
	vxge_os_spin_unlock(&umq->channel.post_lock);
#elif defined(VXGE_HAL_UMQ_POST_IRQ)
	vxge_os_spin_unlock_irq(&umq->channel.post_lock, flags);
#endif

	vxge_hal_trace_log_umq(hldev, vpath->vp_id,
	    "<==  %s:%s:%d  Result: %d",
	    __FILE__, __func__, __LINE__, status);

	return (status);
}

/*
 * __hal_process_umq - Process UMQ
 * @hldev: Hal device handle
 * @vp_id: Virtual Path Id
 *
 * This function processes messages posted by adapter.
 */

vxge_hal_status_e
__hal_process_umq(__hal_device_t *hldev, u32 vp_id)
{
	vxge_hal_up_msg_h umsgh;
	__hal_umq_t *umq = NULL;
	vxge_hal_opaque_handle_t *op_handle;
	vxge_hal_obj_id_t obj_id;
	__hal_vpath_handle_t *vp;
	vxge_hal_status_e status;
	u32 msg_id;
	u32 msg_type;
	u32 compl_dmq;
	u32 length;
	u32 result;

	vxge_assert(hldev != NULL);

	vxge_hal_trace_log_umq(hldev, vp_id, "==>  %s:%s:%d",
	    __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_umq(hldev, vp_id,
	    "hldev = 0x"VXGE_OS_STXFMT", vp_id = %d", (ptr_t)hldev, vp_id);

	if (hldev->virtual_paths[vp_id].vp_open == VXGE_HAL_VP_NOT_OPEN) {
		vxge_hal_trace_log_umq(hldev, vp_id,
		    "<==  %s:%s:%d  Result: %d", __FILE__, __func__,
		    __LINE__, VXGE_HAL_ERR_VPATH_NOT_OPEN);
		return (VXGE_HAL_ERR_VPATH_NOT_OPEN);
	}

	umq = (__hal_umq_t *)hldev->virtual_paths[vp_id].umqh;

	status = __hal_umq_ready_msg_check(umq);

	if (status != VXGE_HAL_OK) {
		vxge_hal_trace_log_umq(hldev, vp_id, "<==  %s:%s:%d  Result: 0",
		    __FILE__, __func__, __LINE__);

		return (status);
	}

	while (__hal_umq_next_ready_msg_get(umq, &umsgh) == VXGE_HAL_OK) {

		vxge_hal_up_msg_header_get(umsgh, &msg_id, &msg_type,
		    &compl_dmq, &op_handle,
		    &length, &result);

		vp = (__hal_vpath_handle_t *)
		    VXGE_HAL_OPAQUE_HANDLE_GET_VPATH_HANDLE(op_handle);

		obj_id = (vxge_hal_obj_id_t)
		    VXGE_HAL_OPAQUE_HANDLE_GET_HAL_PRIV(op_handle);

		switch (msg_type) {
			case VXGE_HAL_MSG_TYPE_NCE_CREATE_RESP:

				if (result == VXGE_HAL_OK)
					hldev->virtual_paths[vp_id].sw_stats->
					    obj_counts.no_nces++;

				break;

			case VXGE_HAL_MSG_TYPE_NCE_DELETE_RESP:

				if (result == VXGE_HAL_OK) {
					__hal_vp_oid_free(vp->vpath,
					    &vp->vpath->nce_oid_db,
					    (u32)obj_id);
					hldev->virtual_paths[vp_id].sw_stats->
					    obj_counts.no_nces--;
				}

				break;

			case VXGE_HAL_MSG_TYPE_NCE_RCH_TIME_EXCEEDED:
			case VXGE_HAL_MSG_TYPE_NCE_UPDATE_MAC_RESP:
			case VXGE_HAL_MSG_TYPE_NCE_UPDATE_RCH_TIME_RESP:
			case VXGE_HAL_MSG_TYPE_NCE_QUERY_RESP:

				break;

			case VXGE_HAL_MSG_TYPE_CQRQ_CREATE_RESP:

				if (result == VXGE_HAL_OK)
					result = __hal_cqrq_fcqebw(
					    vp, obj_id, op_handle);

				if (result == VXGE_HAL_PENDING) {
					__hal_umq_complete(umq, umsgh);
					vxge_hal_trace_log_umq(hldev,
					    vp_id, "<==  %s:%s:%d  Result: 0",
					    __FILE__, __func__, __LINE__);

					return (VXGE_HAL_OK);
				} else {
					__hal_cqrq_delete(vp,
					    (__hal_cqrq_t *)(ptr_t)obj_id);
				}

				break;

			case VXGE_HAL_MSG_TYPE_CQRQ_DELETE_RESP:

				if (result == VXGE_HAL_OK) {
					__hal_bitmap_cqrq_reset(
					    (__hal_cqrq_t *)(ptr_t)obj_id);
					__hal_cqrq_delete(vp,
					    (__hal_cqrq_t *)(ptr_t)obj_id);
				}

				break;

			case VXGE_HAL_MSG_TYPE_CQRQ_EVENT_NOTIF:
			case VXGE_HAL_MSG_TYPE_CQRQ_MODIFY_RESP:
			case VXGE_HAL_MSG_TYPE_CQRQ_QUERY_RESP:
			case VXGE_HAL_MSG_TYPE_CQRQ_ARM_RESP:
				obj_id = (vxge_hal_obj_id_t)
				    VXGE_HAL_OPAQUE_HANDLE_GET_HAL_PRIV(
				    op_handle);

				break;

			case VXGE_HAL_MSG_TYPE_CQRQ_FIRST_CQE_BW_NOTIF_RESP:

				if (result == VXGE_HAL_OK) {
					__hal_bitmap_cqrq_set(
					    (__hal_cqrq_t *)(ptr_t)obj_id);
				} else {
					__hal_cqrq_delete(vp,
					    (__hal_cqrq_t *)(ptr_t)obj_id);
				}

				vp->cb_fn(vp->client_handle,
				    NULL,
				    VXGE_HAL_MSG_TYPE_CQRQ_CREATE_RESP,
				    obj_id,
				    result,
				    op_handle);

				__hal_umq_complete(umq, umsgh);

				vxge_hal_trace_log_umq(hldev, vp_id,
				    "<==  %s:%s:%d  Result: 0",
				    __FILE__, __func__, __LINE__);

				continue;

			case VXGE_HAL_MSG_TYPE_SRQ_CREATE_RESP:

				if (result == VXGE_HAL_OK) {
					__hal_bitmap_srq_set(
					    (__hal_srq_t *)(ptr_t)obj_id);
				} else {
					__hal_srq_delete(vp,
					    (__hal_srq_t *)(ptr_t)obj_id);
				}

				break;

			case VXGE_HAL_MSG_TYPE_SRQ_DELETE_RESP:

				if (result == VXGE_HAL_OK) {
					__hal_bitmap_srq_reset(
					    (__hal_srq_t *)(ptr_t)obj_id);
					__hal_srq_delete(vp,
					    (__hal_srq_t *)(ptr_t)obj_id);
				}

				break;

			case VXGE_HAL_MSG_TYPE_SRQ_FIRST_WQE_BW_NOTIF_RESP:

				__hal_srq_fwqebw_resp(vp, obj_id, result);

				__hal_umq_complete(umq, umsgh);

				vxge_hal_trace_log_umq(hldev, vp_id,
				    "<==  %s:%s:%d  Result: 0",
				    __FILE__, __func__, __LINE__);

				continue;

			case VXGE_HAL_MSG_TYPE_SRQ_EVENT_NOTIF:
			case VXGE_HAL_MSG_TYPE_SRQ_MODIFY_RESP:
			case VXGE_HAL_MSG_TYPE_SRQ_QUERY_RESP:
			case VXGE_HAL_MSG_TYPE_SRQ_ARM_RESP:
			case VXGE_HAL_MSG_TYPE_SRQ_WQE_BLOCKS_ADDED_NOTIF_RESP:
			case VXGE_HAL_MSG_TYPE_SRQ_RETURN_UNUSED_WQES_RESP:
			case VXGE_HAL_MSG_TYPE_STAG_ALLOCATE_NSMR_RESP:
			case VXGE_HAL_MSG_TYPE_STAG_ALLOCATE_MW_RESP:
			case VXGE_HAL_MSG_TYPE_STAG_DE_ALLOCATE_RESP:
			case VXGE_HAL_MSG_TYPE_STAG_REGISTER_NSMR_RESP:
			case VXGE_HAL_MSG_TYPE_STAG_RE_REGISTER_NSMR_RESP:
			case VXGE_HAL_MSG_TYPE_STAG_REGISTER_SMR_RESP:
			case VXGE_HAL_MSG_TYPE_STAG_QUERY_RESP:
			case VXGE_HAL_MSG_TYPE_STAG_VALID_LOCAL_TAG_RESP:
			case VXGE_HAL_MSG_TYPE_STAG_INVALID_LOCAL_TAG_RESP:
			case VXGE_HAL_MSG_TYPE_STAG_BIND_MW_RESP:
			case VXGE_HAL_MSG_TYPE_STAG_FAST_REGISTER_NSMR_RESP:

				break;

			case VXGE_HAL_MSG_TYPE_TCP_OPEN_RESP:

				if (result == VXGE_HAL_OK) {
					__hal_bitmap_sq_set(
					    (__hal_sq_t *)(ptr_t)obj_id);
					hldev->virtual_paths[vp_id].sw_stats->
					    obj_counts.no_sessions++;
					VXGE_HAL_SQ_STATE_SET(
					    (__hal_sq_t *)(ptr_t)obj_id,
					    VXGEHAL_SQ_STATE_SESS_ESTED);
				} else
					VXGE_HAL_SQ_STATE_SET(
					    (__hal_sq_t *)(ptr_t)obj_id,
					    VXGEHAL_SQ_STATE_INITIALIZED);


				break;

			case VXGE_HAL_MSG_TYPE_TCP_DELETE_RESP:
			case VXGE_HAL_MSG_TYPE_TCP_ABORT_RESP:

				if (result == VXGE_HAL_OK) {
					__hal_bitmap_sq_reset(
					    (__hal_sq_t *)(ptr_t)obj_id);
					hldev->virtual_paths[vp_id].sw_stats->
					    obj_counts.no_sessions--;
					VXGE_HAL_SQ_STATE_SET(
					    (__hal_sq_t *)(ptr_t)obj_id,
					    VXGEHAL_SQ_STATE_SESS_CLOSED);
				} else
					VXGE_HAL_SQ_STATE_SET(
					    (__hal_sq_t *)(ptr_t)obj_id,
					    VXGEHAL_SQ_STATE_SESS_ESTED);

				break;

			case VXGE_HAL_MSG_TYPE_TCP_PROMOTE_TO_IWARP_RESP:
			case VXGE_HAL_MSG_TYPE_TCP_MODIFY_RESP:
			case VXGE_HAL_MSG_TYPE_TCP_RELINK_TO_NCE_RESP:
			case VXGE_HAL_MSG_TYPE_TCP_QP_LIMIT_EXCEEDED:
			case VXGE_HAL_MSG_TYPE_TCP_RDMA_TERMINATE_RECEIVED:
			case VXGE_HAL_MSG_TYPE_TCP_ESTABLISHED:
			case VXGE_HAL_MSG_TYPE_TCP_FIN_RECEIVED:
			case VXGE_HAL_MSG_TYPE_TCP_TIME_WAIT_DONE:
			case VXGE_HAL_MSG_TYPE_TCP_FIN_ACK_RECEIVED:
			case VXGE_HAL_MSG_TYPE_TCP_UPDATE_FAILURE:

				break;

			case VXGE_HAL_MSG_TYPE_LRO_SESSION_CANDIDATE_NOTIF:

				break;

			case VXGE_HAL_MSG_TYPE_LRO_OPEN_RESP:

				if (result == VXGE_HAL_OK) {
					VXGE_HAL_LRO_SESSION_STATE_SET(
					    (__hal_lro_session_t *)
					    (ptr_t)obj_id,
					    VXGEHAL_LRO_SESS_STATE_OPENED);
					hldev->virtual_paths[vp_id].sw_stats->
					    obj_counts.no_sessions++;
				} else {
					VXGE_HAL_LRO_SESSION_STATE_SET(
					    (__hal_lro_session_t *)
					    (ptr_t)obj_id,
					    VVXGEHAL_LRO_SESS_STATE_INITED);
					__hal_lro_session_free(hldev,
					    &hldev->virtual_paths[vp_id],
					    obj_id);
				}

				break;

			case VXGE_HAL_MSG_TYPE_LRO_END_CLASSIF_RESP:

				if (result == VXGE_HAL_OK) {
					VXGE_HAL_LRO_SESSION_STATE_SET(
					    (__hal_lro_session_t *)
					    (ptr_t)obj_id,
					    VXGEHAL_LRO_SESS_STATE_DECLASSED);
				} else {
					VXGE_HAL_LRO_SESSION_STATE_SET(
					    (__hal_lro_session_t *)
					    (ptr_t)obj_id,
					    VXGEHAL_LRO_SESS_STATE_OPENED);
				}

				break;

			case VXGE_HAL_MSG_TYPE_LRO_DELETE_RESP:

				if (result == VXGE_HAL_OK) {
					VXGE_HAL_LRO_SESSION_STATE_SET(
					    (__hal_lro_session_t *)
					    (ptr_t)obj_id,
					    VXGEHAL_LRO_SESS_STATE_CLOSED);
					__hal_lro_session_free(hldev,
					    &hldev->virtual_paths[vp_id],
					    obj_id);
					hldev->virtual_paths[vp_id].sw_stats->
					    obj_counts.no_sessions--;
				} else {
				VXGE_HAL_LRO_SESSION_STATE_SET(
				    (__hal_lro_session_t *)(ptr_t)obj_id,
				    VXGEHAL_LRO_SESS_STATE_DECLASSED);
				}

				break;

			case VXGE_HAL_MSG_TYPE_SPDM_OPEN_RESP:

				if (result == VXGE_HAL_OK)
					hldev->virtual_paths[vp_id].sw_stats->
					    obj_counts.no_sessions++;

				break;

			case VXGE_HAL_MSG_TYPE_SPDM_DELETE_RESP:

				if (result == VXGE_HAL_OK) {
					__hal_vp_oid_free(vp->vpath,
					    &vp->vpath->session_oid_db,
					    (u32)obj_id);
					hldev->virtual_paths[vp_id].sw_stats->
					    obj_counts.no_sessions--;
				}

				break;

			case VXGE_HAL_MSG_TYPE_SESSION_EVENT_NOTIF:
			case VXGE_HAL_MSG_TYPE_SESSION_QUERY_RESP:
			case VXGE_HAL_MSG_TYPE_SESSION_RETURN_IN_PROG_WQES:
			case VXGE_HAL_MSG_TYPE_SESSION_FRAME_WRITE:

				break;

			default:
				vxge_assert(FALSE);
				__hal_umq_complete(umq, umsgh);
				continue;
		}

		vp->cb_fn(vp->client_handle,
		    umsgh,
		    msg_type,
		    obj_id,
		    result,
		    op_handle);

		__hal_umq_complete(umq, umsgh);
	}

	vxge_hal_trace_log_umq(hldev, vp_id, "<==  %s:%s:%d  Result: 0",
	    __FILE__, __func__, __LINE__);

	return (VXGE_HAL_OK);
}

/*
 * __hal_umq_poll - Poll UMQ for message
 * @umq: UMQ
 * @buffer: Buffer to return message
 * @lenght: Lenght of the buffer
 *
 * This function polls a umq for umq messages and copies into the buffer passed
 */

vxge_hal_status_e
__hal_umq_poll(
	__hal_umq_t *umq,
	u8 *buffer,
	u32 length)
{
	vxge_hal_status_e status;
	vxge_hal_up_msg_h umsgh;
	__hal_device_t *hldev;
	vxge_hal_opaque_handle_t *op_handle;
	u32 msg_id;
	u32 msg_type;
	u32 compl_dmq;
	u32 msg_length;
	u32 result;

	vxge_assert((umq != NULL) && (buffer != NULL) && (length != 0));

	hldev = (__hal_device_t *)umq->channel.devh;

	vxge_hal_trace_log_umq(hldev,
	    ((__hal_vpath_handle_t *)umq->channel.vph)->vpath->vp_id,
	    "==>  %s:%s:%d", __FILE__, __func__, __LINE__);

	vxge_hal_trace_log_umq(hldev,
	    ((__hal_vpath_handle_t *)umq->channel.vph)->vpath->vp_id,
	    "umq = 0x"VXGE_OS_STXFMT", buffer = 0x"VXGE_OS_STXFMT", "
	    "length = %d", (ptr_t)umq, (ptr_t)buffer, length);

	if ((status = __hal_umq_ready_msg_check(umq)) == VXGE_HAL_OK) {
		if ((status = __hal_umq_next_ready_msg_get(umq, &umsgh)) ==
		    VXGE_HAL_OK) {

			vxge_hal_up_msg_header_get(umsgh, &msg_id, &msg_type,
			    &compl_dmq, &op_handle,
			    &msg_length, &result);

			if (msg_length <= length) {
				vxge_os_memcpy(buffer, umsgh, msg_length);
			} else {
				status = VXGE_HAL_ERR_OUT_OF_SPACE;
			}

			__hal_umq_complete(umq, umsgh);
		}
	}

	vxge_hal_trace_log_umq(hldev,
	    ((__hal_vpath_handle_t *)umq->channel.vph)->vpath->vp_id,
	    "<==  %s:%s:%d  Result: %d", __FILE__, __func__, __LINE__, status);

	return (status);
}
