/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */
/*
 * Copyright (c) 2002-2010 Neterion, Inc.
 * All right Reserved.
 *
 * FileName :   vxgehal-message.h
 *
 * Description:  HAL Messaging structures
 *
 * Created:       07 Dec 2005
 */

#ifndef	VXGE_HAL_MSG_H
#define	VXGE_HAL_MSG_H

__EXTERN_BEGIN_DECLS

#define	VXGE_HAL_MSG_STAG_IDX_FROM_STAG_ID(sid)		(sid>>8)
#define	VXGE_HAL_MSG_STAG_KEY_FROM_STAG_ID(sid)		(sid&0xff)
#define	VXGE_HAL_MSG_STAG_ID_FROM_IDX_KEY(id, key)		((sid<<8)| \
								(key&0xff))

/* =========================== NCE MESSAGES ============================ */
/*
 * __hal_nce_create_req_dmq_t - The NCE Create Request DMQ
 *
 * @nce_opaque_handle: NCE opaque handle
 * @control_0: Bits 0 to 15 - Reserved
 *	       Bits 16 to 63 - Source Mac Address
 * @control_1: Bits 0 to 15 - Reserved
 *	       Bits 16 to 63 - Remote Mac Address
 * @control_2: Bits 0 to 19 - Reserved
 *	       Bits 20 to 31 - Valn Id
 *	       Bits 32 to 63 - Host Reach Time
 *
 * The NCE Create Request DMQ message is used by the host to create an NCE
 * on the adapter.
 */
typedef  struct  __hal_nce_create_req_dmq_t {
	u64			nce_opaque_handle;

	u64			control_0;
#define	VXGE_HAL_NCE_CREATE_REQ_GET_SRC_MAC(ctrl0)	    bVAL48(ctrl0, 16)
#define	VXGE_HAL_NCE_CREATE_REQ_SRC_MAC(val)		    vBIT(val, 16, 48)

	u64			control_1;
#define	VXGE_HAL_NCE_CREATE_REQ_GET_RMT_MAC(ctrl1)	    bVAL48(ctrl1, 16)
#define	VXGE_HAL_NCE_CREATE_REQ_RMT_MAC(val)		    vBIT(val, 16, 48)

	u64			control_2;
#define	VXGE_HAL_NCE_CREATE_REQ_GET_VLAN_ID(ctrl2)	    bVAL12(ctrl2, 20)
#define	VXGE_HAL_NCE_CREATE_REQ_VLAN_ID(val)		    vBIT(val, 20, 12)

#define	VXGE_HAL_NCE_CREATE_REQ_GET_RCH_TIME(ctrl2)	    bVAL32(ctrl2, 32)
#define	VXGE_HAL_NCE_CREATE_REQ_RCH_TIME(val)		    vBIT(val, 32, 32)

}__hal_nce_create_req_dmq_t;

/*
 * __hal_nce_delete_resp_umq_t - The NCE Delete Response UMQ
 *
 * @nce_opaque_handle: NCE opaque handle
 *
 * The NCE Delete Response UMQ message is received by the host after deleting
 * a NCE on the adapter.
 */
typedef  struct  __hal_nce_delete_resp_umq_t {
	u64			nce_opaque_handle;
}__hal_nce_delete_resp_umq_t;

/*
 * __hal_nce_update_mac_addr_req_dmq_t - The NCE Update MAC Request DMQ
 *
 * @control_0: Bits 0 to 15 - Reserved
 *	       Bits 16 to 63 - Mac Address
 *
 * This message is used by the host to modify the MAC address for an NCE
 * on the adapter.
 */
typedef  struct  __hal_nce_update_mac_addr_req_dmq_t {
	u64			control_0;
#define	VXGE_HAL_NCE_UPDATE_MAC_REQ_GET_MAC_ADDR(ctrl0)	    bVAL48(ctrl0, 16)
#define	VXGE_HAL_NCE_UPDATE_MAC_REQ_MAC_ADDR(val)	    vBIT(val, 16, 48)

}__hal_nce_update_mac_addr_req_dmq_t;

/*
 * __hal_nce_update_rch_time_req_dmq_t - The NCE Update RCH Time Request DMQ
 *
 * @control_0: Bits 0 to 31 - Reserved
 *	       Bits 32 to 63 - Host reachability time. The last time (known to
 *		 KA) that the host confirmed neighbour reachability. This is
 *		 in units of the PE tcp_now clock. When the NCE is offloaded,
 *		 the host sets this to the number of KA tcp_now clock ticks
 *		 since the host last confirmed reachability (the "host
 *		 reachability delta" or relative time instead of absolute
 *		 time). The host performs any necessary conversion from the
 *		 OS clock rate to the KA clock rate. On offload, the KA
 *		 subtracts this from the current tcp_now value and stores the
 *		 absolute reachability time -
 *		 host_rch_time (abs) = tcp_now - host_rch_time (delta)
 *
 * This message is used by the host to modify the host reachability time for an
 * NCE on the adapter.
 */
typedef  struct  __hal_nce_update_rch_time_req_dmq_t {
	u64			control_0;
#define	VXGE_HAL_NCE_UPDATE_RCH_REQ_GET_RCH_TIME(ctrl0)	    bVAL32(ctrl0, 32)
#define	VXGE_HAL_NCE_UPDATE_RCH_REQ_RCH_TIME(val)	    vBIT(val, 32, 32)

}__hal_nce_update_rch_time_req_dmq_t;

/*
 * __hal_nce_query_resp_umq_t - The NCE Query Response UMQ
 *
 * @nce_opaque_handle: NCE opaque handle
 * @control_0: Bits 0 to 15 - Reserved
 *	       Bits 16 to 63 - Source Mac Address
 * @control_1: Bits 0 to 15 - Reserved
 *	       Bits 16 to 63 - Remote Mac Address
 * @control_2: Bits 0 to 31 - Host Reachability Time
 *	       Bits 32 to 63 - Adapter Reachability Time
 * @control_3: Bits 0 to 39 - Reserved
 *	       Bits 40 to 47 - valid
 *	       Bits 48 to 51 - Reserved
 *	       Bits 52 to 63 - Vlan Id
 *
 * The NCE Query Response UMQ message is sent in response to the NCE Query
 * Request DMQ message from the host to indicate the status of the operation.
 */
typedef  struct  __hal_nce_query_resp_umq_t {
	u64			nce_opaque_handle;

	u64			control_0;
#define	VXGE_HAL_NCE_QUERY_RESP_GET_SRC_MAC(ctrl0)	    bVAL48(ctrl0, 16)
#define	VXGE_HAL_NCE_QUERY_RESP_SRC_MAC(val)		    vBIT(val, 16, 48)

	u64			control_1;
#define	VXGE_HAL_NCE_QUERY_RESP_GET_RMT_MAC(ctrl1)	    bVAL48(ctrl1, 16)
#define	VXGE_HAL_NCE_QUERY_RESP_RMT_MAC(val)		    vBIT(val, 16, 48)

	u64			control_2;
#define	VXGE_HAL_NCE_QUERY_RESP_GET_HOST_RCH_TIME(ctrl2)    bVAL32(ctrl2, 0)
#define	VXGE_HAL_NCE_QUERY_RESP_HOST_RCH_TIME(val)	    vBIT(val, 0, 32)

#define	VXGE_HAL_NCE_QUERY_RESP_GET_ADPT_RCH_TIME(ctrl2)    bVAL32(ctrl2, 32)
#define	VXGE_HAL_NCE_QUERY_RESP_ADPT_RCH_TIME(val)	    vBIT(val, 32, 32)

	u64			control_3;
#define	VXGE_HAL_NCE_QUERY_RESP_GET_VALID_FLAG(ctrl3)	    bVAL8(ctrl3, 40)
#define	VXGE_HAL_NCE_QUERY_RESP_VALID_FLAG(val)		    vBIT(val, 40, 8)

#define	VXGE_HAL_NCE_QUERY_RESP_GET_VLAN_ID(ctrl3)	    bVAL12(ctrl3, 52)
#define	VXGE_HAL_NCE_QUERY_RESP_VLAN_ID(val)		    vBIT(val, 52, 12)

}__hal_nce_query_resp_umq_t;

/* ============================= CQRQ MESSAGES ============================== */
/*
 * __hal_cqrq_create_req_dmq_t - The CQRQ Create Request DMQ
 *
 * @cq_opaque_handle: Opaque handle for the use of host
 * @control_0: Bits 0 to 7 - Reserved
 *	       Bits 8 to 15 - This field identifies the interrupt used for this
 *		CQRQ by TIM to signal the writing of a CQE to host memory. CMG
 *		hands this parameter to RPE which forwards it to ROCRC which
 *		relays it to TIM
 *	       Bits 16 to 31 - This field identifies the bitmap position used
 *		by TIM to signal the writing of a CQE to host memory. CMG hands
 *		this parameter to RPE which forwards it to ROCRC which relays it
 *		to TIM.Bitmap_identifier is not necessarily the same as CQRQ ID,
 *		nor is it necessarily unique to a single CQRQ ID.
 *	       Bits 32 to 47 - This field prevents the CQRQ from getting more
 *		than its share of adapter resources. This field sets the maximum
 *		number of adapter CQE group blocks this CQRQ can own at any one
 *		time. This is the maximum number of CQE group blocks this CQRQ
 *		can have on the adapter at any one time.
 *	       Bits 48 to 63 - This field provides a hysteresis upper bound for
 *		automatic adapter CQRQ CQE refill operations. The adapter
 *		attempts to move CQEs from the portion of CQRQ in host memory
 *		to the portion of the CQRQ on the adapter. The adapter CQRQ
 *		attempts to obtain more CQEs from host memory when the previous
 *		attempt did not replenish the number of CQEs in the adapter CQRQ
 *		to the level identified by this field. This field is in terms of
 *		number of CQEs.
 * @control_1: Bits 0 to 15 - This field provides a hysteresis lower bound for
 *		automatic adapter CQRQ CQE refill operations. Once the number of
 *		CQEs in the adapter portion of the CQRQ falls to this level, the
 *		adapter attempts to move CQEs from the host memory portion of
 *		CQRQ to the adapter portion of the CQRQ until occupancy equals
 *		or exceeds the refill_high threshold. This refill operation may
 *		require repeated DMA operations.This field is in terms of number
 *		of CQEs.
 *	       Bits 16 to 31 - The interrupt.low_threshold interrupt and or
 *		umq_message.low_threshold message is optionally generated
 *		whenever the number of CQEs in the adapter portion of the CQRQ
 *		falls below the threshold identified by this field.
 *	       Bits 32 to 39 - Reserved
 *	       Bits 40 to 47 - The event notification mask is a mask used to
 *		determine what events the adapter will send a notification to
 *		the host about. The 8 bit mask looks like
 *		  MSB 0 - EOL UMQ
 *		1 - EOL interrupt
 *		2 - Low Threshold UMQ
 *		3 - Low Threshold interrupt
 *		4 - Drained UMQ
 *		5 - Drained interrupt
 *		6 - SRQ Limit UMQ
 *		LSB 7 - SRQ Limit interrupt
 *	       Bits 48 to 63 - This field identifies the size allocated to each
 *		CQE in terms of  number of bytes. It allows the CMG to count the
 *		number of CQEs in a CQE block
 * @cqrq_ir:This is CQRQ IR (Indication Register) which is the host system
 *		address to which this CQRQ reports the number of returned CQEs.
 * The CQRQ Create Request DMQ message is used by the host to create
 * a CQRQ on the adapter.
 */
typedef  struct  __hal_cqrq_create_req_dmq_t {
	u64			cq_opaque_handle;

	u64			control_0;
#define	VXGE_HAL_CQRQ_CREATE_REQ_GET_INT_NUM(ctrl0)	    bVAL8(ctrl0, 8)
#define	VXGE_HAL_CQRQ_CREATE_REQ_INT_NUM(val)		    vBIT(val, 8, 8)

#define	VXGE_HAL_CQRQ_CREATE_REQ_GET_BITMAP_ID(ctrl0)	    bVAL16(ctrl0, 16)
#define	VXGE_HAL_CQRQ_CREATE_REQ_BITMAP_ID(val)		    vBIT(val, 16, 16)

#define	VXGE_HAL_CQRQ_CREATE_REQ_GET_MAX_CQE_GRPS(ctrl0)    bVAL16(ctrl0, 32)
#define	VXGE_HAL_CQRQ_CREATE_REQ_MAX_CQE_GRPS(val)	    vBIT(val, 32, 16)

#define	VXGE_HAL_CQRQ_CREATE_REQ_GET_REF_TH_HIGH(ctrl0)	    bVAL16(ctrl0, 48)
#define	VXGE_HAL_CQRQ_CREATE_REQ_REF_TH_HIGH(val)	    vBIT(val, 48, 16)

	u64			control_1;
#define	VXGE_HAL_CQRQ_CREATE_REQ_GET_REF_TH_LOW(ctrl1)	    bVAL16(ctrl1, 0)
#define	VXGE_HAL_CQRQ_CREATE_REQ_REF_TH_LOW(val)	    vBIT(val, 0, 16)

#define	VXGE_HAL_CQRQ_CREATE_REQ_GET_LOW_TH_NOTIF(ctrl1)    bVAL16(ctrl1, 16)
#define	VXGE_HAL_CQRQ_CREATE_REQ_LOW_TH_NOTIF(val)	    vBIT(val, 16, 16)

#define	VXGE_HAL_CQRQ_CREATE_REQ_GET_EVENT_MASK(ctrl1)	    bVAL8(ctrl1, 40)
#define	VXGE_HAL_CQRQ_CREATE_REQ_EVENT_MASK(val)	    vBIT(val, 40, 8)
#define	VXGE_HAL_CQRQ_CREATE_REQ_EVENT_MASK_EOL_UMQ	    0x80
#define	VXGE_HAL_CQRQ_CREATE_REQ_EVENT_MASK_EOL_INT	    0x40
#define	VXGE_HAL_CQRQ_CREATE_REQ_EVENT_MASK_LOW_TH_UMQ	    0x20
#define	VXGE_HAL_CQRQ_CREATE_REQ_EVENT_MASK_LOW_TH_INT	    0x10
#define	VXGE_HAL_CQRQ_CREATE_REQ_EVENT_MASK_DRAIN_UMQ	    0x08
#define	VXGE_HAL_CQRQ_CREATE_REQ_EVENT_MASK_DRAIN_INT	    0x04
#define	VXGE_HAL_CQRQ_CREATE_REQ_EVENT_MASK_SRQ_LIM_UMQ	    0x02
#define	VXGE_HAL_CQRQ_CREATE_REQ_EVENT_MASK_SRQ_LIM_INT	    0x01

#define	VXGE_HAL_CQRQ_CREATE_REQ_GET_CQE_PERIOD(ctrl1)	    bVAL16(ctrl1, 48)
#define	VXGE_HAL_CQRQ_CREATE_REQ_CQE_PERIOD(val)	    vBIT(val, 48, 16)

	u64			cqrq_ir;
}__hal_cqrq_create_req_dmq_t;

/*
 * __hal_cqrq_delete_resp_umq_t - The CQRQ Delete Response UMQ
 *
 * @cq_opaque_handle: CQ opaque handle
 *
 * The CQRQ Delete Response UMQ message is received by the host after deleting
 * a CQRQ  on the adapter.
 */
typedef  struct  __hal_cqrq_delete_resp_umq_t {
	u64			cq_opaque_handle;
}__hal_cqrq_delete_resp_umq_t;

/*
 * __hal_cqrq_modify_req_dmq_t - The CQRQ Modify Request DMQ
 *
 * @cq_opaque_handle: Opaque handle for the use of host
 * @control_0: Bits 0 to 15 - This field prevents the CQRQ from getting more
 *		than its share of adapter resources. This field sets the maximum
 *		number of adapter CQE group this CQRQ can own at any one time.
 *		This is the maximum number of CQE group this CQRQ can have on
 *		the adapter at any one time.
 *	       Bits 16 to 31 - This field provides a hysteresis upper bound for
 *		automatic adapter CQE refill operations.The adapter attempts
 *		to move CQEs from the portion of the CQRQ in host memory to the
 *		portion of the CQRQ on the adapter. The adapter CQRQ attempts to
 *		obtain more CQEs from host memory when the previous attempt did
 *		not replenish the number of CQEs in the adapter CQRQ to the
 *		level identified by this field. This field is in terms of number
 *		of CQEs.
 *	       Bits 32 to 47 - This field provides a hysteresis lower bound for
 *		automatic adapter CQRQ CQE refill operations. Once the number of
 *		CQEs in the adapter portion of the CQRQ falls to this level, the
 *		adapter attempts to move CQEs from the host memory portion of
 *		CQRQ to the adapter portion of the CQRQ until occupancy equals
 *		or exceeds the refill_high threshold. This refill operation may
 *		require repeated DMA operations.This field is in terms of number
 *		of CQEs
 *	       Bits 48 to 63 - If enabled, the host is notified(by interrupt or
 *		by UMQ message) whenever the number of CQEs in the adapter
 *		portion of the CQRQ falls below this number.
 * @control_1: Bits 0 to 55 - Reserved
 *	       Bits 56 to 63 - The event notification mask is a mask used to
 *		determine what events the adapter will send a notification to
 *		the host about. The 8 bit mask looks like
 *		  MSB 0 - EOL UMQ
 *		1 - EOL interrupt
 *		2 - Low Threshold UMQ
 *		3 - Low Threshold interrupt
 *		4 - Drained UMQ
 *		5 - Drained interrupt
 *		6 - Reserved
 *		LSB 7 - Reserved
 *
 * The CQRQ Modify Request DMQ message is used by the host to modify fields for
 * an CQRQ on the adapter. The adapter will make the following checks
 * - The CQRQ ID is valid
 * All other checks must be performed by the host software.
 */
typedef  struct  __hal_cqrq_modify_req_dmq_t {
	u64			control_0;
#define	VXGE_HAL_CQRQ_MODIFY_REQ_GET_MAX_CQE_GRPS(ctrl0)    bVAL16(ctrl0, 0)
#define	VXGE_HAL_CQRQ_MODIFY_REQ_MAX_CQE_GRPS(val)	    vBIT(val, 0, 16)

#define	VXGE_HAL_CQRQ_MODIFY_REQ_GET_REF_TH_HIGH(ctrl0)	    bVAL16(ctrl0, 16)
#define	VXGE_HAL_CQRQ_MODIFY_REQ_REF_TH_HIGH(val)	    vBIT(val, 16, 16)

#define	VXGE_HAL_CQRQ_MODIFY_REQ_GET_REF_TH_LOW(ctrl0)	    bVAL16(ctrl0, 32)
#define	VXGE_HAL_CQRQ_MODIFY_REQ_REF_TH_LOW(val)	    vBIT(val, 32, 16)

#define	VXGE_HAL_CQRQ_MODIFY_REQ_GET_LOW_TH_NOTIF(ctrl0)    bVAL16(ctrl0, 48)
#define	VXGE_HAL_CQRQ_MODIFY_REQ_LOW_TH_NOTIF(val)	    vBIT(val, 48, 16)

	u64			control_1;
#define	VXGE_HAL_CQRQ_MODIFY_REQ_GET_EVENT_MASK(ctrl1)	    bVAL8(ctrl1, 56)
#define	VXGE_HAL_CQRQ_MODIFY_REQ_EVENT_MASK(val)	    vBIT(val, 56, 8)
#define	VXGE_HAL_CQRQ_MODIFY_REQ_EVENT_MASK_EOL_UMQ	    0x80
#define	VXGE_HAL_CQRQ_MODIFY_REQ_EVENT_MASK_EOL_INT	    0x40
#define	VXGE_HAL_CQRQ_MODIFY_REQ_EVENT_MASK_LOW_TH_UMQ	    0x20
#define	VXGE_HAL_CQRQ_MODIFY_REQ_EVENT_MASK_LOW_TH_INT	    0x10
#define	VXGE_HAL_CQRQ_MODIFY_REQ_EVENT_MASK_DRAIN_UMQ	    0x08
#define	VXGE_HAL_CQRQ_MODIFY_REQ_EVENT_MASK_DRAIN_INT	    0x04

}__hal_cqrq_modify_req_dmq_t;

/*
 * __hal_cqrq_query_resp_umq_t - The CQRQ Query Response UMQ
 *
 * @cq_opaque_handle: CQRQ opaque handle echoed in response
 * @control_0: Bits 0 to 15 - size allocated to each CQE in terms of  number of
 *		bytes.It allows the CMG to count the number of CQEs in CQE block
 *	       Bits 16 to 31 - This field identifies the bitmap position used
 *		by TIM to signal the writing of a CQE to host memory. CMG hands
 *		this parameter to RPE which forwards it to ROCRC which relays it
 *		to TIM.Bitmap_identifier is not necessarily the same as CQRQ ID,
 *		nor is it necessarily unique to a single CQRQ ID.
 *	       Bits 32 to 47 - This field prevents the CQRQ from getting more
 *		than its share of adapter resources. This field sets the maximum
 *		number of adapter CQE group blocks this CQRQ can own at any one
 *		time. This is the maximum number of CQE group blocks this CQRQ
 *		can have on the adapter at any one time.
 *	       Bits 48 to 63 - This field provides a hysteresis upper bound for
 *		automatic adapter CQRQ CQE refill operations. The adapter
 *		attempts to move CQEs from the CQRQ in host memory
 *		to the portion of the CQRQ on the adapter. The adapter CQRQ
 *		attempts to obtain more CQEs from host memory when the previous
 *		attempt did not replenish the number of CQEs in the adapter CQRQ
 *		to the level identified by this field. This field is in terms of
 *		number of CQEs.
 * @control_1: Bits 0 to 15 - This field provides a hysteresis upper bound for
 *		automatic adapter CQRQ CQE refill operations. Once the number of
 *		CQEs in the adapter portion of the CQRQ falls to this level, the
 *		adapter attempts to move CQEs from the host memory portion of
 *		CQRQ to the adapter portion of the CQRQ until occupancy equals
 *		or exceeds the refill_high threshold. This refill operation may
 *		require repeated DMA operations.This field is in terms of number
 *		of CQEs.
 *	       Bits 16 to 31 - The interrupt.low_threshold interrupt and or
 *		umq_message.low_threshold message is optionally generated
 *		whenever the number of CQEs in the adapter portion of the CQRQ
 *		falls below the threshold identified by this field.
 *	       Bits 32 to 34 - Reserved
 *	       Bits 35 to 39 - The identity of the UMQ through which this CQRQ
 *		communicates messages to the host.All UMQ messages are signalled
 *		directly to MP. MP relays each message to the host system and
 *		co-ordinates upward messages with accompanying interrupts
 *	       Bits 40 to 47 - The event notification mask is a mask used to
 *		determine what events the adapter will send a notification to
 *		the host about. The 8 bit mask looks like
 *		  MSB 0 - EOL UMQ
 *		1 - EOL interrupt
 *		2 - Low Threshold UMQ
 *		3 - Low Threshold interrupt
 *		4 - Drained UMQ
 *		5 - Drained interrupt
 *		6 - SRQ Limit UMQ
 *		LSB 7 - SRQ Limit interrupt
 *	       Bits 48 to 55 - This field identifies the interrupt used for
 *		this CQRQ by TIM to signal the writing of a CQE to host memory.
 *		CMG hands this parameter to RPE which forwards it to ROCRC which
 *		relays it to TIM
 *	       Bits 56 to 58 - Reserved
 *	       Bit 59 - Enable
 *		1'b1 - enable CQRQ operations
 *		1'b0 - disable CQRQ operations
 *		  The CQRQ is normally joined to the free list when disabled.
 *	       Bit 60 - tombstoned
 *		1'b1 - CQRQ CQE refill operations continue but the CQRQ does not
 *		   provide CQEs to sessions. Instead, only the CMG Xtensa
 *		   processor can obtain CQEs from the CQRQ.
 *		  1'b0 - CQRQ tombstone operation is not in effect.
 *		Only one refill operation can be in progress at any time
 *	       Bit 61 - This flag indicates whether this CQRQ is in the
 *		end-of-list condition
 *	       Bit 63 - CQRQ is in the drained condition.
 *	       Bit 63 - Refill
 *		1'b1 - CQRQ CQE refill operation in progress
 *		1'b0 - no CQRQ CQE refill operation in progress
 *		Only one refill operation can be in progress at any time
 * @cqe_block_wrapper_hsa: This field holds the host system address within the
 *		wrapper of the last CQE block DMAed to the adapter.
 * @control_2: Bits 0 to 31 - This field holds the number of bytess (in next
 *		host CQE block) specified by the wrapper of the last CQE block
 *		DMAed to the adapter.
 *	       Bits 32 to 63 - This field identifies the number of CQEs
 *		provided to the CQRQ. The value is mod 2^32 since time equals
 *		zero.
 * @cqrq_ir:This is the host system address to which this CQRQ reports
 *		the number of returned CQEs.
 *
 * The CQRQ Query Response UMQ message is sent in response to the CQRQ Query
 * Request DMQ message from the host to indicate the status of the operation and
 * return any CQRQ properties to the host.
 */
typedef  struct  __hal_cqrq_query_resp_umq_t {
	u64			cq_opaque_handle;

	u64			control_0;
#define	VXGE_HAL_CQRQ_QUERY_RESP_GET_CQE_PERIOD(ctrl0)	    bVAL16(ctrl0, 0)
#define	VXGE_HAL_CQRQ_QUERY_RESP_CQE_PERIOD(val)	    vBIT(val, 0, 16)

#define	VXGE_HAL_CQRQ_QUERY_RESP_GET_BITMAP_ID(ctrl0)	    bVAL16(ctrl0, 16)
#define	VXGE_HAL_CQRQ_QUERY_RESP_BITMAP_ID(val)		    vBIT(val, 16, 16)

#define	VXGE_HAL_CQRQ_QUERY_RESP_GET_MAX_CQE_GRPS(ctrl0)    bVAL16(ctrl0, 32)
#define	VXGE_HAL_CQRQ_QUERY_RESP_MAX_CQE_GRPS(val)	    vBIT(val, 32, 16)

#define	VXGE_HAL_CQRQ_QUERY_RESP_GET_REF_TH_HIGH(ctrl0)	    bVAL16(ctrl0, 48)
#define	VXGE_HAL_CQRQ_QUERY_RESP_REF_TH_HIGH(val)	    vBIT(val, 48, 16)

	u64			control_1;
#define	VXGE_HAL_CQRQ_QUERY_RESP_GET_REF_TH_LOW(ctrl1)	    bVAL16(ctrl1, 0)
#define	VXGE_HAL_CQRQ_QUERY_RESP_REF_TH_LOW(val)	    vBIT(val, 0, 16)

#define	VXGE_HAL_CQRQ_QUERY_RESP_GET_LOW_TH_NOTIF(ctrl1)    bVAL16(ctrl1, 16)
#define	VXGE_HAL_CQRQ_QUERY_RESP_LOW_TH_NOTIF(val)	    vBIT(val, 16, 16)

#define	VXGE_HAL_CQRQ_QUERY_RESP_GET_UMQ(ctrl1)		    bVAL5(ctrl1, 35)
#define	VXGE_HAL_CQRQ_QUERY_RESP_UMQ(val)		    vBIT(val, 35, 5)

#define	VXGE_HAL_CQRQ_QUERY_RESP_GET_EVENT_MASK(ctrl1)	    bVAL8(ctrl1, 40)
#define	VXGE_HAL_CQRQ_QUERY_RESP_EVENT_MASK(val)	    vBIT(val, 40, 8)
#define	VXGE_HAL_CQRQ_QUERY_RESP_EVENT_MASK_EOL_UMQ	    0x80
#define	VXGE_HAL_CQRQ_QUERY_RESP_EVENT_MASK_EOL_INT	    0x40
#define	VXGE_HAL_CQRQ_QUERY_RESP_EVENT_MASK_LOW_TH_UMQ	    0x20
#define	VXGE_HAL_CQRQ_QUERY_RESP_EVENT_MASK_LOW_TH_INT	    0x10
#define	VXGE_HAL_CQRQ_QUERY_RESP_EVENT_MASK_DRAIN_UMQ	    0x08
#define	VXGE_HAL_CQRQ_QUERY_RESP_EVENT_MASK_DRAIN_INT	    0x04
#define	VXGE_HAL_CQRQ_QUERY_RESP_EVENT_MASK_SRQ_LIM_UMQ	    0x02
#define	VXGE_HAL_CQRQ_QUERY_RESP_EVENT_MASK_SRQ_LIM_INT	    0x01

#define	VXGE_HAL_CQRQ_QUERY_RESP_GET_INT_NUM(ctrl1)	    bVAL8(ctrl1, 48)
#define	VXGE_HAL_CQRQ_QUERY_RESP_INT_NUM(val)		    vBIT(val, 48, 8)

#define	VXGE_HAL_CQRQ_QUERY_RESP_GET_ENABLE(ctrl1)	    bVAL1(ctrl1, 59)
#define	VXGE_HAL_CQRQ_QUERY_RESP_ENABLE			    mBIT(59)
#define	VXGE_HAL_CQRQ_QUERY_RESP_ENABLE_ENABLE		    0x1
#define	VXGE_HAL_CQRQ_QUERY_RESP_ENABLE_DISABLE		    0x0

#define	VXGE_HAL_CQRQ_QUERY_RESP_GET_TOMBSTONED(ctrl1)	    bVAL1(ctrl1, 60)
#define	VXGE_HAL_CQRQ_QUERY_RESP_TOMBSTONED		    mBIT(60)
#define	VXGE_HAL_CQRQ_QUERY_RESP_TOMBSTONED_ENABLE	    0x1
#define	VXGE_HAL_CQRQ_QUERY_RESP_TOMBSTONED_DISABLE	    0x0

#define	VXGE_HAL_CQRQ_QUERY_RESP_GET_EOL(ctrl1)		    bVAL1(ctrl1, 61)
#define	VXGE_HAL_CQRQ_QUERY_RESP_EOL			    mBIT(61)

#define	VXGE_HAL_CQRQ_QUERY_RESP_GET_DRAINED(ctrl1)	    bVAL1(ctrl1, 62)
#define	VXGE_HAL_CQRQ_QUERY_RESP_DRAINED		    mBIT(62)

#define	VXGE_HAL_CQRQ_QUERY_RESP_GET_REFILL(ctrl1)	    bVAL1(ctrl1, 63)
#define	VXGE_HAL_CQRQ_QUERY_RESP_REFILL			    mBIT(63)
#define	VXGE_HAL_CQRQ_QUERY_RESP_REFILL_IN_PROGRESS	    0x1
#define	VXGE_HAL_CQRQ_QUERY_RESP_REFILL_NO_OPERATION	    0x0

	u64			cqe_block_wrapper_hsa;

	u64			control_2;
#define	VXGE_HAL_CQRQ_QUERY_RESP_GET_CBW_BYTES(ctrl2)	    bVAL32(ctrl2, 0)
#define	VXGE_HAL_CQRQ_QUERY_RESP_CBW_BYTES(val)		    vBIT(val, 0, 32)

#define	VXGE_HAL_CQRQ_QUERY_RESP_GET_CBW_CQES(ctrl2)	    bVAL32(ctrl2, 32)
#define	VXGE_HAL_CQRQ_QUERY_RESP_CBW_CQES(val)		    vBIT(val, 32, 32)

	u64			cqrq_ir;

}__hal_cqrq_query_resp_umq_t;

/*
 * __hal_cqrq_arm_req_dmq_t - The CQRQ Arm Request DMQ
 *
 * @control_0: Bits 0 to 55 - Reserved
 *	       Bits 56 to 63 - The armed state of the CQRQ.
 *		2'b00 - unarmed.
 *		2'bx1 - armed.
 *		2'b1x -  armed for solicited event.
 *
 * The CQRQ Arm Request DMQ message is used by the host to change the armed
 * state of a CQRQ on the adapter. The armed state determines how the adapter
 * will interrupt the host when RDMA messages arrive.
 */
typedef  struct  __hal_cqrq_arm_req_dmq_t {
	u64			control_0;
#define	VXGE_HAL_CQRQ_ARM_REQ_GET_ARM_TYPE(ctrl0)	    bVAL8(ctrl0, 56)
#define	VXGE_HAL_CQRQ_ARM_REQ_ARM_TYPE(val)		    vBIT(val, 56, 8)
#define	VXGE_HAL_CQRQ_ARM_REQ_ARM_TYPE_UNARMED		    0
#define	VXGE_HAL_CQRQ_ARM_REQ_ARM_TYPE_ARMED		    1
#define	VXGE_HAL_CQRQ_ARM_REQ_ARM_TYPE_ARMED_SE		    2

}__hal_cqrq_arm_req_dmq_t;

/*
 * __hal_cqrq_event_notification_umq_t - The CQRQ Arm Response UMQ
 *
 * @control_0: Bits 0 to 55 - Reserved
 *	       Bits 56 to 63 - The type of the notification
 *		1 = EOL
 *		2 = low threshold
 *		3 = drained
 *
 * The CQRQ Event Notification UMQ message is sent to host when the adapter
 * encounters a problem when DMAing CQEs from host memory. There are three
 * conditions, EOL, Low Threshold, Drained.
 */
typedef  struct  __hal_cqrq_event_notification_umq_t {

	u64			control_0;
#define	VXGE_HAL_CQRQ_EVENT_NOTIF_GET_EVENT_TYPE(ctrl0)	    bVAL8(ctrl0, 56)
#define	VXGE_HAL_CQRQ_EVENT_NOTIF_EVENT_TYPE(val)	    vBIT(val, 56, 8)
#define	VXGE_HAL_CQRQ_EVENT_NOTIF_EVENT_TYPE_EOL	    1
#define	VXGE_HAL_CQRQ_EVENT_NOTIF_EVENT_TYPE_LOW_TH	    2
#define	VXGE_HAL_CQRQ_EVENT_NOTIF_EVENT_TYPE_DRAINED	    3

}__hal_cqrq_event_notification_umq_t;

/*
 * __hal_cqrq_fcqebw_req_dmq_t - The CQRQ First CQE BW Notification Request
 *				DMQ
 *
 * @cqe_block_wrapper_hsa: Address of the first block wrapper in host memory.
 * @control_0: Bits 0 to 31 - Reserved
 *	       Bits 32 to 63 - The number of bytes in the first block wrapper.
 *
 * The CQRQ First CQE BW Notification Request DMQ message is used by the host
 * to notify the adapter after it has configured the first CQE block wrapper(s).
 * It is required to pass the host address and number of bytes of the first CQE
 * block wrapper in host memory.
 */
typedef  struct  __hal_cqrq_fcqebw_req_dmq_t {
	u64			cqe_block_wrapper_hsa;

	u64			control_0;
#define	VXGE_HAL_CQRQ_FCQEBW_GET_LENGTH(ctrl0)		    bVAL32(ctrl0, 32)
#define	VXGE_HAL_CQRQ_FCQEBW_LENGTH(val)		    vBIT(val, 32, 32)

}__hal_cqrq_fcqebw_req_dmq_t;

/* ========================= SRQ MESSAGES ============================= */
/*
 * __hal_srq_create_req_dmq_t - The SRQ Create Request DMQ
 *
 * @srq_opaque_handle: Opaque handle for the use of host
 * @control_0: Bits 0 to 15 - This field represents the maximum number of WQE
 *		Headers/OD Groups that this S-RQ can own at any one time.
 *	       Bits 16 to 31 - This field provides a hysteresis upper bound for
 *		automatic adapter (S-)RQ WQE replenishment/refill operations.
 *		When in refill mode, the adapter moves WQEs from the portion of
 *		the (S-)RQ in host memory to the portion of the (S-)RQ on the
 *		adapter until the number of WQEs on the adapter reaches the
 *		number identified by this field.
 *	       Bits 32 to 47 - This field provides a hysteresis lower bound for
 *		automatic adapter (S-)RQ WQEs replenishment/refill operations.
 *		Once the number of WQEs in the adapter portion of the (S-)RQ
 *		falls to this level, the adapter attempts to move WQEs from the
 *		host memory portion of the (S-)RQ to the adapter portion of the
 *		(S-)RQ until the refill_threshold.high level is met.
 *	       Bits 48 to 63 - If enabled, the host is notified (by interrupt
 *		or by UMQ message) whenever the number of WQEs in the adapter
 *		portion of the SRQ falls below this number.
 * @control_1: Bits 0 to 13 - Reserved
 *	       Bit 13 - Indicates whether the S-RQ is being used for LRO or
 *		Bytestream/iWarp.
 *	       Bit 14 - This slects the mode for the EOL operation of this S-RQ
 *		If set to 1'b1 the mode is poll and WQEs polling will be done
 *		if the EOL is reached. If set to 1'b0 then the mode is "Directed
 *		Reading" where a user doorbell message is used to indicate that
 *		more WQEs have been added to the end of the queue
 *	       Bit 15 - Arm the SRQ Limit
 *		   1 - Arm the SRQ limit
 *		   0 - Disarm the SRQ limit
 *	       Bits 16 to 31 - The event notification mask is a mask used to
 *		determine what events the adapter will send a notification to
 *		the host about. The 8 bit mask looks like
 *		  MSB 0 - 5 - Reserved
 *		   6 - EOL UMQ
 *		 7 - EOL interrupt
 *		 8 - Low Threshold UMQ
 *		 9 - Low Threshold interrupt
 *		10 - Drained UMQ
 *		11 - Drained interrupt
 *		12 - SRQ Limit UMQ
 *		  13 - SRQ Limit interrupt
 *		14 - No WQE OD Groups UMQ
 *		LSB 15 - No WQE OD Groups interrupt
 *	       Bits 32 to 63 - If the number of WQEs in the (S-)RQ (both
 *		on-adapter and on-host) falls below this value, an (S-)RQ Limit
 *		violation is asserted - if armed. Not that the SRQ limit cannot
 *		be armed when creating an SRQ. This must be done later because
 *		the first WQE block wrapper is not available at this point.
 * @control_1: Bits 0 to 39 - Reserved
 *	       Bits 40 to 47 - This field identifies the interrupt used by
 *		this (S-)RQ.
 *	       Bits 48 to 63 - The bitmap field indicating which bit in the
 *		interrupt bitmap is set for event notifications for this SRQ.
 *		To disable the bitmap and use the interrupt_number directly
 *		the bitmap should be set to 0xFFFF.
 * @srq_ir: The SRQ indication register
 *
 * The SRQ Create Request DMQ message is used by the host to create an SRQ on
 * the adapter.
 */
typedef  struct  __hal_srq_create_req_dmq_t {
	u64			srq_opaque_handle;

	u64			control_0;
#define	VXGE_HAL_SRQ_CREATE_REQ_GET_MAX_WQE_OD_GRPS(ctrl0)  bVAL16(ctrl0, 0)
#define	VXGE_HAL_SRQ_CREATE_REQ_MAX_WQE_OD_GRPS(val)	    vBIT(val, 0, 16)

#define	VXGE_HAL_SRQ_CREATE_REQ_GET_REF_TH_HIGH(ctrl0)	    bVAL16(ctrl0, 16)
#define	VXGE_HAL_SRQ_CREATE_REQ_REF_TH_HIGH(val)	    vBIT(val, 16, 16)

#define	VXGE_HAL_SRQ_CREATE_REQ_GET_REF_TH_LOW(ctrl0)	    bVAL16(ctrl0, 32)
#define	VXGE_HAL_SRQ_CREATE_REQ_REF_TH_LOW(val)		    vBIT(val, 32, 16)

#define	VXGE_HAL_SRQ_CREATE_REQ_GET_LOW_TH_NOTIF(ctrl0)	    bVAL16(ctrl0, 48)
#define	VXGE_HAL_SRQ_CREATE_REQ_LOW_TH_NOTIF(val)	    vBIT(val, 48, 16)

	u64			control_1;
#define	VXGE_HAL_SRQ_CREATE_REQ_GET_LRO_BS(ctrl1)	    bVAL1(ctrl1, 13)
#define	VXGE_HAL_SRQ_CREATE_REQ_LRO_BS			    mBIT(13)
#define	VXGE_HAL_SRQ_CREATE_REQ_LRO_BS_LRO		    1
#define	VXGE_HAL_SRQ_CREATE_REQ_LRO_BS_BS		    0

#define	VXGE_HAL_SRQ_CREATE_REQ_GET_POLL_DOORBELL(ctrl1)    bVAL1(ctrl1, 14)
#define	VXGE_HAL_SRQ_CREATE_REQ_POLL_DOORBELL		    mBIT(14)
#define	VXGE_HAL_SRQ_CREATE_REQ_POLL_DOORBELL_POLL	    1
#define	VXGE_HAL_SRQ_CREATE_REQ_POLL_DOORBELL_DOORBELL	    0

#define	VXGE_HAL_SRQ_CREATE_REQ_GET_SRQ_ARM(ctrl1)	    bVAL1(ctrl1, 15)
#define	VXGE_HAL_SRQ_CREATE_REQ_SRQ_ARM			    mBIT(15)
#define	VXGE_HAL_SRQ_CREATE_REQ_SRQ_ARM_ARM		    1
#define	VXGE_HAL_SRQ_CREATE_REQ_SRQ_ARM_DISARM		    0

#define	VXGE_HAL_SRQ_CREATE_REQ_GET_EVENT_MASK(ctrl1)	    bVAL16(ctrl1, 16)
#define	VXGE_HAL_SRQ_CREATE_REQ_EVENT_MASK(val)		    vBIT(val, 16, 16)
#define	VXGE_HAL_SRQ_CREATE_REQ_EVENT_MASK_EOL_UMQ	    0x200
#define	VXGE_HAL_SRQ_CREATE_REQ_EVENT_MASK_EOL_INT	    0x100
#define	VXGE_HAL_SRQ_CREATE_REQ_EVENT_MASK_LOW_TH_UMQ	    0x080
#define	VXGE_HAL_SRQ_CREATE_REQ_EVENT_MASK_LOW_TH_INT	    0x040
#define	VXGE_HAL_SRQ_CREATE_REQ_EVENT_MASK_DRAIN_UMQ	    0x020
#define	VXGE_HAL_SRQ_CREATE_REQ_EVENT_MASK_DRAIN_INT	    0x010
#define	VXGE_HAL_SRQ_CREATE_REQ_EVENT_MASK_SRQ_LIM_UMQ	    0x008
#define	VXGE_HAL_SRQ_CREATE_REQ_EVENT_MASK_SRQ_LIM_INT	    0x004
#define	VXGE_HAL_SRQ_CREATE_REQ_EVENT_MASK_NO_WQE_OD_UMQ    0x002
#define	VXGE_HAL_SRQ_CREATE_REQ_EVENT_MASK_NO_WQE_OD_INT    0x001

#define	VXGE_HAL_SRQ_CREATE_REQ_GET_SRQ_LIMIT(ctrl2)	    bVAL32(ctrl2, 32)
#define	VXGE_HAL_SRQ_CREATE_REQ_SRQ_LIMIT(val)		    vBIT(val, 32, 32)

	u64			control_2;
#define	VXGE_HAL_SRQ_CREATE_REQ_GET_INT_NUM(ctrl2)	    bVAL8(ctrl2, 40)
#define	VXGE_HAL_SRQ_CREATE_REQ_INT_NUM(val)		    vBIT(val, 40, 8)

#define	VXGE_HAL_SRQ_CREATE_REQ_GET_BITMAP_ID(ctrl2)	    bVAL16(ctrl2, 48)
#define	VXGE_HAL_SRQ_CREATE_REQ_BITMAP_ID(val)		    vBIT(val, 48, 16)

	u64			srq_ir;
}__hal_srq_create_req_dmq_t;

/*
 * __hal_srq_delete_resp_umq_t - The SRQ Delete Response UMQ
 *
 * @srq_opaque_handle: SRQ opaque handle
 *
 * The SRQ Delete Response UMQ message is received by the host after deleting
 * a SRQ  on the adapter.
 */
typedef  struct  __hal_srq_delete_resp_umq_t {
	u64			srq_opaque_handle;
}__hal_srq_delete_resp_umq_t;

/*
 * __hal_srq_modify_req_dmq_t - The SRQ Modify Request DMQ
 *
 * @control_0: Bits 0 to 15 - This field represents the maximum number of WQE
 *		Headers/OD Groups that this S-RQ can own at any one time.
 *	       Bits 16 to 31 - This field provides a hysteresis upper bound for
 *		automatic adapter (S-)RQ WQE replenishment/refill operations.
 *		When in refill mode, the adapter moves WQEs from the portion of
 *		the (S-)RQ in host memory to the portion of the (S-)RQ on the
 *		adapter until the number of WQEs on the adapter reaches the
 *		number identified by this field.
 *	       Bits 32 to 47 - This field provides a hysteresis lower bound for
 *		automatic adapter (S-)RQ WQEs replenishment/refill operations.
 *		Once the number of WQEs in the adapter portion of the (S-)RQ
 *		falls to this level, the adapter attempts to move WQEs from the
 *		host memory portion of the (S-)RQ to the adapter portion of the
 *		(S-)RQ until the refill_threshold.high level is met.
 *	       Bits 48 to 63 - If enabled, the host is notified (by interrupt
 *		or by UMQ message) whenever the number of WQEs in the adapter
 *		portion of the SRQ falls below this number.
 * @control_1: Bits 0 to 14 - Reserved
 *	       Bit 15 - Arm the SRQ Limit
 *		   1 - Arm the SRQ limit
 *		   0 - Disarm the SRQ limit
 *	       Bits 16 to 31 - The event notification mask is a mask used to
 *		determine what events the adapter will send a notification to
 *		the host about. The 8 bit mask looks like
 *		  MSB 0 - EOL UMQ
 *		1 - EOL interrupt
 *		2 - Low Threshold UMQ
 *		3 - Low Threshold interrupt
 *		4 - Drained UMQ
 *		5 - Drained interrupt
 *		6 - SRQ Limit UMQ
 *		LSB 7 - SRQ Limit interrupt
 *	       Bits 32 to 63 - If the number of WQEs in the (S-)RQ (both
 *		on-adapter and on-host) falls below this value, an (S-)RQ
 *		Limit violation is asserted - if armed. Not that the SRQ
 *		limit cannot be armed when creating an SRQ. This must be done
 *		later because the first WQE block wrapper is not available at
 *		this point.
 *
 * The SRQ Modify Request DMQ message is used by the host to modify an SRQ on
 * the adapter. The host must send down all the fields to modify. To simplify
 * the adapter firmware there will be no mask to modify individual fields.
 */
typedef  struct  __hal_srq_modify_req_dmq_t {
	u64			control_0;
#define	VXGE_HAL_SRQ_MODIFY_REQ_GET_MAX_WQE_OD_GRPS(ctrl0)  bVAL16(ctrl0, 0)
#define	VXGE_HAL_SRQ_MODIFY_REQ_MAX_WQE_OD_GRPS(val)	    vBIT(val, 0, 16)

#define	VXGE_HAL_SRQ_MODIFY_REQ_GET_REF_TH_HIGH(ctrl0)	    bVAL16(ctrl0, 16)
#define	VXGE_HAL_SRQ_MODIFY_REQ_REF_TH_HIGH(val)	    vBIT(val, 16, 16)

#define	VXGE_HAL_SRQ_MODIFY_REQ_GET_REF_TH_LOW(ctrl0)	    bVAL16(ctrl0, 32)
#define	VXGE_HAL_SRQ_MODIFY_REQ_REF_TH_LOW(val)		    vBIT(val, 32, 16)

#define	VXGE_HAL_SRQ_MODIFY_REQ_GET_LOW_TH_NOTIF(ctrl0)	    bVAL16(ctrl0, 48)
#define	VXGE_HAL_SRQ_MODIFY_REQ_LOW_TH_NOTIF(val)	    vBIT(val, 48, 16)

	u64			control_1;
#define	VXGE_HAL_SRQ_MODIFY_REQ_GET_SRQ_ARM(ctrl1)	    bVAL1(ctrl1, 15)
#define	VXGE_HAL_SRQ_MODIFY_REQ_SRQ_ARM			    mBIT(15)
#define	VXGE_HAL_SRQ_MODIFY_REQ_SRQ_ARM_ARM		    1
#define	VXGE_HAL_SRQ_MODIFY_REQ_SRQ_ARM_DISARM		    0

#define	VXGE_HAL_SRQ_MODIFY_REQ_GET_EVENT_MASK(ctrl1)	    bVAL16(ctrl1, 16)
#define	VXGE_HAL_SRQ_MODIFY_REQ_EVENT_MASK(val)		    vBIT(val, 16, 16)
#define	VXGE_HAL_SRQ_MODIFY_REQ_EVENT_MASK_EOL_UMQ	    0x80
#define	VXGE_HAL_SRQ_MODIFY_REQ_EVENT_MASK_EOL_INT	    0x40
#define	VXGE_HAL_SRQ_MODIFY_REQ_EVENT_MASK_LOW_TH_UMQ	    0x20
#define	VXGE_HAL_SRQ_MODIFY_REQ_EVENT_MASK_LOW_TH_INT	    0x10
#define	VXGE_HAL_SRQ_MODIFY_REQ_EVENT_MASK_DRAIN_UMQ	    0x08
#define	VXGE_HAL_SRQ_MODIFY_REQ_EVENT_MASK_DRAIN_INT	    0x04
#define	VXGE_HAL_SRQ_MODIFY_REQ_EVENT_MASK_SRQ_LIM_UMQ	    0x02
#define	VXGE_HAL_SRQ_MODIFY_REQ_EVENT_MASK_SRQ_LIM_INT	    0x01

#define	VXGE_HAL_SRQ_MODIFY_REQ_GET_SRQ_LIMIT(ctrl1)	    bVAL32(ctrl1, 32)
#define	VXGE_HAL_SRQ_MODIFY_REQ_SRQ_LIMIT(val)		    vBIT(val, 32, 32)

}__hal_srq_modify_req_dmq_t;

/*
 * __hal_srq_query_resp_dmq_t - The SRQ Query Response UMQ
 *
 * @srq_opaque_handle: This field is a host-specified constant allowing the
 *		(S-)RQ to identify itself to the host within any UMQ message
 *		generated on behalf of the (S-)RQ.
 * @control_0: Bits 0 to 7 - Reserved
 *	       Bits 8 to 15 - The identity of the Virtual Path to which this
 *		S-RQ belongs. Also the identity of the UMQ through which this
 *		S-RQ comunicates messages to the host. All messages are sent
 *		indirectly to the MP via the CXP
 *	       Bits 16 to 31 - This field represents the maximum number of WQE
 *		Headers/OD Groups that this S-RQ can own at any one time.
 *	       Bits 32 to 47 - This field provides a hysteresis upper bound for
 *		automatic adapter (S-)RQ WQE replenishment/refill operations.
 *		When in refill mode, the adapter moves WQEs from the portion of
 *		the (S-)RQ in host memory to the portion of the (S-)RQ on the
 *		adapter until the number of WQEs on the adapter reaches the
 *		number identified by this field.
 *	       Bits 48 to 63 - This field provides a hysteresis lower bound for
 *		automatic adapter (S-)RQ WQEs replenishment/refill operations.
 *		Once the number of WQEs in the adapter portion of the (S-)RQ
 *		falls to this level, the adapter attempts to move WQEs from the
 *		host memory portion of the (S-)RQ to the adapter portion of the
 *		(S-)RQ until the refill_threshold.high level is met.
 * @control_1: Bits 0 to 15 - If enabled, the host is notified (by interrupt or
 *		by UMQ message) whenever the number of WQEs in the adapter
 *		portion of the SRQ falls below this number.
 *	       Bits 16 to 31 - The event notification mask is a mask used to
 *		determine what events the adapter will send a notification to
 *		the host about. The 8 bit mask looks like
 *		  MSB 0 - 5 - Reserved
 *		   6 - EOL UMQ
 *		 7 - EOL interrupt
 *		 8 - Low Threshold UMQ
 *		 9 - Low Threshold interrupt
 *		10 - Drained UMQ
 *		11 - Drained interrupt
 *		12 - SRQ Limit UMQ
 *		  13 - SRQ Limit interrupt
 *		14 - No WQE OD Groups UMQ
 *		LSB 15 - No WQE OD Groups interrupt
 *	       Bits 32 to 53 - Reserved
 *	       Bit 54 - This flag indicates whether the (S-)RQ Limit is armed.
 *		It is set via the WQE Doorbell. It is cleared when an (S-)RQ
 *		Limit violation condition occurs.
 *	       Bit 55 - Indicates whether the S-RQ is being used for LRO or
 *		Bytestream/iWarp.
 *	       Bits 56 to 60 - Reserved.
 *	       Bits 61 to 63 - These bits reflect the current state of the
 *		(S-)RQ.
 *		0 - Free: (S-)RQ belongs to free list
 *		1 - Initialize: (S-)RQ has been created but has not been
 *		initialized with WQEs
 *		2 - Normal (in-service): (S-)RQ is in service and operating
 *		normally
 *		3 - Refill (in-service): (S-)RQ is in service but needs to be
 *		refilled with WQEs from the host
 *		4 - End of list (in-service): (S-)RQ is in service, requires
 *		refill, but has reached the end of the on-host WQEs
 *		5 - Drained: S-RQ is drained, that is, there are no WQEs left
 *		on the adapter.
 *		6 - Tombstone: (S-)RQ operations disabled .WQE replenishment
 *		operations continue but the (S-)RQ does not provide WQEs
 *		to sessions. Instead, only the CMG Xtensa processor can
 *		obtain WQEs from the (S-)RQ"
 * @control_2: Bits 0 to 31 - Reserved
 *	       Bits 32 to 63 - If the number of WQEs in the (S-)RQ (both
 *		on-adapter and on-host) falls below this value, an (S-)RQ Limit
 *		violation is asserted - if armed
 * @control_3: Bits 0 to 7 - Reserved
 *	       Bits 8 to 15 - This field identifies the interrupt used by
 *		this (S-)RQ.
 *	       Bits 16 to 31 - The bitmap field indicating which bit in the
 *		interrupt bitmap is set for event notifications for this SRQ.
 *		To disable the bitmap and use the interrupt_number directly
 *		the bitmap should be set to 0xFFFF.
 *	       Bits 32 to 63 - This field holds the number of bytess (in next
 *		host WQE block) specified by the wrapper of the last WQE block
 *		DMAed to the adapter.
 * @wb_host_address: This field holds the host system address within the
 *		wrapper of the last WQE block DMAed to the adapter.
 * @srq_ir: This is the host system address to which this SRQ reports the number
 *		of DMAed WQEs from host memory.
 *
 * The SRQ Query Response UMQ message is sent in response to the SRQ Query
 * Request DMQ message from the host to indicate the status of the operation
 * and return any SRQ properties to the host
 */
typedef  struct  __hal_srq_query_resp_umq_t {
	u64			srq_opaque_handle;

	u64			control_0;
#define	VXGE_HAL_SRQ_QUERY_RESP_GET_VIRTUAL_PATH(ctrl0)	    bVAL8(ctrl0, 8)
#define	VXGE_HAL_SRQ_QUERY_RESP_VIRTUAL_PATH(val)	    vBIT(val, 8, 8)

#define	VXGE_HAL_SRQ_QUERY_RESP_GET_MAX_WQE_OD_GRPS(ctrl0)  bVAL16(ctrl0, 16)
#define	VXGE_HAL_SRQ_QUERY_RESP_MAX_WQE_OD_GRPS(val)	    vBIT(val, 16, 16)

#define	VXGE_HAL_SRQ_QUERY_RESP_GET_REF_TH_HIGH(ctrl0)	    bVAL16(ctrl0, 32)
#define	VXGE_HAL_SRQ_QUERY_RESP_REF_TH_HIGH(val)	    vBIT(val, 32, 16)

#define	VXGE_HAL_SRQ_QUERY_RESP_GET_REF_TH_LOW(ctrl0)	    bVAL16(ctrl0, 48)
#define	VXGE_HAL_SRQ_QUERY_RESP_REF_TH_LOW(val)		    vBIT(val, 48, 16)

	u64			control_1;
#define	VXGE_HAL_SRQ_QUERY_RESP_GET_LOW_TH_NOTIF(ctrl1)	    bVAL16(ctrl1, 0)
#define	VXGE_HAL_SRQ_QUERY_RESP_LOW_TH_NOTIF(val)	    vBIT(val, 0, 16)

#define	VXGE_HAL_SRQ_QUERY_RESP_GET_EVENT_MASK(ctrl1)	    bVAL16(ctrl1, 16)
#define	VXGE_HAL_SRQ_QUERY_RESP_EVENT_MASK(val)		    vBIT(val, 16, 16)
#define	VXGE_HAL_SRQ_QUERY_RESP_EVENT_MASK_EOL_UMQ	    0x200
#define	VXGE_HAL_SRQ_QUERY_RESP_EVENT_MASK_EOL_INT	    0x100
#define	VXGE_HAL_SRQ_QUERY_RESP_EVENT_MASK_LOW_TH_UMQ	    0x080
#define	VXGE_HAL_SRQ_QUERY_RESP_EVENT_MASK_LOW_TH_INT	    0x040
#define	VXGE_HAL_SRQ_QUERY_RESP_EVENT_MASK_DRAIN_UMQ	    0x020
#define	VXGE_HAL_SRQ_QUERY_RESP_EVENT_MASK_DRAIN_INT	    0x010
#define	VXGE_HAL_SRQ_QUERY_RESP_EVENT_MASK_SRQ_LIM_UMQ	    0x008
#define	VXGE_HAL_SRQ_QUERY_RESP_EVENT_MASK_SRQ_LIM_INT	    0x004
#define	VXGE_HAL_SRQ_QUERY_RESP_EVENT_MASK_NO_WQE_OD_UMQ    0x002
#define	VXGE_HAL_SRQ_QUERY_RESP_EVENT_MASK_NO_WQE_OD_INT    0x001

#define	VXGE_HAL_SRQ_QUERY_RESP_GET_SRQ_LIM_ARMED(ctrl1)    bVAL1(ctrl1, 54)
#define	VXGE_HAL_SRQ_QUERY_RESP_SRQ_LIM_ARMED		    mBIT(54)

#define	VXGE_HAL_SRQ_QUERY_RESP_GET_LRO_BS(ctrl1)	    bVAL1(ctrl1, 55)
#define	VXGE_HAL_SRQ_QUERY_RESP_LRO_BS			    mBIT(55)

#define	VXGE_HAL_SRQ_QUERY_RESP_GET_SRQ_STATE(ctrl1)	    bVAL3(ctrl1, 61)
#define	VXGE_HAL_SRQ_QUERY_RESP_SRQ_STATE(val)		    vBIT(val, 61, 3)
#define	VXGE_HAL_SRQ_QUERY_RESP_SRQ_STATE_FREE		    0
#define	VXGE_HAL_SRQ_QUERY_RESP_SRQ_STATE_INITIALIZE	    1
#define	VXGE_HAL_SRQ_QUERY_RESP_SRQ_STATE_NORMAL	    2
#define	VXGE_HAL_SRQ_QUERY_RESP_SRQ_STATE_REFILL	    3
#define	VXGE_HAL_SRQ_QUERY_RESP_SRQ_STATE_EOL		    4
#define	VXGE_HAL_SRQ_QUERY_RESP_SRQ_STATE_DRAINED	    5
#define	VXGE_HAL_SRQ_QUERY_RESP_SRQ_STATE_TOMBSTONED	    6

	u64			control_2;
#define	VXGE_HAL_SRQ_QUERY_RESP_GET_SRQ_LIM_VALUE(ctrl2)    bVAL32(ctrl2, 32)
#define	VXGE_HAL_SRQ_QUERY_RESP_SRQ_LIM_VALUE(val)	    vBIT(val, 32, 32)

	u64			control_3;
#define	VXGE_HAL_SRQ_QUERY_RESP_GET_INT_NUM(ctrl3)	    bVAL8(ctrl3, 8)
#define	VXGE_HAL_SRQ_QUERY_RESP_INT_NUM(val)		    vBIT(val, 8, 8)

#define	VXGE_HAL_SRQ_QUERY_RESP_GET_BITMAP_ID(ctrl3)	    bVAL16(ctrl3, 16)
#define	VXGE_HAL_SRQ_QUERY_RESP_BITMAP_ID(val)		    vBIT(val, 16, 16)

#define	VXGE_HAL_SRQ_QUERY_RESP_GET_WB_BYTES(ctrl3)	    bVAL32(ctrl3, 32)
#define	VXGE_HAL_SRQ_QUERY_RESP_WB_BYTES(val)		    vBIT(val, 32, 32)

	u64			wb_host_address;

	u64			srq_ir;

}__hal_srq_query_resp_umq_t;

/*
 * __hal_srq_arm_req_dmq_t - The SRQ Arm Request DMQ
 *
 * @control_0: Bits 0 to 31 - Reserved
 *	       Bits 32 to 63 - The SRQ limit to set if 'srq_limit_set' is set.
 *
 * The SRQ Arm Request DMQ message is sent to the adapter to arm or re-arm
 * the SRQ limit.
 */
typedef  struct  __hal_srq_arm_req_dmq_t {
	u64			control_0;
#define	VXGE_HAL_SRQ_ARM_REQ_GET_LIMIT(ctrl0)		    bVAL32(ctrl0, 32)
#define	VXGE_HAL_SRQ_ARM_REQ_LIMIT(val)			    vBIT(val, 32, 32)

}__hal_srq_arm_req_dmq_t;

/*
 * __hal_srq_event_notification_umq_t - The SRQ Event Notification UMQ
 *
 * @control_0: Bits 0 to 55 - Reserved
 *	       Bits 56 to 63 - The type of the notification:
 *		1 = EOL
 *		2 = low threshold
 *		3 = drained
 *		4 = SRQ limit
 *
 * The SRQ Event Notification UMQ is used to alert the host that the adapter
 * has encountered one of the following conditions when DMAing WQEs from host
 * memory:
 * EOL (End of list of WQEs in host memory)
 * Low Threshold (The adapter is running low on available WQEs)
 * Drained (Adapter out of WQEs because of EOL condition or adapter use faster
 * than DMA)
 * SRQ Limit (The number of available WQEs on adapter + host less than SRQ
 * limit and the SRQ limit is armed)
 */
typedef  struct  __hal_srq_event_notification_umq_t {
	u64			control_0;
#define	VXGE_HAL_SRQ_EVENT_NOTIF_GET_EVENT_TYPE(ctrl0)	    bVAL8(ctrl0, 56)
#define	VXGE_HAL_SRQ_EVENT_NOTIF_EVENT_TYPE(val)	    vBIT(val, 56, 8)
#define	VXGE_HAL_SRQ_EVENT_NOTIF_EVENT_TYPE_EOL		    1
#define	VXGE_HAL_SRQ_EVENT_NOTIF_EVENT_TYPE_LOW_TH	    2
#define	VXGE_HAL_SRQ_EVENT_NOTIF_EVENT_TYPE_DRAINED	    3
#define	VXGE_HAL_SRQ_EVENT_NOTIF_EVENT_TYPE_SRQ_LIMIT	    4

}__hal_srq_event_notification_umq_t;

/*
 * __hal_srq_fwqebw_req_dmq_t - The SRQ First WQE BW Notification Request DMQ
 * @wqe_block_host_address: The host address of the first WQE block wrapper in
 *		host memory.
 * @control_0: Bits 0 to 31 - The number of bytes in the first WQE block wrapper
 *		in host memory.
 *	       Bits 32 to 63 - The WQE sequence number(WSN) of the tail or last
 *		WQE that was added to the list.
 *
 * The SRQ First WQE BW Notification Request DMQ is used to alert the adapter of
 * the location of the first WQE block wrapper after initially creating the SRQ.
 * It is required because the host cannot pre-post WQEs when creating the SRQ.
 */
typedef  struct  __hal_srq_fwqebw_req_dmq_t {
	u64			wqe_block_host_address;

	u64			control_0;
#define	VXGE_HAL_SRQ_FWQEBW_REQ_GET_WB_BYTES(ctrl0)	    bVAL32(ctrl0, 0)
#define	VXGE_HAL_SRQ_FWQEBW_REQ_WB_BYTES(val)		    vBIT(val, 0, 32)

#define	VXGE_HAL_SRQ_FWQEBW_REQ_GET_ON_TAIL_WSN(ctrl0)	    bVAL32(ctrl0, 32)
#define	VXGE_HAL_SRQ_FWQEBW_REQ_ON_TAIL_WSN(val)	    vBIT(val, 32, 32)

}__hal_srq_fwqebw_req_dmq_t;

/*
 * __hal_srq_wqebadd_req_dmq_t - The SRQ WQE Blocks Added Notification Request
 *				DMQ
 * @control_0: Bits 0 to 31 - Reserved.
 *	       Bits 32 to 63 - The sequence number of the last or tail WQE added
 *	       to the list of WQEs in host memory.
 *
 * The SRQ WQE Blocks Added Notification Request is used to alert the adapter
 * that new WQEs have been posted in host memory. This is required in order for
 * the adapter to support the concept of SRQ limit.
 */
typedef  struct  __hal_srq_wqebadd_req_dmq_t {
	u64			control_0;
#define	VXGE_HAL_SRQ_WQEBADD_REQ_GET_ON_TAIL_WSN(ctrl0)	    bVAL32(ctrl0, 32)
#define	VXGE_HAL_SRQ_WQEBADD_REQ_ON_TAIL_WSN(val)	    vBIT(val, 32, 32)

}__hal_srq_wqebadd_req_dmq_t;

/*
 * __hal_srq_return_unused_wqes_req_dmq_t - The SRQ Return Unused WQEs request
 *					   DMQ
 *
 * @control_0: Bits 0 to 31 - Reserved
 *	       Bits 32 to 63 - The number of WQEs host is reclaiming
 *
 * The SRQ Return unused WQEs request DMQ is used when deleting a TCP session or
 * tearing down an SRQ  to notify the adapter.
 */
typedef  struct  __hal_srq_return_unused_wqes_req_dmq_t {
	u64			control_0;
#define	VXGE_HAL_SRQ_RETURN_WQES_REQ_GET_NUM_WQES(ctrl0)    bVAL32(ctrl0, 32)
#define	VXGE_HAL_SRQ_RETURN_WQES_REQ_NUM_WQES(val)	    vBIT(val, 32, 32)

}__hal_srq_return_unused_wqes_req_dmq_t;

/*
 * __hal_srq_return_unused_wqes_resp_umq_t - The SRQ Return unused WQEs response
 *					   UMQ
 *
 * @srq_opaque_handle: SRQ opaque handle echoed in response
 * @control_0: Bits 0 to 31 - The number of WQE opaque handles in this message
 *	       Bits 32 to 63 - The number of WQE opaque handles left to send
 * @wqe_opaque_handles: Array of the 64 bit opaque handle of the WQE retrieved
 *		from SRQ. Use the 'number_of_wqes_in_msg' to determine how many
 *		of these handle is valid.
 *
 * The SRQ Return WQEs UMQ is used when deleting a TCP session or tearing down
 * an SRQ  to return any WQEs that might still be in play for the SRQ.
 */
typedef  struct  __hal_srq_return_unused_wqes_resp_umq_t {
	u64			srq_opaque_handle;

	u64			control_0;
#define	VXGE_HAL_SRQ_RETURN_WQES_RESP_GET_NUM_IN_MSG(ctrl0) bVAL32(ctrl0, 0)
#define	VXGE_HAL_SRQ_RETURN_WQES_RESP_NUM_IN_MSG(val)	    vBIT(val, 0, 32)

#define	VXGE_HAL_SRQ_RETURN_WQES_RESP_GET_NUM_REMAIN(ctrl0) bVAL32(ctrl0, 32)
#define	VXGE_HAL_SRQ_RETURN_WQES_RESP_NUM_REMAIN(val)	    vBIT(val, 32, 32)

	u64			wqe_opaque_handles[10];

}__hal_srq_return_unused_wqes_resp_umq_t;

/* ========================== STAG MESSAGES ================================ */
/*
 * __hal_stag_allocate_nsmr_req_dmq_t - STAG Allocate NSMR Request DMQ
 *
 * @pd_id: Protection domain to which the new NSMR belongs
 * @control_0: Bits 0 to 31 - The number of PBLEs that the new NSMR must be able
 *		to accommodate.
 *	       Bits 32 to 55 - The STAG Index of the NSMR to be created.
 *	       Bits 56 to 62 - Reserved.
 *	       Bit 63 - Specifies if this NSMR will be allowed to be used
 *		for any remote access operations (read, write, invalidate) when
 *		it is subsequently registered.
 *
 * This downward message commands the adapter to create a new non-shared memory
 * region (NSMR) in the invalid state. This message is used to implement the
 * Allocate Non-Shared Memory Region memory management verb.
 */
typedef  struct  __hal_stag_allocate_nsmr_req_dmq_t {
	u64			pd_id;

	u64			control_0;
#define	VXGE_HAL_STAG_ALLOC_NSMR_REQ_GET_PBL_LEN(ctrl0)	    bVAL32(ctrl0, 0)
#define	VXGE_HAL_STAG_ALLOC_NSMR_REQ_PBL_LEN(val)	    vBIT(val, 0, 32)

#define	VXGE_HAL_STAG_ALLOC_NSMR_REQ_GET_STAG_IDX(ctrl0)    bVAL24(ctrl0, 32)
#define	VXGE_HAL_STAG_ALLOC_NSMR_REQ_STAG_IDX(val)	    vBIT(val, 32, 24)

#define	VXGE_HAL_STAG_ALLOC_NSMR_REQ_GET_RAC_FLAG(ctrl0)    bVAL1(ctrl0, 63)
#define	VXGE_HAL_STAG_ALLOC_NSMR_REQ_RAC_FLAG		    mBIT(63)

}__hal_stag_allocate_nsmr_req_dmq_t;

/*
 * __hal_stag_allocate_nsmr_resp_umq_t - STAG Allocate NSMR Response UMQ
 *
 * @control_0: Bits 0 to 23 - The STag index of the NSMR assigned by the host.
 *	       Bits 24 to 31 - Reserved.
 *	       Bits 32 to 63 - The number of PBLEs that the newly-allocated NSMR
 *	       is capable of accommodating.If the command finished successfully,
 *	       this value will be greater than or equal to the
 *	       Requested-PBL-Length value
 *
 * This upward message communicates to the host the success or failure of the
 * corresponding STAG Allocate NSMR Request DMQ message
 */
typedef  struct  __hal_stag_allocate_nsmr_resp_umq_t {
	u64			control_0;
#define	VXGE_HAL_STAG_ALLOC_NSMR_RESP_GET_STAG_IDX(ctrl0)   bVAL24(ctrl0, 0)
#define	VXGE_HAL_STAG_ALLOC_NSMR_RESP_STAG_IDX(val)	    vBIT(val, 0, 24)

#define	VXGE_HAL_STAG_ALLOC_NSMR_RESP_GET_PBL_LEN(ctrl0)    bVAL32(ctrl0, 0)
#define	VXGE_HAL_STAG_ALLOC_NSMR_RESP_PBL_LEN(val)	    vBIT(val, 32, 32)

}__hal_stag_allocate_nsmr_resp_umq_t;

/*
 * __hal_stag_allocate_mw_req_dmq_t - STAG Allocate MW Request DMQ
 *
 * @control_0: Bits 0 to 23 - The STag index of the NSMR assigned by the host.
 *	       Bits 24 to 63 - Reserved.
 * @pd_id: Protection domain to which the new MW belongs.
 *
 * This downward message commands the adapter to allocate a new memory window
 * (MW). This message is used to implement the Allocate Memory Window memory
 * management verb.
 */
typedef  struct  __hal_stag_allocate_mw_req_dmq_t {
	u64			control_0;
#define	VXGE_HAL_STAG_ALLOC_MW_REQ_GET_STAG_IDX(ctrl0)	    bVAL24(ctrl0, 0)
#define	VXGE_HAL_STAG_ALLOC_MW_REQ_STAG_IDX(val)	    vBIT(val, 0, 24)

	u64			pd_id;

}__hal_stag_allocate_mw_req_dmq_t;

/*
 * __hal_stag_allocate_mw_resp_umq_t - STAG Allocate MW Response UMQ
 *
 * @control_0: Bits 0 to 23 - The STag index of the NSMR assigned by the host.
 *	       Bits 24 to 63 - Reserved.
 *
 * This upward message notifies host the result of allocate mw request
 */
typedef  struct  __hal_stag_allocate_mw_resp_umq_t {
	u64			control_0;
#define	VXGE_HAL_STAG_ALLOC_MW_RESP_GET_STAG_IDX(ctrl0)	    bVAL24(ctrl0, 0)
#define	VXGE_HAL_STAG_ALLOC_MW_RESP_STAG_IDX(val)	    vBIT(val, 0, 24)

}__hal_stag_allocate_mw_resp_umq_t;

/*
 * __hal_stag_deallocate_req_dmq_t - STAG De-allocate Request DMQ
 *
 * @control_0: Bits 0 to 23 - The index of the STag being de-allocated on the
 *		adapter.
 *	       Bits 24 to 62 - Reserved
 *	       Bit 63 - Specifies if the PBL that underlies this STag is also
 *		to be deallocated with the STag.This field is necessary since
 *		multiple STags can refer to a single PBL and (only) the host
 *		does the tracking to determine when that PBL can be deallocated.
 *
 * This downward message commands the adapter to deallocate the specified
 * STag, freeing up any on-adapter resources
 */
typedef  struct  __hal_stag_deallocate_req_dmq_t {
	u64			control_0;
#define	VXGE_HAL_STAG_DEALLOC_REQ_GET_STAG_IDX(ctrl0)	    bVAL24(ctrl0, 0)
#define	VXGE_HAL_STAG_DEALLOC_REQ_STAG_IDX(val)		    vBIT(val, 0, 24)

#define	VXGE_HAL_STAG_DEALLOC_REQ_GET_PBL_FLAG(ctrl0)	    bVAL1(ctrl0, 63)
#define	VXGE_HAL_STAG_DEALLOC_REQ_PBL_FLAG		    mBIT(63)

}__hal_stag_deallocate_req_dmq_t;

/*
 * __hal_stag_deallocate_resp_umq_t - STAG De-allocate Response UMQ
 *
 * @control_0: Bits 0 to 23 - The STag index of the de-allocated stag.
 *	       Bits 24 to 63 - Reserved.
 *
 * This upward message notifies host the result of de-allocate request
 */
typedef  struct  __hal_stag_deallocate_resp_umq_t {
	u64			control_0;
#define	VXGE_HAL_STAG_DEALLOC_RESP_GET_STAG_IDX(ctrl0)	    bVAL24(ctrl0, 0)
#define	VXGE_HAL_STAG_DEALLOC_RESP_STAG_IDX(val)	    vBIT(val, 0, 24)

}__hal_stag_deallocate_resp_umq_t;

/*
 * __hal_stag_register_nsmr_req_dmq_t - STAG Register NSMR Request DMQ
 *
 * @control_0: Bits 0 to 23 - The host assigned STag index to associate with the
 *		NSMR being registered on the adapter..
 *	       Bits 24 to 31 - Instance identifier for this validation of this
 *		NSMR
 *	       Bits 32 to 63 - The size in bytes of each physical buffer in the
 *		PBL.
 * @pd_id: Protection domain to which the new NSMR belongs
 * @first_pbl_block_ptr:Starting address in host memory of the 1st PBL block for
 *		the NSMRs PBL
 * @control_1: Bits 0 to 31 - Number of PBLEs in the 1st PBL block
 *	       Bits 32 to 63 - The total number of PBLEs in the PBL.
 * @control_2: Bits 0 to 24 - Reserved.
 *	       Bit 25 - Specifies if the NSMR will use virtual addressing.
 *	       Bit 26 - Specifies if this NSMR will be allowed to be used for
 *		any remote access operations (read, write, invalidate) on this
 *		and subsequent registrations. Only meaningful if
 *		Fast-Register-Flag is cleared.
 *	       Bit 27 - Specifies if the adapter can write to this NSMR.
 *		Allows this STag to be used in an Rx OD list..
 *	       Bit 28 - Specifies if the adapter can read from this NSMR.
 *		Allows this STag to be used in a Tx OD list..
 *	       Bit 29 - Specifies if remote peers are allowed to write to this
 *		NSMR.Allows this STag to be the sink for RDMA Write or RDMA Read
 *	       Bit 30 - Specifies if remote peers are allowed to read from this
 *		NSMR. Allows this STag to be the source for RDMA Read.
 *	       Bit 31 - Specifies if this NSMR may have MWs bound to it by
 *		subsequent PostSQ.BindMW work requests..
 *	       Bits 32 to 63 - The number of bytes into the first PBLE where
 *		the MR begins.
 * @mr_length: The size in bytes of the NSMR.
 * @base_to:The virtual address of the first byte of the NSMR.Only meaningful if
 *		VA-Base-Flag is set.
 *
 * This downward message commands the adapter to register a non-shared memory
 * region.This message is used to implement Register NSMR memory management verb
 * Fast registration cannot be performed with this DMQ. It can only be done via
 * the PostSQ TOWI.
 */
typedef  struct  __hal_stag_register_nsmr_req_dmq_t {
	u64			control_0;
#define	VXGE_HAL_STAG_REG_NSMR_REQ_GET_STAG_IDX(ctrl0)	    bVAL24(ctrl0, 0)
#define	VXGE_HAL_STAG_REG_NSMR_REQ_STAG_IDX(val)	    vBIT(val, 0, 24)

#define	VXGE_HAL_STAG_REG_NSMR_REQ_GET_STAG_KEY(ctrl0)	    bVAL8(ctrl0, 24)
#define	VXGE_HAL_STAG_REG_NSMR_REQ_STAG_KEY(val)	    vBIT(val, 24, 8)

#define	VXGE_HAL_STAG_REG_NSMR_REQ_GET_PBLE_SIZE(ctrl0)	    bVAL32(ctrl0, 32)
#define	VXGE_HAL_STAG_REG_NSMR_REQ_PBLE_SIZE(val)	    vBIT(val, 32, 32)

	u64			pd_id;

	u64			first_pbl_block_ptr;

	u64			control_1;
#define	VXGE_HAL_STAG_REG_NSMR_REQ_GET_FPBLB_LEN(ctrl1)	    bVAL32(ctrl1, 0)
#define	VXGE_HAL_STAG_REG_NSMR_REQ_FPBLB_LEN(val)	    vBIT(val, 0, 32)

#define	VXGE_HAL_STAG_REG_NSMR_REQ_GET_NUM_PBLS(ctrl1)	    bVAL32(ctrl1, 32)
#define	VXGE_HAL_STAG_REG_NSMR_REQ_NUM_PBLS(val)	    vBIT(val, 32, 32)

	u64			control_2;
#define	VXGE_HAL_STAG_REG_NSMR_REQ_GET_VA_BASE(ctrl2)	    bVAL1(ctrl2, 25)
#define	VXGE_HAL_STAG_REG_NSMR_REQ_VA_BASE		    mBIT(25)

#define	VXGE_HAL_STAG_REG_NSMR_REQ_GET_RAC_FLAG(ctrl2)	    bVAL1(ctrl2, 26)
#define	VXGE_HAL_STAG_REG_NSMR_REQ_RAC_FLAG		    mBIT(26)

#define	VXGE_HAL_STAG_REG_NSMR_REQ_GET_LW_FLAG(ctrl2)	    bVAL1(ctrl2, 27)
#define	VXGE_HAL_STAG_REG_NSMR_REQ_LW_FLAG		    mBIT(27)

#define	VXGE_HAL_STAG_REG_NSMR_REQ_GET_LR_FLAG(ctrl2)	    bVAL1(ctrl2, 28)
#define	VXGE_HAL_STAG_REG_NSMR_REQ_LR_FLAG		    mBIT(28)

#define	VXGE_HAL_STAG_REG_NSMR_REQ_GET_RW_FLAG(ctrl2)	    bVAL1(ctrl2, 29)
#define	VXGE_HAL_STAG_REG_NSMR_REQ_RW_FLAG		    mBIT(29)

#define	VXGE_HAL_STAG_REG_NSMR_REQ_GET_RR_FLAG(ctrl2)	    bVAL1(ctrl2, 30)
#define	VXGE_HAL_STAG_REG_NSMR_REQ_RR_FLAG		    mBIT(30)

#define	VXGE_HAL_STAG_REG_NSMR_REQ_GET_MW_BIND_EN(ctrl2)    bVAL1(ctrl2, 31)
#define	VXGE_HAL_STAG_REG_NSMR_REQ_MW_BIND_EN		    mBIT(31)

#define	VXGE_HAL_STAG_REG_NSMR_REQ_GET_FB_OFFSET(ctrl2)	    bVAL32(ctrl2, 32)
#define	VXGE_HAL_STAG_REG_NSMR_REQ_FB_OFFSET(val)	    vBIT(val, 32, 32)

	u64			mr_length;

	u64			base_to;

}__hal_stag_register_nsmr_req_dmq_t;

/*
 * __hal_stag_register_nsmr_resp_umq_t - STAG Register NSMR Response UMQ
 *
 * @control_0: Bits 0 to 23 - The host assigned STag index of the NSMR that
 *		was registered via the original DMQ. This is returned unmodified
 *		from the DMQ for convenience.
 *	       Bits 24 to 31 - The host assigned STag key of the NSMR that was
 *		registered via the original DMQ.This is returned unmodified from
 *		the DMQ for convenience.
 *	       Bits 32 to 63 - The number of PBLEs that the newly-allocated
 *		NSMR is capable of accommodating. If this command finished
 *		successfully, this value will be greater than or equal to the
 *		PBL-Length value .
 *
 * This upward message communicates to the host the success or failure of the
 * corresponding STAG Register NSMR Request DMQ message.
 */
typedef  struct  __hal_stag_register_nsmr_resp_umq_t {
	u64			control_0;
#define	VXGE_HAL_STAG_REG_NSMR_RESP_GET_STAG_IDX(ctrl0)	    bVAL24(ctrl0, 0)
#define	VXGE_HAL_STAG_REG_NSMR_RESP_STAG_IDX(val)	    vBIT(val, 0, 24)

#define	VXGE_HAL_STAG_REG_NSMR_RESP_GET_STAG_KEY(ctrl0)	    bVAL8(ctrl0, 24)
#define	VXGE_HAL_STAG_REG_NSMR_RESP_STAG_KEY(val)	    vBIT(val, 24, 8)

#define	VXGE_HAL_STAG_REG_NSMR_RESP_GET_PBL_LEN(ctrl0)	    bVAL32(ctrl0, 32)
#define	VXGE_HAL_STAG_REG_NSMR_RESP_PBL_LEN(val)	    vBIT(val, 32, 32)

}__hal_stag_register_nsmr_resp_umq_t;

/*
 * __hal_stag_re_register_nsmr_req_dmq_t - STAG Re-Register NSMR Request DMQ
 *
 * @control_0: Bits 0 to 23 - The STag index of the original NSMR that is being
 *		re-registered. The adapter will use the same STag index for the
 *		re-registered NSMR.
 *	       Bits 24 to 31 - Instance identifier for this validation of this
 *		NSMR
 *	       Bits 32 to 63 - The size in bytes of each physical buffer in the
 *		PBL.
 * @pd_id: Protection domain to which the new NSMR belongs
 * @first_pbl_block_ptr:Starting address in host memory of the 1st PBL block for
 *		the NSMRs PBL
 * @control_1: Bits 0 to 31 - Number of PBLEs in the 1st PBL block
 *	       Bits 32 to 63 - The total number of PBLEs in the PBL.
 * @control_2: Bits 0 to 24 - Reserved.
 *	       Bit 25 - Specifies if the NSMR will use virtual addressing.
 *	       Bit 26 - Specifies if this NSMR will be allowed to be used for
 *		any remote access operations (read, write, invalidate) on this
 *		and subsequent registrations. Only meaningful if
 *		Fast-Register-Flag is cleared.
 *	       Bit 27 - Specifies if the adapter can write to this NSMR.
 *		Allows this STag to be used in an Rx OD list..
 *	       Bit 28 - Specifies if the adapter can read from this NSMR.
 *		Allows this STag to be used in a Tx OD list..
 *	       Bit 29 - Specifies if remote peers are allowed to write to this
 *		NSMR.Allows this STag to be the sink for RDMA Write or RDMA Read
 *	       Bit 30 - Specifies if remote peers are allowed to read from this
 *		NSMR. Allows this STag to be the source for RDMA Read.
 *	       Bit 31 - Specifies if this NSMR may have MWs bound to it by
 *		subsequent PostSQ.BindMW work requests..
 *	       Bits 32 to 63 - The number of bytes into the first PBLE where
 *		the MR begins.
 * @mr_length: The size in bytes of the NSMR.
 * @base_to:The virtual address of the first byte of the NSMR.Only meaningful if
 *		VA-Base-Flag is set.
 *
 * This downward message commands the adapter to change the memory registration
 * of an existing NSMR to create a new NSMR in the valid state.
 * This message is used to implement the Reregister Non-Shared Memory Region
 * memory management verb.
 */
typedef  struct  __hal_stag_re_register_nsmr_req_dmq_t {
	u64			control_0;
#define	VXGE_HAL_STAG_RREG_NSMR_REQ_GET_STAG_IDX(ctrl0)	    bVAL24(ctrl0, 0)
#define	VXGE_HAL_STAG_RREG_NSMR_REQ_STAG_IDX(val)	    vBIT(val, 0, 24)

#define	VXGE_HAL_STAG_RREG_NSMR_REQ_GET_STAG_KEY(ctrl0)	    bVAL8(ctrl0, 24)
#define	VXGE_HAL_STAG_RREG_NSMR_REQ_STAG_KEY(val)	    vBIT(val, 24, 8)

#define	VXGE_HAL_STAG_RREG_NSMR_REQ_GET_PBLE_SIZE(ctrl0)    bVAL32(ctrl0, 32)
#define	VXGE_HAL_STAG_RREG_NSMR_REQ_PBLE_SIZE(val)	    vBIT(val, 32, 32)

	u64			pd_id;

	u64			first_pbl_block_ptr;

	u64			control_1;
#define	VXGE_HAL_STAG_RREG_NSMR_REQ_GET_FPBLB_LEN(ctrl1)    bVAL32(ctrl1, 0)
#define	VXGE_HAL_STAG_RREG_NSMR_REQ_FPBLB_LEN(val)	    vBIT(val, 0, 32)

#define	VXGE_HAL_STAG_RREG_NSMR_REQ_GET_NUM_PBLS(ctrl1)	    bVAL32(ctrl1, 32)
#define	VXGE_HAL_STAG_RREG_NSMR_REQ_NUM_PBLS(val)	    vBIT(val, 32, 32)

	u64			control_2;
#define	VXGE_HAL_STAG_RREG_NSMR_REQ_GET_VA_BASE(ctrl2)	    bVAL1(ctrl2, 25)
#define	VXGE_HAL_STAG_RREG_NSMR_REQ_VA_BASE		    mBIT(25)

#define	VXGE_HAL_STAG_RREG_NSMR_REQ_GET_RAC_FLAG(ctrl2)	    bVAL1(ctrl2, 26)
#define	VXGE_HAL_STAG_RREG_NSMR_REQ_RAC_FLAG		    mBIT(26)

#define	VXGE_HAL_STAG_RREG_NSMR_REQ_GET_LW_FLAG(ctrl2)	    bVAL1(ctrl2, 27)
#define	VXGE_HAL_STAG_RREG_NSMR_REQ_LW_FLAG		    mBIT(27)

#define	VXGE_HAL_STAG_RREG_NSMR_REQ_GET_LR_FLAG(ctrl2)	    bVAL1(ctrl2, 28)
#define	VXGE_HAL_STAG_RREG_NSMR_REQ_LR_FLAG		    mBIT(28)

#define	VXGE_HAL_STAG_RREG_NSMR_REQ_GET_RW_FLAG(ctrl2)	    bVAL1(ctrl2, 29)
#define	VXGE_HAL_STAG_RREG_NSMR_REQ_RW_FLAG		    mBIT(29)

#define	VXGE_HAL_STAG_RREG_NSMR_REQ_GET_RR_FLAG(ctrl2)	    bVAL1(ctrl2, 30)
#define	VXGE_HAL_STAG_RREG_NSMR_REQ_RR_FLAG		    mBIT(30)

#define	VXGE_HAL_STAG_RREG_NSMR_REQ_GET_MW_BIND(ctrl2)	    bVAL1(ctrl2, 31)
#define	VXGE_HAL_STAG_RREG_NSMR_REQ_MW_BIND		    mBIT(31)

#define	VXGE_HAL_STAG_RREG_NSMR_REQ_GET_FB_OFFSET(ctrl2)    bVAL32(ctrl2, 32)
#define	VXGE_HAL_STAG_RREG_NSMR_REQ_FB_OFFSET(val)	    vBIT(val, 32, 32)

	u64			mr_length;

	u64			base_to;

}__hal_stag_re_register_nsmr_req_dmq_t;

/*
 * __hal_stag_re_register_nsmr_resp_umq_t - STAG Re-Register NSMR Response UMQ
 *
 * @control_0: Bits 0 to 23 - The host assigned STag index of the NSMR that
 *		was registered via the original DMQ. This is returned unmodified
 *		from the DMQ for convenience.
 *	       Bits 24 to 31 - The host assigned STag key of the NSMR that was
 *		registered via the original DMQ.This is returned unmodified from
 *		the DMQ for convenience.
 *	       Bits 32 to 63 - The number of PBLEs that the new NSMR is capable
 *		of accommodating. If this command finished successfully, this
 *		value will be greater than or equal to the PBL-Length value.
 *
 * This upward message communicates to the host the success or
 * failure of the corresponding STAG Re-register NSMR Request DMQ message.
 */
typedef  struct  __hal_stag_re_register_nsmr_resp_umq_t {
	u64			control_0;
#define	VXGE_HAL_STAG_RREG_NSMR_RESP_GET_STAG_IDX(ctrl0)    bVAL24(ctrl0, 0)
#define	VXGE_HAL_STAG_RREG_NSMR_RESP_STAG_IDX(val)	    vBIT(val, 0, 24)

#define	VXGE_HAL_STAG_RREG_NSMR_RESP_GET_STAG_KEY(ctrl0)    bVAL8(ctrl0, 24)
#define	VXGE_HAL_STAG_RREG_NSMR_RESP_STAG_KEY(val)	    vBIT(val, 24, 8)

#define	VXGE_HAL_STAG_RREG_NSMR_RESP_GET_PBL_LEN(ctrl0)	    bVAL32(ctrl0, 32)
#define	VXGE_HAL_STAG_RREG_NSMR_RESP_PBL_LEN(val)	    vBIT(val, 32, 32)
}__hal_stag_re_register_nsmr_resp_umq_t;

/*
 * __hal_stag_register_smr_req_dmq_t - STAG Register SMR Request DMQ
 *
 * @control_0: Bits 0 to 23 - Unique identifier for the underlying MR on which
 *		this SMR will be created.
 *	       Bit 24 - Reserved.
 *	       Bit 25 - Specifies if the new SMR will use virtual addressing.
 *	       Bit 26 - Specifies if this SMR will be allowed to be used for
 *		any remote access operations (read, write, invalidate).
 *	       Bit 27 - Specifies if the adapter can write to this SMR.
 *		Allows this STag to be used in an Rx OD list.
 *	       Bit 28 - Specifies if the adapter can read from this SMR.
 *		Allows this STag to be used in a Tx OD list.
 *	       Bit 29 - Specifies if remote peers are allowed to write to this
 *		SMR. Allows this STag to be the sink for RDMA Write or RDMA Read
 *	       Bit 30 - Specifies if remote peers are allowed to read from this
 *		SMR. Allows this STag to be the source for RDMA Read.
 *	       Bit 31 - Specifies if this SMR may have MWs bound to it by
 *		subsequent PostSQ.BindMW work requests.
 *	       Bits 32 to 55 - Unique identifier for the new SMR that is being
 *		created
 *	       Bits 56 to 63 - Instance identifier for the SMR.
 * @pd_id: Protection domain to which the new SMR belongs.
 * @base_to:The virtual address of the first byte of the SMR. Only meaningful if
 *		va_base_flag is set.
 *
 * This downward message commands the adapter to create a shared memory region
 * (SMR) based on an existing memory region  either shared(SMR) or non-shared
 * (NSMR). This message is used to implement the Register Shared Memory Region
 * verb.
 */
typedef  struct  __hal_stag_register_smr_req_dmq_t {
	u64			control_0;
#define	VXGE_HAL_STAG_REG_SMR_REQ_GET_MR_STAG_IDX(ctrl0)    bVAL24(ctrl0, 0)
#define	VXGE_HAL_STAG_REG_SMR_REQ_MR_STAG_IDX(val)	    vBIT(val, 0, 24)

#define	VXGE_HAL_STAG_REG_SMR_REQ_GET_VA_BASE(ctrl0)	    bVAL1(ctrl0, 25)
#define	VXGE_HAL_STAG_REG_SMR_REQ_VA_BASE		    mBIT(25)

#define	VXGE_HAL_STAG_REG_SMR_REQ_GET_RAC_FLAG(ctrl0)	    bVAL1(ctrl0, 26)
#define	VXGE_HAL_STAG_REG_SMR_REQ_RAC_FLAG		    mBIT(26)

#define	VXGE_HAL_STAG_REG_SMR_REQ_GET_LW_FLAG(ctrl0)	    bVAL1(ctrl0, 27)
#define	VXGE_HAL_STAG_REG_SMR_REQ_LW_FLAG		    mBIT(27)

#define	VXGE_HAL_STAG_REG_SMR_REQ_GET_LR_FLAG(ctrl0)	    bVAL1(ctrl0, 28)
#define	VXGE_HAL_STAG_REG_SMR_REQ_LR_FLAG		    mBIT(28)

#define	VXGE_HAL_STAG_REG_SMR_REQ_GET_RW_FLAG(ctrl0)	    bVAL1(ctrl0, 29)
#define	VXGE_HAL_STAG_REG_SMR_REQ_RW_FLAG		    mBIT(29)

#define	VXGE_HAL_STAG_REG_SMR_REQ_GET_RR_FLAG(ctrl0)	    bVAL1(ctrl0, 30)
#define	VXGE_HAL_STAG_REG_SMR_REQ_RR_FLAG		    mBIT(30)

#define	VXGE_HAL_STAG_REG_SMR_REQ_GET_MW_BIND(ctrl0)	    bVAL1(ctrl0, 31)
#define	VXGE_HAL_STAG_REG_SMR_REQ_MW_BIND		    mBIT(31)

#define	VXGE_HAL_STAG_REG_SMR_REQ_GET_STAG_IDX(ctrl0)	    bVAL24(ctrl0, 32)
#define	VXGE_HAL_STAG_REG_SMR_REQ_STAG_IDX(val)		    vBIT(val, 32, 24)

#define	VXGE_HAL_STAG_REG_SMR_REQ_GET_STAG_KEY(ctrl0)	    bVAL8(ctrl0, 56)
#define	VXGE_HAL_STAG_REG_SMR_REQ_STAG_KEY(val)		    vBIT(val, 56, 8)

	u64			pd_id;

	u64			base_to;

}__hal_stag_register_smr_req_dmq_t;

/*
 * __hal_stag_register_smr_resp_umq_t - STAG Register SMR Response UMQ
 *
 * @control_0: Bits 0 to 23 - The host assigned STag index of the SMR that was
 *		registered via the original DMQ. This is returned unmodified
 *		from the DMQ for convenience.
 *	       Bits 24 to 31 - The host assigned STag key of the SMR that was
 *		registered via the original DMQ. This is returned unmodified
 *		from the DMQ for convenience.
 *	       Bits 32 to 63 - Reserved.
 *
 * This upward message communicates to the host the success or
 * failure of the corresponding STAG Register SMR Request DMQ message.
 */
typedef  struct  __hal_stag_register_smr_resp_umq_t {
	u64			control_0;
#define	VXGE_HAL_STAG_REG_SMR_RESP_GET_STAG_IDX(ctrl0)	    bVAL24(ctrl0, 0)
#define	VXGE_HAL_STAG_REG_SMR_RESP_STAG_IDX(val)	    vBIT(val, 0, 24)

#define	VXGE_HAL_STAG_REG_SMR_RESP_GET_STAG_KEY(ctrl0)	    bVAL8(ctrl0, 24)
#define	VXGE_HAL_STAG_REG_SMR_RESP_STAG_KEY(val)	    vBIT(val, 24, 8)

}__hal_stag_register_smr_resp_umq_t;

/*
 * __hal_stag_query_req_dmq_t - STAG Query Request DMQ
 *
 * @control_0: Bits 0 to 23 - The STag index of the existing on-adapter STag
 *		that is being queried.
 *	       Bits 24 to 63 - Reserved.
 *
 * This downward message commands the adapter to return the specifics of the
 * specified STag.
 */
typedef  struct  __hal_stag_query_req_dmq_t {
	u64			control_0;
#define	VXGE_HAL_STAG_QUERY_REQ_GET_STAG_IDX(ctrl1)	    bVAL24(ctrl1, 0)
#define	VXGE_HAL_STAG_QUERY_REQ_STAG_IDX(val)		    vBIT(val, 0, 24)
}__hal_stag_query_req_dmq_t;

/*
 * __hal_stag_query_resp_umq_t - STAG Query Response UMQ
 *
 * @control_0: Bits 0 to 23 - This field uniquely identifies the STag record.
 *	       Bits 24 to 31 - Instance identifier for the STag.
 *		Only meaningful if valid_flag is set.
 *	       Bits 32 to 34 - Reserved.
 *	       Bit 35 - Is this record in use?
 *	       Bit 36 - Is this STag a MR or an MW?
 *	       Bit 37 - Is this STag zero-based
 *	       Bit 38 - Is this STag in the valid state?
 *	       Bit 39 - Is this MR an SMR?
 *	       Bits 40 to 41 - Reserved.
 *	       Bit 42 - Can MWs be bound to this STag?
 *	       Bit 43 - Can this STag be used for remote access (read, write,
 *		invalidate)?
 *	       Bit 44 - Can this STag be used for remote read access?
 *	       Bit 45 - Can this STag be used for remote write access?
 *	       Bit 46 - Can this STag be used for local read access?
 *	       Bit 47 - Can this STag be used for local write access?
 *	       Bits 48 to 63 - Reserved.
 * @pd_id: Protection Domain ID.
 * @control_1: Bits 0 to 23 - Session ID
 *	       Bits 24 to 31 - Session Instance Number
 *	       Bits 32 to 63 - Reserved.
 * @length:length (in bytes) of the MR or MW described by this STag.
 * @mr_base_to: TO of first byte in the MR
 * @mw_base_to:TO of the first byte in the MW relative to the underlying MR.
 * @control_2: Bits 0 to 31 - The offset in bytes into the first physical buffer
 *		where the MR begins.
 *	       Bits 32 to 63 - size in bytes of each underlying physical buffer
 * @control_3: Bits 0 to 23 - Identify of the Underlying PBL
 *	       Bits 24 to 26 - Reserved
 *	       Bits 27 to 31 - Depth of the PBL tree
 *	       Bits 32 to 55 - Identity of the parent STag
 *	       Bits 56 to 60 - Reserved
 *	       Bit 61 - Is there a parent record id?
 *	       Bit 62 - Is there a left subtree for this STag record?
 *	       Bit 63 - Is there a right subtree for this STag record?
 * @control_4: Bits 0 to 23 - Identity of the STag record at the top of the left
 *		subtree
 *	       Bits 24 to 31 - Reserved
 *	       Bits 32 to 55 - Identity of the STag record at the top of the
 *		right subtree
 *	       Bits 56 to 63 - Reserved
 * @control_5: Bits 0 to 23 - The number of MWs bound to this MR
 *	       Bits 24 to 29 - Reserved
 *	       Bits 30 to 31 - "What is the depth imbalance between the left
 *		and right subtrees of this STag?
 *		-1 --- left subtree is one deeper than right subtree
 *		0 --- left and right subtrees are same depth
 *		1 --- right subtree is one deeper than left subtree"
 *	       Bits 32 to 63 - The maximum PBLE capacity of current PBL tree
 * @control_6: Bits 0 to 31 - The actual number of PBLEs in the current PBL tree
 *	       Bits 32 to 55 - Identity of the underlying MR
 *	       Bits 56 to 63 - Reserved
 *
 * This downward message commands the adapter to register a non-shared memory
 * region.This message is used to implement Register NSMR memory management verb
 * Fast registration cannot be performed with this DMQ. It can only be done via
 * the PostSQ TOWI.
 */
typedef  struct  __hal_stag_query_resp_umq_t {
	u64			control_0;
#define	VXGE_HAL_STAG_QUERY_RESP_GET_STAG_IDX(ctrl0)	    bVAL24(ctrl0, 0)
#define	VXGE_HAL_STAG_QUERY_RESP_STAG_IDX(val)		    vBIT(val, 0, 24)

#define	VXGE_HAL_STAG_QUERY_RESP_GET_STAG_KEY(ctrl0)	    bVAL8(ctrl0, 24)
#define	VXGE_HAL_STAG_QUERY_RESP_STAG_KEY(val)		    vBIT(val, 24, 8)

#define	VXGE_HAL_STAG_QUERY_RESP_GET_IS_ALLOC(ctrl0)	    bVAL1(ctrl0, 35)
#define	VXGE_HAL_STAG_QUERY_RESP_IS_ALLOC		    mBIT(35)

#define	VXGE_HAL_STAG_QUERY_RESP_GET_IS_WINDOW(ctrl0)	    bVAL1(ctrl0, 36)
#define	VXGE_HAL_STAG_QUERY_RESP_IS_WINDOW		    mBIT(36)

#define	VXGE_HAL_STAG_QUERY_RESP_GET_IS_ZERO_BASE(ctrl0)    bVAL1(ctrl0, 37)
#define	VXGE_HAL_STAG_QUERY_RESP_IS_ZERO_BASE		    mBIT(37)

#define	VXGE_HAL_STAG_QUERY_RESP_GET_IS_VALID(ctrl0)	    bVAL1(ctrl0, 38)
#define	VXGE_HAL_STAG_QUERY_RESP_IS_VALID		    mBIT(38)

#define	VXGE_HAL_STAG_QUERY_RESP_GET_IS_SHARED(ctrl0)	    bVAL1(ctrl0, 39)
#define	VXGE_HAL_STAG_QUERY_RESP_IS_SHARED		    mBIT(39)

#define	VXGE_HAL_STAG_QUERY_RESP_GET_ALLOW_BIND(ctrl0)	    bVAL1(ctrl0, 42)
#define	VXGE_HAL_STAG_QUERY_RESP_ALLOW_BIND		    mBIT(42)

#define	VXGE_HAL_STAG_QUERY_RESP_GET_ALLOW_RAC(ctrl0)	    bVAL1(ctrl0, 43)
#define	VXGE_HAL_STAG_QUERY_RESP_ALLOW_RAC		    mBIT(43)

#define	VXGE_HAL_STAG_QUERY_RESP_GET_ALLOW_RR(ctrl0)	    bVAL1(ctrl0, 44)
#define	VXGE_HAL_STAG_QUERY_RESP_ALLOW_RR		    mBIT(44)

#define	VXGE_HAL_STAG_QUERY_RESP_GET_ALLOW_RW(ctrl0)	    bVAL1(ctrl0, 45)
#define	VXGE_HAL_STAG_QUERY_RESP_ALLOW_RW		    mBIT(45)

#define	VXGE_HAL_STAG_QUERY_RESP_GET_ALLOW_LR(ctrl0)	    bVAL1(ctrl0, 46)
#define	VXGE_HAL_STAG_QUERY_RESP_ALLOW_LR		    mBIT(46)

#define	VXGE_HAL_STAG_QUERY_RESP_GET_ALLOW_LW(ctrl0)	    bVAL1(ctrl0, 47)
#define	VXGE_HAL_STAG_QUERY_RESP_ALLOW_LW		    mBIT(47)

	u64			pd_id;

	u64			control_1;
#define	VXGE_HAL_STAG_QUERY_RESP_GET_SID(ctrl1)		    bVAL24(ctrl1, 0)
#define	VXGE_HAL_STAG_QUERY_RESP_SID(val)		    vBIT(val, 0, 24)

#define	VXGE_HAL_STAG_QUERY_RESP_GET_SIN(ctrl1)		    bVAL8(ctrl1, 24)
#define	VXGE_HAL_STAG_QUERY_RESP_SIN(val)		    vBIT(val, 24, 8)

	u64			length;

	u64			mr_base_to;

	u64			mw_base_to;

	u64			control_2;
#define	VXGE_HAL_STAG_QUERY_RESP_GET_FBO(ctrl2)		    bVAL32(ctrl2, 0)
#define	VXGE_HAL_STAG_QUERY_RESP_FBO(val)		    vBIT(val, 0, 32)

#define	VXGE_HAL_STAG_QUERY_RESP_GET_PBLE_SIZE(ctrl2)	    bVAL32(ctrl2, 32)
#define	VXGE_HAL_STAG_QUERY_RESP_PBLE_SIZE(val)		    vBIT(val, 32, 32)

	u64			control_3;
#define	VXGE_HAL_STAG_QUERY_RESP_GET_ROOT_PBL_ID(ctrl3)	    bVAL24(ctrl3, 0)
#define	VXGE_HAL_STAG_QUERY_RESP_ROOT_PBL_ID(val)	    vBIT(val, 0, 24)

#define	VXGE_HAL_STAG_QUERY_RESP_GET_PBL_DEPTH(ctrl3)	    bVAL5(ctrl3, 27)
#define	VXGE_HAL_STAG_QUERY_RESP_PBL_DEPTH(val)		    vBIT(val, 27, 5)

#define	VXGE_HAL_STAG_QUERY_RESP_GET_PSTAG_ID(ctrl3)	    bVAL24(ctrl3, 32)
#define	VXGE_HAL_STAG_QUERY_RESP_PSTAG_ID(val)		    vBIT(val, 32, 24)

#define	VXGE_HAL_STAG_QUERY_RESP_GET_PSTAG_VALID(ctrl3)	    bVAL1(ctrl3, 61)
#define	VXGE_HAL_STAG_QUERY_RESP_PSTAG_VALID		    mBIT(61)

#define	VXGE_HAL_STAG_QUERY_RESP_GET_LEFT_VALID(ctrl3)	    bVAL1(ctrl3, 62)
#define	VXGE_HAL_STAG_QUERY_RESP_LEFT_VALID		    mBIT(62)

#define	VXGE_HAL_STAG_QUERY_RESP_GET_RIGHT_VALID(ctrl3)	    bVAL1(ctrl3, 63)
#define	VXGE_HAL_STAG_QUERY_RESP_RIGHT_VALID		    mBIT(63)

	u64			control_4;
#define	VXGE_HAL_STAG_QUERY_RESP_GET_LEFT_ID(ctrl4)	    bVAL24(ctrl4, 0)
#define	VXGE_HAL_STAG_QUERY_RESP_LEFT_ID(val)		    vBIT(val, 0, 24)

#define	VXGE_HAL_STAG_QUERY_RESP_GET_RIGHT_ID(ctrl4)	    bVAL24(ctrl4, 32)
#define	VXGE_HAL_STAG_QUERY_RESP_RIGHT_ID(val)		    vBIT(val, 32, 24)

	u64			control_5;
#define	VXGE_HAL_STAG_QUERY_RESP_GET_NUM_MW_BOUND(ctrl5)    bVAL24(ctrl5, 0)
#define	VXGE_HAL_STAG_QUERY_RESP_NUM_MW_BOUND(val)	    vBIT(val, 0, 24)

#define	VXGE_HAL_STAG_QUERY_RESP_GET_ST_IMBALANCE(ctrl5)    bVAL2(ctrl5, 30)
#define	VXGE_HAL_STAG_QUERY_RESP_ST_IMBALANCE(val)	    vBIT(val, 30, 2)
#define	VXGE_HAL_STAG_QUERY_RESP_ST_IMBALANCE_SAME	    0
#define	VXGE_HAL_STAG_QUERY_RESP_ST_IMBALANCE_RIGHT_DEEP    1
#define	VXGE_HAL_STAG_QUERY_RESP_ST_IMBALANCE_LEFT_DEEP	    2

#define	VXGE_HAL_STAG_QUERY_RESP_GET_MAX_PBL_LEN(ctrl5)	    bVAL32(ctrl5, 32)
#define	VXGE_HAL_STAG_QUERY_RESP_MAX_PBL_LEN(val)	    vBIT(val, 32, 32)

	u64			control_6;
#define	VXGE_HAL_STAG_QUERY_RESP_GET_PBL_LEN(ctrl6)	    bVAL32(ctrl6, 0)
#define	VXGE_HAL_STAG_QUERY_RESP_PBL_LEN(val)		    vBIT(val, 0, 32)

#define	VXGE_HAL_STAG_QUERY_RESP_GET_MR_ID(ctrl6)	    bVAL24(ctrl6, 32)
#define	VXGE_HAL_STAG_QUERY_RESP_MR_ID(val)		    vBIT(val, 32, 24)


}__hal_stag_query_resp_umq_t;

/*
 * __hal_stag_validate_req_dmq_t - STAG Validate Local Tag Request DMQ
 *
 * @control_0: Bits 0 to 23 - The index of the existing on-adapter STag that is
 *		being validated.
 *	       Bits 24 to 31 - The key that to assign to the STag when it is
 *		validated.
 *	       Bits 32 to 63 - Reserved
 *
 * This message commands the adapter to transition an invalid STag to the valid
 * state without changing any of its other attributes..
 */
typedef  struct  __hal_stag_validate_req_dmq_t {
	u64			control_0;
#define	VXGE_HAL_STAG_VALID_LOCAL_REQ_GET_STAG_IDX(ctrl0)	bVAL24(ctrl0, 0)
#define	VXGE_HAL_STAG_VALID_LOCAL_REQ_STAG_IDX(val)		vBIT(val, 0, 24)

#define	VXGE_HAL_STAG_VALID_LOCAL_REQ_GET_STAG_KEY(ctrl0)    bVAL8(ctrl0, 24)
#define	VXGE_HAL_STAG_VALID_LOCAL_REQ_STAG_KEY(val)		vBIT(val, 24, 8)

}__hal_stag_validate_req_dmq_t;

/*
 * __hal_stag_validate_resp_umq_t - STAG Validate Local Tag Response
 *					   UMQ
 *
 * @control_0: Bits 0 to 23 - The host assigned STag index of the STag that was
 *		validated via the original DMQ.
 *	       Bits 24 to 31 - The host assigned STag key of the STag that was
 *		validated via the original DMQ.
 *	       Bits 32 to 63 - Reserved
 *
 * This upward message communicates to the host the success of failure of the
 * corresponding STAG Validate Local Tag Request DMQ message.
 */
typedef  struct  __hal_stag_validate_resp_umq_t {
	u64			control_0;
#define	VXGE_HAL_STAG_VALID_LOCAL_RESP_GET_STAG_IDX(ctrl0)	bVAL24(ctrl0, 0)
#define	VXGE_HAL_STAG_VALID_LOCAL_RESP_STAG_IDX(val)		vBIT(val, 0, 24)

#define	VXGE_HAL_STAG_VALID_LOCAL_RESP_GET_STAG_KEY(ctrl0)	bVAL8(ctrl0, 24)
#define	VXGE_HAL_STAG_VALID_LOCAL_RESP_STAG_KEY(val)		vBIT(val, 24, 8)

}__hal_stag_validate_resp_umq_t;

/*
 * __hal_stag_invalidate_req_dmq_t - STAG InValidate Local Tag Request DMQ
 *
 * @control_0: Bits 0 to 23 - The index of the existing on-adapter STag that
 *		is being invalidated.
 *	       Bits 24 to 31 - The key that is currently assigned to the STag.
 *	       Bits 32 to 63 - Reserved
 * @pd_id: The PD ID of the session or QP.
 *
 * The STAG Invalidate Local Tag Request DMQ message is used by the host to
 * invalidate a local STAG. This message provides an alternative route for the
 * normal TOWI based STAG Invalidation. It allows a kernel mode process to
 * invalidate an STAG without to write a TOWI.
 */
typedef  struct  __hal_stag_invalidate_req_dmq_t {
	u64			control_0;
#define	VXGE_HAL_STAG_INVALID_REQ_GET_STAG_IDX(ctrl0)    bVAL24(ctrl0, 0)
#define	VXGE_HAL_STAG_INVALID_REQ_STAG_IDX(val)	    vBIT(val, 0, 24)

#define	VXGE_HAL_STAG_INVALID_REQ_GET_STAG_KEY(ctrl0)    bVAL8(ctrl0, 24)
#define	VXGE_HAL_STAG_INVALID_REQ_STAG_KEY(val)	    vBIT(val, 24, 8)

	u64			pd_id;
}__hal_stag_invalidate_req_dmq_t;

/*
 * __hal_stag_invalidate_resp_umq_t - STAG InValidate Local Tag Response UMQ
 *
 * @control_0: Bits 0 to 23 - The index of the existing on-adapter STag that
 *		is being invalidated.
 *	       Bits 24 to 31 - The key that is currently assigned to the STag.
 *	       Bits 32 to 63 - Reserved
 *
 * This upward message communicates to the host the success or failure of the
 * corresponding STAG Invalidate Local Tag Request DMQ message.
 */
typedef  struct  __hal_stag_invalidate_resp_umq_t {
	u64			control_0;
#define	VXGE_HAL_STAG_INVALID_RESP_GET_STAG_IDX(ctrl0)   bVAL24(ctrl0, 0)
#define	VXGE_HAL_STAG_INVALID_RESP_STAG_IDX(val)	    vBIT(val, 0, 24)

#define	VXGE_HAL_STAG_INVALID_RESP_GET_STAG_KEY(ctrl0)   bVAL8(ctrl0, 24)
#define	VXGE_HAL_STAG_INVALID_RESP_STAG_KEY(val)	    vBIT(val, 24, 8)

}__hal_stag_invalidate_resp_umq_t;

/*
 * __hal_stag_bind_mw_req_dmq_t - STAG Bind MW Request DMQ
 *
 * @control_0: Bits 0 to 23 - The STag Index of the underlying MR to bind to.
 *	       Bits 24 to 31 - The STag Key of the underlying MR to bind to.
 *	       Bits 32 to 55 - The existing Memory Window's STag Index.
 *	       Bits 56 to 63 - The STag Key to use for the MW.
 * @control_1: Bits 0 to 31 - The existing Memory Region's STAG index.
 *		Least significant 8 bits is the key.
 *	       Bits 32 to 39 - Control flags for Memory Window.
 *		3'bxx1 Virtual Addressing Type
 *		3'bx1x Remote Write Enabled
 *		3'b1xx Remote Read Enabled"
 *	       Bits 40 to 63 - Reserved
 * @mw_length: Memory window length in bytes.
 * @tagged_offset: Memory window tagged offset in bytes.
 *
 * This downward message commands the adapter to bind an existing (invalid)
 * MW to an existing (valid) MR. This message provides an alternative to
 * the TOWI based implementation allowing the DMQ path be used for MW binding
 */
typedef  struct  __hal_stag_bind_mw_req_dmq_t {
	u64			control_0;
#define	VXGE_HAL_STAG_BIND_MW_REQ_GET_MR_STAG_IDX(ctrl0)    bVAL24(ctrl0, 0)
#define	VXGE_HAL_STAG_BIND_MW_REQ_MR_STAG_IDX(val)	    vBIT(val, 0, 24)

#define	VXGE_HAL_STAG_BIND_MW_REQ_GET_MR_STAG_KEY(ctrl0)    bVAL8(ctrl0, 24)
#define	VXGE_HAL_STAG_BIND_MW_REQ_MR_STAG_KEY(val)	    vBIT(val, 24, 8)

#define	VXGE_HAL_STAG_BIND_MW_REQ_GET_MW_STAG_IDX(ctrl0)    bVAL24(ctrl0, 32)
#define	VXGE_HAL_STAG_BIND_MW_REQ_MW_STAG_IDX(val)	    vBIT(val, 32, 24)

#define	VXGE_HAL_STAG_BIND_MW_REQ_GET_MW_STAG_KEY(ctrl0)    bVAL8(ctrl0, 56)
#define	VXGE_HAL_STAG_BIND_MW_REQ_MW_STAG_KEY(val)	    vBIT(val, 56, 8)

	u64			control_1;
#define	VXGE_HAL_STAG_BIND_MW_REQ_GET_MR_STAG(ctrl1)	    bVAL32(ctrl1, 0)
#define	VXGE_HAL_STAG_BIND_MW_REQ_MR_STAG(val)		    vBIT(val, 0, 32)

#define	VXGE_HAL_STAG_BIND_MW_REQ_GET_MW_CTRL(ctrl1)	    bVAL8(ctrl1, 32)
#define	VXGE_HAL_STAG_BIND_MW_REQ_MW_CTRL(val)		    vBIT(val, 32, 8)
#define	VXGE_HAL_STAG_BIND_MW_REQ_MW_CTRL_VA_TYPE	    1
#define	VXGE_HAL_STAG_BIND_MW_REQ_MW_CTRL_RW_ENABLED	    2
#define	VXGE_HAL_STAG_BIND_MW_REQ_MW_CTRL_RR_ENABLED	    4

	u64			mw_length;

	u64			tagged_offset;

}__hal_stag_bind_mw_req_dmq_t;

/*
 * __hal_stag_bind_mw_resp_umq_t - STAG Bind MW Response UMQ
 *
 * @control_0: Bits 0 to 23 - The host assigned STag index MW that was bound.
 *	       This is returned unmodified from the request for convenience..
 *	       Bits 24 to 31 - Host assigned STag key of the MW that was bound.
 *	       This is returned unmodified from the request for convenience.
 *	       Bits 32 to 63 - Reserved
 *
 * This upward message communicates to the host the success or failure of the
 * corresponding STAG Bind MW Request DMQ message.
 */
typedef  struct  __hal_stag_bind_mw_resp_umq_t {
	u64			control_0;
#define	VXGE_HAL_STAG_BIND_MW_RESP_GET_STAG_IDX(ctrl0)	    bVAL24(ctrl0, 0)
#define	VXGE_HAL_STAG_BIND_MW_RESP_STAG_IDX(val)	    vBIT(val, 0, 24)

#define	VXGE_HAL_STAG_BIND_MW_RESP_GET_STAG_KEY(ctrl0)	    bVAL8(ctrl0, 24)
#define	VXGE_HAL_STAG_BIND_MW_RESP_STAG_KEY(val)	    vBIT(val, 24, 8)

}__hal_stag_bind_mw_resp_umq_t;

/*
 * __hal_stag_fast_register_nsmr_req_dmq_t - STAG Fast Register NSMR
 *						 Request DMQ
 *
 * @control_0: Bits 0 to 31 - Reserved.
 *	       Bits 32 to 63 - Length in bytes of the Memory Region control
 *		structure..
 * @mr_control_ptr: Pointer to Memory Region control structure in host memory.
 *
 * The STAG Fast Register NSMR Request DMQ provides an alternative way to fast
 * register an NSMR instead of going to the TOWI path.
 */
typedef  struct  __hal_stag_fast_register_nsmr_req_dmq_t {
	u64			control_0;
#define	VXGE_HAL_STAG_FRNSMR_REQ_GET_MR_CTRL_SIZE(ctrl0)    bVAL32(ctrl0, 32)
#define	VXGE_HAL_STAG_FRNSMR_REQ_MR_CTRL_SIZE(val)	    vBIT(val, 32, 32)

	u64			mr_control_ptr;

}__hal_stag_fast_register_nsmr_req_dmq_t;

/*
 * __hal_stag_fast_register_nsmr_resp_umq_t - STAG Fast Register NSMR Response
 *					UMQ
 *
 * @control_0: Bits 0 to 23 - The host assigned STag index of the NSMR that
 *	       was fast-registered. This is returned unmodified from the request
 *             for convenience.
 *	       Bits 24 to 31 - The host assigned STag key of the NSMR that was
 *             fast-registerd. This is returned unmodified from the request for
 *             convenience.
 *	       Bits 32 to 63 - Reserved
 *
 * This upward message communicates to the host the success or failure of the
 * corresponding STAG Fast Register NSMR DMQ message.
 */
typedef  struct  __hal_stag_fast_register_nsmr_resp_umq_t {
	u64			control_0;
#define	VXGE_HAL_STAG_FRNSMR_RESP_GET_STAG_IDX(ctrl0)	    bVAL24(ctrl0, 0)
#define	VXGE_HAL_STAG_FRNSMR_RESP_STAG_IDX(val)		    vBIT(val, 0, 24)

#define	VXGE_HAL_STAG_FRNSMR_RESP_GET_STAG_KEY(ctrl0)	    bVAL8(ctrl0, 24)
#define	VXGE_HAL_STAG_FRNSMR_RESP_STAG_KEY(val)		    vBIT(val, 24, 8)

}__hal_stag_fast_register_nsmr_resp_umq_t;


/* ======================== SESSION MESSAGES ============================= */

/*
 * __hal_tcp_open_req_dmq_t - TCP Open Request DMQ
 *
 * @session_opaque_handle:An opaque handle used by the host to reference data
 *		structures in host memory related to this session
 * @control_0: Bits 0 to 14 - Reserved
 *	       Bit 15 -  Active Opener
 *	       Bits 16 to 31 - Layer 4 source port
 *	       Bits 32 to 47 - Layer 4 destination port
 *	       Bit 48 - Does this session use IPv6 addressing?
 *			1'b1 - IPv6
 *			 1'b0 - IPv4
 *	       Bit 49 - Indicates whether or not this session supports VLAN.
 *			1 - VLAN supported and vlan_id read by adapter
 *			0 - VLAN disabled and vlan_id ignored by adapter.
 *	       Bits 50 to 51 - Reserved
 *	       Bits 52 to 63 - The VLAN ID used by this session.
 *		Only valid if use_vlan == 1.
 * @ip_sa: IPv4 or IPv6 source address
 *		  ip_sa.high - Higher 64 bits
 *		ip_sa.low - Lower 64 bits
 * @ip_da: IPv4 or IPv6 destination address
 *		  ip_da.high - Higher 64 bits
 *		ip_da.low - Lower 64 bits
 * @control_1: Bits 0 to 7 - Instance number of the CQRQ.
 *	       Bits 8 to 31 - Adapter ID of the bytestream CQRQ for the bystream
 *		session.
 *	       Bits 32 to 39 - Instance number of the SRQ.
 *	       Bits 40 to 63 - Adapter ID of the bytestream SRQ for the
 *		bytestream session.
 * @control_2: Bits 0 to 5 - Reserved
 *	       Bit 6 - If == 1, a merge will be broken if a frame with PSH == 1
 *	       is received
 *	       Bit 7 -  Merging of TCP payloads is enabled if this bit is set
 *	       Bits 8 to 11 - Reserved
 *	       Bits 12 to 15 - If no_wqe_cnt is >= this threshold, the RPE will
 *	       inform the RxPE, which will most likely initiate session teardown
 *	       Bits 16 to 23 - Reserved
 *	       Bits 24 to 31 - If the frame meets all the low-latency criteria,
 *	       its RDMA payload will be placed in an extended CQE.
 *		Size of the CQEs produced by this session
 *		0 -  48B, CQE header only
 *		1 - 176B, CQE header + 128B of immediate data
 *	       Bits 32 to 63 - Reserved
 * @control_3: Bits 0 to 1 - Reserved
 *	       Bit 2 -  Force VLAN tag insertion on transmit frames for this
 *	       session, even if the vlan_id in the NCE context is zero, so that
 *	       the user_priority value is sent. VLAN tags are generated if this
 *	       is 1 or the vlan_id in the NCE context is non-zero. The tag value
 *	       is always {session.vlan_user_priority, session.vlan_cfi,
 *	       nce.vlan_id}.
 *	       Bit 3 -  Enable window scaling. This controls whether the window
 *	       scale option is sent and accepted on SYN or SYN/ACK segments
 *	       during the 2 way handshake.
 *			1 - Enable window scaling (default)
 *			0 - Disable window scaling
 *	       The window scaling values are defined by the snd_wscale and
 *             rcv_wscale fields..
 *	       Bit 4 -  Enable for SACKs. (note that protocol SEEMS to allow
 *	       asymmetric setting but there is no real use for it; we should
 *	       allow only symmetric operation so this applies to rx and tx).
 *	       See tstamp_ok.
 *	       Bit 5 -  Enable for Explicit Congestion Notification.
 *	       See tstamp_ok.
 *	       Bit 6 -  Enable timestamps.
 *			1 - Enable timestamps (default)
 *			0 - Disable timestamps
 *	       Active opener sets this to allow timestamps; adapter may clear it
 *	       based on negotiation. Passive opener sets this to allow
 *	       timestamps and if the far end has indicated timestamp capable.
 *	       The adapter will not clear it.
 *	       Bit 7 - Enable for keepalive probes for this connection.
 *			1 - Enable keepalive (default)
 *			0 - Disable keepalive
 *	       Bits 8 to 15 - Interrupt number for the session. This is
 *	       included in ROCRC commands for HW-CQ-SQ and HW-CQ-RR updates
 *		(ie it is used for traffic interrupts only).
 *	       Bits 16 to 31 - Interrupt bitmap number for the session.
 *	       Bits 32 to 63 - INCE # for this session.
 * @control_4: Bits 0 to 7 - Value to insert in the user_priority field when
 *		VLAN tags are generated on tx frames.
 *	       Bits 8 to 15 - Reserved.
 *	       Bits 16 to 23 - Duplicate ACK threshold for fast retransmit.
 *		Default is 3.
 *	       Bits 24 to 31 - Value to place in the IPv4 TOS field or the Ipv6
 *		Traffic Class field. The 2 lower bits are overwritten if ECN is
 *		enabled.
 *	       Bits 32 to 39 - Value to place in the IPv4 Flags field. This is
 *		included in PE context so the PE firmware could choose to
 *		override them (particularly the DF flag).
 *	       Bits 40 to 43 - Reserved.
 *	       Bits 44 to 63 - Value to place in the IPv6 Flow Label field.
 * @control_5: Bits 0 to 31 - Keepalive idle timer - the idle time before
 *		sending the first keepalive probe (default is 2 hours).
 *	       Bits 32 to 63 - Keepalive retry time interval  idle time before
 *		resending an unanswered keepalive probe.
 * @control_6: Bits 0 to 31 - Maximum time to spent retransmitting one segment.
 *		This is the equivelant of retx_cnt_max but measured in time
 *		instead of number of retransmits (ie threshold R2 in RFC1122
 *		section 4.2.3.5). This is required by the M/S Chimney. It
 *		appears to apply in addition to retx_cnt_max, not instead of it,
 *		although that is not explicitly stated.
 *	       Bits 32 to 39 - Reserved
 *	       Bits 40 to 47 - Sender window shift scale value(ie value received
 *		from far end which is applied to window values that we receive).
 *		Supplied by host on create for passive open. Set by the adapter
 *		on establishment for active open.
 *	       Bits 48 to 63 - Keepalive maximum probe count - maximum number of
 *		unanswered keepalive probes before giving up on session.
 * @control_7: Bits 0 to 31 - Next sequence number to send. This is slightly
 *		different from some TCP stacks in that it always reflects the
 *		highest sequence number ever sent. In case of retransmit (fast
 *		retx or retx timeout), snd_retx stores the next sequence number
 *		to retransmit. Set to ISS value supplied by host on creation for
 *		both active and passive open.
 *	       Bits 32 to 47 - Max segment size for transmit. This is defined as
 *		the maximum permitted TCP payload size, accounting for the
 *		maximum TCP option size (40B if both SACK and timestamps are
 *		enabled) - snd_mss = MSS - max_tcp_options.
 *		max_tcp_options is 40B assuming both SACK and timestamps are
 *		enabled(note that the firmware might choose to use a fixed value
 *		of 40 even if SACK or timestamps are not enabled - this is TBD).
 *		The definition is slightly different from most TCP stacks which
 *		store the true MSS value (max payload assuming no IP or TCP
 *		options). It allows the same amount of payload whether SACK is
 *		sent or not - important for sending aligned DDP segments. Most
 *		stacks typically pack the max possible data into a segment, and
 *		then resegment on retransmit if the options size has grown.
 *		When sending the MSS option on SYN or SYN/ACK segments, the PE
 *		firmware calculates the true MSS value based on this field (ie
 *		MSS = snd_mss + max_tcp_options). Supplied by host for both
 *		active and passive open. In active open case, adapter may adjust
 *		value to the minimum of the host supplied value and the value
 *		from the received MSS option.
 *	       Bits 48 to 63 - Value of MSS option received from far end on the
 *		SYN or SYN/ACK segment.The PE applies this as a limit on snd_mss
 *		when the host signals path MTU changes as follows -
 *		snd_mss = min(new_mss_from_host, snd_mss_limit)-max_tcp_options
 *		Supplied by host for passive open.Set by adapter for active open
 *		based on the MSS option value on the received SYN or SYN/ACK
 *		segment.
 * @control_8: Bits 0 to 31 - Next byte expected to be received. This is the
 *		last ACK value sent to the far end(except in the case of delayed
 *		ACKs when it is the value we want to send on next ACK). Supplied
 *		by host for passive open. Host might supply IRS instead of
 *		rcv_nxt (different by 1). Initialized by adapter for active open
 *		on receipt of SYN/ACK or SYN segment.
 *	       Bits 32 to 39 - Reserved
 *	       Bits 40 to 47 - Receiver window shift scale value (ie value sent
 *		to far end which is applied to window values that we generate).
 *		Desired value is supplied by host, but adapter may change it to
 *		zero in the active open case if far end disapproves of window
 *		scaling.
 *	       Bits 48 to 63 - Max segment size for receive. This is the value
 *		advertised to the far end in the MSS option of SYN or SYN/ACK
 *		sent by this end.This is required by the delayed acknowledgement
 *		algorithm, as specified in 1122 and 2581 (send ack when 2 full
 *		MSS sized segments have been received). The rcv_mss value is
 *		used not the send_mss. NOTE-current plan is only to support ack
 *		every N segments, NOT after receipt of 2 x MSS worth of data.
 *		Hence this is not required for Rx SWS avoidance. But it might
 *		be required for deciding whether to send an ack on host window
 *		updates.
 * @control_9: Bits 0 to 31 - Maximum receive buffer size that the connection
 *		is allowed to consume. Units are TCP payload bytes (ie sequence
 *		number bytes).
 *	       Bits 32 to 63 - Maximum value for receive window to be advertised
 *		This can be less than rcv_buf_size, so that in cases when the
 *		transmit rate is not limited by the receive application, the
 *		advertised window will remain constant. This is allows less
 *		frequent buffer usage updates from host without causing bursty
 *		behaviour from the sender.
 * @control_10: Bits 0 to 31 - For bytestream sessions - lowest TCP sequence
 *		number buffered in the connection's receive buffer. This is
 *		updated by the host as data is consumed by the ULP/application.
 *		Must increase monotonically to avoid window shrinks (either
 *		check in PE when host updates it, or just make it a requirement
 *		of the host). For iWARP sessions - the DDP mainland RHS TCP
 *		sequence number. Represents the last aligned DDP mainland
 *		placement. The DDP mainland edge differs from the TCP mainland
 *		edge (rcv_nxt) if MPA misaligned frames are arriving (ie session
 *		is ""DDP OOO"" and iwarp_align_state != aligned). This field is
 *		only updated when misaligned. When aligned, including in the
 *		normal fast path DDP in order case, this is not updated to save
 *		fast path instructions; instead rcv_nxt defines both the TCP and
 *		DDP mainland edge. Supplied by host for passive open.Initialized
 *		by adapter for active open on receipt of SYN/ACK or SYN segment.
 *		In both cases the proper initial value is IRS + 1 (same as
 *		rcv_nxt). This is set equal to rcv_nxt on promote to iWARP.
 *	       Bits 32 to 63 - Configured minimum RTT value for this session.
 *		BSD allows this to be configured per session; global register
 *		might be enough (if so, remove this from context).
 * @control_11: Bits 0 to 31 - Reserved
 *	       Bits 32 to 63 - Timestamps Enabled
 *		ts_recent value from RFC1323 - most recent timestamp from
 *		received packets. This is echoed in outgoing timestamps and
 *		used for PAWS. Value of zero means that ts_recent is invalid.
 *		Note - zero meaning invalid is used like in BSD and linux
 *		instead of keeping a separate valid flag. It requires slightly
 *		more h/w in the rx fast path decision (compare 32 bits to 0
 *		instead of 1) but less h/w on transmit which is supposed to send
 *		tsecr=0 when ts_recent is invalid. It is more efficient in the
 *		software fast path (only one variable to load).
 *		Timestamps Disabled
 *		Records transmit time of packet to be measured (formerly called
 *		rttm_time_sent). This is non-zero if a packet is currently being
 *		timed. It is set to zero (indicating invalid) after the packet
 *		is acknowledged or retransmitted.
 *		Required on create for passive opener when timestamps are
 *		enabled, to supply the timestamp tsval from the SYN segment.
 *		Set to 0 for active open or when timestamps are disabled.
 * @nxt_towi_ptr: Pointer to the next TOWI (or TOWI block wrapper) to be read
 *		from host memory. It points to the first TOWI including or
 *		after nxt_towi which is not cached in the TOWI cache (ie to
 *		nxt_towi + current cache fill) or to the TOWI block wrapper
 *		preceding this TOWI. Initialized to the base address of the
 *		first TOWI Block on session creation.
 * @control_12: Bits 0 to 31 - Remaining number of TOWIs in the block containing
 *		*nxt_towi_ptr, including *nxt_towi_ptr. 0 indicates that
 *		nxt_towi_ptr is pointing to a TOWI block wrapper.
 *		Initialized to the number of TOWIs in the first TOWI Block on
 *		session creation.
 *	       Bits 32 to 63 - Host supplied limit on the maximum number of ACK
 *		list blocks that can be held by this session at any one time.
 *		Note that there is no need to make this limit optional. The
 *		check can be effectively disabled by setting the limit higher
 *		than the total number of ACK blocks available.
 * @hw_cq_sq: Pointer to the "hw-cq-sq" location on the host. The PE writes
 *		back the highest acknowledged TOWI number to this location.
 *
 * The TCP Open Request DMQ message is sent by the host to open a TCP connection
 * on the adapter.
 */
typedef  struct  __hal_tcp_open_req_dmq_t {
	u64			session_opaque_handle;

	u64			control_0;
#define	VXGE_HAL_TCP_OPEN_REQ_GET_ACTIVE_OPENER(ctrl0)	    bVAL1(ctrl0, 15)
#define	VXGE_HAL_TCP_OPEN_REQ_ACTIVE_OPENER		    mBIT(15)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_TCP_SRC_PORT(ctrl0)	    bVAL16(ctrl0, 16)
#define	VXGE_HAL_TCP_OPEN_REQ_TCP_SRC_PORT(val)		    vBIT(val, 16, 16)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_TCP_DEST_PORT(ctrl0)	    bVAL16(ctrl0, 32)
#define	VXGE_HAL_TCP_OPEN_REQ_TCP_DEST_PORT(val)	    vBIT(val, 32, 16)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_USE_IPV6(ctrl0)	    bVAL1(ctrl0, 48)
#define	VXGE_HAL_TCP_OPEN_REQ_USE_IPV6			    mBIT(48)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_USE_VLAN(ctrl0)	    bVAL1(ctrl0, 49)
#define	VXGE_HAL_TCP_OPEN_REQ_USE_VLAN			    mBIT(49)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_VLAN_ID(ctrl0)	    bVAL12(ctrl0, 52)
#define	VXGE_HAL_TCP_OPEN_REQ_VLAN_ID(val)		    vBIT(val, 52, 12)

	struct {
		u64		high;
		u64		low;
	}			ip_sa;

	struct {
		u64		high;
		u64		low;
	}			ip_da;


	u64			control_1;
#define	VXGE_HAL_TCP_OPEN_REQ_GET_BS_CQRQ_INST_NUM(ctrl1)    bVAL8(ctrl1, 0)
#define	VXGE_HAL_TCP_OPEN_REQ_BS_CQRQ_INST_NUM(val)	    vBIT(val, 0, 8)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_BS_CQRQ_ID(ctrl1)	    bVAL24(ctrl1, 8)
#define	VXGE_HAL_TCP_OPEN_REQ_BS_CQRQ_ID(val)		    vBIT(val, 8, 24)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_BS_SRQ_INST_NUM(ctrl1)    bVAL8(ctrl1, 32)
#define	VXGE_HAL_TCP_OPEN_REQ_BS_SRQ_INST_NUM(val)	    vBIT(val, 32, 8)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_BS_SRQ_ID(ctrl1)	    bVAL24(ctrl1, 40)
#define	VXGE_HAL_TCP_OPEN_REQ_BS_SRQ_ID(val)		    vBIT(val, 40, 24)

	u64			control_2;
#define	VXGE_HAL_TCP_OPEN_REQ_GET_BREAK_MERGE_EN(ctrl2)	    bVAL1(ctrl2, 6)
#define	VXGE_HAL_TCP_OPEN_REQ_BREAK_MERGE_EN		    mBIT(6)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_DATA_MERGE_EN(ctrl2)	    bVAL1(ctrl2, 7)
#define	VXGE_HAL_TCP_OPEN_REQ_DATA_MERGE_EN		    mBIT(7)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_NO_WQE_TH(ctrl2)	    bVAL4(ctrl2, 12)
#define	VXGE_HAL_TCP_OPEN_REQ_NO_WQE_TH(val)		    vBIT(val, 12, 4)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_CQRQ_LOW_LAT(ctrl2)	    bVAL8(ctrl2, 24)
#define	VXGE_HAL_TCP_OPEN_REQ_CQRQ_LOW_LAT(val)		    vBIT(val, 24, 8)
#define	VXGE_HAL_TCP_OPEN_REQ_CQRQ_LOW_LAT_HEADER_ONLY	    0
#define	VXGE_HAL_TCP_OPEN_REQ_CQRQ_LOW_LAT_HEADER_DATA	    1

	u64			control_3;
#define	VXGE_HAL_TCP_OPEN_REQ_GET_SEND_VLAN_UP(ctrl3)	    bVAL1(ctrl3, 2)
#define	VXGE_HAL_TCP_OPEN_REQ_SEND_VLAN_UP		    mBIT(2)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_WSCALE_OK(ctrl3)	    bVAL1(ctrl3, 3)
#define	VXGE_HAL_TCP_OPEN_REQ_WSCALE_OK			    mBIT(3)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_SACK_OK(ctrl3)	    bVAL1(ctrl3, 4)
#define	VXGE_HAL_TCP_OPEN_REQ_SACK_OK			    mBIT(4)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_ECN_OK(ctrl3)		    bVAL1(ctrl3, 5)
#define	VXGE_HAL_TCP_OPEN_REQ_ECN_OK			    mBIT(5)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_TSTAMP_OK(ctrl3)	    bVAL1(ctrl3, 6)
#define	VXGE_HAL_TCP_OPEN_REQ_TSTAMP_OK			    mBIT(6)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_KEEPALIVE_OK(ctrl3)	    bVAL1(ctrl3, 7)
#define	VXGE_HAL_TCP_OPEN_REQ_KEEPALIVE_OK		    mBIT(7)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_INT_NUM(ctrl3)	    bVAL8(ctrl3, 8)
#define	VXGE_HAL_TCP_OPEN_REQ_INT_NUM(val)		    vBIT(val, 8, 8)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_INT_BITMAP(ctrl3)	    bVAL16(ctrl3, 16)
#define	VXGE_HAL_TCP_OPEN_REQ_INT_BITMAP(val)		    vBIT(val, 16, 16)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_NCE_ID(ctrl3)		    bVAL32(ctrl3, 32)
#define	VXGE_HAL_TCP_OPEN_REQ_NCE_ID(val)		    vBIT(val, 32, 32)

	u64			control_4;
#define	VXGE_HAL_TCP_OPEN_REQ_GET_VLAN_USER_PRI(ctrl4)	    bVAL8(ctrl4, 0)
#define	VXGE_HAL_TCP_OPEN_REQ_VLAN_USER_PRI(val)	    vBIT(val, 0, 8)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_DUP_ACK_TH(ctrl4)	    bVAL8(ctrl4, 16)
#define	VXGE_HAL_TCP_OPEN_REQ_DUP_ACK_TH(val)		    vBIT(val, 16, 8)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_IP_TOS(ctrl4)		    bVAL8(ctrl4, 24)
#define	VXGE_HAL_TCP_OPEN_REQ_IP_TOS(val)		    vBIT(val, 24, 8)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_IP_FLAGS(ctrl4)	    bVAL8(ctrl4, 32)
#define	VXGE_HAL_TCP_OPEN_REQ_IP_FLAGS(val)		    vBIT(val, 32, 8)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_IP_FLOW_LABEL(ctrl4)	    bVAL20(ctrl4, 40)
#define	VXGE_HAL_TCP_OPEN_REQ_IP_FLOW_LABEL(val)	    vBIT(val, 40, 20)

	u64			control_5;
#define	VXGE_HAL_TCP_OPEN_REQ_GET_KPA_IDLE_TIME(ctrl5)	    bVAL32(ctrl5, 0)
#define	VXGE_HAL_TCP_OPEN_REQ_KPA_IDLE_TIME(val)	    vBIT(val, 0, 32)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_KPA_RETRY_TIME(ctrl5)	    bVAL32(ctrl5, 32)
#define	VXGE_HAL_TCP_OPEN_REQ_KPA_RETRY_TIME(val)	    vBIT(val, 32, 32)

	u64			control_6;
#define	VXGE_HAL_TCP_OPEN_REQ_GET_RETX_TIME_MAX(ctrl6)	    bVAL32(ctrl6, 0)
#define	VXGE_HAL_TCP_OPEN_REQ_RETX_TIME_MAX(val)	    vBIT(val, 0, 32)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_SND_WSCALE(ctrl6)	    bVAL8(ctrl6, 40)
#define	VXGE_HAL_TCP_OPEN_REQ_SND_WSCALE(val)		    vBIT(val, 40, 8)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_KPA_PROBES_MAX(ctrl6)	    bVAL16(ctrl6, 48)
#define	VXGE_HAL_TCP_OPEN_REQ_KPA_PROBES_MAX(val)	    vBIT(val, 48, 16)

	u64			control_7;
#define	VXGE_HAL_TCP_OPEN_REQ_GET_SND_NXT(ctrl7)	    bVAL32(ctrl7, 0)
#define	VXGE_HAL_TCP_OPEN_REQ_SND_NXT(val)		    vBIT(val, 0, 32)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_SND_MSS(ctrl7)	    bVAL16(ctrl7, 32)
#define	VXGE_HAL_TCP_OPEN_REQ_SND_MSS(val)		    vBIT(val, 32, 16)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_SND_MSS_LIMIT(ctrl7)	    bVAL16(ctrl7, 48)
#define	VXGE_HAL_TCP_OPEN_REQ_SND_MSS_LIMIT(val)	    vBIT(val, 48, 16)

	u64			control_8;
#define	VXGE_HAL_TCP_OPEN_REQ_GET_RCV_NXT(ctrl8)	    bVAL32(ctrl8, 0)
#define	VXGE_HAL_TCP_OPEN_REQ_RCV_NXT(val)		    vBIT(val, 0, 32)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_RCV_WSCALE(ctrl8)	    bVAL8(ctrl8, 40)
#define	VXGE_HAL_TCP_OPEN_REQ_RCV_WSCALE(val)		    vBIT(val, 40, 8)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_RCV_MSS(ctrl8)	    bVAL16(ctrl8, 48)
#define	VXGE_HAL_TCP_OPEN_REQ_RCV_MSS(val)		    vBIT(val, 48, 16)

	u64			control_9;
#define	VXGE_HAL_TCP_OPEN_REQ_GET_RCV_BUF_SIZE(ctrl9)	    bVAL32(ctrl9, 0)
#define	VXGE_HAL_TCP_OPEN_REQ_RCV_BUF_SIZE(val)		    vBIT(val, 0, 32)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_RCV_MAX_WND(ctrl9)	    bVAL32(ctrl9, 32)
#define	VXGE_HAL_TCP_OPEN_REQ_RCV_MAX_WND(val)		    vBIT(val, 32, 32)

	u64			control_10;
#define	VXGE_HAL_TCP_OPEN_REQ_GET_RCV_BUF(ctrl10)	    bVAL32(ctrl10, 0)
#define	VXGE_HAL_TCP_OPEN_REQ_RCV_BUF(val)		    vBIT(val, 0, 32)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_RTT_MIN(ctrl10)	    bVAL32(ctrl10, 32)
#define	VXGE_HAL_TCP_OPEN_REQ_RTT_MIN(val)		    vBIT(val, 32, 32)

	u64			control_11;
#define	VXGE_HAL_TCP_OPEN_REQ_GET_TS_RECENT(ctrl11)	    bVAL32(ctrl11, 32)
#define	VXGE_HAL_TCP_OPEN_REQ_TS_RECENT(val)		    vBIT(val, 32, 32)

	u64			nxt_towi_ptr;

	u64			control_12;
#define	VXGE_HAL_TCP_OPEN_REQ_GET_NXT_TOWI_CNT(ctrl12)	    bVAL32(ctrl12, 0)
#define	VXGE_HAL_TCP_OPEN_REQ_NXT_TOWI_CNT(val)		    vBIT(val, 0, 32)

#define	VXGE_HAL_TCP_OPEN_REQ_GET_ACK_BLK_LIMIT(ctrl12)	    bVAL32(ctrl12, 32)
#define	VXGE_HAL_TCP_OPEN_REQ_ACK_BLK_LIMIT(val)	    vBIT(val, 32, 32)

	u64			hw_cq_sq;

}__hal_tcp_open_req_dmq_t;

/*
 * __hal_tcp_promote_to_iwarp_req_dmq_t - TCP Promote to iWARP DMQ
 *
 * @control_0: Bits 0 to 4 - Reserved.
 *	       Bit 5 - If set then MPA CRCs are expected to be valid on received
 *		frames. This field gets copied into -  p_SCR.rx_crc_status
 *		c_SCR.crc32c_usage
 *	       Bit 6 - Whether markers exist in the transmit direction or not.
 *		This field gets copied into -  p_SCR.tx_marker_status
 *		c_SCR.marker_status (or'ed with rx_marker_status)
 *	       Bit 7 - Whether markers exist in the receive direction or not.
 *		If this is set to zero for an RDMA session, the RxPE drops all
 *		out of order receive segments.
 *		This field gets copied into -  p_SCR.rx_marker_status
 *		r_SCR.iwarp_rx_markers_enabled c_SCR.marker_status (or'ed with
 *		tx_marker_status)
 *	       Bits 8 to 13 - Reserved.
 *	       Bit 14 - 1- the session is allowed to use STag=0 in Rx WQEs.
 *			 0- the session is not allowed to use STag=0 in Rx WQEs.
 *	       This isn't essential as the host could restrict the posting of Rx
 *		WQEs that use STag=0 to those SRQs that feed sessions enabled to
 *		use STag=0. The adapter just uses the Rx WQEs provided to it
 *	       Bit 15 - 1 - session to accept STag=0 in receive frames.
 *			 0 - session rejects STag=0 in receive frames
 *		 - as per the verbs. The ability to process physical addresses
 *		(STag=0) from the wire is a proprietary enhancement.
 *	       Bits 16 to 21 - Reserved.
 *	       Bit 22 - If the session has remote write permission, incoming
 *		RDMA WRITE or RDMA READ RESPONSE messages can be placed
 *	       Bit 23 - When a SEND* WITH INVALIDATE message is received, the
 *		STag can only be invalidated if the session has remote write
 *		and/or remote read access. Also used to ensure that RIRRs are
 *		allowed on this session
 *	       Bits 24 to 63 - Reserved.
 * @srq_pd_id: Protection domain to which the associated iWARP SRQ belongs.
 *		This is needed to do access checking.
 * @iwarp_qp_pd_id:Protection domain to which this session belongs. This is
 *		needed to do STag access permission checking.
 * @control_1: Bits 0 to 15 - When a low-latency SEND* is passed to the host,
 *		the frame data will be offset from the CQE base address by this
 *		number of bytes
 *	       Bits 16 to 31 - If the number of in-progress WQEs reaches this
 *		value and the Queue Pair limit is armed, the session produces an
 *		asynchronous event
 *	       Bits 32 to 47 - The maximum number of WQEs that this session is
 *		allowed to have in progress on this SRQ.
 *	       Bits 48 to 55 - Reserved.
 *	       Bits 56 to 63 - If set, the Queue Pair limit is armed at the
 *		iwarp_qp_limit_value
 * @control_2: Bits 0 to 7 - Instance Number of CQRQ for the iWARP session.
 *	       Bits 8 to 31 - Adapter ID of the CQRQ for the iWARP session.
 *	       Bits 32 to 39 - Instance Number of SRQ for the iWARP session.
 *	       Bits 40 to 63 - Adapter ID of the SRQ for the iWARP session.
 * @control_3: Bits 0 to 5 - Reserved
 *	       Bits 6 to 7 - The RDMA version number
 *		  -- 01 - for IETF RNICs
 *		  -- 00 - for RDMAC RNICs
 *	       Bits 8 to 13 - Reserved
 *	       Bits 14 to 15 - Direct Data Placement Protocol Version
 *		  -- 01 - for protocol described in
 *		http://ietf.org/internet-drafts/draft-ietf-rddp-ddp-06.txt
 *	       Bits 16 to 31 - Reserved
 *	       Bits 32 to 47 - This field represents the size of TCP payload
 *		of MPA reply frame (in bytes) on the Responder. This value is
 *		relevant only for the MPA Responder. It allows the Tx PE to
 *		determine the TCP SN at which iWarp will begin (ie the TxIMSN).
 *	       Bits 48 to 54 - Reserved
 *	       Bit 55 - This allows the PE to take distinquish whether it is
 *		the MPA Initiator or the MPA Responder.
 *	       Bits 56 to 63 - Used to restrict session's ability to use special
 *		TOWI atoms. It is used to enforce checks such as preventing the
 *		QP from performing a Fast Register NSMR operation.
 * @control_4: Bits 0 to 39 - Reserved
 *	       Bits 40 to 63 - Host supplied IRD limit for this session.
 * @hw_cq_rr_ir: Pointer to the "hw-cq-rr" location on the host.
 *
 * The TCP Promote to iWARP Request DMQ message is sent from the host to the
 * adapter in order to migrate an existing bytestream session to iWARP mode.
 */
typedef  struct  __hal_tcp_promote_to_iwarp_req_dmq_t {

	u64			control_0;
#define	VXGE_HAL_TCP_PROMO_REQ_GET_MPA_CRC_VALID(ctrl0)	    bVAL1(ctrl0, 5)
#define	VXGE_HAL_TCP_PROMO_REQ_MPA_CRC_VALID		    mBIT(5)

#define	VXGE_HAL_TCP_PROMO_REQ_GET_TX_MARK_STAT(ctrl0)	    bVAL1(ctrl0, 6)
#define	VXGE_HAL_TCP_PROMO_REQ_TX_MARK_STAT		    mBIT(6)

#define	VXGE_HAL_TCP_PROMO_REQ_GET_RX_MARK_STAT(ctrl0)	    bVAL1(ctrl0, 7)
#define	VXGE_HAL_TCP_PROMO_REQ_RX_MARK_STAT		    mBIT(7)

#define	VXGE_HAL_TCP_PROMO_REQ_GET_IWLS_ZERO_OK(ctrl0)	    bVAL1(ctrl0, 14)
#define	VXGE_HAL_TCP_PROMO_REQ_IWLS_ZERO_OK		    mBIT(14)

#define	VXGE_HAL_TCP_PROMO_REQ_GET_IWWS_ZERO_OK(ctrl0)	    bVAL1(ctrl0, 15)
#define	VXGE_HAL_TCP_PROMO_REQ_IWWS_ZERO_OK		    mBIT(15)

#define	VXGE_HAL_TCP_PROMO_REQ_GET_IBWRR_EN(ctrl0)	    bVAL1(ctrl0, 22)
#define	VXGE_HAL_TCP_PROMO_REQ_IBWRR_EN			    mBIT(22)

#define	VXGE_HAL_TCP_PROMO_REQ_GET_IBRR_EN(ctrl0)	    bVAL1(ctrl0, 23)
#define	VXGE_HAL_TCP_PROMO_REQ_IBRR_EN			    mBIT(23)

	u64			srq_pd_id;

	u64			iwarp_qp_pd_id;

	u64			control_1;
#define	VXGE_HAL_TCP_PROMO_REQ_GET_CQRQ_LLSFD_OFF(ctrl1)    bVAL16(ctrl1, 0)
#define	VXGE_HAL_TCP_PROMO_REQ_CQRQ_LLSFD_OFF(val)	    vBIT(val, 0, 16)

#define	VXGE_HAL_TCP_PROMO_REQ_GET_QP_LIMIT_VALUE(ctrl1)    bVAL16(ctrl1, 16)
#define	VXGE_HAL_TCP_PROMO_REQ_QP_LIMIT_VALUE(val)	    vBIT(val, 16, 16)

#define	VXGE_HAL_TCP_PROMO_REQ_GET_RQ_MAX_LENGTH(ctrl1)	    bVAL16(ctrl1, 32)
#define	VXGE_HAL_TCP_PROMO_REQ_RQ_MAX_LENGTH(val)	    vBIT(val, 32, 16)

#define	VXGE_HAL_TCP_PROMO_REQ_GET_QP_LIMIT_ARMED(ctrl1)    bVAL8(ctrl1, 56)
#define	VXGE_HAL_TCP_PROMO_REQ_QP_LIMIT_ARMED(val)	    vBIT(val, 56, 8)

	u64			control_2;
#define	VXGE_HAL_TCP_PROMO_REQ_GET_CQRQ_INST_NUM(ctrl2)	    bVAL8(ctrl2, 0)
#define	VXGE_HAL_TCP_PROMO_REQ_CQRQ_INST_NUM(val)	    vBIT(val, 0, 8)

#define	VXGE_HAL_TCP_PROMO_REQ_GET_CQRQ_ID(ctrl2)	    bVAL24(ctrl2, 8)
#define	VXGE_HAL_TCP_PROMO_REQ_CQRQ_ID(val)		    vBIT(val, 8, 24)

#define	VXGE_HAL_TCP_PROMO_REQ_GET_SRQ_INST_NUM(ctrl2)	    bVAL8(ctrl2, 32)
#define	VXGE_HAL_TCP_PROMO_REQ_SRQ_INST_NUM(val)	    vBIT(val, 32, 8)

#define	VXGE_HAL_TCP_PROMO_REQ_GET_SRQ_ID(ctrl2)	    bVAL24(ctrl2, 40)
#define	VXGE_HAL_TCP_PROMO_REQ_SRQ_ID(val)		    vBIT(val, 40, 24)

	u64			control_3;
#define	VXGE_HAL_TCP_PROMO_REQ_GET_RDMAP_VERSION(ctrl3)	    bVAL2(ctrl3, 6)
#define	VXGE_HAL_TCP_PROMO_REQ_RDMAP_VERSION(val)	    vBIT(val, 6, 2)
#define	VXGE_HAL_TCP_PROMO_REQ_RDMAP_VERSION_IETF	    1
#define	VXGE_HAL_TCP_PROMO_REQ_RDMAP_VERSION_RDMAC	    0

#define	VXGE_HAL_TCP_PROMO_REQ_GET_DDP_VERSION(ctrl3)	    bVAL2(ctrl3, 14)
#define	VXGE_HAL_TCP_PROMO_REQ_DDP_VERSION(val)		    vBIT(val, 14, 2)
#define	VXGE_HAL_TCP_PROMO_REQ_DDP_VERSION_IETF_RDP_DDP	    1

#define	VXGE_HAL_TCP_PROMO_REQ_GET_MPA_FRAME_SIZE(ctrl3)    bVAL16(ctrl3, 32)
#define	VXGE_HAL_TCP_PROMO_REQ_MPA_FRAME_SIZE(val)	    vBIT(val, 32, 16)

#define	VXGE_HAL_TCP_PROMO_REQ_GET_MPA_INITIATOR(ctrl3)	    bVAL1(ctrl3, 55)
#define	VXGE_HAL_TCP_PROMO_REQ_MPA_INITIATOR		    mBIT(55)

#define	VXGE_HAL_TCP_PROMO_REQ_GET_BLOCK_PERMISSIONS(ctrl3) bVAL8(ctrl3, 56)
#define	VXGE_HAL_TCP_PROMO_REQ_BLOCK_PERMISSIONS(val)	    vBIT(val, 56, 8)

	u64			control_4;
#define	VXGE_HAL_TCP_PROMO_REQ_GET_IRD_LIMIT(ctrl4)	    bVAL24(ctrl4, 40)
#define	VXGE_HAL_TCP_PROMO_REQ_IRD_LIMIT(val)		    vBIT(val, 40, 24)

	u64			hw_cq_rr_ir;

}__hal_tcp_promote_to_iwarp_req_dmq_t;

/*
 * __hal_tcp_promote_to_iwarp_resp_umq_t - TCP promote_to_iwarp Response UMQ
 *
 * @control_0: Bits 0 to 31 - The Rx_IMSN is a 32b per-session constant that
 *		identifies the TCP SN where the first MPA marker starts. The
 *		adapter will know this value as part of the promotion to iWarp
 *		process. The host will need this value if it has to do MPA
 *		misalignment handling.
 *	       Bits 32 to 63 - Reserved.
 *
 * The TCP Promote to iWARP Response UMQ message is sent to the host to indicate
 * the status of promoting an existing bytestream session to iWARP mode.
 */
typedef  struct  __hal_tcp_promote_to_iwarp_resp_umq_t {
	u64			control_0;
#define	VXGE_HAL_TCP_PROMO_RESP_GET_RX_IMSN(ctrl0)	    bVAL32(ctrl0, 0)
#define	VXGE_HAL_TCP_PROMO_RESP_RX_IMSN(val)		    vBIT(val, 0, 32)
}__hal_tcp_promote_to_iwarp_resp_umq_t;

/*
 * __hal_tcp_modify_req_dmq_t - TCP Modify Request DMQ
 *
 * @control_0: Bits 0 to 4 - Reserved.
 *	       Bit 5 - If set then MPA CRCs are expected to be valid on received
 *		frames. The adapter always calculates MPA CRCs in the transmit
 *		direction. This field is ignored for a bytestream session.
 *	       Bit 6 - if MPA markers exist in the transmit direction or not.
 *		-- 1 - enable MPA markers in transmit frames
 *		-- 0 - disable MPA markers in transmit frames
 *		This field is ignored for a bytestream session.
 *	       Bit 7 - Whether markers exist in the receive direction or not.
 *		-- 1 - enable MPA markers in receive frames
 *		-- 0 - disable MPA markers in receive frames
 *	       If this is set to zero for an RDMA session,the RxPE drops all out
 *	       of order receive segments. This field is ignored for a bytestream
 *		session.
 *	       Bits 8 to 13 - Reserved.
 *	       Bits 14 to 15 - The RDMA version number
 *		  -- 01 - for IETF RNICs
 *		  -- 00 - for RDMAC RNICs
 *	       Bits 16 to 21 - Reserved
 *	       Bits 22 to 23 - Direct Data Placement Protocol Version
 *		  -- 01 - for protocol described in
 *		http://ietf.org/internet-drafts/draft-ietf-rddp-ddp-06.txt
 *	       Bits 24 to 39 - Reserved
 *	       Bits 40 to 47 - If set, the Queue Pair limit is armed at the
 *		iwarp_qp_limit_value. This field is ignored for a bytestream
 *		session.
 *	       Bits 48 to 63 - If the number of in-progress WQEs reaches this
 *		value and the Queue Pair limit is armed, the session produces
 *		an asynchronous event. This field is ignored for a bytestream
 *		session.
 * @control_1: Bits 0 to 15 - The maximum number of WQEs that this session is
 *		allowed to have in progress on this SRQ.
 *	       Bits 16 to 23 - Reserved
 *	       Bits 24 to 32 - Used to restrict session's ability to use special
 *		TOWI atoms. It is used to enforce checks such as preventing the
 *		QP from performing a Fast Register NSMR operation.
 *	       Bits 32 to 39 - Reserved
 *	       Bits 40 to 63 - Host supplied IRD limit for this session.
 *
 * The TCP Modify Request DMQ message is sent by the host to modify the
 * attributes associated with a bytestream or iWARP session.
 */
typedef  struct  __hal_tcp_modify_req_dmq_t {

	u64			control_0;
#define	VXGE_HAL_TCP_MODIFY_REQ_GET_MPA_CRC_VALID(ctrl0)    bVAL1(ctrl0, 5)
#define	VXGE_HAL_TCP_MODIFY_REQ_MPA_CRC_VALID		    mBIT(5)

#define	VXGE_HAL_TCP_MODIFY_REQ_GET_TX_MARK_MODE(ctrl0)	    bVAL1(ctrl0, 6)
#define	VXGE_HAL_TCP_MODIFY_REQ_TX_MARK_MODE		    mBIT(6)

#define	VXGE_HAL_TCP_MODIFY_REQ_GET_RX_MARK_MODE(ctrl0)	    bVAL1(ctrl0, 7)
#define	VXGE_HAL_TCP_MODIFY_REQ_RX_MARK_MODE		    mBIT(7)

#define	VXGE_HAL_TCP_MODIFY_REQ_GET_RDMAP_VERSION(ctrl0)    bVAL2(ctrl0, 14)
#define	VXGE_HAL_TCP_MODIFY_REQ_RDMAP_VERSION(val)	    vBIT(val, 14, 2)
#define	VXGE_HAL_TCP_MODIFY_REQ_RDMAP_VERSION_IETF	    1
#define	VXGE_HAL_TCP_MODIFY_REQ_RDMAP_VERSION_RDMAC	    0

#define	VXGE_HAL_TCP_MODIFY_REQ_GET_DDP_VERSION(ctrl0)	    bVAL2(ctrl0, 22)
#define	VXGE_HAL_TCP_MODIFY_REQ_DDP_VERSION(val)	    vBIT(val, 22, 2)
#define	VXGE_HAL_TCP_MODIFY_REQ_DDP_VERSION_IETF_RDP_DDP    1

#define	VXGE_HAL_TCP_MODIFY_REQ_GET_QP_LIMIT_ARMED(ctrl0)   bVAL8(ctrl0, 40)
#define	VXGE_HAL_TCP_MODIFY_REQ_QP_LIMIT_ARMED(val)	    vBIT(val, 40, 8)

#define	VXGE_HAL_TCP_MODIFY_REQ_GET_QP_LIMIT_VALUE(ctrl0)   bVAL16(ctrl0, 48)
#define	VXGE_HAL_TCP_MODIFY_REQ_QP_LIMIT_VALUE(val)	    vBIT(val, 48, 16)

	u64			control_1;
#define	VXGE_HAL_TCP_MODIFY_REQ_GET_RQ_MAX_LEN(ctrl1)	    bVAL16(ctrl1, 16)
#define	VXGE_HAL_TCP_MODIFY_REQ_RQ_MAX_LEN(val)		    vBIT(val, 0, 16)

#define	VXGE_HAL_TCP_MODIFY_REQ_GET_BLOCK_PERMISSIONS(ctrl1) bVAL8(ctrl1, 24)
#define	VXGE_HAL_TCP_MODIFY_REQ_BLOCK_PERMISSIONS(val)	    vBIT(val, 24, 8)

#define	VXGE_HAL_TCP_MODIFY_REQ_GET_IRD_LIMIT(ctrl1)	    bVAL24(ctrl1, 40)
#define	VXGE_HAL_TCP_MODIFY_REQ_IRD_LIMIT(val)		    vBIT(val, 40, 24)

}__hal_tcp_modify_req_dmq_t;

/*
 * __hal_tcp_delete_resp_umq_t - The TCP Delete Response UMQ
 *
 * @session_opaque_handle: Session opaque handle
 *
 * The TCP Delete Response UMQ message is received by the host after deleting
 * a TCP connection on the adapter.
 */
typedef  struct  __hal_tcp_delete_resp_umq_t {
	u64			session_opaque_handle;
}__hal_tcp_delete_resp_umq_t;

/*
 * __hal_tcp_update_rxwin_req_dmq_t - TCP Update RXWIN DMQ
 *
 * @control_0: Bits 0 to 28 - Reserved
 *	       Bit 29 - Does the rcv_buf field contain a value to be updated?
 *		  1 = rcv_buf field is valid
 *		  0 = rcv_buf field is not valid
 *	       Bit 30 - Does the rcv_buf_size field contain a value to be
 *              updated?
 *		  1 = rcv_buf_size field is valid
 *		  0 = rcv_buf_size is not valid
 *	       Bit 31 - Does the rcv_max_win field contain a value to be
 *              updated?
 *		  1 = rcv_max_win valid
 *		  0 = rcv_max_win not valid
 *	       Bits 32 to 63 -For bytestream sessions:lowest TCP sequence number
 *	       buffered in the connection's receive buffer. This is updated by
 *	       the host as data is consumed by the ULP/application.Must increase
 *	       monotonically to avoid window shrinks(either check in PE when
 *	       host updates it, or just make it a requirement of the host).
 *	       For iWARP sessions: the DDP mainland RHS TCP sequence number.
 *	       Represents the last aligned DDP mainland placement. The DDP
 *	       mainland edge differs from the TCP mainland edge (rcv_nxt) if MPA
 *	       misaligned frames are arriving (ie session is ""DDP OOO"" and
 *	       iwarp_align_state != aligned). This field is only updated when
 *	       misaligned. When aligned, including in the normal fast path DDP
 *	       in order case, this is not updated to save fast path instructions
 *	       instead rcv_nxt defines both the TCP and DDP mainland edge.
 *	       Supplied by host for passive open. Initialized by adapter for
 *	       active open on receipt of SYN/ACK or SYN segment.
 *	       In both cases the proper initial value is IRS+1(same as rcv_nxt)
 *	       This is set equal to rcv_nxt on promote to iWARP
 * @control_1: Bits 0 to 31 - Maximum receive buffer size that the connection is
 *		allowed to consume. Units are TCP payload bytes (ie sequence
 *		number bytes).
 *	       Bits 32 to 63 - Maximum value for receive window to advertise
 *		This can be less than rcv_buf_size, so that in cases when the
 *		transmit rate is not limited by the receive application, the
 *		advertised window will remain constant. This is allows less
 *		frequent buffer usage updates from host without causing bursty
 *		behaviour from the sender.
 *
 * This message is used for receive window updates, both for rx window flow
 * control updates (updates to rcv_buf as data is consumed by the application
 * on the host) and for maximum receive window size updates (when the receive
 * buffer size changes on the host)
 * If the update is successful there will be no response from the adapter.
 * If an error occurred the adapter will generate a TCP Update Failure UMQ
 * message with the reason for the failure.
 */
typedef  struct __hal_tcp_update_rxwin_req_dmq_t {
	u64			control_0;
#define	VXGE_HAL_TCP_UPDATE_RXWIN_GET_RB_VALID(ctrl0)	    bVAL1(ctrl0, 29)
#define	VXGE_HAL_TCP_UPDATE_RXWIN_RB_VALID		    mBIT(29)

#define	VXGE_HAL_TCP_UPDATE_RXWIN_GET_RBSZ_VALID(ctrl0)	    bVAL1(ctrl0, 30)
#define	VXGE_HAL_TCP_UPDATE_RXWIN_RBSZ_VALID		    mBIT(30)

#define	VXGE_HAL_TCP_UPDATE_RXWIN_GET_RW_VALID(ctrl0)	    bVAL1(ctrl0, 31)
#define	VXGE_HAL_TCP_UPDATE_RXWIN_RW_VALID		    mBIT(31)

#define	VXGE_HAL_TCP_UPDATE_RXWIN_GET_RBUF(ctrl0)	    bVAL32(ctrl0, 32)
#define	VXGE_HAL_TCP_UPDATE_RXWIN_RBUF(val)		    vBIT(val, 32, 32)

	u64			control_1;

#define	VXGE_HAL_TCP_UPDATE_RXWIN_GET_RBUF_SIZE(ctrl1)	    bVAL32(ctrl1, 0)
#define	VXGE_HAL_TCP_UPDATE_RXWIN_RBUF_SIZE(val)	    vBIT(val, 0, 32)

#define	VXGE_HAL_TCP_UPDATE_RXWIN_GET_RWND_MAX(ctrl1)	    bVAL32(ctrl1, 32)
#define	VXGE_HAL_TCP_UPDATE_RXWIN_RWND_MAX(val)		    vBIT(val, 32, 32)

} __hal_tcp_update_rxwin_req_dmq_t;

/*
 * __hal_tcp_update_mss_req_dmq_t - TCP Update MSS DMQ
 *
 * @control_0: Bits 0 to 29 - Reserved
 *	       Bit 30 - Does the send_mss field contain a value to be updated?
 *		  1 = snd_mss field is valid
 *		  0 = snd_mss field is not valid
 *	       Bit 31 - Does the send_mss_limit field contain a value to be
 *		updated?
 *		  1 = snd_mss_limit field is valid
 *		  0 = snd_mss_limit field is not valid
 *	       Bits 32 to 47 - Max segment size for transmit.This is defined as
 *		the maximum permitted TCP payload size, accounting for the
 *		maximum TCP option size (40B if both SACK and timestamps are
 *		enabled)-snd_mss = MSS-max_tcp_options max_tcp_options is 40B
 *		assuming both SACK and timestamps are enabled (note that the
 *		firmware might choose to use a fixed value of 40 even if SACK or
 *		timestamps are not enabled - this is TBD). The definition is
 *		slightly different from most TCP stacks which store the true MSS
 *		value (max payload assuming no IP or TCP options). It allows the
 *		same amount of payload whether SACK is sent or not - important
 *		for sending aligned DDP segments. Most stacks typically pack the
 *		max possible data into a segment,and resegment on retransmit
 *		if the options size has grown.When sending the MSS option on SYN
 *		or SYN/ACK segments, the PE firmware calculates the true MSS
 *		value based on this field (ie MSS = snd_mss + max_tcp_options).
 *		Supplied by host for both active and passive open.In active open
 *		case,adapter may adjust value to the minimum of the host
 *		supplied value and the value from the received MSS option.
 *	       Bits 48 to 63 - Value of MSS option received from far end on the
 *		SYN or SYN/ACK segment.The PE applies this as a limit on snd_mss
 *		when the host signals path MTU changes as follows -
 *		snd_mss = min(new_mss_from_host, snd_mss_limit)-max_tcp_options
 *		Supplied by host for passive open.
 *		Set by adapter for active open based on the MSS option value on
 *		the received SYN or SYN/ACK segment.
 *
 * This DMQ is sent by the host to the adapter to update the MSS for the session
 * If the update is successful there will be no response from the adapter. If an
 * error occurred the adapter will generate a TCP Update Failure UMQ message
 * with the reason for the failure.
 */
typedef  struct __hal_tcp_update_mss_req_dmq_t {
	u64			control_0;
#define	VXGE_HAL_TCP_UPDATE_MSS_GET_MSS_VALID(ctrl0)	    bVAL1(ctrl0, 30)
#define	VXGE_HAL_TCP_UPDATE_RXWIN_MSS_VALID		    mBIT(30)

#define	VXGE_HAL_TCP_UPDATE_MSS_GET_MSS_LIM_VALID(ctrl0)    bVAL1(ctrl0, 31)
#define	VXGE_HAL_TCP_UPDATE_MSS_MSS_LIM_VALID		    mBIT(31)

#define	VXGE_HAL_TCP_UPDATE_MSS_GET_MSS(ctrl0)		    bVAL16(ctrl0, 32)
#define	VXGE_HAL_TCP_UPDATE_MSS_MSS(val)		    vBIT(val, 32, 16)

#define	VXGE_HAL_TCP_UPDATE_MSS_GET_MSS_LIMIT(ctrl0)	    bVAL16(ctrl0, 48)
#define	VXGE_HAL_TCP_UPDATE_MSS_MSS_LIMIT(val)		    vBIT(val, 48, 16)

} __hal_tcp_update_mss_req_dmq_t;

/*
 * __hal_tcp_update_ip_header_req_dmq_t - TCP Update IP Header DMQ
 *
 * @control_0: Bits 0 to 21 - Reserved
 *	       Bit 22 - Does the ip_tos field contain a value to be updated?
 *		 1 = ip_tos valid
 *		 0 = ip_tos not valid
 *	       Bit 23 - Does the ip_flowlabel field contain a value to be
 *		updated?
 *		 1 = ip_flowlabel valid
 *		 0 = ip_flowlabel not valid"
 *	       Bit 24 to 31 - Value to place in the IPv4 TOS field or the Ipv6
 *		Traffic Class field. The 2 lower bits are overwritten if ECN
 *		is enabled.
 *	       Bits 32 to 43 - Reserved
 *	       Bit 44 to 63 - Value to place in the IPv6 Flow Label field.
 *		This might be moved to the H2L context.
 *
 * The TCP Update IP Header DMQ is used to update the IP TOS and IP flow label
 * in the IP header. If the update is successful there will be no response from
 * the adapter. If an error occurred the adapter will generate a TCP Update
 * Failure UMQ message with the reason for the failure.
 */
typedef  struct __hal_tcp_update_ip_header_req_dmq_t {
	u64			control_0;
#define	VXGE_HAL_TCP_UPDATE_IPHEAD_GET_TOS_VALID(ctrl0)	    bVAL1(ctrl0, 22)
#define	VXGE_HAL_TCP_UPDATE_IPHEAD_TOS_VALID		    mBIT(22)

#define	VXGE_HAL_TCP_UPDATE_IPHEAD_GET_FL_VALID(ctrl0)	    bVAL1(ctrl0, 23)
#define	VXGE_HAL_TCP_UPDATE_IPHEAD_FL_VALID		    mBIT(23)

#define	VXGE_HAL_TCP_UPDATE_IPHEAD_GET_TOS(ctrl0)	    bVAL8(ctrl0, 24)
#define	VXGE_HAL_TCP_UPDATE_IPHEAD_TOS(val)		    vBIT(val, 24, 8)

#define	VXGE_HAL_TCP_UPDATE_IPHEAD_GET_FLOW_LABEL(ctrl0)    bVAL20(ctrl0, 44)
#define	VXGE_HAL_TCP_UPDATE_IPHEAD_FLOW_LABEL(val)	    vBIT(val, 44, 20)

} __hal_tcp_update_ip_header_req_dmq_t;

/*
 * __hal_tcp_update_keepalive_req_dmq_t - TCP Update Keepalive DMQ
 *
 * @control_0: Bits 0 to 46 - Reserved
 *	       Bit 47 - Enable for keepalive probes for this connection.
 *	       Bits 48 to 63 - Keepalive maximum probe count: maximum number of
 *		unanswered keepalive probes before giving up on session.
 * @control_1: Bits 0 to 31 - Keepalive idle timer: the idle time before sending
 *		the first keepalive probe (default is 2 hours).
 *	       Bits 32 to 63 - Keepalive retry time interval: idle time before
 *		resending an unanswered keepalive probe.
 *
 * The TCP Update Keepalive DMQ message is sent from the host to the adapter to
 * update the keep-alive timer for the session.If the update is successful there
 * will be no response from the adapter.If an error occurred the adapter will
 * generate a TCP Update Failure UMQ message with the reason for the failure.
 */
typedef  struct __hal_tcp_update_keepalive_req_dmq_t {
	u64			control_0;
#define	VXGE_HAL_TCP_UPDATE_KPA_GET_KPA_OK(ctrl0)	    bVAL1(ctrl0, 47)
#define	VXGE_HAL_TCP_UPDATE_KPA_KPA_OK			    mBIT(47)

#define	VXGE_HAL_TCP_UPDATE_KPA_GET_PROBES_MAX(ctrl0)	    bVAL16(ctrl0, 48)
#define	VXGE_HAL_TCP_UPDATE_KPA_PROBES_MAX(val)		    vBIT(val, 48, 16)

	u64			control_1;
#define	VXGE_HAL_TCP_UPDATE_KPA_GET_IDLE_TIME(ctrl1)	    bVAL32(ctrl1, 0)
#define	VXGE_HAL_TCP_UPDATE_KPA_IDLE_TIME(val)		    vBIT(val, 0, 32)

#define	VXGE_HAL_TCP_UPDATE_KPA_GET_RETRY_TIME(ctrl1)	    bVAL32(ctrl1, 32)
#define	VXGE_HAL_TCP_UPDATE_KPA_RETRY_TIME(val)		    vBIT(val, 32, 32)

} __hal_tcp_update_keepalive_req_dmq_t;

/*
 * __hal_tcp_relink_to_nce_req_dmq_t - TCP Relink to NCE Request DMQ
 *
 * @control_0: Bits 0 to 39 - Reserved.
 *	       Bits 40 to 63 - NCE ID that this session is being linked to.
 *
 * The TCP Relink to NCE Request DMQ would be used to change the NCE entry
 * associated with a particular bytestream or iWARP session. This message
 * could be used to change the NCE of a group of sessions if a particular
 * path went down and need to be replaced by a new path.
 * The host is responsible for tracking the mapping of sessions to NCEs so
 * that when de-allocating an NCE it does not de-allocate on that is still
 * in use by a particular session.
 */
typedef  struct  __hal_tcp_relink_to_nce_req_dmq_t {
	u64			control_0;
#define	VXGE_HAL_TCP_RLINK_NCE_REQ_GET_NCE_ID(ctrl0)	    bVAL24(ctrl0, 40)
#define	VXGE_HAL_TCP_RLINK_NCE_REQ_NCE_ID(val)		    vBIT(val, 40, 24)

}__hal_tcp_relink_to_nce_req_dmq_t;

/*
 * __hal_lro_open_req_dmq_t - LRO Open Request DMQ
 *
 * @session_opaque_handle:An opaque handle used by the host to reference data
 *		structures in host memory related to this LRO session
 * @control_0: Bits 0 to 7 - Reserved
 *	       Bits 8 to 15 - There are two modes of LRO and the value of this
 *		field determines which mode is being used:
 *		0 = Pre-configured or session based LRO
 *		1 = Adaptive LRO
 *		In Adaptive LRO the adapter uses a block of session context
 *		space on the adapter to optimize merging performance. To this
 *		it needs a base Session ID and the number of session context
 *		records reserved for this purpose. The Session ID (SID) passed
 *		in the DMQ header defines the base SID to use for Adaptive LRO
 *		sessions. This base SID must be zero, or a multiple of 1024.
 *		The num_lro_sessions determines how many session context
 *		records are reserved for it's use.
 *	       Bits 16 to 31 - The number of LRO sessions to create.This must
 *		be 1 for Pre-Configured LRO mode and a multiple of 1024 (up to
 *		maximum of 65k) for Adaptive LRO.
 *	       Bits 32 to 47 - TCP source port
 *	       Bits 48 to 63 - TCP destination port
 * @ip_sa:IPv4 or IPv6 source address
 *		  ip_sa.high - High 64 bits
 *		  ip_sa.low - Low 64 bits
 * @ip_da:IPv4 or IPv6 destination address
 *		  ip_da.high - High 64 bits
 *		  ip_da.low - Low 64 bits
 * @control_1: Bits 0 to 15 - Reserved
 *	       Bit 16 - Does this session use IPv6 addressing?
 *		 1 - IPv6, 0 - IPv4
 *	       Bit 17 - Use vlan Id
 *	       Bits 18 to 19 - Reserved
 *	       Bits 20 to 31 - Vlan Id
 *	       Bits 32 to 55 - Reserved
 *	       Bits 56 to 63 - If no_wqe_cnt is >= this threshold, the RPE will
 *		inform the RxPE,which will most likely initiate session teardown
 * @control_2: Bits 0 to 7 - Instance number of the LRO CQRQ for the LRO
 *	       Bits 8 to 31 - Adapter ID of the LRO CQRQ for the LRO
 *	       Bits 32 to 39 - Instance number of the LRO SRQ for the LRO
 *	       Bits 40 to 63 - Adapter ID of the LRO SRQ for the LRO
 * @control_3: Bits 0 to 4 - Reserved
 *	       Bit 5 - The RPE detects duplicate frames if this bit is set
 *	       Bit 6 - Merging of frames with TCP payload is enabled if this
 *	       Bit is set
 *	       Bit 7 - Merging of frames without TCP payload is enabled if this
 *	       Bit is set
 *	       Bits 8 to 19 - Reserved
 *	       Bits 20 to 21 - llc_header_mode can take the values:
 *		0 - place the length of the frame payload in the Ethernet header
 *		even if this length exceeds 1500 bytes.
 *		1 - place the length of the frame payload in the Ethernet header
 *		if the length is <= 1500 bytes. For all lengths > 1500 bytes,
 *		saturate the L/T field at 1500 (0x5dc).
 *		2 - place the length of the frame payload in the Ethernet header
 *		if the length is <= 1500 bytes. For all lengths > 1500 bytes,
 *		convert to jumbo frames by setting the L/T field to 0x8870.
 *	       Bits 22 to 23 - snap_header_mode can take the values
 *		0 - place the length of the frame payload in the Ethernet header
 *		even if this length exceeds 1500 bytes.
 *		1 - place the length of the frame payload in the Ethernet header
 *		if the length is <= 1500 bytes. For all lengths > 1500 bytes,
 *		saturate the L/T field at 1500 (0x5dc).
 *		2 - place the length of the frame payload in the Ethernet header
 *		if the length is <= 1500 bytes. For all lengths > 1500 bytes,
 *		convert to jumbo frames by setting the L/T field to 0x8870."
 *	       Bits 24 to 29 - Reserved
 *	       Bit 30 - If set,Explicit Congestion Notification will be checked
 *		for this session.If the congestion-experienced codepoint is set,
 *		the frame will be considered non-mergeable (left or right).
 *	       Bit 31 - If set, enable for ECN with nonces. The LRO behaviour
 *		is the same as for session_ecn_enabled, except the IP.ECN and
 *		TCP.NS fields are updated for each frame of the merge.
 *	       Bits 32 to 37 - Reserved
 *	       Bits 38 to 39 - 2'b00 -  scatter mode A
 *				2'b01 - scatter mode B
 *				2'b10 - scatter mode C
 *	       Bits 40 to 44 - Reserved
 *	       Bits 45 to 47 - 3'b001 -  1-buffer mode
 *				3'b011 -  3-buffer mode
 *				3'b101 -  5-buffer mode"
 *	       Bits 48 to 63 - Reserved
 * @control_4: Bits 0 to 23 - Left mergeability mask for frames with TCP payload
 *	       Bits 24 to 31 - Mask used when detecting duplicate or window
 *		update Acks
 *	       Bits 32 to 39 - Mask used when detecting maskable
 *		non-mergeability conditions
 *	       Bits 40 to 63 - Left mergeability mask for frames without TCP
 *		payload
 * @control_5: Bits 0 to 15 - For IPv4, the maximum size of the merged IP
 *		datagram (maximum value 64KB - 1). For IPv6, the maximum size
 *		of the merged IP datagram payload (maximum value 64KB - 1).
 *		Used when determining right mergeability criteria. Note that
 *		the underlying WQE structure makes it highly unlikely that the
 *		merge will ever reach the 64KB - 1 maximum value.
 *	       Bits 16 to 23 - Reserved
 *	       Bits 24 to 31 - Right mergeability mask for frames with TCP
 *		payload
 *	       Bits 32 to 47 - Reserved
 *	       Bits 48 to 55 - Number of frames allowed in the merge. Valid
 *		values 1 - 255. Undefined behaviour if a value of 0 is used
 *	       Bits 40 to 47 - Push flag threshold in merge. Valid values
 *		1 - 255. When this number of PSH flags is reached, the merge
 *		is ended
 *	       Bits 56 to 63 - Push flag threshold in merge. Valid values
 *		1 - 255. When this number of PSH flags is reached, the merge
 *		is ended
 * @control_6: Bits 0 to 15 - Reserved
 *	       Bits 16 to 31 - If the frame is shorter than the MTU threshold,
 *		the merge is ended (however, frames with TCP payload lengths
 *		of zero will also be allowed to account for the combined data
 *		and ack merge case). Values 0 - (2^16 - 1).
 *	       Bits 32 to 47 - If the TCP payload is shorter than the MSS
 *		threshold, the merge is ended (however, payload lengths of zero
 *		will also be allowed to account for the combined data and ack
 *		merge case). Values 0 - (2^16 - 1).
 *	       Bits 48 to 63 - Used for monotonically non-decreasing timestamp
 *		comparisons. Valid values 0 to (2^16 - 1).
 * @control_7: Bits 0 to 7 - Reserved
 *	       Bits 8 to 31 - Used to ensure that the ack number does not
 *		advance too far from the first ack number in the merge.In theory
 *		the maximum permitted value is 2^31  1, but in practice, this
 *		should be smaller than the session's maximum allowed window size
 *	       Bits 32 to 47 - Length of OD 0 for this S-RQ.OD lengths are only
 *		required for LRO sessions because of the format of the WQE for
 *		LRO sessions.
 *	       Bits 48 to 63 - Length of OD 1 for this S-RQ
 * @control_8: Bits 0 to 15 - Length of OD 2 for this S-RQ
 *	       Bits 16 to 31 - Length of OD 3 for this S-RQ
 *	       Bits 32 to 47 - Length of OD 4 for this S-RQ
 *	       Bits 48 to 63 - The minimum number of bytes that must be free in
 *		an in-progress WQE if it is to be considered right-mergeable.
 *
 * The LRO Open Request DMQ message is sent by the host to open a LRO connection
 * on the adapter. There is no PE context for an LRO session. The PE is involved
 * for timer purposes and transferring messages to the RPE but it contains no
 * session context.
 */
typedef  struct  __hal_lro_open_req_dmq_t {
	u64			session_opaque_handle;

	u64			control_0;
#define	VXGE_HAL_LRO_OPEN_REQ_GET_LRO_MODE(ctrl0)	    bVAL8(ctrl0, 8)
#define	VXGE_HAL_LRO_OPEN_REQ_LRO_MODE(val)		    vBIT(val, 8, 8)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_NUM_LRO_SESSIONS(ctrl0)    bVAL16(ctrl0, 16)
#define	VXGE_HAL_LRO_OPEN_REQ_NUM_LRO_SESSIONS(val)	    vBIT(val, 16, 16)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_TCP_SRC_PORT(ctrl0)	    bVAL16(ctrl0, 32)
#define	VXGE_HAL_LRO_OPEN_REQ_TCP_SRC_PORT(val)		    vBIT(val, 32, 16)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_TCP_DEST_PORT(ctrl0)	    bVAL16(ctrl0, 48)
#define	VXGE_HAL_LRO_OPEN_REQ_TCP_DEST_PORT(val)	    vBIT(val, 48, 16)

	struct {
		u64		high;
		u64		low;
	}			ip_sa;

	struct {
		u64		high;
		u64		low;
	}			ip_da;

	u64			control_1;
#define	VXGE_HAL_LRO_OPEN_REQ_GET_USE_IPV6(ctrl1)	    bVAL1(ctrl1, 16)
#define	VXGE_HAL_LRO_OPEN_REQ_USE_IPV6			    mBIT(16)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_USE_VLAN(ctrl1)	    bVAL1(ctrl1, 17)
#define	VXGE_HAL_LRO_OPEN_REQ_USE_VLAN			    mBIT(17)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_VLAN_ID(ctrl1)	    bVAL12(ctrl1, 20)
#define	VXGE_HAL_LRO_OPEN_REQ_VLAN_ID(val)		    vBIT(val, 20, 12)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_NO_WQE_THRESHOLD(ctrl1)    bVAL8(ctrl1, 56)
#define	VXGE_HAL_LRO_OPEN_REQ_NO_WQE_THRESHOLD(val)	    vBIT(val, 56, 8)

	u64			control_2;
#define	VXGE_HAL_LRO_OPEN_REQ_GET_LRO_CQRQ_INST_NUM(ctrl2)    bVAL8(ctrl2, 0)
#define	VXGE_HAL_LRO_OPEN_REQ_LRO_CQRQ_INST_NUM(val)	    vBIT(val, 0, 8)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_LRO_CQRQ_ID(ctrl2)	    bVAL24(ctrl2, 8)
#define	VXGE_HAL_LRO_OPEN_REQ_LRO_CQRQ_ID(val)		    vBIT(val, 8, 24)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_LRO_SRQ_INST_NUM(ctrl2)    bVAL8(ctrl2, 32)
#define	VXGE_HAL_LRO_OPEN_REQ_LRO_SRQ_INST_NUM(val)	    vBIT(val, 32, 8)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_LRO_SRQ_ID(ctrl2)	    bVAL24(ctrl2, 40)
#define	VXGE_HAL_LRO_OPEN_REQ_LRO_SRQ_ID(val)		    vBIT(val, 40, 24)

	u64			control_3;
#define	VXGE_HAL_LRO_OPEN_REQ_GET_DUP_ACK_DET_EN(ctrl3)	    bVAL1(ctrl3, 5)
#define	VXGE_HAL_LRO_OPEN_REQ_DUP_ACK_DET_EN		    mBIT(5)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_DATA_MERGE_EN(ctrl3)	    bVAL1(ctrl3, 6)
#define	VXGE_HAL_LRO_OPEN_REQ_DATA_MERGE_EN		    mBIT(6)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_ACK_MERGE_EN(ctrl3)	    bVAL1(ctrl3, 7)
#define	VXGE_HAL_LRO_OPEN_REQ_ACK_MERGE_EN		    mBIT(7)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_LLC_HDR_MODE(ctrl3)	    bVAL2(ctrl3, 20)
#define	VXGE_HAL_LRO_OPEN_REQ_LLC_HDR_MODE(val)		    vBIT(val, 20, 2)
#define	VXGE_HAL_LRO_OPEN_REQ_LLC_HDR_MODE_PLACE	    0
#define	VXGE_HAL_LRO_OPEN_REQ_LLC_HDR_MODE_SAT_LT	    1
#define	VXGE_HAL_LRO_OPEN_REQ_LLC_HDR_MODE_CONVERT	    2

#define	VXGE_HAL_LRO_OPEN_REQ_GET_SNAP_HDR_MODE(ctrl3)	    bVAL2(ctrl3, 22)
#define	VXGE_HAL_LRO_OPEN_REQ_SNAP_HDR_MODE(val)	    vBIT(val, 22, 2)
#define	VXGE_HAL_LRO_OPEN_REQ_SNAP_HDR_MODE_PLACE	    0
#define	VXGE_HAL_LRO_OPEN_REQ_SNAP_HDR_MODE_SAT_LT	    1
#define	VXGE_HAL_LRO_OPEN_REQ_SNAP_HDR_MODE_CONVERT	    2

#define	VXGE_HAL_LRO_OPEN_REQ_GET_SESS_ECN_EN(ctrl3)	    bVAL1(ctrl3, 30)
#define	VXGE_HAL_LRO_OPEN_REQ_SESS_ECN_EN		    mBIT(30)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_S_ECN_EN_NONCE(ctrl3)	    bVAL1(ctrl3, 31)
#define	VXGE_HAL_LRO_OPEN_REQ_S_ECN_EN_NONCE		    mBIT(31)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_SCATTER_MODE(ctrl3)	    bVAL2(ctrl3, 38)
#define	VXGE_HAL_LRO_OPEN_REQ_SCATTER_MODE(val)		    vBIT(val, 38, 2)
#define	VXGE_HAL_LRO_OPEN_REQ_SCATTER_MODE_B		    0x1
#define	VXGE_HAL_LRO_OPEN_REQ_SCATTER_MODE_C		    0x2

#define	VXGE_HAL_LRO_OPEN_REQ_GET_BUFFER_MODE(ctrl3)	    bVAL3(ctrl3, 45)
#define	VXGE_HAL_LRO_OPEN_REQ_BUFFER_MODE(val)		    vBIT(val, 45, 3)
#define	VXGE_HAL_LRO_OPEN_REQ_BUFFER_MODE_1_BUF		    0x1
#define	VXGE_HAL_LRO_OPEN_REQ_BUFFER_MODE_3_BUF		    0x3
#define	VXGE_HAL_LRO_OPEN_REQ_BUFFER_MODE_5_BUF		    0x5

	u64			control_4;
#define	VXGE_HAL_LRO_OPEN_REQ_GET_DATA_LM_MASK(ctrl4)	    bVAL24(ctrl4, 0)
#define	VXGE_HAL_LRO_OPEN_REQ_DATA_LM_MASK(val)		    vBIT(val, 0, 24)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_DUPACK_MASK(ctrl4)	    bVAL8(ctrl4, 24)
#define	VXGE_HAL_LRO_OPEN_REQ_DUPACK_MASK(val)		    vBIT(val, 24, 8)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_LRO_NM_MASK(ctrl4)	    bVAL8(ctrl4, 32)
#define	VXGE_HAL_LRO_OPEN_REQ_LRO_NM_MASK(val)		    vBIT(val, 32, 8)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_ACK_LM_MASK(ctrl4)	    bVAL24(ctrl4, 40)
#define	VXGE_HAL_LRO_OPEN_REQ_ACK_LM_MASK(val)		    vBIT(val, 40, 24)

	u64			control_5;
#define	VXGE_HAL_LRO_OPEN_REQ_GET_MAX_IP_DG_SIZE(ctrl5)	    bVAL16(ctrl5, 0)
#define	VXGE_HAL_LRO_OPEN_REQ_MAX_IP_DG_SIZE(val)	    vBIT(val, 0, 16)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_LRO_RM_MASK(ctrl5)	    bVAL8(ctrl5, 24)
#define	VXGE_HAL_LRO_OPEN_REQ_LRO_RM_MASK(val)		    vBIT(val, 24, 8)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_FRAME_TH(ctrl5)	    bVAL8(ctrl5, 48)
#define	VXGE_HAL_LRO_OPEN_REQ_FRAME_TH(val)		    vBIT(val, 48, 8)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_PSH_TH(ctrl5)		    bVAL8(ctrl5, 56)
#define	VXGE_HAL_LRO_OPEN_REQ_PSH_TH(val)		    vBIT(val, 56, 8)

	u64			control_6;
#define	VXGE_HAL_LRO_OPEN_REQ_GET_MTU_TH(ctrl5)		    bVAL16(ctrl5, 16)
#define	VXGE_HAL_LRO_OPEN_REQ_MTU_TH(val)		    vBIT(val, 16, 16)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_MSS_TH(ctrl6)		    bVAL16(ctrl6, 16)
#define	VXGE_HAL_LRO_OPEN_REQ_MSS_TH(val)		    vBIT(val, 16, 16)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_MAX_TCP_TSV_DEL(ctrl6)    bVAL16(ctrl6, 48)
#define	VXGE_HAL_LRO_OPEN_REQ_MAX_TCP_TSV_DEL(val)	    vBIT(val, 48, 16)

	u64			control_7;
#define	VXGE_HAL_LRO_OPEN_REQ_GET_MAX_ACK_NBR_DEL(ctrl6)    bVAL24(ctrl6, 8)
#define	VXGE_HAL_LRO_OPEN_REQ_MAX_ACK_NBRV_DEL(val)	    vBIT(val, 8, 24)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_BUF0_LENGTH(ctrl7)	    bVAL16(ctrl7, 32)
#define	VXGE_HAL_LRO_OPEN_REQ_BUF0_LENGTH(val)		    vBIT(val, 32, 16)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_BUF1_LENGTH(ctrl7)	    bVAL16(ctrl7, 48)
#define	VXGE_HAL_LRO_OPEN_REQ_BUF1_LENGTH(val)		    vBIT(val, 48, 16)

	u64			control_8;
#define	VXGE_HAL_LRO_OPEN_REQ_GET_BUF2_LENGTH(ctrl7)	    bVAL16(ctrl7, 0)
#define	VXGE_HAL_LRO_OPEN_REQ_BUF2_LENGTH(val)		    vBIT(val, 0, 16)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_BUF3_LENGTH(ctrl7)	    bVAL16(ctrl7, 16)
#define	VXGE_HAL_LRO_OPEN_REQ_BUF3_LENGTH(val)		    vBIT(val, 16, 16)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_BUF4_LENGTH(ctrl8)	    bVAL16(ctrl8, 32)
#define	VXGE_HAL_LRO_OPEN_REQ_BUF4_LENGTH(val)		    vBIT(val, 32, 16)

#define	VXGE_HAL_LRO_OPEN_REQ_GET_MIN_SPARE_WQE_CAPACITY(ctrl8)\
							    bVAL16(ctrl8, 48)
#define	VXGE_HAL_LRO_OPEN_REQ_MIN_SPARE_WQE_CAPACITY(val)   vBIT(val, 48, 16)

}__hal_lro_open_req_dmq_t;

/*
 * __hal_lrp_delete_resp_umq_t - The LRO Delete Response UMQ
 *
 * @session_opaque_handle: Session opaque handle
 *
 * The LRO Delete Response UMQ message is received by the host after deleting
 * a LRO session on the adapter.
 */
typedef  struct  __hal_lro_delete_resp_umq_t {
	u64			session_opaque_handle;
}__hal_lro_delete_resp_umq_t;

/*
 * __hal_lro_session_candidate_umq_t - LRO Session Candidate Notification UMQ
 *
 * @control_0: Bits 0 to 15 - Layer 4 source port
 *	       Bits 16 to 31 - Layer 4 destination port
 *	       Bits 32 to 62 - Reserved for alignment
 *	       Bit 63 - 1'b1 - IPv4
 *			1'b0 - IPv6"
 * @ip_sa:IPv4 or IPv6 source address
 *		  ip_sa.high - High 64 bits
 *		  ip_sa.low - Low 64 bits
 * @ip_da:IPv4 or IPv6 destination address
 *		  ip_da.high - High 64 bits
 *		  ip_da.low - Low 64 bits
 *
 * This msg indicates to the host that the adapter's autoLRO feature has
 * identified a candidate LRO session. No response from the host is required.
 * (If the host did decide to act on this information from the adapter, the host
 * would use the usual LRO Open Request DMQ)
 */
typedef  struct __hal_lro_session_candidate_umq_t {
	u64			control_0;
#define	VXGE_HAL_LRO_SESS_CAND_GET_L4_SRC_PORT(ctrl0)	    bVAL16(ctrl0, 0)
#define	VXGE_HAL_LRO_SESS_CAND_L4_SRC_PORT(val)		    vBIT(val, 0, 16)

#define	VXGE_HAL_LRO_SESS_CAND_GET_L4_DEST_PORT(ctrl0)	    bVAL16(ctrl0, 16)
#define	VXGE_HAL_LRO_SESS_CAND_L4_DEST_PORT(val)	    vBIT(val, 16, 16)

#define	VXGE_HAL_LRO_SESS_CAND_GET_IPV4_NOT_IPV6(ctrl0)	    bVAL1(ctrl0, 63)
#define	VXGE_HAL_LRO_SESS_CAND_IPV4_NOT_IPV6		    mBIT(63)

	struct {
		u64		high;
		u64		low;
	}			ip_sa;

	struct {
		u64		high;
		u64		low;
	}			ip_da;


}__hal_lro_session_candidate_umq_t;

/*
 * __hal_spdm_open_req_dmq_t - The SPDM Open Request DMQ
 *
 * @session_opaque_handle:An opaque handle used by the host to reference data
 *		structures in host memory related to this SPDM session
 * @control_0: Bits 0 to 15 - Reserved
 *	       Bits 16 to 31 - Layer 4 source port
 *	       Bits 32 to 47 - Layer 4 destination port
 *	       Bit 48 - Does this session use IPv6 addressing?
 *		1 - IPv6, 0 - IPv0
 *	       Bit 49 - Indicates if this session uses Vlan
 *	       Bits 50 to 51 - Reserved
 *	       Bits 52 to 63 - vlan Id, used only if use_vlan is set
 * @ip_sa:IPv4 or IPv6 source address
 *		  ip_sa.high - High 64 bits
 *		  ip_sa.low - Low 64 bits
 * @ip_da:IPv4 or IPv6 destination address
 *		  ip_da.high - High 64 bits
 *		  ip_da.low - Low 64 bits
 *
 * The SPDM Open Request DMQ message is sent by the host to open an SPDM
 * connection on the adapter. There is no RPE or PE context for an SPDM session.
 * The ONE is not involved in this type of classification.
 */
typedef  struct __hal_spdm_open_req_dmq_t {
	u64			session_opaque_handle;

	u64			control_0;
#define	VXGE_HAL_SPDM_OPEN_REQ_GET_L4_SRC_PORT(ctrl0)	    bVAL16(ctrl0, 16)
#define	VXGE_HAL_SPDM_OPEN_REQ_L4_SRC_PORT(val)		    vBIT(val, 16, 16)

#define	VXGE_HAL_SPDM_OPEN_REQ_GET_L4_DEST_PORT(ctrl0)	    bVAL16(ctrl0, 32)
#define	VXGE_HAL_SPDM_OPEN_REQ_L4_DEST_PORT(val)	    vBIT(val, 32, 16)

#define	VXGE_HAL_SPDM_OPEN_REQ_GET_USE_IPV6(ctrl0)	    bVAL1(ctrl0, 48)
#define	VXGE_HAL_SPDM_OPEN_REQ_USE_IPV6			    mBIT(48)

#define	VXGE_HAL_SPDM_OPEN_REQ_GET_USE_VLAN(ctrl0)	    bVAL1(ctrl0, 49)
#define	VXGE_HAL_SPDM_OPEN_REQ_USE_VLAN			    mBIT(49)

#define	VXGE_HAL_SPDM_OPEN_REQ_GET_VLAN_ID(ctrl0)	    bVAL12(ctrl0, 52)
#define	VXGE_HAL_SPDM_OPEN_REQ_VLAN_ID(val)		    vBIT(val, 52, 12)

	struct {
		u64		high;
		u64		low;
	}			ip_sa;

	struct {
		u64		high;
		u64		low;
	}			ip_da;


}__hal_spdm_open_req_dmq_t;

/*
 * __hal_spdm_open_resp_umq_t - SPDM Open Response UMQ
 *
 * @control_0: Bits 0 to 31 - Classification hash used to classify the session
 *		context.
 *	       Bits 32 to 47 - Reserved.
 *
 * The SPDM Open Response UMQ message is sent in response to a SPDM Open Request
 * DMQ message to indicate the status of creating the SPDM session
 */
typedef  struct  __hal_spdm_open_resp_umq_t {
	u64			control_0;
#define	VXGE_HAL_SPDM_OPEN_RESP_GET_SESS_HASH(ctrl0)	    bVAL32(ctrl0, 0)
#define	VXGE_HAL_SPDM_OPEN_RESP_SESS_HASH(val)		    vBIT(val, 0, 32)
}__hal_spdm_open_resp_umq_t;

/*
 * __hal_spdm_delete_resp_umq_t - The SPDM Delete Response UMQ
 *
 * @session_opaque_handle: Session opaque handle
 *
 * The SPDM Delete Response UMQ message is received by the host after deleting
 * a SPDM session on the adapter.
 */
typedef  struct  __hal_spdm_delete_resp_umq_t {
	u64			session_opaque_handle;
}__hal_spdm_delete_resp_umq_t;

/*
 * __hal_session_query_resp_umq_t - The Session query response UMQ message
 *
 * @control_0: Bits 0 to 2 - The type of offloaded session. SPDM session will
 *		have a session_type of zero.
 *		-- 0 - Reserved
 *		-- 1 - LRO
 *		-- 2 - Bytestream
 *		-- 3 - iWarp
 *		-- 4 - MxP
 *		-- 5-7 - Reserved
 *	       Bits 3 to 7 - virtual path that this session is associated with.
 *	       Bits 8 to 15 - The session instance number.
 *	       Bit 16 - Indicates whether the session is an SPDM session or not
 *		-- 1 - The session is SPDM
 *		-- 0 - The session is not SPDM
 *		This field is isolated from the session_type strictly for the
 *		MAC's purposes.
 *	       Bit 17 - Indicates if this session uses IPv6
 *		-- 1 - This session uses IPv6 addressing
 *		-- 0 - This session does not use IPv6 addressing
 *	       Bit 18 - Indicates if this session uses a VLAN.
 *		-- 1 - This session uses a VLAN
 *		-- 0 - This session does not use a VLAN"
 *	       Bit 19 - Reserved for future use.
 *	       Bits 20 to 31 - The ID of the VLAN associated with the session.
 *		This is valid only if use_vlanid is set to 1.
 *	       Bits 32 to 63 - Receive direction Initial Marker Sequence Number
 *		Identifies the TCP sequence number of the first byte of the
 *		first MPA marker. Only meaningful if use_mpa_markers==1.
 * @control_1: Bits 0 to 31 - Used by the firmware to store the session hash for
 *		lookup purposes during session teardown.
 *	       Bits 32 to 47 - TCP Source Port
 *	       Bits 48 to 63 - TCP Destination Port
 * @ip_sa:	 IPv4 or IPv6 source address
 *	       Bits 0-127:   IPv6 address
 *	       Bits 96-127: IPv4 address
 * @ip_da:	 IPv4 or IPv6 destination address
 *	       Bits 0-127:   IPv6 address
 *	       Bits 96-127: IPv4 address
 * @control_2: Bits 0 to 61 - Reserved for future use.
 *	       Bit 62 - Indicates if this session uses MPA CRC32C. This is only
 *		applicable to iWARP sessions.
 *		-- 1 - The session uses MPA CRC32
 *		-- 0 - The session does not use MPA CRC"
 *	       Bit 63 - Indicates if this session uses MPA markers in the Rx
 *		direction.
 *		This is only applicable to iWARP sessions.
 *		-- 1 - The session expects MPA markers in Rx direction
 *		-- 0 - The session does not expect MPA markers in Rx direction
 *
 * The Session Query Response UMQ message is sent in response to a Session Query
 * Request DMQ message to return the attributes associated with the specified
 * session
 */
typedef struct __hal_session_query_resp_umq_t {
	u64			control_0;
#define	VXGE_HAL_SESSION_QUERY_RESP_GET_SESS_TYPE(ctrl0)    bVAL3(ctrl0, 0)
#define	VXGE_HAL_SESSION_QUERY_RESP_SESS_TYPE(val)	    vBIT(val, 0, 3)
#define	VXGE_HAL_SESSION_QUERY_RESP_SESS_TYPE_LRO	    1
#define	VXGE_HAL_SESSION_QUERY_RESP_SESS_TYPE_BYTE_STREAM   2
#define	VXGE_HAL_SESSION_QUERY_RESP_SESS_TYPE_IWARP	    3
#define	VXGE_HAL_SESSION_QUERY_RESP_SESS_TYPE_MXP	    4

#define	VXGE_HAL_SESSION_QUERY_RESP_GET_VP_ID(ctrl0)	    bVAL5(ctrl0, 5)
#define	VXGE_HAL_SESSION_QUERY_RESP_VP_ID(val)		    vBIT(val, 3, 5)

#define	VXGE_HAL_SESSION_QUERY_RESP_GET_INST_NUM(ctrl0)	    bVAL8(ctrl0, 8)
#define	VXGE_HAL_SESSION_QUERY_RESP_INST_NUM(val)	    vBIT(val, 8, 8)

#define	VXGE_HAL_SESSION_QUERY_RESP_GET_IS_SPDM(ctrl0)	    bVAL1(ctrl0, 16)
#define	VXGE_HAL_SESSION_QUERY_RESP_IS_SPDM		    mBIT(16)

#define	VXGE_HAL_SESSION_QUERY_RESP_GET_USE_IPV6(ctrl0)	    bVAL1(ctrl0, 17)
#define	VXGE_HAL_SESSION_QUERY_RESP_USE_IPV6		    mBIT(17)

#define	VXGE_HAL_SESSION_QUERY_RESP_GET_USE_VLAN(ctrl0)	    bVAL1(ctrl0, 18)
#define	VXGE_HAL_SESSION_QUERY_RESP_USE_VLAN		    mBIT(18)

#define	VXGE_HAL_SESSION_QUERY_RESP_GET_VLAN_ID(ctrl0)	    bVAL12(ctrl0, 20)
#define	VXGE_HAL_SESSION_QUERY_RESP_VLAN_ID(val)	    vBIT(val, 20, 12)

#define	VXGE_HAL_SESSION_QUERY_RESP_GET_RX_IMSN(ctrl0)	    bVAL32(ctrl0, 32)
#define	VXGE_HAL_SESSION_QUERY_RESP_RX_IMSN(val)	    vBIT(val, 32, 32)

	u64			control_1;
#define	VXGE_HAL_SESSION_QUERY_RESP_GET_FW_CTRL(ctrl1)	    bVAL32(ctrl1, 0)
#define	VXGE_HAL_SESSION_QUERY_RESP_FW_CTRL(val)	    vBIT(val, 0, 32)

#define	VXGE_HAL_SESSION_QUERY_RESP_GET_SRC_PORT(ctrl1)	    bVAL16(ctrl1, 32)
#define	VXGE_HAL_SESSION_QUERY_RESP_SRC_PORT(val)	    vBIT(val, 32, 16)

#define	VXGE_HAL_SESSION_QUERY_RESP_GET_DEST_PORT(ctrl1)    bVAL16(ctrl1, 48)
#define	VXGE_HAL_SESSION_QUERY_RESP_DEST_PORT(val)	    vBIT(val, 48, 16)

	struct {
		u64		high;
		u64		low;
	}			ip_sa;

	struct {
		u64		high;
		u64		low;
	}			ip_da;

	u64			control_2;
#define	VXGE_HAL_SESSION_QUERY_RESP_GET_USE_MPA_CRC32C(ctrl2)	bVAL1(ctrl2, 62)
#define	VXGE_HAL_SESSION_QUERY_RESP_USE_MPA_CRC32C	    mBIT(62)

#define	VXGE_HAL_SESSION_QUERY_RESP_GET_USE_MPA_MARKERS(ctrl2)	bVAL1(ctrl2, 63)
#define	VXGE_HAL_SESSION_QUERY_RESP_USE_MPA_MARKERS	    mBIT(63)

} __hal_session_query_resp_umq_t;

/*
 * __hal_session_return_in_progress_wqes_umq_t - The Session return
 *			in progress WQEs UMQ message
 *
 * @control_0: Bits 0 to 31 - The number of WQE opaque handles in this message
 *	       Bits 32 to 63 - The number of WQE opaque handles left to send
 * @wqe_opaque_handles: Array of the 64 bit opaque handle of the WQE retrieved
 *		from SRQ. Use the 'number_of_wqes_in_msg' to determine how many
 *		of these handle is valid.
 *
 * This message is generated by the adapter during deletion of a session to
 * return any WQEs that may be in the in-progress list for the session. If a
 * WQE is inthe in-progress list it is owned by the session and cannot be
 * returned to the head of the WQE list for an SRQ because of ordering issues.
 * Therefore, it must be returned to the host at which point the host may
 * choose to destroy the resource or simply re-post the WQE for re-use.
 */
typedef  struct  __hal_session_return_in_progress_wqes_umq_t {
	u64			control_0;
#define	VXGE_HAL_SESSION_RETURN_WQES_GET_NUM_IN_MSG(ctrl0)  bVAL32(ctrl0, 0)
#define	VXGE_HAL_SESSION_RETURN_WQES_NUM_IN_MSG(val)	    vBIT(val, 0, 32)

#define	VXGE_HAL_SESSION_RETURN_WQES_GET_NUM_REMAIN(ctrl0)  bVAL32(ctrl0, 32)
#define	VXGE_HAL_SESSION_RETURN_WQES_NUM_REMAIN(val)	    vBIT(val, 32, 32)

	u64			wqe_opaque_handles[10];

}__hal_session_return_in_progress_wqes_umq_t;

/*
 * __hal_session_frame_write_umq_t - The Session Frame Write UMQ message
 *
 * @control_0: Bits 0 to 31 - Reserved
 *	       Bits 32 to 39 - A code provided to the RPE by the RxPE firmware.
 *		Other than the 8'd0 code, which is implemented in hardware, all
 *		these may be subject to redefinition at a later date.
 *		If the MSB is 0:
 *		-- 0:  Transfer of complete Ethernet frame initiated by RPE
 *		   hardware because ""send frame up debug UMQ"" context bit
 *		   was set.
 *		-- 1:  Transfer initiated by the RxPE firmware, but no other
 *		   transfer code was applicable.
 *		-- 2:  Contains an RDMA Terminate message or message segment
 *		If the MSB is 1, this is a misaligned or errored frame, and the
 *		subsequent bits have the following meanings:
 *		-- Bits 1 - 3:  frame_alignment:
 *		--	0 - SA
 *		--	1 - PSA
 *		--	2 - MA
 *		--	3 - PMA
 *		--	4 - MAX
 *		--	5 - PMAX
 *		--	6 - DME
 *		-- Bit 4:  fpdu_length_issue
 *		-- Bit 5:  mpa_marker_issue
 *		-- Bit 6:  mpa_crc32_issue
 *		-- Bit 7:  mpa_leftmost (If == 1, the first marker in the frame
 *		pointed to the first byte of TCP payload)
 *		All other values are reserved for future use."
 *	       Bits 40 to 42 - Reserved
 *	       Bits 43 to 47 - The number of markers contained in the frame, or
 *		portion of the frame, that will be passed up the UMQ. Helps the
 *		host to perform marker stripping.
 *	       Bits 48 to 49 - Reserved
 *	       Bits 50 to 63 - The byte index of the first marker,
 *		if nbr_mkrs > 0. The first byte of frame data passed to the host
 *		is considered to be Byte 0.
 * @fdata: This will contain the variable length data of the frame pulled from
 *		the frame buffer by the ROCRC. It will not actually be attached
 *		by the MXP but the MXP will leave space for placing the frame
 *		data when it generates the UMQ header.
 *
 * The Frame Write UMQ message is generated by the adapter in order to send
 * certain frames to the host via the UMQ path instead of the normal path.
 * Frames will be sent to the host under the following conditions:
 *	1) mis-aligned frames that the adapter cannot place
 *	2) during debugging to look at the contents of the frame
 * In addition to this, the RDMA terminate message will also be sent via a UMQ
 * message but in this case it will be sent in a TCP RDMA Terminate Received UMQ
 * message. Frames arriving in the UMQ will not have markers stripped. Instead
 * the host will be responsible for stripping markers and taking appropriate
 * action on the received frame.
 */
typedef  struct  __hal_session_frame_write_umq_t {
	u64			control_0;
#define	VXGE_HAL_SESSION_FRAME_WRITE_GET_UMQ_OPAQUE_DATA(ctrl0)	bVAL8(ctrl0, 32)
#define	VXGE_HAL_SESSION_FRAME_WRITE_UMQ_OPAQUE_DATA(val)    vBIT(val, 32, 8)

#define	VXGE_HAL_SESSION_FRAME_WRITE_GET_NUM_MARKERS(ctrl0) bVAL5(ctrl0, 43)
#define	VXGE_HAL_SESSION_FRAME_WRITE_NUM_MARKERS(val)	    vBIT(val, 43, 5)

#define	VXGE_HAL_SESSION_FRAME_WRITE_GET_FIRST_MARKER_IDX(ctrl0)\
							    bVAL14(ctrl0, 50)
#define	VXGE_HAL_SESSION_FRAME_WRITE_FIRST_MARKER_IDX(val)  vBIT(val, 50, 14)

	u64			fdata[2];

}__hal_session_frame_write_umq_t;

#define	VXGE_HAL_DOWN_MSG_HEADER_SIZE	24

/*
 * struct __hal_down_msg_t - Downward Message
 * @control_0: Bits 0 to 31 - Message ID Number.
 *	       Bits 32 to 47 - Message Type.
 *	       Bits 48 to 55 - Reserved for Message Id Selector.
 *	       Bit 56 - Interrupt Control.
 *	       Bit 57 - UMQ on Completion.
 *	       Bits 58 to 63 - Reserved.
 * @control_1: Contains Opaque handle which is copied to UMQ.
 * @control_2: Bits 0 to 3 - Protocol version number.
 *	       Bits 4 to 15 - Lenght of  the message.
 *	       Bits 16 to 63 - Reserved.
 * @union: union of dmq message structures
 *		__hal_nce_create_req_dmq_t		  nce_create;
 *		__hal_nce_update_mac_addr_req_dmq_t	  nce_update_mac;
 *		__hal_nce_update_rch_time_req_dmq_t	  nce_update_rch;
 *		__hal_cqrq_create_req_dmq_t		  cqrq_create;
 *		__hal_cqrq_modify_req_dmq_t		  cqrq_modify;
 *		__hal_cqrq_arm_req_dmq_t		  cqrq_arm;
 *		__hal_cqrq_fcqebw_req_dmq_t		  cqrq_fcqebw;
 *		__hal_srq_create_req_dmq_t		  srq_create;
 *		__hal_srq_modify_req_dmq_t		  srq_modify;
 *		__hal_srq_arm_req_dmq_t			  srq_arm;
 *		__hal_srq_fwqebw_req_dmq_t		  srq_fwqebw;
 *		__hal_srq_wqebadd_req_dmq_t		  srq_wqebadd;
 *		  __hal_srq_return_unused_wqes_req_dmq_t  srq_return_wqes;
 *		__hal_stag_allocate_nsmr_req_dmq_t	  stag_allocate_nsmr;
 *		__hal_stag_allocate_mw_req_dmq_t	  stag_allocate_mw;
 *		__hal_stag_deallocate_req_dmq_t		  stag_deallocate;
 *		__hal_stag_register_nsmr_req_dmq_t	  stag_reg_nsmr;
 *		__hal_stag_re_register_nsmr_req_dmq_t	  stag_rereg_nsmr;
 *		__hal_stag_register_smr_req_dmq_t	  stag_reg_smr;
 *		  __hal_stag_query_req_dmq_t		  stag_query;
 *		__hal_stag_validate_req_dmq_t		  stag_validate;
 *		__hal_stag_invalidate_req_dmq_t		  stag_invalidate;
 *		  __hal_stag_bind_mw_req_dmq_t		  stag_bind_mw;
 *		__hal_stag_fast_register_nsmr_req_dmq_t	  stag_frnsmr;
 *		__hal_tcp_open_req_dmq_t		  tcp_open;
 *		__hal_tcp_promote_to_iwarp_req_dmq_t	  tcp_promote_iwarp;
 *		__hal_tcp_modify_req_dmq_t		  tcp_modify;
 *		  __hal_tcp_update_rxwin_req_dmq_t	  tcp_rxwin;
 *		  __hal_tcp_update_mss_req_dmq_t	  tcp_mss;
 *		  __hal_tcp_update_ip_header_req_dmq_t	  tcp_iphead;
 *		  __hal_tcp_update_keepalive_req_dmq_t	  tcp_keepalive;
 *		  __hal_tcp_relink_to_nce_req_dmq_t	  tcp_relink;
 *		__hal_lro_open_req_dmq_t		  lro_open;
 *		__hal_spdm_open_req_dmq_t		  spdm_open;
 * Created and appended to the DMQ list by the host. The message is signaled
 * via the DMQ Doorbell Entry (Kernel Doorbell FIFOs). The message is
 * consumed by the adapter based message processor (from the PFC).
 * Please refer to UG for details.
 */
typedef __vxge_os_attr_cacheline_aligned struct __hal_down_msg_t {
	u64			control_0;
#define	VXGE_HAL_DOWN_MSG_GET_ID_NUMBER(ctrl0)		    bVAL32(ctrl0, 0)
#define	VXGE_HAL_DOWN_MSG_ID_NUMBER(val)		    vBIT(val, 0, 32)

#define	VXGE_HAL_DOWN_MSG_GET_MESSAGE_TYPE(ctrl0)	    bVAL16(ctrl0, 32)
#define	VXGE_HAL_DOWN_MSG_MESSAGE_TYPE(val)		    vBIT(val, 32, 16)

#define	VXGE_HAL_DOWN_MSG_GET_INTERRUPT_CONTROL(ctrl0)	    bVAL1(ctrl0, 56)
#define	VXGE_HAL_DOWN_MSG_INTERRUPT_CONTROL(val)	    mBIT(56)
#define	VXGE_HAL_DOWN_MSG_INTERRUPT_CONTROL_INT_REQUIRED    0x1
#define	VXGE_HAL_DOWN_MSG_INTERRUPT_CONTROL_NOT_REQUIRED    0x0

#define	VXGE_HAL_DOWN_MSG_GET_GEN_COMPL_UMQ(ctrl0)	    bVAL1(ctrl0, 57)
#define	VXGE_HAL_DOWN_MSG_GEN_COMPL_UMQ(val)		    mBIT(57)
#define	VXGE_HAL_DOWN_MSG_GEN_COMPL_UMQ_REQUIRED	    0x1
#define	VXGE_HAL_DOWN_MSG_GEN_COMPL_UMQ_NOT_REQUIRED	    0x0

	u64			control_1;

	u64			control_2;
#define	VXGE_HAL_DOWN_MSG_GET_PROTOCOL_VERSION(ctrl2)	    bVAL4(ctrl2, 0)
#define	VXGE_HAL_DOWN_MSG_PROTOCOL_VERSION(val)		    vBIT(val, 0, 4)

#define	VXGE_HAL_DOWN_MSG_GET_LENGTH(ctrl2)		    bVAL12(ctrl2, 4)
#define	VXGE_HAL_DOWN_MSG_LENGTH(val)			    vBIT(val, 4, 12)

	union {
	__hal_nce_create_req_dmq_t			nce_create;
	__hal_nce_update_mac_addr_req_dmq_t		nce_update_mac;
	__hal_nce_update_rch_time_req_dmq_t		nce_update_rch;
	__hal_cqrq_create_req_dmq_t			cqrq_create;
	__hal_cqrq_modify_req_dmq_t			cqrq_modify;
	__hal_cqrq_arm_req_dmq_t			cqrq_arm;
	__hal_cqrq_fcqebw_req_dmq_t			cqrq_fcqebw;
	__hal_srq_create_req_dmq_t			srq_create;
	__hal_srq_modify_req_dmq_t			srq_modify;
	__hal_srq_arm_req_dmq_t				srq_arm;
	__hal_srq_fwqebw_req_dmq_t			srq_fwqebw;
	__hal_srq_wqebadd_req_dmq_t			srq_wqebadd;
	__hal_srq_return_unused_wqes_req_dmq_t		srq_return_wqes;
	__hal_stag_allocate_nsmr_req_dmq_t		stag_allocate_nsmr;
	__hal_stag_allocate_mw_req_dmq_t		stag_allocate_mw;
	__hal_stag_deallocate_req_dmq_t			stag_deallocate;
	__hal_stag_register_nsmr_req_dmq_t		stag_reg_nsmr;
	__hal_stag_re_register_nsmr_req_dmq_t		stag_rereg_nsmr;
	__hal_stag_register_smr_req_dmq_t		stag_reg_smr;
	__hal_stag_query_req_dmq_t			stag_query;
	__hal_stag_validate_req_dmq_t			stag_validate;
	__hal_stag_invalidate_req_dmq_t			stag_invalidate;
	__hal_stag_bind_mw_req_dmq_t			stag_bind_mw;
	__hal_stag_fast_register_nsmr_req_dmq_t		stag_frnsmr;
	__hal_tcp_open_req_dmq_t			tcp_open;
	__hal_tcp_promote_to_iwarp_req_dmq_t		tcp_promote_iwarp;
	__hal_tcp_modify_req_dmq_t			tcp_modify;
	__hal_tcp_update_rxwin_req_dmq_t		tcp_rxwin;
	__hal_tcp_update_mss_req_dmq_t			tcp_mss;
	__hal_tcp_update_ip_header_req_dmq_t		tcp_ipheader;
	__hal_tcp_update_keepalive_req_dmq_t		tcp_keepalive;
	__hal_tcp_relink_to_nce_req_dmq_t		tcp_relink;
	__hal_lro_open_req_dmq_t			lro_open;
	__hal_spdm_open_req_dmq_t			spdm_open;
	} m;

} __hal_down_msg_t;

#define	VXGE_HAL_UP_MSG_HEADER_SIZE	24

/*
 * struct __hal_up_msg_t - Upward Message
 * @control_0: Bits 0 to 31 - Message ID Number.
 *	       Bits 32 to 47 - Message Type.
 *	       Bits 48 to 55 - Reserved for Message Id Selector.
 *	       Bit 56 to 62 - Reserved.
 *	       Bit 63 - UMQ on Completion.
 * @control_1: Contains Opaque handle which is copied from DMQ.
 * @control_2: Bits 0 to 3 - Protocol version number.
 *	       Bits 4 to 15 - Lenght of the message.
 *	       Bits 16 to 47 - Reserved.
 *	       Bits 48 to 63 - Result code.
 * @union: Union of UMQ message structures
 *		  __hal_nce_delete_resp_umq_t		   nce_delete;
 *		  __hal_nce_query_resp_umq_t		   nce_query;
 *		  __hal_cqrq_delete_resp_umq_t		   cqrq_delete;
 *		  __hal_cqrq_query_resp_umq_t		   cqrq_query;
 *		  __hal_cqrq_event_notification_umq_t	   cqrq_event_notif;
 *		  __hal_srq_delete_resp_umq_t		   srq_delete;
 *		  __hal_srq_query_resp_umq_t		   srq_query;
 *		  __hal_srq_event_notification_umq_t	   srq_event_notif;
 *		  __hal_srq_return_unused_wqes_resp_umq_t  srq_return_wqes;
 *		  __hal_stag_allocate_nsmr_resp_umq_t	   stag_allocate_nsmr;
 *		__hal_stag_allocate_mw_resp_umq_t	   stag_allocate_mw;
 *		__hal_stag_deallocate_resp_umq_t	   stag_deallocate;
 *		  __hal_stag_register_nsmr_resp_umq_t	   stag_reg_nsmr;
 *		  __hal_stag_re_register_nsmr_resp_umq_t   stag_rereg_nsmr;
 *		__hal_stag_register_smr_resp_umq_t	   stag_reg_smr;
 *		  __hal_stag_query_resp_umq_t		   stag_query;
 *		__hal_stag_validate_resp_umq_t		   stag_validate;
 *		__hal_stag_invalidate_rep_umq_t		   stag_invalidate;
 *		  __hal_stag_bind_mw_resp_umq_t		   stag_bind_mw;
 *		__hal_stag_fast_register_nsmr_resp_umq_t   stag_frnsmr;
 *		  __hal_tcp_delete_resp_umq_t		   tcp_delete;
 *		  __hal_tcp_promote_to_iwarp_resp_umq_t	   tcp_promote_iwarp;
 *		  __hal_lro_delete_resp_umq_t		   lro_delete;
 *		  __hal_lro_session_candidate_umq_t	   lro_sess_cand;
 *		  __hal_spdm_delete_resp_umq_t		   spdm_delete;
 *		  __hal_spdm_open_resp_umq_t		   spdm_open;
 *		__hal_session_query_resp_umq_t		   session_query;
 *		__hal_session_return_in_progress_wqes_umq_t session_return_wqes;
 *		__hal_session_frame_write_umq_t		   session_frame_write;
 *
 * Created by the adapter (MP) and consumed by the host.
 */
typedef __vxge_os_attr_cacheline_aligned struct __hal_up_msg_t {
	u64		control_0;
#define	VXGE_HAL_UP_MSG_GET_ID_NUMBER(ctrl0)		    bVAL32(ctrl0, 0)
#define	VXGE_HAL_UP_MSG_ID_NUMBER(val)			    vBIT(val, 0, 32)

#define	VXGE_HAL_UP_MSG_GET_MESSAGE_TYPE(ctrl0)		    bVAL16(ctrl0, 32)
#define	VXGE_HAL_UP_MSG_MESSAGE_TYPE(val)		    vBIT(val, 32, 16)

#define	VXGE_HAL_UP_MSG_GET_GEN_COMPL_DMQ(ctrl0)	    bVAL1(ctrl0, 63)
#define	VXGE_HAL_UP_MSG_GEN_COMPL_DMQ(val)		    mBIT(63)
#define	VXGE_HAL_UP_MSG_GEN_COMPL_DMQ_REQUIRED		    0x1
#define	VXGE_HAL_UP_MSG_GEN_COMPL_DMQ_NOT_REQUIRED	    0x0

	u64		control_1;

	u64		control_2;
#define	VXGE_HAL_UP_MSG_GET_PROTOCOL_VERSION(ctrl2)	    bVAL4(ctrl2, 0)
#define	VXGE_HAL_UP_MSG_PROTOCOL_VERSION(val)		    vBIT(val, 0, 4)

#define	VXGE_HAL_UP_MSG_GET_LENGTH(ctrl2)		    bVAL12(ctrl2, 4)
#define	VXGE_HAL_UP_MSG_LENGTH(val)			    vBIT(val, 4, 12)

#define	VXGE_HAL_UP_MSG_GET_RESULT(ctrl2)		    bVAL16(ctrl2, 48)
#define	VXGE_HAL_UP_MSG_RESULT(val)			    vBIT(val, 48, 16)

	union {
	__hal_nce_delete_resp_umq_t			nce_delete;
	__hal_nce_query_resp_umq_t			nce_query;
	__hal_cqrq_delete_resp_umq_t			cqrq_delete;
	__hal_cqrq_query_resp_umq_t			cqrq_query;
	__hal_cqrq_event_notification_umq_t		cqrq_event_notif;
	__hal_srq_delete_resp_umq_t			srq_delete;
	__hal_srq_query_resp_umq_t			srq_query;
	__hal_srq_event_notification_umq_t		srq_event_notif;
	__hal_srq_return_unused_wqes_resp_umq_t	srq_return_wqes;
	__hal_stag_allocate_nsmr_resp_umq_t		stag_allocate_nsmr;
	__hal_stag_allocate_mw_resp_umq_t		stag_allocate_mw;
	__hal_stag_deallocate_resp_umq_t		stag_deallocate;
	__hal_stag_register_nsmr_resp_umq_t		stag_reg_nsmr;
	__hal_stag_re_register_nsmr_resp_umq_t		stag_rereg_nsmr;
	__hal_stag_register_smr_resp_umq_t		stag_reg_smr;
	__hal_stag_query_resp_umq_t			stag_query;
	__hal_stag_validate_resp_umq_t			stag_validate;
	__hal_stag_invalidate_resp_umq_t		stag_invalidate;
	__hal_stag_bind_mw_resp_umq_t			stag_bind_mw;
	__hal_stag_fast_register_nsmr_resp_umq_t	stag_frnsmr;
	__hal_tcp_delete_resp_umq_t			tcp_delete;
	__hal_tcp_promote_to_iwarp_resp_umq_t		tcp_promote_iwarp;
	__hal_lro_delete_resp_umq_t			lro_delete;
	__hal_lro_session_candidate_umq_t		lro_sess_cand;
	__hal_spdm_delete_resp_umq_t			spdm_delete;
	__hal_spdm_open_resp_umq_t			spdm_open;
	__hal_session_query_resp_umq_t			session_query;
	__hal_session_return_in_progress_wqes_umq_t	 session_return_wqes;
	__hal_session_frame_write_umq_t			session_frame_write;
	} m;

} __hal_up_msg_t;

/*
 * struct __hal_umq_dmq_ir_t - Upward/Downward Massaging Queue
 *			     Indication Register
 * @control_0: Bits 1 to 31 - Reserved
 *	       Bits 32 to 63 - UMQ byte count
 * @control_1: DMQ last NOT used DMQ Block Host address.
 * @pad: Space for cache alignment
 *
 * Created and initialized at init time. Location for this structure
 * is provided via a register at init time and updated by adapter (MP) as
 * downward messages are consumed.
 */
typedef __vxge_os_attr_cacheline_aligned struct __hal_umq_dmq_ir_t {
	u64		control_0;
#define	VXGE_HAL_UMQ_DMQ_IR_GET_NO_OF_BYTES_WRITTEN(ctrl0)  bVAL32(ctrl0, 32)
#define	VXGE_HAL_UMQ_DMQ_IR_NO_OF_BYTES_WRITTEN(val)	    vBIT(val, 32, 32)

	u64		control_1;
#define	VXGE_HAL_UMQ_DMQ_IR_GET_DMQ_LAST_NOT_USED_BLOCK_ADDR(ctrl1)	ctrl1
#define	VXGE_HAL_UMQ_DMQ_IR_DMQ_LAST_NOT_USED_BLOCK_ADDR(val)		val

	u8		pad[(((15+__vxge_os_cacheline_size)/
			__vxge_os_cacheline_size)*__vxge_os_cacheline_size)-8];
} __hal_umq_dmq_ir_t;


#define	VXGE_HAL_DMSG_LENGTH_GET(dmsg)					    \
	(u32)VXGE_HAL_DOWN_MSG_GET_LENGTH(vxge_os_ntohll((dmsg)->control_2))

#define	VXGE_HAL_DMSG_LENGTH_SET(dmsg, length)				    \
	(dmsg)->control_2 |= vxge_os_htonll(VXGE_HAL_DOWN_MSG_LENGTH(length))

#define	VXGE_HAL_UMSG_LENGTH_GET(umsg)					    \
	(u32)VXGE_HAL_UP_MSG_GET_LENGTH(vxge_os_ntohll((umsg)->control_2))

#define	VXGE_HAL_UMSG_LENGTH_SET(umsg, length)				    \
	(umsg)->control_2 |= vxge_os_htonll(VXGE_HAL_UP_MSG_LENGTH(length))


void
__hal_down_msg_header_set(vxge_hal_down_msg_h dmsgh, u32 msg_id,
			u32 msg_type, u32 intr_ctrl, u32 compl_umq,
			vxge_hal_opaque_handle_t *opaque_handle, u32 length);

void
vxge_hal_up_msg_header_get(vxge_hal_up_msg_h umsgh,
			    u32 *msg_id,
			    u32 *msg_type,
			    u32 *compl_dmq,
			    vxge_hal_opaque_handle_t **opaque_handle,
			    u32 *length, u32 *result);


void
__hal_nce_create_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	vxge_hal_opaque_handle_t *nce_opaque_handle,
	u64 src_mac_addr,
	u64 rem_mac_addr,
	u32 vlanId,
	u32 host_rch_time);

void
__hal_nce_delete_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle);

void
vxge_hal_nce_delete_resp_umsg_get(
	vxge_hal_up_msg_h umsgh,
	vxge_hal_opaque_handle_t **nce_opaque_handle);

void
__hal_nce_update_mac_addr_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u64 mac_addr);


void
__hal_nce_update_rch_time_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u64 rch_time);

void
__hal_nce_query_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle);

void
vxge_hal_nce_query_resp_umsg_get(
	vxge_hal_up_msg_h umsgh,
	vxge_hal_opaque_handle_t **nce_opaque_handle,
	u64 *src_mac_addr,
	u64 *rem_mac_addr,
	u32 *host_rch_time,
	u32 *adapter_rch_time,
	u32 *valid_flag,
	u32 *vlan_id);

void
__hal_cqrq_create_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	vxge_hal_opaque_handle_t *cq_opaque_handle,
	u32 interrupt_number,
	u32 bitmap_id,
	u32 max_cqe_groups,
	u32 refill_threshold_high,
	u32 refill_threshold_low,
	u32 low_threshold_notify,
	u32 event_notification_mask,
	u32 cqe_period,
	u64 cqrq_ir);

void
__hal_cqrq_delete_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle);

void
vxge_hal_cqrq_delete_resp_umsg_get(
	vxge_hal_up_msg_h umsgh,
	vxge_hal_opaque_handle_t **cq_opaque_handle);

void
__hal_cqrq_modify_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 max_cqe_groups,
	u32 refill_threshold_high,
	u32 refill_threshold_low,
	u32 low_threshold_notify,
	u32 event_notification_mask);

void
__hal_cqrq_query_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle);

void
__hal_cqrq_arm_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 arm_type);

void
__hal_cqrq_fcqebw_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u64 cqe_block_wrapper_hsa,
	u32 cqe_block_wrapper_bytes);

void
__hal_srq_create_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	vxge_hal_opaque_handle_t *srq_opaque_handle,
	u32 max_wqe_od_groups,
	u32 refill_threshold_high,
	u32 refill_threshold_low,
	u32 low_threshold_notify,
	u32 lro_bytestream,
	u32 poll_doorbell,
	u32 srq_arm,
	u32 event_notification_mask,
	u32 srq_limit_value,
	u32 interrupt_number,
	u32 bitmap_id,
	u64 srq_ir);

void
__hal_srq_delete_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle);

void
vxge_hal_srq_delete_resp_umsg_get(
	vxge_hal_up_msg_h umsgh,
	vxge_hal_opaque_handle_t **srq_opaque_handle);

void
__hal_srq_modify_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 max_wqe_od_groups,
	u32 refill_threshold_high,
	u32 refill_threshold_low,
	u32 low_threshold_notify,
	u32 srq_arm,
	u32 event_notification_mask,
	u32 srq_limit_value);

void
__hal_srq_query_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle);

void
__hal_srq_arm_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 srq_limit);

void
__hal_srq_fwqebw_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u64 wqe_block_host_address,
	u32 wqe_block_bytes,
	u32 on_host_tail_wsn);

void
__hal_srq_wqebadd_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 on_host_tail_wsn);

void
__hal_srq_return_unused_wqes_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 number_of_wqes);

void
__hal_stag_allocate_nsmr_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u64 pd_id,
	u32 num_of_pbles,
	u32 stag_index,
	u32 remote_access_flag);

void
vxge_hal_stag_allocate_nsmr_resp_umsg_get(
	vxge_hal_up_msg_h umsgh,
	u32 *stag_index,
	u32 *num_of_pbles);

void
__hal_stag_allocate_mw_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 stag_index,
	u64 pd_id);

void
vxge_hal_stag_allocate_mw_resp_umsg_get(
	vxge_hal_up_msg_h umsgh,
	u32 *stag_index);

void
__hal_stag_deallocate_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 stag_index,
	u32 dealloc_pbl);

void
vxge_hal_stag_deallocate_resp_umsg_get(
	vxge_hal_up_msg_h umsgh,
	u32 *stag_index);

void
__hal_stag_register_nsmr_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 stag_index,
	u32 stag_key,
	u32 pble_size,
	u64 pd_id,
	u64 first_pbl_block_ptr,
	u32 first_pbl_block_length,
	u32 pbl_length,
	u32 va_base_flag,
	u32 remote_access_flag,
	u32 local_write_flag,
	u32 local_read_flag,
	u32 remote_write_flag,
	u32 remote_read_flag,
	u32 enable_mw_binding_flag,
	u32 first_byte_offset,
	u64 mr_length,
	u64 base_to);

void
vxge_hal_stag_register_nsmr_resp_umsg_get(
	vxge_hal_up_msg_h umsgh,
	u32 *stag_index,
	u32 *stag_key,
	u32 *num_of_pbles);

void
__hal_stag_re_register_nsmr_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 stag_index,
	u32 stag_key,
	u32 pble_size,
	u64 pd_id,
	u64 first_pbl_block_ptr,
	u32 first_pbl_block_length,
	u32 pbl_length,
	u32 va_base_flag,
	u32 remote_access_flag,
	u32 local_write_flag,
	u32 local_read_flag,
	u32 remote_write_flag,
	u32 remote_read_flag,
	u32 enable_mw_binding_flag,
	u32 first_byte_offset,
	u64 mr_length,
	u64 base_to);

void
vxge_hal_stag_re_register_nsmr_resp_umsg_get(
	vxge_hal_up_msg_h umsgh,
	u32 *stag_index,
	u32 *stag_key,
	u32 *num_of_pbles);

void
__hal_stag_register_smr_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 base_mr_stag_index,
	u32 va_base_flag,
	u32 remote_access_flag,
	u32 local_write_flag,
	u32 local_read_flag,
	u32 remote_write_flag,
	u32 remote_read_flag,
	u32 enable_mw_binding_flag,
	u32 stag_index,
	u32 stag_key,
	u64 pd_id,
	u64 base_to);

void
vxge_hal_stag_register_smr_resp_umsg_get(
	vxge_hal_up_msg_h umsgh,
	u32 *stag_index,
	u32 *stag_key);

void
__hal_stag_query_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 stag_index);

void
vxge_hal_stag_query_resp_umsg_get(
	vxge_hal_up_msg_h umsgh,
	vxge_hal_stag_info_t *stag_info);

void
__hal_stag_validate_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 stag_index,
	u32 stag_key);

void
vxge_hal_stag_validate_resp_umsg_get(
	vxge_hal_up_msg_h umsgh,
	u32 *stag_index,
	u32 *stag_key);

void
__hal_stag_invalidate_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 stag_index,
	u32 stag_key,
	u64 pd_id);

void
vxge_hal_stag_invalidate_resp_umsg_get(
	vxge_hal_up_msg_h umsgh,
	u32 *stag_index,
	u32 *stag_key);

void
__hal_stag_bind_mw_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 mr_stag_index,
	u32 mr_stag_key,
	u32 mw_stag_index,
	u32 mw_stag_key,
	u32 mr_stag,
	u32 mw_ctrl,
	u64 mw_length,
	u64 tagged_offset);

void
vxge_hal_stag_bind_mw_resp_umsg_get(
	vxge_hal_up_msg_h umsgh,
	u32 *stag_index,
	u32 *stag_key);

void
__hal_stag_fast_register_nsmr_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 mr_control_size,
	u64 mr_control_ptr);

void
vxge_hal_stag_fast_register_nsmr_resp_umsg_get(
	vxge_hal_up_msg_h umsgh,
	u32 *stag_index,
	u32 *stag_key);

void
__hal_tcp_promote_to_iwarp_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 mpa_crc_valid,
	u32 tx_marker_status,
	u32 rx_marker_status,
	u32 iwarp_local_stag_zero_allowed,
	u32 iwarp_wire_stag_zero_allowed,
	u32 inbound_write_readresp_enable,
	u32 inbound_readreq_enable,
	u64 srq_pd_id,
	u64 iwarp_qp_pd_id,
	u32 cqrq_ll_send_frame_data_offset,
	u32 iwarp_qp_limit_value,
	u32 rq_max_length,
	u32 iwarp_qp_limit_armed,
	u32 iwarp_cqrq_sin,
	u32 iwarp_cqrq_sid,
	u32 iwarp_srq_sin,
	u32 iwarp_srq_sid,
	u32 rdmap_version,
	u32 ddp_version,
	u32 mpa_reply_frame_size,
	u32 is_mpa_initiator,
	u32 block_permissions,
	u32 ird_limit,
	u64 hw_cq_rr_ir);

void
__hal_tcp_modify_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 mpa_crc_valid,
	u32 tx_marker_mode,
	u32 rx_marker_mode,
	u32 rdmap_version,
	u32 ddp_version,
	u32 iwarp_qp_limit_armed,
	u32 iwarp_qp_limit_value,
	u32 rq_max_length,
	u32 block_permissions,
	u32 ird_limit);

void
vxge_hal_tcp_promote_to_iwarp_resp_umsg_get(
	vxge_hal_up_msg_h umsgh,
	u32 *rx_imsn);

void
__hal_tcp_delete_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle);

void
vxge_hal_tcp_delete_resp_umsg_get(
	vxge_hal_up_msg_h umsgh,
	vxge_hal_opaque_handle_t **session_opaque_handle);

void
__hal_tcp_abort_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle);

void
__hal_tcp_update_rxwin_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 rcv_buf_valid,
	u32 rcv_buf_size_valid,
	u32 rcv_max_wnd_valid,
	u32 rcv_buf,
	u32 rcv_buf_size,
	u32 rcv_max_wnd);

void
__hal_tcp_update_mss_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 snd_mss_valid,
	u32 snd_mss_limit_valid,
	u32 snd_mss,
	u32 snd_msss_limit);

void
__hal_tcp_update_ip_header_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 ip_tos_valid,
	u32 ip_flowlabel_valid,
	u32 ip_tos,
	u32 ip_flowlabel);

void
__hal_tcp_update_keepalive_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 kpa_ok,
	u32 probes_max,
	u32 idle_time,
	u32 retry_time);

void
__hal_tcp_relink_to_nce_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 nce_id);

void
__hal_lro_end_classification_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle);

void
__hal_lro_delete_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle);

void
vxge_hal_lro_delete_resp_umsg_get(
	vxge_hal_up_msg_h umsgh,
	vxge_hal_opaque_handle_t **session_opaque_handle);


void
__hal_spdm_open_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle,
	vxge_hal_opaque_handle_t *session_opaque_handle,
	u32 l4sp,
	u32 l4dp,
	u32 use_ipv6,
	u32 use_vlan,
	u32 vlan_id,
	u64 ip_sa_high,
	u64 ip_sa_low,
	u64 ip_da_high,
	u64 ip_da_low);

void
__hal_spdm_delete_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle);

void
vxge_hal_spdm_delete_resp_umsg_get(
	vxge_hal_up_msg_h umsgh,
	vxge_hal_opaque_handle_t **session_opaque_handle);

void
__hal_session_query_req_dmsg_set(
	vxge_hal_down_msg_h dmsgh,
	u32 msg_id,
	u32 intr_ctrl,
	u32 compl_umq,
	vxge_hal_opaque_handle_t *opaque_handle);

#ifdef	RNIC_LOOPBACK

void
__hal_down_msg_header_get(vxge_hal_down_msg_h dmsgh, u32 *msg_id,
			u32 *msg_type, u32 *intr_ctrl, u32 *compl_umq,
			vxge_hal_opaque_handle_t **opaque_handle, u32 *length);
void inline
__hal_up_msg_header_set(vxge_hal_up_msg_h umsgh,
			    u32 msg_id,
			    u32 msg_type,
			    u32 compl_dmq,
			    vxge_hal_opaque_handle_t *opaque_handle,
			    u32 length,
			    u32 result);

void
__hal_nce_create_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	vxge_hal_opaque_handle_t **nce_opaque_handle,
	u64 *src_mac_addr,
	u64 *rem_mac_addr,
	u32 *vlanId,
	u32 *host_rch_time);

void
__hal_nce_create_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result);

void
__hal_nce_delete_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	vxge_hal_opaque_handle_t *nce_opaque_handle,
	u32 result);

void
__hal_nce_update_mac_addr_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u64 *mac_addr);

void
__hal_nce_update_mac_addr_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result);

void
__hal_nce_update_rch_time_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u64 *rch_time);

void
__hal_nce_update_rch_time_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result);

void
__hal_nce_query_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result,
	vxge_hal_opaque_handle_t *nce_opaque_handle,
	u64 src_mac_addr,
	u64 rem_mac_addr,
	u64 host_rch_time,
	u64 adapter_rch_time,
	u32 valid_flag,
	u32 vlan_id);

void
__hal_cqrq_create_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	vxge_hal_opaque_handle_t **cq_opaque_handle,
	u32 *interrupt_number,
	u32 *bitmap_id,
	u32 *max_cqe_groups,
	u32 *refill_threshold_high,
	u32 *refill_threshold_low,
	u32 *low_threshold_notify,
	u32 *event_notification_mask,
	u32 *cqe_period,
	u64 *cqrq_ir);

void
__hal_cqrq_create_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result);

void
__hal_cqrq_delete_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	vxge_hal_opaque_handle_t *cq_opaque_handle,
	u32 result);

void
__hal_cqrq_modify_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u32 *max_cqe_groups,
	u32 *refill_threshold_high,
	u32 *refill_threshold_low,
	u32 *low_threshold_notify,
	u32 *event_notification_mask);

void
__hal_cqrq_modify_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result);

void
__hal_cqrq_query_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result,
	vxge_hal_opaque_handle_t *cq_opaque_handle,
	u32 cqe_period,
	u32 bitmap_id,
	u32 max_cqe_groups,
	u32 refill_threshold_high,
	u32 refill_threshold_low,
	u32 low_threshold_notify,
	u32 umq,
	u32 event_notification_mask,
	u32 interrupt_number,
	u32 enable,
	u32 tombstoned,
	u32 eol,
	u32 drained,
	u32 refill,
	u64 cqe_block_wrapper_hsa,
	u32 cqe_block_wrapper_num_bytes,
	u32 cq_indication_cqes_reported,
	u64 cqrq_ir);

void
__hal_cqrq_arm_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u32 *arm_type);

void
__hal_cqrq_arm_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result);
void
__hal_cqrq_event_notification_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 event_type);

void
__hal_cqrq_fcqebw_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u64 *cqe_block_wrapper_hsa,
	u32 *cqe_block_wrapper_bytes);

void
__hal_cqrq_fcqebw_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result);

void
__hal_srq_create_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	vxge_hal_opaque_handle_t **srq_opaque_handle,
	u32 *max_wqe_od_groups,
	u32 *refill_threshold_high,
	u32 *refill_threshold_low,
	u32 *low_threshold_notify,
	u32 *lro_bytestream,
	u32 *poll_doorbell,
	u32 *srq_arm,
	u32 *event_notification_mask,
	u32 *srq_limit_value,
	u32 *interrupt_number,
	u32 *bitmap_id,
	u64 *srq_ir);

void
__hal_srq_create_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result);

void
__hal_srq_delete_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	vxge_hal_opaque_handle_t *srq_opaque_handle,
	u32 result);

void
__hal_srq_modify_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u32 *max_wqe_od_groups,
	u32 *refill_threshold_high,
	u32 *refill_threshold_low,
	u32 *low_threshold_notify,
	u32 *srq_arm,
	u32 *event_notification_mask,
	u32 *srq_limit_value);

void
__hal_srq_modify_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result);

void
__hal_srq_query_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result,
	vxge_hal_opaque_handle_t *srq_opaque_handle,
	u32 virtual_path,
	u32 max_num_wqe_od_grps,
	u32 refill_threshold_high,
	u32 refill_threshold_low,
	u32 low_threshold_notify,
	u32 event_notification_mask,
	u32 srq_limit_armed,
	u32 lro_bytestream,
	u32 srq_state,
	u32 srq_limit_value,
	u32 interrupt_number,
	u32 bitmap_id,
	u32 wb_bytes,
	u64 wb_host_address,
	u64 srq_ir);

void
__hal_srq_arm_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u32 *srq_limit);

void
__hal_srq_arm_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result);

void
__hal_srq_event_notification_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 event_type);

void
__hal_srq_fwqebw_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u64 *wqe_block_host_address,
	u32 *wqe_block_bytes,
	u32 *on_host_tail_wsn);

void
__hal_srq_fwqebw_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result);

void
__hal_srq_wqebadd_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u32 *on_host_tail_wsn);

void
__hal_srq_wqebadd_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result);

void
__hal_srq_return_unused_wqes_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u32 *number_of_wqes);

void
__hal_srq_return_unused_wqes_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	vxge_hal_opaque_handle_t *srq_opaque_handle,
	u32 number_of_wqes_in_msg,
	u32 number_of_wqes_remaining,
	u64 *wqe_opaque_handles);

void
__hal_stag_allocate_nsmr_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u64 *pd_id,
	u32 *num_of_pbles,
	u32 *stag_index,
	u32 *remote_access_flag);

void
__hal_stag_allocate_nsmr_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 stag_index,
	u32 num_of_pbles,
	u32 result);

void
__hal_stag_allocate_mw_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u32 *stag_index,
	u64 *pd_id);

void
__hal_stag_allocate_mw_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 stag_index,
	u32 result);

void
__hal_stag_deallocate_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u32 *stag_index,
	u32 *dealloc_pbl);

void
__hal_stag_deallocate_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 stag_index,
	u32 result);

void
__hal_stag_register_nsmr_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u32 *stag_index,
	u32 *stag_key,
	u32 *pble_size,
	u64 *pd_id,
	u64 *first_pbl_block_ptr,
	u32 *first_pbl_block_length,
	u32 *pbl_length,
	u32 *va_base_flag,
	u32 *remote_access_flag,
	u32 *local_write_flag,
	u32 *local_read_flag,
	u32 *remote_write_flag,
	u32 *remote_read_flag,
	u32 *enable_mw_binding_flag,
	u32 *first_byte_offset,
	u64 *mr_length,
	u64 *base_to);

void
__hal_stag_register_nsmr_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 stag_index,
	u32 stag_key,
	u32 num_of_pbles,
	u32 result);

void
__hal_stag_re_register_nsmr_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u32 *stag_index,
	u32 *stag_key,
	u32 *pble_size,
	u64 *pd_id,
	u64 *first_pbl_block_ptr,
	u32 *first_pbl_block_length,
	u32 *pbl_length,
	u32 *va_base_flag,
	u32 *remote_access_flag,
	u32 *local_write_flag,
	u32 *local_read_flag,
	u32 *remote_write_flag,
	u32 *remote_read_flag,
	u32 *enable_mw_binding_flag,
	u32 *first_byte_offset,
	u64 *mr_length,
	u64 *base_to);

void
__hal_stag_re_register_nsmr_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 stag_index,
	u32 stag_key,
	u32 num_of_pbles,
	u32 result);

void
__hal_stag_register_smr_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u32 *base_mr_stag_index,
	u32 *va_base_flag,
	u32 *remote_access_flag,
	u32 *local_write_flag,
	u32 *local_read_flag,
	u32 *remote_write_flag,
	u32 *remote_read_flag,
	u32 *enable_mw_binding_flag,
	u32 *stag_index,
	u32 *stag_key,
	u64 *pd_id,
	u64 *base_to);

void
__hal_stag_register_smr_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 stag_index,
	u32 stag_key,
	u32 result);

void
__hal_stag_query_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u32 *stag_index);

void
__hal_stag_query_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result,
	u32 stag_index,
	u32 stag_key,
	u32 is_allocated,
	u32 is_window,
	u32 is_zero_based,
	u32 is_valid,
	u32 is_shared,
	u32 allow_bind,
	u32 allow_remote_access,
	u32 allow_remote_read,
	u32 allow_remote_write,
	u32 allow_local_read,
	u32 allow_local_write,
	u64 pd_id,
	u32 sid,
	u32 sin,
	u64 length,
	u64 mr_base_to,
	u64 mw_base_to,
	u32 fbo,
	u32 pble_size,
	u32 root_pbl_record_id,
	u32 pbl_tree_depth,
	u32 parent_stag_record_id,
	u32 parent_stag_valid,
	u32 left_subtree_valid,
	u32 right_subtree_valid,
	u32 left_subtree_record_id,
	u32 right_subtree_record_id,
	u32 num_mws_bound,
	u32 subtree_imbalance,
	u32 max_pbl_length,
	u32 actual_pbl_length,
	u32 underlying_mr_record_id);

void
__hal_stag_validate_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u32 *stag_index,
	u32 *stag_key);

void
__hal_stag_validate_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 stag_index,
	u32 stag_key,
	u32 result);

void
__hal_stag_invalidate_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u32 *stag_index,
	u32 *stag_key,
	u64 *pd_id);

void
__hal_stag_invalidate_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 stag_index,
	u32 stag_key,
	u32 result);

void
__hal_stag_bind_mw_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u32 *mr_stag_index,
	u32 *mr_stag_key,
	u32 *mw_stag_index,
	u32 *mw_stag_key,
	u32 *mr_stag,
	u32 *mw_ctrl,
	u64 *mw_length,
	u64 *tagged_offset);

void
__hal_stag_bind_mw_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 stag_index,
	u32 stag_key,
	u32 result);

void
__hal_stag_fast_register_nsmr_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u32 *mr_control_size,
	u64 *mr_control_ptr);

void
__hal_stag_fast_register_nsmr_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 stag_index,
	u32 stag_key,
	u32 result);

void
__hal_tcp_open_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	vxge_hal_opaque_handle_t **session_opaque_handle,
	u32 *active_opener,
	u32 *tcp_source_port,
	u32 *tcp_dest_port,
	u32 *use_ipv6,
	u32 *use_vlan,
	u32 *vlan_id,
	u64 *ip_sa_high,
	u64 *ip_sa_low,
	u64 *ip_da_high,
	u64 *ip_da_low,
	u32 *bytestream_cqrq_sin,
	u32 *bytestream_cqrq_sid,
	u32 *bytestream_srq_sin,
	u32 *bytestream_srq_sid,
	u32 *break_merge_on_psh,
	u32 *data_merging_enabled,
	u32 *no_wqe_threshold,
	u32 *cqrq_low_latency,
	u32 *send_vlan_up,
	u32 *wscale_ok,
	u32 *sack_ok,
	u32 *ecn_ok,
	u32 *tstamp_ok,
	u32 *kpalive_ok,
	u32 *int_num,
	u32 *int_bitmap,
	u32 *nce_id,
	u32 *vlan_user_priority,
	u32 *dup_ack_thres,
	u32 *ip_tos,
	u32 *ip_flags,
	u32 *ip_flowlabel,
	u32 *kpa_idle_time,
	u32 *kpa_retry_time,
	u32 *retx_time_max,
	u32 *snd_wscale,
	u32 *kpa_probes_max,
	u32 *snd_nxt,
	u32 *snd_mss,
	u32 *snd_mss_limit,
	u32 *rcv_nxt,
	u32 *rcv_wscale,
	u32 *rcv_mss,
	u32 *rcv_buf_size,
	u32 *rcv_max_wnd,
	u32 *rcv_buf,
	u32 *rttmin,
	u32 *ts_recent,
	u64 *nxt_towi_ptr,
	u32 *nxt_towi_cnt,
	u32 *ack_blk_limit,
	u64 *hw_cq_sq);

void
__hal_tcp_open_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result);

void
__hal_tcp_promote_to_iwarp_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u32 *mpa_crc_valid,
	u32 *tx_marker_status,
	u32 *rx_marker_status,
	u32 *iwarp_local_stag_zero_allowed,
	u32 *iwarp_wire_stag_zero_allowed,
	u32 *inbound_write_readresp_enable,
	u32 *inbound_readreq_enable,
	u64 *srq_pd_id,
	u64 *iwarp_qp_pd_id,
	u32 *cqrq_ll_send_frame_data_offset,
	u32 *iwarp_qp_limit_value,
	u32 *rq_max_length,
	u32 *iwarp_qp_limit_armed,
	u32 *iwarp_cqrq_sin,
	u32 *iwarp_cqrq_sid,
	u32 *iwarp_srq_sin,
	u32 *iwarp_srq_sid,
	u32 *rdmap_version,
	u32 *ddp_version,
	u32 *mpa_reply_frame_size,
	u32 *is_mpa_initiator,
	u32 *block_permissions,
	u32 *ird_limit,
	u64 *hw_cq_rr_ir);

void
__hal_tcp_promote_to_iwarp_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 rx_imsn,
	u32 result);

void
__hal_tcp_modify_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u32 *mpa_crc_valid,
	u32 *tx_marker_mode,
	u32 *rx_marker_mode,
	u32 *rdmap_version,
	u32 *ddp_version,
	u32 *iwarp_qp_limit_armed,
	u32 *iwarp_qp_limit_value,
	u32 *rq_max_length,
	u32 *block_permissions,
	u32 *ird_limit);

void
__hal_tcp_modify_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result);

void
__hal_tcp_delete_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	vxge_hal_opaque_handle_t *session_opaque_handle,
	u32 result);

void
__hal_tcp_abort_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result);

void
__hal_tcp_established_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle);

void
__hal_tcp_fin_received_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle);

void
__hal_tcp_time_wait_done_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle);

void
__hal_tcp_update_rxwin_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u32 *rcv_buf_valid,
	u32 *rcv_buf_size_valid,
	u32 *rcv_max_wnd_valid,
	u32 *rcv_buf,
	u32 *rcv_buf_size,
	u32 *rcv_max_wnd);

void
__hal_tcp_update_mss_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u32 *snd_mss_valid,
	u32 *snd_mss_limit_valid,
	u32 *snd_mss,
	u32 *snd_msss_limit);

void
__hal_tcp_update_ip_header_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u32 *ip_tos_valid,
	u32 *ip_flowlabel_valid,
	u32 *ip_tos,
	u32 *ip_flowlabel);

void
__hal_tcp_update_keepalive_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u32 *kpa_ok,
	u32 *probes_max,
	u32 *idle_time,
	u32 *retry_time);

void
__hal_tcp_update_failure_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result);

void
__hal_tcp_fin_ack_received_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle);

void
__hal_tcp_relink_to_nce_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	u32 *nce_id);

void
__hal_tcp_relink_to_nce_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result);

void
__hal_tcp_qp_limit_exceeded_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result);

void
__hal_tcp_rdma_terminate_received_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result);

void
__hal_lro_open_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	vxge_hal_opaque_handle_t **session_opaque_handle,
	u32 *lro_mode,
	u32 *max_sessions,
	u32 *tcp_src_port,
	u32 *tcp_dest_port,
	u64 *ip_sa_high,
	u64 *ip_sa_low,
	u64 *ip_da_high,
	u64 *ip_da_low,
	u32 *use_ipv6,
	u32 *use_vlan,
	u32 *vlan_id,
	u32 *no_wqe_threshold,
	u32 *lro_cqrq_sin,
	u32 *lro_cqrq_sid,
	u32 *lro_srq_sin,
	u32 *lro_srq_sid,
	u32 *dupack_detection_enabled,
	u32 *data_merging_enabled,
	u32 *ack_merging_enabled,
	u32 *llc_hdr_mode,
	u32 *snap_hdr_mode,
	u32 *session_ecn_enabled,
	u32 *session_ecn_enabled_nonce,
	u32 *scatter_mode,
	u32 *buffer_mode,
	u32 *data_lm_mask,
	u32 *dupack_mask,
	u32 *lro_nm_mask,
	u32 *ack_lm_mask,
	u32 *max_ip_datagram_size,
	u32 *lro_rm_mask,
	u32 *frame_threshold,
	u32 *psh_threshold,
	u32 *mtu_threshold,
	u32 *mss_threshold,
	u32 *max_tcp_tsval_delta,
	u32 *max_ack_nbr_delta,
	u32 *buffer0_length,
	u32 *buffer1_length,
	u32 *buffer2_length,
	u32 *buffer3_length,
	u32 *buffer4_length,
	u32 *min_spare_wqe_capacity);

void
__hal_lro_open_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result);

void
__hal_lro_end_classification_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result);

void
__hal_lro_delete_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	vxge_hal_opaque_handle_t *session_opaque_handle,
	u32 result);

void
__hal_lro_session_candidate_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 l4sp,
	u32 l4dp,
	u32 ipv4_not_ipv6,
	u64 ip_sa_low,
	u64 ip_sa_high,
	u64 ip_da_low,
	u64 ip_da_high);

void
__hal_spdm_open_req_dmsg_get(
	vxge_hal_down_msg_h dmsgh,
	vxge_hal_opaque_handle_t **session_opaque_handle,
	u32 *l4sp,
	u32 *l4dp,
	u32 *use_ipv6,
	u32 *use_vlan,
	u32 *vlan_id,
	u64 *ip_sa_high,
	u64 *ip_sa_low,
	u64 *ip_da_high,
	u64 *ip_da_low);

void
__hal_spdm_open_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 session_hash,
	u32 result);

void
__hal_spdm_delete_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	vxge_hal_opaque_handle_t *session_opaque_handle,
	u32 result);

void
__hal_session_event_notification_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result);

void
__hal_session_query_resp_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 result,
	u32 sess_type,
	u32 vp_id,
	u32 sess_inst_num,
	u32 is_spdm,
	u32 use_ipv6,
	u32 use_vlan,
	u32 vlan_id,
	u32 rx_imsn,
	u32 fw_ctrl,
	u32 src_port,
	u32 dest_port,
	u64 ip_sa_high,
	u64 ip_sa_low,
	u64 ip_da_high,
	u64 ip_da_low,
	u32 use_mpa_crc232,
	u32 use_mpa_markers);

void
__hal_session_return_in_progress_wqes_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 number_of_wqes_in_msg,
	u32 number_of_wqes_remaining,
	u64 *wqe_opaque_handles);

void
__hal_session_frame_write_umsg_set(
	vxge_hal_up_msg_h umsgh,
	u32 msg_id,
	u32 compl_dmq,
	vxge_hal_opaque_handle_t *opaque_handle,
	u32 umq_opaque_data,
	u32 num_markers,
	u32 first_marker_index,
	u64 fdata[2]);

#endif

__EXTERN_END_DECLS

#endif /* VXGE_HAL_MSG_H */
